/*** A_DEL_PSTAT.   Delete a person from a conference.
/
/    ok = a_del_pstat (pstat, mask, wait);
/
/    Parameters:
/       int       ok;       (return status)
/       Pstat     pstat;    (object containing status info)
/       long      mask;     (ignored)
/       int       wait;     (synchronicity flag)
/
/    Purpose:
/       Delete a person from a conference.  Delete their participation
/       status record, and remove them from the membership list for
/       that conference.
/
/    How it works:
/       Caller must fill in pstat->cnum, pstat->owner.
/
/    Returns: A_OK on success
/             A_NOCONF    if no such conference
/             A_NOPERSON  if no such person
/             A_NOTOWNER  owner is not current userid
/             A_NOTMEMBR  not a member
/
/    Error Conditions:
/
/    Side effects: 
/
/    Related functions:
/
/    Called by:  UI
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  api/agetpsta.c
/ */

/*** Copyright (C) 1992 Camber-Roth.  All rights reserved. */

/*: CR 12/09/92 22:40 New function. */
/*: CR 12/13/92 22:32 Add comments. */
/*: CR  2/22/93 23:31 Add new args to a_cache() calls. */
/*: CP  5/06/92 12:30 Add 0L as modnames 'item' arg. */
/*: CP  8/01/93  9:37 Add debugging stuff. */

#include <stdio.h>
#include "caucus.h"
#include "chixuse.h"
#include "api.h"
#include "done.h"

FUNCTION  a_del_pstat (pstat, mask, wait)
   Pstat  pstat;
   long   mask;
   int    wait;
{
   int      success, access, additem, chgresp, ok, chgsubj;
   Chix     userid, str;
   Person   person;
   Namelist add, del, nlnode();

   /*** Check validity of arguments. */
   if (pstat == (Pstat) NULL  ||  pstat->tag != T_PSTAT)  return (A_BADARG);
   if (pstat->cnum <= 0)                                  return (A_NOCONF);

   /*** Allocate miscellaneous stuff. */
   person = a_mak_person("adelpsta person");
   userid = chxalloc (20L, THIN, "a_del_ps userid");
   str    = chxalloc (40L, THIN, "a_del_ps str");

   /*** Regular users can only delete themselves; organizers can delete 
   /    anyone. */
   sysuserid (userid);
   a_cache (V_GET, pstat->cnum, &access, &additem, &chgresp, 
                   &chgsubj, nullchix);
   if (NOT chxeq (userid, pstat->owner)  &&  access < 3)  DONE   (A_NOTOWNER);

   /*** Make sure user is a member of this conference. */
   if (NOT unit_lock  (XUPA, WRITE, L(pstat->cnum), 0L, 0L, 0L, userid))
                                                          DONE   (A_NOTMEMBR);
   if (NOT unit_check (XUPA))     { unit_unlk (XUPA);     DONE   (A_NOTMEMBR); }

   /*** Delete participation record. */
   unit_kill (XUPA);
   unit_unlk (XUPA);

   /*** Get person's name... */
   chxcpy (person->owner, userid);
   ok = a_get_person (person, P_NAME | P_LASTNM, A_WAIT);
   if (ok != A_OK)   DONE (ok);

   /*** ... and then remove the user from the membership list for this 
   /    conference. */
   tomember (str, person->lastname, person->name, userid);
   add = nlnode(1);
   del = nlnode(1);
   nladd  (del, str, 0);
   modnames (XCMD, XCMN, nullchix, pstat->cnum, 0L, add, del, NOPUNCT);
   nlfree (add);
   nlfree (del);

   pstat->ready = 1;
   DONE (A_OK);

done:
   a_fre_person (person);
   chxfree (userid);
   chxfree (str);
   return  (success);
}
