/*** A_FRE_CLIST.   Free (deallocate) a Clist object.
/
/    a_fre_clist (clist);
/
/    Parameters:
/       Clist     clist;   (object to be freed)
/
/    Purpose:
/       Deallocate and release an object created by a_mak_clist().
/
/    How it works:
/
/    Returns: 1 if proper object type
/             0 if clist is null or not an Clist.
/
/    Error Conditions:
/  
/    Side effects: 
/
/    Related functions:  a_mak_clist()
/
/    Called by:  UI
/
/    Operating system dependencies:
/
/    Known bugs:      none
/
/    Home:  api/afreclist.c
/ */

/*** Copyright (C) 1992 Camber-Roth.  All rights reserved. */

/*: CR 11/15/92 17:47 New function. */
/*: CR 12/07/92  0:40 Make Lint happier. */
/*: CP  8/01/93  9:37 Add debugging stuff. */

#include <stdio.h>
#include "chixuse.h"
#include "api.h"

extern int api_debug;

FUNCTION  a_fre_clist (clist)
   Clist  clist;
{
   Clist  next;

   if (clist == (Clist) NULL)  return (0);
   if (clist->tag != T_CLIST)  return (0);

   for (;   clist != (Clist) NULL;   clist=next) {
      if (api_debug) a_debug (A_FREE, (Ageneric) clist);
      clist->tag = 0;
      chxfree (clist->lname);
      chxfree (clist->tname);
      next = clist->next;
      free ( (char *) clist);
   }

   return  (1);
}
