/*** A_MAK_CLIST.   Make (allocate) an Clist object.
/
/    clist = a_mak_clist (name);
/
/    Parameters:
/       Clist   clist   (returned/created object)
/       char   *name;   (name for debugging purposes)
/
/    Purpose:
/       Allocate and initialize an Clist object.
/
/    How it works:
/
/    Returns: newly created object
/             NULL on error
/
/    Error Conditions: insufficient heap memory, or heap catastrophe.
/  
/    Side effects: 
/
/    Related functions:  a_fre_clist()
/
/    Called by:  UI
/
/    Operating system dependencies:
/
/    Known bugs:      none
/
/    Home:  api/amakclis.c
/ */

/*** Copyright (C) 1992 Camber-Roth.  All rights reserved. */

/*: CR 11/15/92 17:47 New function. */
/*: CP  8/01/93  9:37 Add 'name' argument, debugging stuff. */

#include <stdio.h>
#include "chixuse.h"
#include "api.h"

extern int api_debug;

FUNCTION  Clist  a_mak_clist (name)
   char  *name;
{
   Clist  clist;
   char  *malloc();

   clist = (Clist) malloc (sizeof (struct Clist_t));
   if (clist == (Clist) NULL)  return (clist);

   clist->tag    = T_CLIST;
   A_SET_NAME (clist->dname, name);
   clist->lname  = chxalloc (20L, THIN, "a_mak_cl lname");
   clist->tname  = chxalloc (20L, THIN, "a_mak_cl tname");
   clist->cnum   = 0;
   clist->member = 0;
   clist->access = 0;
   clist->next   = (Clist) NULL;
   clist->ready  = 0;

   if (api_debug) a_debug (A_ALLOC, (Ageneric) clist);

   return (clist);
}
