
/*** CARVSTAT.  Produce status report from list of carv-marked files.
/
/    Invoked as:
/       carvstat_x  group   input   output
/
/    Arguments:
/       group      group (directory) prefix, e.g. Caucus "ced"
/       input      name of file containing list of files to be examined
/       output     name of file to which status report is appended
/
/    Purpose:
/       Carvstat examines the carv-marks in a list of files, and
/       produces a report summarizing the number and signature of
/       the carv'd changes for all of the files.
/
/    How it works:
/       Report lines consist of the group, the file name, and the list
/       of initials from the carve-marked lines in a file.  For example,
/       in group CED, file CEDEDCOM.C, with marks by AA, CR, and JV in that
/       order, the report line would look like "ced cededcom.c AA CR JV".
/   
/       The status file is in turn used by another utility, CARVCHEK, which
/       reports on the relative status of the marked sources on two
/       different machines by comparing their CARVSTAT status files.
/
/    Exit status: 0
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  carves/carvstat.c
/ */

/*** Copyright (C) 1988 Camber-Roth.  All rights reserved. */

/*: CR  3/01/89 15:56 Start carve-marking carve source code. */
/*: CR 12/08/89 16:12 Use expanded system type names and "typesys.h". */
/*: CR  9/06/91 14:01 Expand maxline size to 400. */
/*: CR  8/21/92 13:17 Remove #define sys types to systype.h. */
/*: CR  9/11/92 12:39 New header comments; parse "#:" carvmarks. */
/*: CR 11/16/92 11:24 Raise MAXLINE to 1000. */

#include <stdio.h>
#include "typesys.h"

#define  MAXLINE  1000
#define  NOT    !
#define  FUNCTION

FUNCTION main (argc, argv)
   int   argc;
   char *argv[];
{
   FILE *in, *ot, *fopen();
   char  file[MAXLINE];

   if (argc < 4) {
      printf ("Usage:  carvstat_x  group  input   output\n");
      exit   (1);
   }

   if ( (in = fopen (argv[2], "r")) == NULL) {
      printf ("Can't open input file '%s'\n", argv[2]);
      exit   (1);
   }

#if UNIX | MSDOS | VAXVMS
   if ( (ot = fopen (argv[3], "a"))  == NULL) {
#endif
#if PRIMOS
   if ( (ot = fopen (argv[3], "wa")) == NULL) {
#endif
      printf ("Can't open output file '%s' for append\n", argv[3]);
      exit   (1);
   }

   strlower (argv[1]);

   while (fgets (file, 100, in) != NULL) {
      stripnl (file);
      process (file, ot, argv[1]);
   }

   fclose (in);
   fclose (ot);
}

/*-------------------------------------------------------*/
FUNCTION stripnl (file)
   char  file[];
{
   int   i;

   /*** Remove trailing newlines, blanks, and tabs. */
   for (i = strlen (file);
        file[i-1]=='\n'  ||  file[i-1]==' '  ||  file[i-1]=='\t';
        --i) ;
   file[i] = '\0';

   /*** Remove leading blanks. */
   for (i=0;   file[i]==' ';   ++i) ;
   strcpy (file, file+i);
}

/*-------------------------------------------------------*/

FUNCTION process (file, ot, group)
   char  file[], group[];
   FILE *ot;
{
   FILE *fp, *fopen();
   int   this_carv, found_carv;
   char  initials[MAXLINE], line[MAXLINE];

   if ( (fp = fopen (file, "r")) == NULL)  return (0);

   sprintf (initials, "%-8s %-10s: ", group, file);
   for (found_carv=0;   fgets (line, 120, fp) != NULL; ) {
      this_carv = 0;

      if (line[0]=='/'  &&  line[1]=='*'  &&  line[2]==':') {
         this_carv = found_carv = 1;
         line[6] = '\0';
         strcat (initials, line+4);
         strcat (initials, " ");
      }

      if (line[0]=='#'  &&  line[1]==':') {
         this_carv = found_carv = 1;
         line[5] = '\0';
         strcat (initials, line+3);
         strcat (initials, " ");
      }

      if (found_carv  &&  NOT this_carv)  break;
   }
   fprintf (ot, "%s\n", initials);
   fclose  (fp);
   return  (1);
}

/*-------------------------------------------------------*/

#include <ctype.h>

FUNCTION strlower (s)
   char  *s;
{
   for (;   *s;   ++s)  if (isupper(*s))  *s = tolower(*s);
}

/*-------------------------------------------------------*/

FUNCTION streq (a, b)
   char  *a, *b;
{
   while (*a == *b) {
      if (NOT *a)  return (1);
      ++a;
      ++b;
   }
   return (0);
}
