/*** CEDINTEXT.  Caucus EDitor INsert TEXT.
/
/    Prompt for text from the user, and insert it at the current
/    Mark in the CED editing file.  If TRUNCATE is true, truncate
/    (with a warning) all lines longer than 77 characters. */

/*** Copyright (C) 1984 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:32 Source code master starting point */
/*: CR  4/25/89 15:13 Return on reading EOT from user. */
/*: CW  5/18/89 15:20 Add XW unit codes. */
/*: CW  7/21/89 15:57 Echoinput() input text line. */
/*: CR  4/08/91 18:48 Add new unit_lock argument. */
/*: UL 12/11/91 18:41 Make unit_lock args 3-6 long.*/
/*: JX  3/28/92 11:00 Rewrite to use memory instead of disk. Chixify. */
/*: JX  5/16/92 16:10 Fix Chix. */
/*: CL 12/10/92 15:22 Long chxindex args. */
/*: CL 12/11/92 11:12 Long Chxcatsub args. */
/*: CK  9/20/93 16:48 Complain if cedinfile can't read all of redirected file. */

#include <stdio.h>
#include "caucus.h"
#include "chixuse.h"
#include "ced.h"

extern struct flag_template  flag;
extern Userreg               thisuser;
extern union  null_t         null;
extern Chix   ss_dotnl, ss_inred;

FUNCTION  cedintext (ced, truncate)
   int   truncate;
   Ced  *ced;
{
   static Chix s_quest;
   Chix  file, str, prompt;
   short  length, redirect;
   long   redlen;

   ENTRY ("cedintext", "");

   str  = chxalloc (100L, THIN, "cedint str");
   file = chxalloc (40L, THIN, "cedint file");
   prompt = chxalloc (40L, THIN, "cedint prompt");

   length = (truncate       ? 78 : 200);
   mdstr (prompt, "ced_Kquest", &s_quest);
   if (flag.no1prompt)  chxclear (prompt);

   redlen = chxlen (ss_inred);

   while (1) {

      /*** Make sure we haven't overflowed max number of lines. */
      if (ced->size >= MAXCED - 1) {
         mdwrite (XWER, "ced_Tmaxlin", null.md);
         goto done;
      }

      /*** Get the next line (or command) from the user.  Break out of
      /    insert loop and return if we hit EOT. */
      if (prompter (prompt, nullchix, 0, "ced_Hin",
                    length, str, nullchix) < -3)
         break;
      if (chxeq (str, thisuser->eot))  {
         chxclear (str);  
         break;
      }
      echoinput (prompt, str);

      /*** Test for input redirection? */
      redirect = (chxindex (str, 0L, ss_inred) == 0);

      if (redirect  &&  NOT flag.redirect)  
         mdwrite (XWER, "err_Tinred", null.md);

      /*** Handle input redirection. */
      else if (redirect) {
         chxcatsub (file, str, redlen, L(1000));
         unit_lock (XIRE, READ, 0L, 0L, 0L, 0L, file);
         if (NOT unit_view (XIRE)) {
            mdwrite    (XWER, "ced_Tnoopen", null.md);
            unit_write (XWER, file);
            unit_write (XWER, ss_dotnl);
         }
         else {
            mdwrite    (XWTX, "ced_Treading", null.md);
            unit_write (XWTX, file);
            mdwrite    (XWTX, "ced_Tdot3",    null.md);
            if (NOT cedinfile  (ced, XIRE)) 
               mdwrite (XWER, "ced_Epartial", null.md);
            unit_close (XIRE);
            unit_unlk  (XIRE);
            mdwrite    (XWTX, "ced_Tcontype", null.md);
         }
      }

      else   cedinline (ced, str);
   }
done:
   chxfree (str);
   chxfree (file);
   chxfree (prompt);
   RETURN (1);
}
