/*** COPY_REDIRECT.  Read from input redirection file, write to unit N.
/
/    copy_redirect (n, name);
/  
/    Parameters:
/       int    n;        (write contents of NAME to unit N)
/       Chix   name;     (name of input redirection file)
/  
/    Purpose:
/      COPY_REDIRECT takes care of the messy details of locking, opening,
/      and reading from an input redirection file NAME, while writing to
/      another (already locked and opened) file unit N.
/  
/    How it works:
/  
/    Returns: 1 on success
/             0 on failure to open NAME.
/  
/    Error Conditions:
/  
/    Side Effects:
/  
/    Related to:
/  
/    Called by: 
/  
/    Home: ced/copyredi.c
/ */

/*** Copyright (C) 1990-1992 Camber-Roth.  All rights reserved. */

/*: CR  4/16/90 15:57 New function. */
/*: CR  4/08/91 18:48 Add new unit_lock argument. */
/*: UL 12/11/91 18:41 Make unit_lock args 3-6 long.*/
/*: JX  3/28/92 11:45 Chixify. */
/*: JX  5/16/92 16:47 Fix Chix. */
/*: CR 10/21/92 11:40 Rename from input_redirect() to copy_redirect(). */

#include <stdio.h>
#include "caucus.h"
#include "chixuse.h"

extern struct flag_template flag;
extern union                null_t null;
extern Chix                 ss_dotnl;

FUNCTION  copy_redirect (n, name)
   int    n;
   Chix   name;
{
   int    lock, success;
   Chix   str;

   ENTRY ("copy_redirect", "");

   /*** Make sure input redirection is allowed. */
   if (NOT flag.redirect) {
      mdwrite (XWER, "err_Tinred", null.md);
      RETURN (0);
   }
 
   str = chxalloc (100L, THIN, "inputr str");  
   success = 1;
   lock    = unit_lock (XIRE, READ, 0L, 0L, 0L, 0L, name);
   if (NOT lock  ||  NOT unit_view (XIRE)) {
      success = 0;
      mdwrite    (XWER, "ced_Tnoopen", null.md);
      unit_write (XWER, name);
      unit_write (XWER, ss_dotnl);
   }
   else {
      mdwrite    (XWTX, "ced_Treading", null.md);
      unit_write (XWTX, name);
      mdwrite    (XWTX, "ced_Tdot3",    null.md);
      while   (unit_read (XIRE, str, 0L)) {
         sysclean(str);
         fixtabs (str);
         unit_write (n, str);
         unit_write (n, CQ("\n"));
      }
      unit_close (XIRE);
      mdwrite (XWTX, "ced_Tcontype", null.md);
   }
   if (lock) unit_unlk (XIRE);

   chxfree (str);
   RETURN (success);
}
