/*** ISEDITREQ.   Is the string STR an edit request? 
/
/    ISEDITREQ is supplied with a string STR, the name of the DEFaultEDITOR,
/    and a table of optional OPTWORDS that will be recognized as an edit
/    request keyword.
/
/    Edit requests are one or two words, and can take the forms:
/         edit                        (or "e" or "ed" or "edi")
/         edit name_of_editor
/         :
/         :name_of_editor
/         'word'                      (where 'word' is a member of OPTWORDS)
/         'word' name_of_editor
/
/    In the one-word cases, ISEDITREQ sets EDITOR to the supplied DEFEDITOR,
/    and returns 1.  In the two-word cases (includes :name_of_editor),
/    ISEDITREQ sets EDITOR to the named editor, and returns 1.  In all other
/    cases ISEDITREQ returns 0.
/
/    Note: ISEDITREQ recognizes various abbreviations of "edit caucus",
/    e.g. "edit cau" is expanded to "edit caucus".  */

/*: AA  7/29/88 17:32 Source code master starting point */
/*: AM  2/07/89 10:24 Jcaucus changes*/
/*: JX  3/28/92 11:50 Chixify. */
/*: JX  5/16/92 16:50 Fix Chix. */
/*: CL 12/11/92 11:12 Long Chxcatsub args. */
/*: JV  6/28/93 15:54 Fix "edit edit" bug. */

#include "caucus.h"
#include "chixuse.h"
#include <stdio.h>

extern union null_t null;
extern Chix ss_caucus;

FUNCTION  iseditreq (str, optwords, defeditor, editor)
   Chix   str, optwords, defeditor, editor;
{
   static Chix isedtable;
   Chix   com, word1, word2, word3;
   int    words, success;

   ENTRY ("iseditreq", "");

   com = chxalloc (70L, THIN, "ised com");
   word1 = chxalloc (70L, THIN, "ised word1");
   word2 = chxalloc (70L, THIN, "ised word2");
   word3 = chxalloc (70L, THIN, "ised word3");

   mdtable ("ced_Aised", &isedtable);
   chxcatsub (com, str, 0L, L(68));
   jixreduce (com);

   words  = chxtoken (word1, nullchix, 1, com) > -1;
   words += chxtoken (word2, nullchix, 2, com) > -1;
   words += chxtoken (word3, nullchix, 3, com) > -1;

   if (words==3)  FAIL;

   if (tablematch (isedtable, word1) == 0  ||
       tablematch (optwords,  word1) == 0) {
      if (words==1)  chxcpy (editor, defeditor);
      if (words==2) {
         if (chxabr (ss_caucus, word2) == 2 &&  chxlen (word2) >= 3)
                     chxcpy (editor, ss_caucus);
         else        chxcpy (editor, word2);
      }
      SUCCEED;
   }

   FAIL;
done:
   chxfree (com);
   chxfree (word1);
   chxfree (word2);
   chxfree (word3);
   RETURN (success);
}
