/*** TEXTFORM.  Format text.
/
/    TEXTFORM reformats the text in file number IN according to TRUNCATE,
/    and places the reformatted text back in file IN.
/
/    If TRUNCATE is false, the text is reformatted so as to pack lines
/    as close to 76 characters as possible.  Blank lines are left alone;
/    lines starting with a blank or tab start a new line.
/
/    If TRUNCATE is true, TEXTFORM scans IN for any overlong (>76) lines.
/    If there are any, it prompts the user to make sure s/he wants to
/    truncate those lines.  The user can chose to truncate, reformat, or
/    edit the text.  In the first two cases the appropriate action is
/    taken, and TEXTFORM RETURNs 1.  In the last case no action is
/    taken, and TEXTFORM RETURNs 0.
/    It also returns 0 on any error.  */

/*: AA  7/29/88 17:32 Source code master starting point */
/*: CR  4/08/91 18:48 Add new unit_lock argument. */
/*: UL 12/11/91 18:41 Make unit_lock args 3-6 long.*/
/*: JX  3/28/92 13:10 Chixify. */
/*: JX  5/16/92 16:55 TRUNCATE should be a Flag, not an int! */

#include "caucus.h"
#include "chixuse.h"
#include <stdio.h>

FUNCTION  textform (in, truncate, editor)
   int    in;
   Flag   truncate;
   Chix   editor;
{
   Chix   str;
   int    worked, choice, success;

   ENTRY ("textform", "");

   str = chxalloc (100L, THIN, "textf str");

   /*** Open the in & XITY files. */
   if (NOT unit_view (in)  )   {success =  -1; goto done; }
   unit_lock (XITY, WRITE, 0L, 0L, 0L, 0L, nullchix);
   if (NOT unit_make (XITY) )  {
      unit_close (in);
      unit_unlk  (XITY);
      success = -1; goto done;
   }

   if (truncate) {
      choice = texttrunc (in, XITY, str, editor);
      if      (choice == 0)  worked = 1;
      else if (choice == 2)  worked = 0;
      else if (choice == 1)  {
         unit_close (in);    unit_view (in);
         unit_close (XITY);  unit_make (XITY);
         truncate = 0;
      }
   }

   if (NOT truncate) {
      textreform (in, XITY, str);
      worked = 1;
   }

   unit_close (in);
   unit_close (XITY);

   if (worked) {
      unit_kill (in);
      unit_move (XITY, in);
   }

   unit_unlk   (XITY);

done:
   chxfree (str);
   if (success == -1) worked = -1;
   RETURN (worked);
}
