
/*** STORE_ULIST.   Store a conference userlist 'ulist' into a file.
/
/    ok = store_ulist (cnum, u, ulist);
/
/    Parameters:
/       int       ok;         (success?)
/       int       cnum;       (conference number)
/       int       u;          (unit number of file; usually XCUS)
/       Chix     *ulist;      (write this userlist)
/
/    Purpose:
/       Store a conference userlist ULIST into a file.
/
/    How it works:
/
/    Returns: 1 on success.
/             0 if file cannot be written, or other error.
/
/    Error Conditions
/  
/    Side effects:
/
/    Related functions:
/
/    Called by: 
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  ulist/storeulis.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: CR 10/30/91 16:42 New function. */
/*: CR 10/31/91 12:26 Fix double ;;. */
/*: CR 10/31/91 12:36 Check each ulist[index] for nullchix. */
/*: CR 11/06/91 11:14 Decl strtoken(). */
/*: UL 12/11/91 18:41 Make unit_lock args 3-6 long.*/
/*: DE  3/17/92 16:57 Chixified */
/*: JX  5/13/92 11:45 Removed xtra vars. */
/*: CR 12/07/92 15:12 Add new U argument; chxfree word2! */
/*: CL 12/11/92 11:13 Long Chxcatsub args. */

#include <stdio.h>
#include "caucus.h"

extern union  null_t         null;

FUNCTION  store_ulist (cnum, u, ulist)
   int    cnum, u;
   Chix  *ulist;
{
   static short order[] = { 5, 2, 4, 3};
   static Chix inextab;
   int    index, success=0;
   long   para, subp;
   Chix   temp, word, word2;

   ENTRY ("store_ulist", "");

   temp   = chxalloc (200L, THIN, "storeuli temp");
   word   = chxalloc (40L,  THIN, "storeuli word"); 
   word2  = chxalloc (40L,  THIN, "storeuli word2"); 

   /*** Open the userlist file for writing. */
   if (NOT unit_lock (u, WRITE, L(cnum), 0L, 0L, 0L, nullchix)) FAIL;
   if (NOT unit_make (u))                    { unit_unlk (u);   FAIL; }

   mdtable  ("mai_Ainex", &inextab);

   for (index=0;   index<MAXULIST;   ++index) {
      para = (index + 3) / 4;
      subp = (index + 3) % 4;

      if (ulist[index] == nullchix)  continue;

      if (index > 0) {
         chxtoken   (word, nullchix, order[subp], inextab);
         chxclear   (word2);
         chxcatsub  (word2, word, L(1), ALLCHARS);
         chxclear   (temp);
         chxformat  (temp, CQ(":%d.%d %s\n"), para, subp, word2, nullchix);
         unit_write (u, temp);
      }

      unit_write (u, ulist[index]);
   }

   unit_close (u);
   unit_unlk  (u);

   SUCCEED;

 done:

   chxfree ( temp );
   chxfree ( word );
   chxfree ( word2 );

   RETURN  ( success );
}
