/*** CHXINDEX.  Find position of chix Y in chix X, starting at position POS.
/
/    n = chxindex (x, pos, y);
/
/    Parameters:
/       long  n;     (character position in X where Y starts)
/       Chix  x;     (search in this chix...)
/       long  pos;   (...starting at this position...)
/       Chix  y;     (...for this chix.)
/
/    Purpose:
/       Search chix X, starting at position POS, for the first occurence of 
/       chix Y.  Return the number of the character position in X where Y 
/       starts.  (Note that, as usual, Y may be found in X, even if they have
/       different types, so long as their corresponding characters match
/       after promotion.)
/
/    How it works:
/       Starting at POS, for each character in X, see if that character begins
/       a match with Y.  Call chxscan() to check for the match.
/
/    Returns: N, number of character position in X where Y starts.
/       The first character of X is number 0.
/
/       Returns -1 if:  Y is not found in X,
/                       Y or X are empty.
/                       Y or X are not chix.
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  application
/
/    Operating system dependencies: none
/
/    Known bugs:      none
/
/    Home:  chx/chxindex.c
/ */

/*** Copyright (C) 1990 Camber-Roth.  All rights reserved. */

/*: CX 12/10/90 23:36 New function. */
/*: CR 10/18/91 14:54 Add POS argument. */
/*: CR 12/10/92 14:56 Change POS & return value to long. */
/*: CL 12/11/92 13:47 Long Chxvalue N. */

#include <stdio.h>
#include "chixdef.h"

FUNCTION  long chxindex (x, pos, y)
   Chix   x, y;
   long   pos;
{
   long   x_end, y_end, sum;
   long   i, j;
   char  *p;

   ENTRY ("chxindex", "");

   /*** Input validation. */
   if (NOT chxcheck (x, "chxindex"))   RETURN (-1);
   if (NOT chxcheck (y, "chxindex"))   RETURN (-1);
   if (chxvalue (y, 0L) == 0L)         RETURN (-1);
   if (x->actlen < 0)  chxlen (x);
   if (pos >= x->actlen)               RETURN (-1);

   /*** Starting at POS, examine each character in X.  Does it start a
   /    match with Y? */
   for (i=pos, p = x->p + pos * x->type;    (1);    ++i, p += x->type) {

      /*** Are we at the end of X? */
      for (sum=0L, j=0;   j<x->type;   ++j)  sum |= p[j];
      if  (sum == 0L)  break;

      chxscan (p, x->type, y->p, y->type, &x_end, &y_end);
      if (y_end == 0L)  RETURN (i);
   }

   RETURN (-1);
}
