
/*** CHXLEN.  Compute length of a chix, in characters.
/
/    i = chxlen (x);
/
/    Parameters:
/       long  i;
/       Chix  x;
/
/    Purpose:
/       Count up and return the number of characters (not bytes) in
/       chix X.
/
/    How it works:
/       Scans each character in X until it finds one that has *all*
/       bytes equal to 0.  (If it doesn't find such a zero, force one
/       at the maximum length of the string, and complain.)
/
/    Returns: n >= 0 for a valid chix
/                  0 for a nullchix
/                 -1 on error
/
/    Error Conditions:  invalid chix (also calls chxerror to handle errors)
/  
/    Side effects:      none
/
/    Related functions:
/
/    Called by:  chx...() functions, and application.
/
/    Operating system dependencies: none
/
/    Known bugs:      none
/
/    Home:  chx/chxlen.c
/ */

/*** Copyright (C) 1990 Camber-Roth.  All rights reserved. */

/*: CX 12/10/90 23:36 New function. */
/*: CR  7/23/91 20:06 Allow X to be a nullchix. */
/*: CR 10/15/91 18:51 Set actlen on forced termination. */
/*: CR 12/10/92 14:57 Change return value to long. */

#include <stdio.h>
#include "chixdef.h"

FUNCTION  long chxlen (x)
   Chix   x;
{
   long   i, j, n;
   int    sum;

   ENTRY ("chxlen", "");

   if (x == nullchix)                  RETURN ( 0);
   if (NOT chxcheck (x, "chxlen"))     RETURN (-1);
   
   /*** If we already know the length, return it. */
   if (x->actlen >= 0)                 RETURN ((int) x->actlen);

   /*** For each character, add up all the bytes.  If they sum to
   /    0, we've hit the end of the string. */
   for (i=n=0;   n < x->maxlen;   i += x->type, n++) {
      for (j=sum=0;   j < x->type;   ++j)  sum |= x->p[i+j];
      if  (sum==0)  { x->actlen = n;   RETURN (n); }
   }

   /*** If we got here, there was no 0 termination!  Complain, and
   /    force a termination. */
   i = --n * x->type;
   for (j=0;   j < x->type;   ++j)  x->p[i+j] = 0;
   chxerror (CXE_NOEND, "chxlen", x->name);
   x->actlen = n;
   RETURN   (n);
}
