
/*** CHXLONG.  Scan a chix for a long integer.
/
/    v = chxlong (a, i);
/
/    Parameters:
/       long  v;
/       Chix  a;
/       long *i;    (scan from this character position in A.)
/
/    Purpose:
/       Scan a chix for an integer.
/
/    How it works:
/       Scan chix A, starting at position I, for an integer.  The integer
/       may consist of leading blanks and a leading plus or minus sign, 
/       followed by the digits 0 through 9.  I is set to the position of 
/       the first character in A after the integer.
/
/       If an error occurs, or there is no integer at position I, I is
/       unchanged.
/
/    Returns: value of integer
/             (undefined on error)
/
/    Error Conditions:
/       A not a chix
/       I is NULL
/      *I is out of range
/  
/    Side effects:
/
/    Related functions: chxnum().
/
/    Called by:  Application, chxnum().
/
/    Operating system dependencies: none
/
/    Known bugs:      none
/
/    Home:  chx/chxlong.c
/ */

/*** Copyright (C) 1990 Camber-Roth.  All rights reserved. */

/*: CX 12/10/90 23:36 New function. */
/*: CR  7/23/91 23:19 Allow leading '+' sign. */
/*: CL 12/11/92 13:47 Long Chxvalue N. */

#include <stdio.h>
#include "chixdef.h"

#define  DIGIT(x)   ('0'<=x  &&  x<='9')

FUNCTION  long chxlong (a, i)
   Chix   a;
   long  *i;
{
   long   j, j0;
   int    minus = 0;
   long   result, c, sign;

   ENTRY ("chxlong", "");

   if (NOT chxcheck (a, "chxlong")  ||
       i == (long *) NULL           ||
      *i <  0                       ||
      *i >= a->maxlen)                   RETURN (0L);
   
   j = *i;
   while (chxvalue (a, j) == ' ')   ++j;
   sign = chxvalue (a, j);
   if (sign == '-')               { ++j;   minus = 1; }
   if (sign == '+')                 ++j;

   for (j0=j, result=0L;   c = chxvalue (a, j), DIGIT(c);   ++j) 
      result = 10L * result + (c - '0');
   
   if (j0 == j)                          RETURN (0L);
   *i = j;
   RETURN (minus ? -result : result);
}
