
/*** CHXNUM.  Convert digits in a chix to a number.
/
/    success = chxnum (a, v);
/
/    Parameters:
/       int   success;   (found & converted a number?)
/       Chix  a;         (convert digits in this chix...)
/       long *v;         (and put them in this long)
/
/    Purpose:
/       Convert the digits found at the beginning of chix A to a long
/       integer.  Ignores leading blanks, and understands a leading
/       minus sign.
/
/    How it works:
/       Most of the work is done by calling chxlong().  If no number is
/       found, V is set to 0.
/
/    Returns: 1 if any digits were found (and converted)
/             0 if no  digits were found (or other error)
/
/    Error Conditions:
/       no digits were found
/       A is not a chix
/       V is NULL
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  Application.
/
/    Operating system dependencies: none
/
/    Known bugs:      none
/
/    Home:  chx/chxnum.c
/ */

/*** Copyright (C) 1990 Camber-Roth.  All rights reserved. */

/*: CR  6/09/91 16:21 New function. */
/*: CR 10/11/92 21:32 Expand comments. */
/*: CR 12/11/92 15:32 Long chxlong arg. */

#include <stdio.h>
#include "chixdef.h"

FUNCTION  chxnum (a, v)
   Chix   a;
   long  *v;
{
   long   i;

   ENTRY ("chxnum", "");

   if (NOT chxcheck (a, "chxnum")  ||  v == (long *) NULL)  RETURN (0);
   i = 0;
   *v = chxlong (a, &i);

   RETURN (i ? 1 : 0);
}
