/*** ADDMESS.   Add a message (Send a message) command.
/
/    SEND          or    ADD MESSAGE          prompts the user for a message;
/    SEND AGAIN    or    ADD MESSAGE AGAIN    uses the text of the last message 
/                                             the user sent;
/    SEND TO name  or    ADD MESSAGE TO name  sends the message to 'name'.
/
/    Other allowed variations:
/       SEND AGAIN TO name   or ADD MESSAGE AGAIN TO name
/       SEND TO name AGAIN   or ADD MESSAGE TO name AGAIN
/
/    In either case, ADDMESS calls MESS_SEND to prompt for a subject;
/    prompt for editing the message; and prompt for one or more
/    recipients. */

/*** Copyright (C) 1987 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:38 Source code master starting point*/
/*: JV  9/21/88 16:25 Added debug diagnostic.*/
/*: CR  9/30/88 16:13 Replace unit_copy() with direct copying code. */
/*: CR 10/17/88 13:10 if (debug) printf --> if (debug) unit_write... */
/*: CR 12/05/88 13:38 Add unit_view(in) ! */
/*: CR  8/21/89 12:11 Remove unit_unlk(in). */
/*: CW  5/17/89 16:21 Use XW unitcodes in place of XITT.*/
/*: JV  3/16/90 18:24 Add SEND TO name code */
/*: CR  4/24/90 21:06 Fix bad use of null.str; remove unused stuff. */
/*: CR  4/25/90 16:53 Reformat, add comments. */
/*: CR  9/26/90 10:19 got_id unused.  What is it for? */
/*: CR  2/28/91 13:40 Clear screen (w NEWPAGE) after checking syntax. */
/*: CR  4/08/91 18:49 Add new unit_lock argument. */
/*: CR  5/04/91 17:15 Expand modifier, name, userid buffers. */
/*: CR  7/11/91 16:24 Use new Address object instead of userid. */
/*: CX 11/11/91 23:20 Chixify. */
/*: UL 12/11/91 18:42 Make unit_lock args 3-6 long.*/
/*: CR 12/20/91 13:14 Add ERROR arg to choose_name() call. */
/*: CR 12/26/91 11:55 ...but do it correctly! */
/*: JX  5/19/92 21:27 Fix Chix. */
/*: CX  6/01/92 11:38 Make Address use consistent. */
/*: CX  6/05/92 16:20 XKEY --> u_keybd. */
/*: CI 10/06/92 12:55 Chix Integration. */
/*: TP 10/19/92 15:22 Remove u_keybd. */
/*: CP  8/10/93  0:14 Replace arg 'in' with 'red'. */

#include <stdio.h>
#include "caucus.h"
#include "redirect.h"

extern Userreg           thisuser;
extern union  null_t     null;
extern int               debug;
extern Chix              ss_qt1, ss_qt2, ss_dotnl;

FUNCTION   addmess (red, comline)
   Redirect *red;
   Chix      comline;
{
   static  Chix  messmodtab = nullchix;
   short   word=3, messtype=MSEND, choice;
   int     success, error;
   Chix    name, keep[2], line, modifier;
   Address address, pattern;
   Address make_address();

   ENTRY ("addmess", "");

   modifier = chxalloc (80L, THIN, "addmess modifier");
   name     = chxalloc (80L, THIN, "addmess name");
   keep[0]  = chxalloc (80L, THIN, "addmess keep0");
   keep[1]  = chxalloc (80L, THIN, "addmess keep1");
   line     = chxalloc (80L, THIN, "addmess line");
   address  = make_address();
   pattern  = make_address();
   success  = 0;

   apply_redirect (red);

   into_address (name, address);
   if (debug)  unit_write (XWDB, CQ("Got to Addmess.\n"));

   if (NOT mdtable ("mes_Amessmod", &messmodtab))  FAIL;

   /*** Check for 'AGAIN' or 'TO' modifiers. */
   while (chxtoken (modifier, null.chx, word, comline) >= 0) {
      jixreduce (modifier);
      choice = tablematch (messmodtab, modifier);
      switch (choice) {
         case (-2): mdwrite (XWER, "mes_Eambig", null.md);
                    SUCCEED;

         /*** AGAIN modifier.  Set message type to MAGAIN. */
         case  (0): messtype = MAGAIN;
                    word++; break;                /* Check for more words */

         /*** TO modifier.  Look for name following it.  If name is in
         /    quotes, look for other modifiers afterwards. */
         case  (1): 
            if (chxinside (comline, ss_qt1, name, ss_qt1) != -1 ||
                chxinside (comline, ss_qt2, name, ss_qt2) != -1) {
               chxquote (comline, 0, keep);         /* Remove name from cmd */
               word++; word++; break;   /* Must jump over strquote marker! */
            }
            else if (chxtoken (null.chx, name, word+1, comline) >= 0) {
               word=1000;
               break;
            } else {                   /* No name after the TO modifier */
               mdwrite (XWER, "mes_Enoname", null.md); 
               FAIL;
            }

         case (-1):
         default:
            mdwrite (XWER, "mes_Ebadcom", null.md);
            SUCCEED;
      }
   }

   unit_write (XWTX, NEWCPAGE);

   /*** If any name was found, try to turn it into an address. */
   if (NOT EMPTYCHX(name)) {
      into_address (name, pattern);
      if (NOT choose_name (pattern, address, &error)) {
         if (error == NOPERSON) {
            mdwrite    (XWER, "mes_Enopart", null.md);
            unit_write (XWER, name);
            unit_write (XWER, ss_dotnl);
         }
         FAIL;
      }
   }
 
   /*** If text of message to send comes from input redirection, put
   /    the text in the last-message-sent file. */
   if (red->unit != XKEY) {
      unit_lock  (XILM, WRITE, 0L, 0L, 0L, 0L, thisuser->id);
      unit_make  (XILM);
      unit_view  (red->unit);
      while (unit_read (red->unit, line, 0L)) {
         unit_write (XILM, line);
         unit_write (XILM, NEWLINE);
      }
      unit_close (red->unit);
      unit_close (XILM);
      unit_unlk  (XILM);
      mess_send  (0, MFILE, address);
      SUCCEED;
   }
  
   mess_send (0, messtype, address);  
   SUCCEED;

done:
   free_address (address);
   free_address (pattern);
   chxfree (modifier);
   chxfree (name);
   chxfree (keep[0]);
   chxfree (keep[1]);
   chxfree (line);
   RETURN  (success);
}
