
/*** ADDRESP.  CLI: Make a response to an item. */

/*** Copyright (C) 1987 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:38 Source code master starting point*/
/*: JV  9/03/89 16:34 Check verbose flag before printing concise message. */
/*: JV  9/04/89 11:12 Add definition of 'in' argument! */
/*: CW  5/17/89 16:21 Use XW unitcodes in place of XITT.*/
/*: CR  9/05/89 22:04 Add code for SET MYTEXT options handling. */
/*: CR  9/15/89 16:59 Check thisuser->read_only. */
/*: CR  3/04/90 22:36 Add context argument to enterresp() call. */
/*: CR  4/08/91 18:49 Add new unit_lock argument. */
/*: CX 11/12/91  0:17 Chixify. */
/*: UL 12/11/91 18:42 Make unit_lock args 3-6 long.*/
/*: CR  1/20/92 16:40 Add cnum arg to frozen() and load_header(). */
/*: JX  5/26/92 16:44 Fixify. */
/*: CX  6/05/92 16:20 XKEY --> u_keybd. */
/*: CR 10/11/92 21:53 Remove CQ from (md)prompter() call. */
/*: TP 10/19/92 15:23 Remove u_keybd. */
/*: TP 10/20/92 14:56 Determine source of resp input via input_control(). */
/*: JV  4/02/93 11:23 Add attach arg to load_header(). */
/*: JV  6/08/93 17:34 Don't need header or title. */
/*: CN  5/15/93 15:20 Replace parse_range with new_parse(). */
/*: CT  5/28/93 13:47 Replace SELECT with Rlist. */
/*: CP  6/23/93 23:40 Use new form of new_parse(). */
/*: CP  6/25/93 17:33 Initialize FOUND. */
/*: CP  7/14/93 15:31 Integrate 2.5 changes. */
/*: CP  8/09/93 11:41 Add 'attach' arg to new_parse() call. */
/*: CP  8/10/93  0:10 Replace arg 'in' with 'red'. */
/*: CK  8/12/93 15:33 Move ATTACH section before multiple resp test. */
/*: CP  8/12/93 17:00 Use return value from one_resp(), add_attach(). */
/*: CK  8/13/93 17:24 Change add_attach() -> hook_attachment(). */
/*: CP  9/09/93 17:09 Replace new_parse() with parse_or_ask(). */
/*: CK  9/15/93 14:04 Handle items with no responses; handle MULTIPLE mod. */
/*: CK 10/07/93 17:05 Use &thisitem in parse_or_ask() call. */
/*: CK 10/13/93 18:48 Skip entering response text if frozen item. */
/*: CK 10/14/93 13:50 Handle deciding if ADD ITEM or ADD RESP.  */
/*: CR 10/22/93 15:11 Add new rule for new resp & SET MYTEXT NEVER. */
/*: CP 11/11/93 14:11 Check return from get_resp_text(). */

#include <stdio.h>
#include "caucus.h"
#include "inputcon.h"
#include "api.h"
#include "modifier.h"
#include "redirect.h"
#define   RLNULL   ( (Rlist) NULL )

extern struct master_template master;
extern Userreg                thisuser;
extern struct partic_t        thisconf;
extern struct fresh_template  fresh;
extern struct this_template   thisitem;
extern union  null_t          null;
extern short                  confnum;

FUNCTION  addresp (red, command)
   Redirect *red;
   Chix      command;
{
   int    changed, i;
   int    found, item, resp;
   int    success, source, in, error, errpos, ok;
   long   arg;
   char   modifier[MODIFIERS+5];
   char   errstr[40];
   Chix   object, title, str, attach, errformat;
   Rlist  rptr, result;

   ENTRY ("addresp", "");

   if (thisuser->read_only) {
      mdwrite (XWER, "ent_Ereadonly", null.md);
      RETURN  (0);
   }

   jixreduce (command);
   object  = chxalloc (80L, THIN, "addresp object");
   title   = chxalloc (80L, THIN, "addresp title");
   attach  = chxalloc (80L, THIN, "addresp attach");
   str     = chxalloc (80L, THIN, "addresp str");
   result  = RLNULL;
   success = 0;

   /*** Try parsing the item/resp range given on the command line.  There
   /    are several possible results:
   /     (a) "ADD ITEM", with no instances.  Means add a new item.
   /     (b) "ADD", with an erroneous instance.  Complain.
   /     (c) "ADD", with no instances.  Try again, via parse_or_ask().
   /     (d) Valid ADD RESP or ADD ATTACH. */
   chxtoken (null.chx, object, 2, command);
   ok = new_parse (object, &result, modifier, attach, confnum, &thisitem, 
                          1, 0, 1, &errpos);

   /*** Case (a): if we have "ITEM" but nothing else, just add a new item. */
   if (NOT ok  &&  errpos == -1  &&  modifier[M_ITEMS])   DONE (additem (red));

   /*** Cases (b) and (c).  Let parse_or_ask() do all the work. */
   if (NOT ok) {
      a_fre_rlist (result);
      if (NOT parse_or_ask (object, &result, modifier, attach, confnum, 
                                    &thisitem, 0))  FAIL;
   }

   /*** From here on, we are in case (d) no matter what. */

   /*** Handle adding an attachment to an existing response.  The real 
   /    work is done by add_attach(). */
   if (modifier[M_ATTACH]) {

      /*** Use the first item:resp instance given. */
      if (NOT one_response (&item, &resp, result, &master)) {
         mdwrite (XWER, "res_Tnoitem", null.md);
         FAIL;
      }

      /*** Add the attachment to that item:response. */
      if (hook_attachment (confnum, item, resp, attach, red, 1, &error))
         SUCCEED;
      
      /*** If the attachment failed, explain why. */
      if (error < 0)  { sprintf (errstr, "add_Eat%d", -error);  arg = item; }
      else            { strcpy  (errstr, "add_Eat0");           arg = error;}
      sprintf (errstr, "add_Eat%d",  (error < 0 ? -error : 0));
      errformat = chxalloc (200L, THIN, "addresp errformat");
      if (NOT mdstr (errformat, errstr, null.md)) 
         chxofascii (errformat, "\nUnknown attachment error!\n");
      chxclear   (str);
      chxformat  (str, errformat, arg, L(resp), attach, nullchix);
      unit_write (XWER, str);
      chxfree    (errformat);
      FAIL;
   }
  
   /*** Find which item was specified.  (And how many!). */
   found = 0;
   item  = 0;
   for (rptr=result->next;   rptr!=RLNULL;   rptr=rptr->next) {
      for (i=rptr->i0;   i<=rptr->i1;   ++i) {
         if (i == 0)   continue;
         ++found;
         if (item==0)  item = i;
      }
   }
  
   /*** If more than one item was specified, complain. */
   if ( (NOT modifier[M_MULTIPLE])  &&  found > 1) {
      mdwrite (XWER, "add_Etoomany", null.md);
      FAIL;
   }

   /*** If frozen, skip asking for response text entirely. */
   if (frozen (confnum, item, 1))   FAIL;

   /*** If this is interactive, display the header and ask the user
   /    to enter their response. */
   apply_redirect (red);
   in = red->unit;
   if (in == XKEY) {
      if (NOT load_header (confnum, item, str, title, (Attachment) NULL))
         FAIL;
      unit_write (XWTX, NEWCPAGE);
      if (NOT show_header (XWTX, item, str, title, 0))  FAIL;
      if (NOT get_resp_text(0))                         FAIL;
      source = XITX;
      unit_lock (XITX, READ, 0L, 0L, 0L, 0L, CQ(""));
   }
   else source = in;
  
   /*** For each item in RESULT (probably only one unless MULTIPLE
   /    modifier given), add the response, and update the user's
   /    record of what they've "seen". */
   changed = 0;
   for (rptr=result->next;   rptr!=RLNULL;   rptr=rptr->next) {
      for (item=rptr->i0;   item<=rptr->i1;   ++item) {
         if (item == 0)                           continue;
         if (frozen (confnum, item, 1))           continue;
         if (NOT enterresp (item, 0, source))     continue;
            /* May want to produce an error message? */
            /* mdwrite (XWER, "ent_Tcancel", null.md); */

         /*** If the user's responses are NEVER marked NEW, mark all of the
         /    responses as seen. */
         if (thisuser->mytext == NEVER) {
            if (thisconf.responses[item] == master.responses[item] - 1) {
                thisconf.responses[item] =  master.responses[item];
                changed = 1;
            }
            if (thisconf.backresp[item] != 0)  changed = 1;
            thisconf.backresp [item] = 0;
         }
      
         /*** If user's responses now show up new LATER (after someone else
         /    responses), *and* we've seen all the previous responses,
         /    mark ALL of them as seen, but remind CAUCUS to backup one more
         /    the next time we consider the new responses to this item. */
         if (thisuser->mytext == LATER) {
            if (thisconf.responses[item]  == master.responses[item]-1) {
               thisconf.responses[item]++;
               thisconf.backresp [item]++;
               changed = 1;
            }
            if (fresh.responses[item] == master.responses[item]-1) {
               fresh.responses[item]++;
               fresh.backresp [item]++;
            }
         }
      }
   }
   if (source == XITX)  unit_unlk (source);

   if (changed) {
      sysuserid (str);
      unit_lock (XUPA, WRITE, L(confnum), 0L, 0L, 0L, str);
      store_user(&thisconf);
      unit_unlk (XUPA);
   }
   SUCCEED;

done:
   a_fre_rlist (result);
   chxfree (str);
   chxfree (title);
   chxfree (attach);
   chxfree (object);
   RETURN  (success);
}

