/*** DELPERSON.   Remove a person from this conference.
/
/    DELPERSON may be used by the Organizer to remove any one person
/    from the conference (except the organizer!).
/
/    DELPERSON used by anyone else removes that user from the conference,
/    i.e. resigns that user.  Both forms prompt for confirmation before
/    actually removing anyone.
/
/    DELPERSON returns 1 if a user has successfully removed him/her self
/    from the conference, and 0 otherwise.  */

/*: AA  7/29/88 17:38 Source code master starting point*/
/*: JV  9/19/89 17:36 Add _delete_person AFM. */
/*: CW  5/17/89 16:22 Use XW unitcodes in place of XITT.*/
/*: CR 10/17/89 15:17 Replace 2nd arg of FIRE_ARM with "". */
/*: CR  7/14/90 11:42 Use thisuser->is_organizer. */
/*: CR  8/12/90 15:31 Do not explicitly display userid; it's part of name. */
/*: CR  2/01/91 12:29 Use person_format() to display user's name. */
/*: CR  4/08/91 18:49 Add new unit_lock argument. */
/*: CR  7/16/91 17:08 Add HOST argument to person_format() calls. */
/*: CR  7/30/91 18:14 Add 3rd arg to loadreg(). */
/*: CR  8/02/91 12:44 Add NOPUNCT as new last arg to modnames() call. */
/*: CX 11/12/91 15:30 Chixify. */
/*: CR 11/18/91 17:20 Add CNUM argument to modnames() call. */
/*: UL 12/11/91 18:42 Make unit_lock args 3-6 long.*/
/*: CX  3/17/92 15:44 Replace FIRE_AFM with fire_afm(). */
/*: JX  7/07/92 11:02 Don't nlfree(briefs) or (print). */
/*: JV  3/31/93 13:07 Confirm resign for org's deleting self. */
/*: JV  6/11/93 14:30 Fix '=' bug. */
/*: CP  5/06/92 12:41 Add 0L as 'item' arg to modnames() call. */
/*: CP  7/14/93 14:41 Integrate 2.5 changes. */

#include <stdio.h>
#include "caucus.h"

extern struct master_template master;
extern Userreg                thisuser;
extern union  null_t          null;
extern Chix                   afm_name;
extern Chix                   yesnotable;
extern short                  confnum;

FUNCTION  delperson (command)
   Chix   command;
{
   struct  namlist_t *wholist, *add, *del, *nluserid(), *nlnode();
   short   nogood, number;
   int     success;
   char   *message, *prompt, *help;
   Chix    str, who, answer, no, format, result, tmpstr;
   Userreg person;
   Chix    person_format();
   Userreg make_userreg();

   ENTRY ("delperson", "");

   no     = chxalloc (80L, THIN, "delperson no");
   str    = chxalloc (80L, THIN, "delperson str");
   who    = chxalloc (80L, THIN, "delperson who");
   answer = chxalloc (80L, THIN, "delperson answer");
   format = chxalloc (80L, THIN, "delperson format");
   tmpstr = chxalloc (80L, THIN, "delperson tmpstr");
   person = make_userreg();

   /*** If this is the Organizer, get the name of person to be removed
   /    from the conference (from the command line, or by prompting.)
   /    Find the list of userids that match this name, and if there's
   /    only one (not the organizer's!), remove that user. */

   if (thisuser->is_organizer) {
      if (chxtoken (null.chx, str, 3, command) < 0) {
         if (mdprompter ("del_Pwho", null.chx, 0, "del_Hwho", 80, str, CQ(""))
               < -1)  FAIL;
      }
      wholist = nluserid (str, 1);
      number  = nlsize   (wholist);
      nogood  = 1;

      if (number == 1) {
         message = "del_Eorgan";
         chxcpy (who, (wholist->next)->str);
         if (NOT chxeq (who, master.organizerid))  nogood = 0;
      }
      else if (number == 0)   message = "del_Epers0";
      else if (number  > 1)   message = "del_Epers2";
      nlfree (wholist);

      if (nogood) { mdwrite (XWER, message, null.md);   FAIL; }
      if (chxvalue (str, 0L) == '=') {
         chxcatsub (tmpstr, str, 1L, ALLCHARS);
         chxtrim   (tmpstr);
         chxcpy    (str, tmpstr);
      }
      if (chxeq (str, thisuser->id)) {            /* Del self == resign */
         prompt = "del_Presign";
         help   = "del_Hresign";
      } else {
         prompt = "del_Pdelusr";
         help   = "del_Hdelusr";
      }
   }

   /*** Regular users can only delete themselves, i.e., resign. */
   else {
      sysuserid (who);
      prompt = "del_Presign";
      help   = "del_Hresign";
   }

   /*** Load the registration info on the person selected, and
   /    prompt for confirmation on deletion. */
   person->print = person->briefs = null.nl;
   if (NOT loadreg (who, person, NULLSITE))  FAIL;

   /*** If forcing someone else to resign, display name before confirming. */
   if (NOT chxeq (tmpstr, thisuser->id) && thisuser->is_organizer) {
      mdstr (format, "del_Fname", null.md);
      result = person_format (format, person->name,  person->id, null.chx,
                                      person->phone, person->briefs);
      unit_write (XWTX, NEWCPAGE);
      unit_write (XWTX, result);
      chxfree    (result);
   }

   /*** Ask for confirmation... */
   chxtoken (no, null.chx, 1, yesnotable);
   if (mdprompter (prompt, yesnotable, 2, help, 18, answer, no) < 1)  FAIL;
  
   /*** Actually remove this person->.. */

   /*** Remove the user's name from the membership list for this conf. */
   tomember (str, person->lastname, person->name, person->id);
   add = nlnode(1);
   del = nlnode(1);
   nladd (del, str, 0);
   modnames (XCMD, XCMN, null.chx, confnum, 0L, add, del, NOPUNCT);
   nlfree   (add);
   nlfree   (del);

   unit_lock (XUPA, WRITE, L(confnum), 0L, 0L, 0L, person->id);
   unit_kill (XUPA);
   unit_unlk (XUPA);

   /* Now fire off the AFM. */
   fire_afm ("_delete_person", null.chx, null.chx);

   if (thisuser->is_organizer)  mdwrite (XWTX, "del_Tpergone", null.md);

   /*** If a user removed him/herself, return 1 ==> leave this conf! */
   success = (chxeq (person->id, thisuser->id));

done:
   chxfree (no);
   chxfree (format);
   chxfree (answer);
   chxfree (who);
   chxfree (str);
   chxfree (tmpstr);
   free_userreg (person);
   RETURN  (success);
}
