/*** DELSUBJ.   Delete a subject.
/
/    DELSUBJ is used to delete a SUBJECT or MYSUBJECT named by
/    the user on the command line.  If no subject is named, the
/    user is prompted for a name.
/
/    MINE means MYSUBJECT if true, and SUBJECT otherwise. */

/*** Copyright (C) 1987 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:38 Source code master starting point*/
/*: CW  5/17/89 16:22 Use XW unitcodes in place of XITT.*/
/*: CR  7/14/90 11:49 Use thisuser->is_organizer. */
/*: CR  7/23/91 15:56 Replace subdel() with chg_dsubj(). */
/*: CR  7/23/91 23:56 Add addxosubj() call. */
/*: CR  7/30/91 13:51 Replace license check with is_distributed(). */
/*: CX 11/12/91 17:28 Chixify. */
/*: CR 10/29/91 11:45 Add confnum arg to getline() call. */
/*: JV 10/31/91 15:09 Add dest_chn arg to addxosubject() call. */
/*: CR 11/07/91 15:11 Add confname arg to chg_dsubject(), submatch() calls. */
/*: JX  2/25/92 11:30 Declare confname. */
/*: JX  5/26/92 19:03 Fixify. */
/*: JX  8/03/92 11:38 CHN's are chix. */
/*: CR 10/11/92 21:54 Remove CQ from (md)prompter() call. */

#include <stdio.h>
#include "caucus.h"
#include "xaction.h"

extern struct master_template master;
extern Textbuf                tbuf;
extern Userreg                thisuser;
extern union  null_t          null;
extern Chix                   yesnotable;
extern short                  confnum;
extern Chix                   confname;

FUNCTION  delsubj (command, mine)
   int    mine;
   Chix   command;
{
   short  utd,utf, usd,usf;
   short  i, subject;
   int    error1, error2;
   Chix   owner, name, line, keep, no;
   int    success;
   Chix   getline();

   ENTRY ("delsubj", "");

   owner = chxalloc (20L, THIN, "delsubj owner");
   name  = chxalloc (60L, THIN, "delsubj name");
   keep  = chxalloc (20L, THIN, "delsubj keep");
   no    = chxalloc (20L, THIN, "delsubj no");

   /*** Setup: determine unit numbers and owner for (MY)SUBJECT. */
   if (mine) {
      usd = XUND;   usf = XUNN;
      utd = XUSD;   utf = XUSF;
   }
   else {
      usd = XCND;   usf = XCNN;
      utd = XCSD;   utf = XCSF;
   }
   if (mine) sysuserid (owner);

   if (NOT mine  &&  NOT thisuser->is_organizer) {
      mdwrite (XWER, "cus_Tonlyorg", null.md);
      FAIL;
   }

   /*** Pluck the subject name off the command line.  If no name, 
   /    prompt the user for one. */
   if (chxtoken (null.chx, name, 3, command) < 0) {
      if (mdprompter ("del_Psubnam", null.chx, 0, "del_Hsubnam", 
                       80, name, null.chx) < -1)  FAIL;
   }
   chxstrip (name);

   /*** Find the number of the subject that uniquely matches NAME. */
   subject = submatch (usd,usf, utd,utf, owner, confnum, confname, name);
   if (subject  < 0) {  mdwrite (XWER, "chg_Esubj2",  null.md);   FAIL; }
   if (subject == 0) {  mdwrite (XWER, "chg_Enosubj", null.md);   FAIL; }

   /*** Display full name of subject (and item numbers) and prompt for
   /    confirmation to delete this subject. */
   unit_write (XWTX, NEWCPAGE);
   for (i=1;
       (line = getline (utd, utf, confnum, owner, 0, subject, i, tbuf)) !=
               null.chx; ++i) {
      unit_write (XWTX, line);
      unit_write (XWTX, NEWLINE);
   }
   chxtoken (no, null.chx, 1, yesnotable);
   if (mdprompter ("del_Psubok", yesnotable, 2, "del_Hsubok", 80, keep, no)
           <= 0) SUCCEED;

   chg_dsubject (XT_DELETE, confnum, confname, owner, name, nullchix, &error1);

   /*** If distributed, write OSUBJECT deletion transaction. */
   if (EMPTYCHX(owner)  &&  is_distributed (confnum)) {
      addxosubject (XT_DELETE, confnum, name, nullchix, nullchix, &error2);
   }
   SUCCEED;

done:
   chxfree (no);
   chxfree (keep);
   chxfree (name);
   chxfree (owner);
   RETURN  (success);
}
