/*** SHOWTEXT.  Display the text of an item on unit OUT.
/
/    ok = showtext (out, item, last, attach);
/
/    Parameters:
/       int    ok;          (Success?)
/       int    out;         (unit number to write output to)
/       int    last;        (number of last response selected)
/       int    attach;      (0=>no attach, 1=>show attach, 2=>askme)
/
/    Purpose:
/       Display the title and text of an item on unit OUT.
/
/    How it works:
/       If the item has an attachment, decide what to do with it
/       based on the ATTACH argument.
/
/    Returns: 1 if the item was displayed
/             0 on error or if the user interrupted with ^C.
/
/    Error Conditions:
/ 
/    Side effects: updates the 'thisitem' global.
/
/    Related functions:
/
/    Called by:  showitem(), consearch()
/
/    Operating system dependencies: none
/
/    Known bugs:      none
/
/    Home:  cmd1/showitem.c
/ */

/*** Copyright (C) 1987 Camber-Roth.  All rights reserved.  */

/*: AA  7/29/88 17:39 Source code master starting point*/
/*: JV  9/15/89 18:09 Add _beg_showitem and _end_showitem AFM's */
/*: JV  9/18/89 16:24 Remove AFM's. */
/*: CW  5/17/89 16:22 Use XW unitcodes in place of XITT.*/
/*: CR 12/01/89 11:19 Set thisitem.number to the item number! */
/*: CR  8/09/90 19:46 Use show_format(). */
/*: CR  2/11/91 18:20 Set thisitem.frozen! */
/*: CR 10/29/91 11:45 Add confnum arg to getline() call. */
/*: CR  1/20/92 16:41 Expand thisitem assignment. */
/*: JX  5/14/92 12:05 Chixify. */
/*: JV  6/08/93 16:09 Show_format doesn't need header, title; start lnum @ 0. */
/*: CP  6/24/93 15:10 Remove "more text" prompting. */
/*: CP  6/30/93 17:25 Add 'resp2' arg to show_format() call. */
/*: CP  7/01/93 21:04 Add 'last' arg. */
/*: CP  7/16/93 16:31 Remove unneeded 'title'. */
/*: CP  8/24/93 15:34 Add 'attach' arg. */
/*: CP  8/25/93 21:40 Add show_attachment() call! */
/*: CP  8/27/93 16:52 Split item text & selected responses display in two. */
/*: JV  9/10/93 11:57 Reset thisitem.rnum. */
/*: CK  9/20/93 15:12 Change syntax of res_Fshowitem2 slightly. */
/*: CP 11/11/93 13:37 Add 'resp1' arg to show_format() call. */

#include <stdio.h>
#include "caucus.h"
#include "chixuse.h"
#include "done.h"
#include "redirect.h"

extern struct this_template thisitem;
extern union null_t         null;
extern short                confnum;

FUNCTION  showtext (out, item, last, attach)
   int  out, item, last, attach;
{
   static Chix s_showitem, s_showitem2;
   int    lnum, success;
   long   number;
   Chix   header, getline();
   Chix   str, format, format2;

   ENTRY ("showtext", "");

   str     = chxalloc ( 60L, THIN, "showtext str");
   format  = chxalloc (100L, THIN, "showtext format");
   format2 = chxalloc (100L, THIN, "showtext format2");

   /*** Get the header for this item.  Ensure it exists. */
   header = getline (XCRD, XCRF, confnum, nullchix, item, 0, 0, (Textbuf) NULL);
   if (header==nullchix)  FAIL;

   chxtoken (str, nullchix, 3, header);
   chxnum   (str, &number);
   thisitem.frozen = number;
   thisitem.number = item;
   thisitem.rnum   = 
   thisitem.cnum   = confnum;

   lnum = 0;
   mdstr (format,  "res_Fshowitem",  &s_showitem);
   mdstr (format2, "res_Fshowitem2", &s_showitem2);

   /*** Display header and text of item. */
   lnum = show_format (out, format, item, 0, 0, last, lnum, nullchix, header);
   if (lnum == 0)     FAIL;

   /*** If there is an attachment, does/can the user want to see it? */
   if (ok_show_attachment (confnum, item, 0, (Redirect *) NULL, 0, 0, attach))
          show_attachment (confnum, item, 0, (Redirect *) NULL, 0);

   if (sysbrktest())  FAIL;

   /*** Tell which responses were selected. */
   show_format (out, format2, item, 0, last, 0, lnum, nullchix, header);
   SUCCEED;

done:
   chxfree (str);
   chxfree (format);
   chxfree (format2);
   RETURN  (success);
}
