/*** FORGET.   CLI: Handle the FORGET command. 
/
/    ok = forget (comline);
/
/    Parameters:
/       int     ok;         (success?)
/       Chix    comline;    (the text of the command line)
/
/    Purpose: Handle the FORGET command.
/
/    How it works:
/
/    Returns: 1 on success,
/             0 on failure (that item doesn't exist)
/
/    Error Conditions:
/
/    Side Effects:
/
/    Related to:
/
/    Called by:  interpret()
/
/    Operating system dependencies: none
/
/    Known bugs:
/
/    Home: cmd2/forget.c
/ */

/*** Copyright (C) 1987-1992 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:39 Source code master starting point */
/*: CR  8/24/88 12:32 Lower-case COMLINE before processing */
/*: JV  8/23/89 10:15 Added error message if no items selected. */
/*: CR 10/09/89 11:30 Use XW unit codes. */
/*: CR 10/11/89 16:11 Revert to "no items selected" if parserange fails. */
/*: CR  4/08/91 18:49 Add new unit_lock argument. */
/*: UL 12/11/91 18:42 Make unit_lock args 3-6 long.*/
/*: JX  2/24/92 18:33 Chixify, add header. */
/*: JX  5/26/92 17:25 Fixify. */
/*: CN  5/15/93 15:20 Replace parse_range with new_parse(). */
/*: CT  5/28/93 13:47 Replace SELECT with Rlist. */
/*: CP  6/25/93 13:59 Add new args to new_parse() call. */
/*: CP  8/01/93 10:50 Add a_mak...() name arg. */
/*: CP  8/09/93 11:41 Add 'attach' arg to new_parse() call. */
/*: CP  8/20/93 14:29 Correct order of args in new_parse() call. */
/*: CP  9/10/93  9:26 Replace new_parse() with parse_or_ask(). */
/*: CK  9/15/93 13:36 Add 'mod1st' arg to new_parse() call. */
/*: CK 10/07/93 17:05 Use &thisitem in parse_or_ask() call. */

#include <stdio.h>
#include "caucus.h"
#include "chixuse.h"
#include "done.h"
#include "api.h"
#include "modifier.h"
#define   RLNULL   ( (Rlist) NULL )

extern struct master_template master;
extern struct this_template   thisitem;
extern struct partic_t        thisconf;
extern union  null_t          null;
extern Chix                   ss_dotnl;
extern short                  confnum;

FUNCTION  forget (comline)
   Chix   comline;
{
   Chix   range, userid, none, nof, s, format;
   int    success, errpos;
   short  high, i;
   char   modifier[MODIFIERS+5];
   Rlist  rptr, result, flist, fptr, remlist, add_to_rlist(), and_rlist();

   ENTRY ("forget", "");

   s      = chxalloc (80L, THIN, "forget s");
   range  = chxalloc (80L, THIN, "forget range");
   userid = chxalloc (40L, THIN, "forget userid");
   format = chxalloc (20L, THIN, "forget format");
   none   = chxalloc (20L, THIN, "forget none");
   nof    = chxalloc (20L, THIN, "forget nof");
   result = RLNULL;

   /*** If an item range was specified on the command line, parse it.
   /    Those are the items we will forget.  */
   jixreduce (comline);
   if ( (chxtoken (range, s, 2, comline)) != -1) {

      if (NOT parse_or_ask (s, &result, modifier, nullchix, confnum,
                               &thisitem, 1))  FAIL;

      /*** Find the HIGHest item in the range. */
      for (high=0, rptr=result->next;   rptr!=RLNULL;   rptr=rptr->next)
         if (rptr->i1 > high)  high = rptr->i1;

      /*** Mark the NEW items below HIGH to now be UNSEEN. */
      for (i=thisconf.items+1;   i<=high;   ++i)  thisconf.responses[i] = -1;

      /*** Mark the selected items forgotten. */
      for (rptr=result->next;   rptr!=RLNULL;   rptr=rptr->next)
         for (i=rptr->i0;    i<=rptr->i1;   ++i)  thisconf.responses[i] = -2;

      thisconf.items = max (thisconf.items, high);
      mdwrite (XWTX, "for_Tgotten", null.md);
   }

   /*** Otherwise, display the list of forgotten items, and prompt the
   /    user for the items to be remembered. */
   else {
      /*** Make and display the list of forgotten items. */
      flist = fptr = a_mak_rlist("forget flist");
      for (i=1;   i<=thisconf.items;      ++i) 
         if (thisconf.responses[i]==-2)   fptr = add_to_rlist (fptr, i, 0, -1);

      mdwrite (XWTX, "for_Tyouhave", null.md);
      if (NOT new_printrange (XWTX, flist, 0))
         mdwrite (XWTX, "for_Tnoitems", null.md);

      mdstr (none, "for_Knone", null.md);
      mdstr (nof,  "for_Knof",  null.md);
      if (mdprompter ("for_Premember", nullchix, 0, "for_Hremember",
                          80, range, none) < -1)                   SUCCEED;
      if (chxeq (range, none))                                     SUCCEED;
      chxcat    (range, CQ(" "));
      chxcat    (range, nof);
      chxcat    (range, CQ(" "));
      if (NOT new_parse (range, &remlist, modifier, nullchix, confnum,
                         &thisitem, 1, 1, 1, &errpos)) {
         if (errpos >= 0)  report_parse_err (range, errpos);
         SUCCEED;
      }

      result = and_rlist (flist, remlist);
      for (rptr=result->next;   rptr!=RLNULL;   rptr=rptr->next)
         for (i=rptr->i0;   i<=rptr->i1;   ++i)  thisconf.responses[i] = -1;

      mdwrite (XWTX, "for_Tunseen", null.md);
      a_fre_rlist (remlist);
      a_fre_rlist (flist);
   }

   sysuserid (userid);
   if (unit_lock (XUPA, WRITE, L(confnum), 0L, 0L, 0L, userid)) {
      store_user(&thisconf);
      unit_unlk (XUPA);
   }
   SUCCEED;

done:
   a_fre_rlist (result);
   chxfree (s);
   chxfree (range);
   chxfree (format);
   chxfree (userid);
   chxfree (none);
   chxfree (nof);
   RETURN  (success);
}
