/*** FREEZE.   Handle the FREEZE command. 
/
/    ok = freeze (comline);
/
/    Parameters:
/       int   ok;        (success?)
/       Chix  comline;   (the command line)
/
/    Purpose:
/       Provide user interface and db alteration for FREEZE.
/
/    How it works:
/
/    Returns: 1 on success,
/             0 on failure
/
/    Error Conditions:
/
/    Side Effects:
/
/    Related to:
/
/    Called by:  interpret()
/
/    Home: cmd2/freeze.c
/ */

/*** Copyright (C) 1987-1992 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:40 Source code master starting point */
/*: CR  4/21/89 17:00 Ensure frozen[] is plenty big for printrange() use. */
/*: JV  8/23/89 10:37 Add error message if itemlist is bad. */
/*: JV  8/23/89 12:21 Add punch line to 'freeze person' mis-command. */
/*: CR  8/03/89 16:43 Add use of frozen codes 1 (org or owner) & 2 (caucus). */
/*: CR 10/09/89 11:30 Use XW unit codes. */
/*: CR 10/11/89 16:11 Revert to "no items selected" if parserange fails. */
/*: JV 12/04/91 14:55 Add global confnum for freezeitem call. */
/*: CR  1/20/92 16:50 Add cnum arg to load_header() call. */
/*: JX  2/25/92 13:30 Chixify. */
/*: JX  5/26/92 20:01 Fixify. */
/*: JV 10/19/92 13:29 Move some code to get_frozen(). */
/*: CR 12/10/92 14:46 Remove frnum.c */
/*: CL 12/11/92 15:52 Long sysmem arg. */
/*: JV  4/02/93 11:27 Remove unused HEADER, TITLE. */
/*: CN  5/15/93 15:06 Replace parse_range() with new_parse(). */
/*: CT  5/28/93 13:47 Replace SELECT with Rlist. */
/*: CP  6/25/93 14:01 Add new args to new_parse() call. */
/*: CP  7/16/93 17:26 Add 'first' arg to get_frozen() call. */
/*: CP  8/06/93 15:12 Skip item 0. */
/*: CP  8/09/93 11:41 Add 'attach' arg to new_parse() call. */
/*: CP  8/20/93 14:31 Correct order of args in new_parse() call. */
/*: CP  9/08/93 15:42 Replace get_frozen() with and_frozen(). */
/*: CP  9/10/93  9:36 Fix 'freeze people'; new_parse() -> parse_or_ask(). */
/*: CK  9/15/93 13:36 Add 'mod1st' arg to new_parse() call. */
/*: CK 10/07/93 17:05 Use &thisitem in parse_or_ask() call. */
/*: CK 10/10/93 14:26 Add ERRCHK, ERRPOS args to new_parse() call. */

#include <stdio.h>
#include "caucus.h"
#include "chixuse.h"
#include "done.h"
#include "api.h"
#include "modifier.h"
#define   RLNULL   ( (Rlist) NULL )

extern struct master_template master;
extern struct this_template   thisitem;
extern struct partic_t        thisconf;
extern union  null_t          null;
extern short                  confnum;
extern Chix                   objectable;
extern Chix                   ss_dotnl;

FUNCTION  freeze (comline)
   Chix   comline;
{
   int    success, errpos;
   short  i;
   char   modifier[MODIFIERS+5];
   Chix   rest, name, none, range, format, message, str;
   Rlist  result, frozen, thawlist, rptr, and_rlist(), rl_all();

   ENTRY ("freeze", "");

   str     = chxalloc (60L, THIN, "str");
   rest    = chxalloc (60L, THIN, "rest");
   name    = chxalloc (60L, THIN, "name");
   none    = chxalloc (20L, THIN, "none");
   range   = chxalloc (80L, THIN, "range");
   format  = chxalloc (60L,  THIN, "format");
   message = chxalloc (100L, THIN, "message");
   frozen  = RLNULL;
   result  = RLNULL;

   /*** If an item range was specified on the command line, parse it.
   /    Those are the items we will freeze.  */
   jixreduce (comline);
   if ( (chxtoken (range, rest, 2, comline)) != -1) {
      if (tablematch (objectable, range) == 5) {
         chxtoken (range, name, 3, comline);
         mdstr (format, "fre_Epeople", null.md);
         chxsimplify (name);
         chxformat  (message, format, 0L, 0L, name, null.chx);
         unit_write (XWTX, message);
         FAIL;
      }

      if (NOT parse_or_ask (rest, &result, modifier, nullchix, confnum, 
                                  &thisitem, 1))  FAIL;

      /*** Freeze the selected items (that the user is allowed to
      /    freeze). */
      for (rptr=result->next;   rptr!=RLNULL;   rptr=rptr->next) {
         for (i=rptr->i0;   i<=rptr->i1;   ++i) {
            if (i == 0)  continue;
            freezeitem ((int) confnum, i, 1);
         }
      }
   }


   /*** Otherwise, display the list of frozen items, and prompt the
   /    user for the items to be thawed. */
   else {
      frozen = rl_all (master.items);
      if (NOT and_frozen (confnum, &master, frozen))  FAIL;

      mdwrite (XWTX, "fre_Titems", null.md);
      if (NOT new_printrange (XWTX, frozen, 0))
         mdwrite (XWTX, "for_Tnoitems", null.md);

      else {
         mdstr (none, "for_Knone", null.md);
         if (mdprompter ("fre_Pthaw", nullchix, 0, "fre_Hthaw", 80,
                         range, none) >= -1
                &&     NOT chxeq   (range, none)) {
            if (NOT new_parse (range, &result, modifier, nullchix, 
                               confnum, &thisitem, 1, 1, 1, &errpos)) {
               if (errpos >= 0)  report_parse_err (range, errpos);
               SUCCEED;
            }

            thawlist = and_rlist (result, frozen);
            for (rptr=thawlist->next;   rptr!=RLNULL;   rptr=rptr->next)
               for (i=rptr->i0;   i<=rptr->i1;   ++i) {
                  if (i == 0)  continue;
                  freezeitem ((int) confnum, i, 0);
               }
            a_fre_rlist (thawlist);
         }
      }
   }
   SUCCEED;

done:
   a_fre_rlist (result);
   a_fre_rlist (frozen);
   chxfree (str);
   chxfree (rest);
   chxfree (name);
   chxfree (none);
   chxfree (range);
   chxfree (format);
   chxfree (message);
   RETURN  (success);
}
