/*** FREEZEITEM.  Check validity of FREEZE command and do if valid.
/
/    freezeitem (cnum, item, freeze);
/
/    Parameters:
/       int cnum,    (Local Conference Number)
/           item,    (Item number)
/           freeze;  (1 for freeze, 0 for thaw)
/
/    Purpose:
/       FREEZEITEM validates a FREEZE command and calls frz_ditem()
/       if the user is the organizer or the owner of the item.
/
/    How it works:
/       The current user must be the organizer or the owner of ITEM.
/       If FREEZE is 1, the item is frozen.  Otherwise, the item is
/       thawed.  
/
/   Returns: 1 on success,
/            0 on failure.
/
/   Error Conditions:
/
/   Side effects:
/      Sometimes loads the relevant item partfile into the textbuf structure,
/      changes it and stores it.  This implies that the text buffer
/      gets overwritten.
/
/   Related functions:
/
/   Called by:  freeze()
/
/   Operating system dependencies: none
/
/   Known bugs:      none
/
/   Home:  cmd2/freezeit.c
/ */

/*** Copyright (C) 1987 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:40 Source code master starting point */
/*: CR  7/14/90 12:00 Use thisuser.is_organizer. */
/*: CR  4/08/91 18:49 Add new unit_lock argument. */
/*: JV 12/04/91 13:25 Add frzxitem call, add cnum to args. */
/*: JV 12/05/91 09:26 Add header, move db change to frz_ditem(). */
/*: JX  2/22/92 16:30 Chixify. */
/*: DE  5/26/92 17:00 Fix chixification */
/*: JX  5/26/92 20:03 More Chix fixes. */
/*: JX  8/03/92 11:40 CHN's are chix. */
/*: CR 10/07/92 15:59 Allocate rest! */

#include <stdio.h>
#include "caucus.h"
#include "chixuse.h"
#include "done.h"

extern Userreg         thisuser;
extern struct master_template master;
extern union  null_t          null;

FUNCTION  freezeitem (cnum, item, freeze)
   int    cnum, item, freeze;
{
   int   error, oitem, success;
   Chix  hdr, owner, ohost, status, rest, getline();

   ENTRY ("freezeitem", "");

   owner  = chxalloc (100L, THIN, "freezeit owner");
   ohost  = chxalloc (100L, THIN, "freezeit ohost");
   status = chxalloc (100L, THIN, "freezeit status");
   rest   = chxalloc (100L, THIN, "freezeit rest");

   if (thisuser->is_organizer)  
      frz_ditem (cnum, item, freeze);
   else {
      hdr = getline (XCRD, XCRF, cnum, nullchix, item, 0, 0, (Textbuf) NULL);

      if ( chxtoken (owner,  nullchix, 2, hdr)  != -1  &&
          (chxtoken (status, rest,     3, hdr)) != -1  && 
              chxeq (thisuser->id, owner) ) 

         frz_ditem (cnum, item, freeze);
      else {
         mdwrite (XWER, "fre_Enoallow", null.md);
         FAIL;
      }
   }

   /*** If distributed, write item freeze transaction. */
   if (is_distributed (cnum)) {
      getoitem (&oitem, ohost, item, &master);
      frzxitem (cnum, oitem, ohost, freeze, nullchix, nullchix,
                nullchix, &error);
   }
   SUCCEED;

done:
   chxfree (owner);
   chxfree (ohost);
   chxfree (status);
   chxfree (rest);
   RETURN  (success);
}
