/*** HELP.   Process user's request for help.
/
/    help  (comline, defaulthelp)
/
/  Parameters:
/     Chix  command
/     char *default_help
/
/  Purpose:
/    Parses the help request in COMMAND, and prints the appropriate help
/    message.  If the help request is merely the word "help", prints the
/    default message string DEFAULTHELP supplied by the caller.
/
/  How it works:
/
/  Returns: 1.
/
/  Error Conditions:
/
/  Side Effects:
/
/  Related to:
/
/  Called by: 
/
/  Home: cmd2/help.c
*/

/*** Copyright (C) 1987-1992 Camber-Roth.  All rights reserved.  */

/*: AA  7/29/88 17:40 Source code master starting point */
/*: JV  9/15/89 17:45 Added _beg_help and _end_help AFM's */
/*: CR 10/09/89 11:24 Use XW unit codes. */
/*: CR 10/17/89 15:17 Replace 2nd arg of FIRE_ARM with "". */
/*: CR  2/27/90 15:10 Help text goes to help window XWHL. */
/*: CR  5/11/90 18:07 Add "HELP -s string" syntax. */
/*: CR  7/14/90 12:00 Use thisuser.is_organizer. */
/*: JX  2/22/92 17:30 Chixify, add header. */
/*: CX  3/17/92 15:44 Replace FIRE_AFM with fire_afm(). */
/*: JX  5/13/92 15:38 Remove unnecessary success. */
/*: CL 12/11/92 13:47 Long Chxvalue N. */
/*: CP  8/30/93 14:57 Replace parse_output() with parse_rout(). */

#include "caucus.h"
#include <stdio.h>
#include "chixuse.h"
#include "done.h"
#include "redirect.h"

extern struct master_template  master;
extern Userreg          thisuser;
extern union  null_t           null;
extern Chix                    ss_dotnl;
extern Chix                    afm_name;

FUNCTION  help (command, default_help)
   Chix command; 
   char *default_help;
{
   static Chix helpptrs;
   short  found, out;
   Chix   syscancel();
   Chix   newconf, helpwords, echotab, word1, word2, word3, message;
   Redirect red;

   ENTRY ("help", "");
   
   newconf    = chxalloc (100L, THIN, "help newconf");
   word1      = chxalloc (100L, THIN, "help word1");
   word2      = chxalloc (100L, THIN, "help word2");
   word3      = chxalloc (100L, THIN, "help word3");
   message    = chxalloc (100L, THIN, "help message");
   echotab    = chxalloc (100L, THIN, "help echotab");
   helpwords  = chxalloc (100L, THIN, "help helpwords");

   /* First fire off initial AFM. */
   fire_afm ("_beg_help", null.chx, newconf);
      
   mdtable ("helpwords", &helpwords);
   mdtable ("ech_Aopts", &echotab);

   /*** Output help text to a file, or the terminal?  If the latter,
   /    display to the help window XWHL, not the regular text window. */
   parse_rout     (&red, command);
   apply_redirect (&red);
   out = red.unit;
   if (out == XWTX)  out = XWHL;
   unit_write (out, NEWCPAGE);

   chxtoken (word1, null.chx, 1, command);   jixreduce (word1);
   chxtoken (word2, null.chx, 2, command);   jixreduce (word2);
   chxtoken (word3, null.chx, 3, command);

   /*** HELP all by itself. */
   if      (EMPTYCHX (word2))   mdwrite (out, default_help, null.md);

   /*** HELP -s dic_string. */
   else if (tablematch (echotab, word2) == 1) {
      if (NOT EMPTYCHX(word3))  mdwrite (out, ascquick(word3), null.md);
   }

   /*** HELP XXX. */
   else {

      if (chxvalue (word2, 0L) == L('?'))  chxcpy (word2, word1);

      found = tablematch (helpwords, word2);
      switch (found) {
         case (-1):
            mdwrite    (XWER, "gen_Tnohelp", null.md);
            unit_write (XWER, word2);
            unit_write (XWER, ss_dotnl);
            break;

         case (-2):     ambiguous (word2);     break;

         default:
            mdtable  ("helpptrs", &helpptrs);
            chxtoken (message, nullchix, found+1, helpptrs);
            mdwrite  (out, ascquick(message), null.md);
            break;
      }

      if (chxeq (message, CQ("h_cancel"))) {
         unit_write (out, syscancel());
         unit_write (out, CQ("\n\n"));
      }
      if (chxeq (message, CQ("h_set")))       sysset    (out, CQ("?"));
      if (chxeq (message, CQ("h_commands"))  &&  thisuser->is_organizer  
          &&  NOT sysbrktest())    mdwrite (out, "gen_Horgcom", null.md);
   }

   if (red.locked) {
      unit_close(red.unit);
      unit_unlk (red.unit);
   }

   /* Finally fire off closing AFM. */   
   fire_afm ("_end_help", null.chx, newconf);

   chxfree (newconf);
   chxfree (word1);
   chxfree (word2);
   chxfree (word3);
   chxfree (message);
   chxfree (echotab);
   chxfree (helpwords);
   RETURN (1);
}
