/*** LOOKTHROUGH.   Look through an item or response for certain words.
/                   and print the surrounding lines if found.
/
/    lookthrough (out, item, resp, words, needed, full)
/
/    Parameters:
/      int    out;               (display hits on this unit)
/      struct namlist_t *words;  (namelist of words to search for)
/      int    item;              (item to search)
/      int    resp;              (resp number to search, 0=>item text)
/      int    needed;            (number of words needed to make a match)
/      int    full;              (show whole response, or just hits?)
/  
/    Purpose:
/      LOOKTHROUGH searches through ITEM, response number RESP, for
/      words in the namelist WORDS.  Each time it finds one of those
/      words, it increments NEEDED.
/  
/    How it works:
/      When NEEDED reaches 0, we declare a match, print the 3 lines
/      surrounding the match on unit OUT.
/  
/    Returns:
/      LOOKTHROUGH returns 1 if a match was found, 0 if no match was found,
/      or -1 if the user interrupted the search.
/  
/    Error Conditions:
/  
/    Side Effects:
/  
/    Related to:
/  
/    Called by:
/  
/    Home: cmd2/lookthro.c
/  */

/*** Copyright (C) 1987-1992 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:40 Source code master starting point */
/*: CR  6/25/89 13:27 Expand size of str[], header[]. */
/*: CW  6/25/89 13:27 Expand size of str[], header[]. */
/*: CR  6/28/90 22:35 Check return from getline() for null. */
/*: CR 10/29/91 11:45 Add confnum arg to getline() call. */
/*: JX  2/25/91 14:21 Chixify, add header. */
/*: DE  5/26/92 16:40 Fix chixification */
/*: CR  9/12/92 19:28 Add FULL argument.  */
/*: CR  9/14/92 11:32 Add SHOWN argument to show_resp() call.  */
/*: CI 10/06/92 17:47 Chix Integration. */
/*: CP  7/01/93 16:05 Add 'brief' arg to show_resp() call. */
/*: CP  8/24/93 15:38 Add 'attach' arg to show_resp() call. */
/*: CK  9/19/93  0:07 Add 'onlyattach' arg to show_resp() call. */

#include <stdio.h>
#include "caucus.h"
#include "chixuse.h"

extern union null_t null;
extern short        confnum;

FUNCTION  lookthrough (out, item, resp, words, needed, full)
   struct namlist_t *words;
   int    out, item, resp, needed, full;
{
   struct namlist_t *p;
   short  line, success, lastline;
   int    shown;
   Chix   str, header, ptr, getline();

   ENTRY ("lookthrough", "");

   str     = chxalloc (100L, THIN, "lookthro str");
   header  = chxalloc (100L, THIN, "lookthro header");

   /*** Save the header for this item/response. */
   if ( (ptr = getline (XCRD, XCRF, confnum, null.chx, item, resp, 0,
                       (Textbuf) NULL)) == null.chx)
      {success = 0; goto done; }
   chxcpy (header, ptr);

   /*** Initialize counters, including "not found" marker for each word. */
   for (p=words->next;   p!=null.nl;   p = p->next)  p->num = 1;
   success    =  0;
   lastline   = -9;

   /*** For each line of text, look for a match. */
   for (line=1;   (ptr = getline (XCRD, XCRF, confnum, null.chx, item,
                                  resp, line, (Textbuf) NULL)) != null.chx;
                                  ++line) {
      if (sysbrktest())  { success = -1;    goto done; }
      chxcpy    (str, ptr);
      jixreduce (str);
      is_match (str, words, &needed);

      if (needed == 0) {
         success  =  1;
         if (full) {
            show_responses (out, confnum, item, resp, resp, 0, 0, 0, &shown, 0);
            break;
         }
         showfound (out, item, resp, line, lastline, header);

         /*** Once one match has been found, any further matches
         /    require only one word to be found. */
         for (p=words->next;   p!=null.nl;   p = p->next)  p->num = 1;
         needed   = -1;
         lastline = line++;
      }
   }

done:
   chxfree (str);
   chxfree (header);
   RETURN  (success);
}
