/*** RIGHTAUTHOR.   Is the author of RESPONSE on ITEM in our AUTHORLIST?
/
/    rightauthor (item, response, authorlist)
/
/    Parameters:
/      int    item                    (
/      int    response                (
/      struct namlist_t *authorlist   (
/  
/    Purpose:
/       Is the author of ITEM:RESPONSE in AUTHORLIST?
/  
/    How it works:
/  
/    Returns:
/      RIGHTAUTHOR returns 1 if:
/         the AUTHORLIST is EMPTY, or if
/         the author of this response/item is in AUTHORLIST.
/  
/      Otherwise, RIGHTAUTHOR returns 0.
/  
/    Error Conditions:
/  
/    Side Effects:
/  
/    Related to:
/  
/    Called by: 
/  
/    Home:  cmd2/rightaut.c
/ */

/*** Copyright (C) 1987-1992 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:40 Source code master starting point */
/*: CR  6/28/90 22:32 Check for null pointer from getline(). */
/*: CR 10/29/91 11:45 Add confnum arg to getline() call. */
/*: JX  2/25/92 14:41 Chixify, add header. */
/*: JX  5/13/92 15:40 Removed unnecessary EMPTY. */
/*: CI 10/07/92 13:30 Chix Integration. */

#include <stdio.h>
#include "caucus.h"
#include "chixuse.h"

extern union null_t null;
extern short        confnum;

FUNCTION  rightauthor (item, response, authorlist)
   int    item, response;
   struct namlist_t *authorlist;
{
   struct namlist_t *p;
   int    success;
   Chix   author, id, ptr, getline();

   ENTRY ("rightauthor", "");

   if (authorlist == null.nl)    RETURN (1);
   if (nlsize (authorlist) == 0) RETURN (1);

   author = chxalloc (40L, THIN, "rightaut author");
   id     = chxalloc (40L, THIN, "rightaut id");

   /*** If the set is restricted to the authors in AUTHORLIST, see if
   /    one of them matches the AUTHOR of this text. */
   if ( (ptr = getline (XCRD, XCRF, confnum, null.chx, item, response, 0,
                       (Textbuf) NULL)) 
               == null.chx) FAIL;
   chxtoken (author, null.chx, 2, ptr);

   for (p=authorlist->next;   p!=null.nl;   p = p->next) {
      chxtoken  (id, null.chx, 2, p->str);
      if (chxeq (id, author))  SUCCEED;
   }
   FAIL;

done:
   chxfree (author);
   chxfree (id);
   RETURN (success);
}
