/*** SEARCH.   Handle the SEARCH command:
/
/    search (comline);
/
/    Parameters:
/       Chix  comline;   (the command line)
/  
/    Purpose:
/       Handle the different forms of the SEARCH command:
/
/          SEARCH [ITEMS | RESPONSES] 
/                   [item_range [EACHOF | ANYOF | FOR words] ] [SAIDBY name]
/
/          SEARCH   PERSONS  name  [EACHOF | ANYOF | FOR words]
/  
/    How it works:
/  
/    Returns: 1.
/  
/    Error Conditions:
/  
/    Side Effects:
/  
/    Related to:
/  
/    Called by:
/  
/    Home: cmd2/search.c
/ */

/*** Copyright (C) 1988-1992 Camber-Roth.   All rights reserved. */

/*: AA  7/29/88 17:40 Source code master starting point */
/*: CR  8/24/88 12:27 Lower-case COMLINE before processing */
/*: JV  8/23/89 17:03 Added err mess about mess searching not allowed. */
/*: CR 10/09/89 11:30 Use XW unit codes. */
/*: JX  2/25/92 14:43 Chixify, add header. */
/*: DE  4/22/92 10:53 Fix 'comline' declaration */
/*: JX  5/27/92 14:59 Chixify strquote. */
/*: CR  7/28/92 18:12 Begin integration with parse_fields(). */
/*: CR  8/23/92 21:58 Add FULL argument. */
/*: CR  9/27/92 16:38 Check return from whosaidit() for null. */
/*: CI 10/07/92 14:40 Chix Integration. */
/*: CL 12/10/92 14:52 Long chxalter args. */
/*: CR  1/25/93 13:06 FAIL on bad answers to mdprompter() questions. */
/*: CP  7/01/93 20:28 Only parse out search-specific stuff. */
/*: CP  8/30/93 15:09 Replace parse_output() with parse_rout(). */

#include <stdio.h>
#include "caucus.h"
#include "redirect.h"

extern Chix         objectable;
extern Chix         ss_qt2;
extern union null_t null;
extern short        confnum;

FUNCTION  search (comline)
   Chix   comline;
{
   static Chix fieldtable, modstr, anyall;
   Chix   wildcard, sfor, person, saidby, data, rest;
   Chix   any, all, temp;
   int    choice, needed, size;
   int    fieldnum, success, anyof, allof;
   char   searchmod[10];
   struct namlist_t *forlist, *saidlist, *names;
   struct namlist_t *nlnames(), *whosaidit(), *nlnode();
   Redirect red;

   ENTRY ("search", "");

   any      = chxalloc (10L, THIN, "search any");
   all      = chxalloc (10L, THIN, "search all");
   temp     = chxalloc (90L, THIN, "search temp");
   rest     = chxalloc (90L, THIN, "search rest");
   wildcard = chxalloc (10L, THIN, "search wildcard");
   person   = chxalloc (40L, THIN, "search person");
   saidby   = chxalloc (40L, THIN, "search saidby");
   sfor     = chxalloc (10L, THIN, "search sfor");
   data     = chxalloc (40L, THIN, "search data");

   forlist = saidlist = null.nl;
   parse_rout     (&red, comline);
   apply_redirect (&red);
   jixreduce (comline);
   arrange   (comline);

   mdtable ("gen_Afieldarg", &fieldtable);
   mdtable ("sch_Amods",     &modstr);
   mdstr   (wildcard, "sch_Kwildname", null.md);

   /*** Must be SEARCH ITEMS/RESPONSES/MESSAGES/PERSON. */
   if ( (chxtoken (temp, rest, 2, comline)) == -1) {
      mdwrite (XWER, "sch_Tsearch", null.md);
      FAIL;
   }
   choice = tablematch (objectable, temp);
   switch (choice) {
      case (-2):   mdwrite (XWER, "sch_Eambobj", null.md);   FAIL;

      case ( 4):
      case ( 3):   mdwrite (XWER, "sch_Ebadobj", null.md);   FAIL;
   }

   /*** Parse out fields that have arguments.  FOR is list of search words.
   /    PERSON is a name,  as in SEARCH PERSON Joe Smith.
   /    SAIDBY is speaker, as in  SEARCH ITEM 5 FOR "hello" SAIDBY Joe Smith.
   /    APPEND keeps parsed-out fields that should be stuck back on to
   /    the end of the object RANGE. */
   anyof = allof = 0;
   while (parse_fields (rest, 1, ss_qt2, fieldtable, 1, &fieldnum, data)) {
      chxtrim (data);
      switch  (fieldnum) {
         case (0):  chxcpy (person, data);                 break;
         case (1):  chxcpy (sfor,   data);                 break;
         case (2):  chxcpy (saidby, data);                 break;
         case (3):  chxcpy (sfor,   data);    anyof = 1;   break;
         case (4):  chxcpy (sfor,   data);    allof = 1;   break;
         default:                                          break;
      }
   }

   /*** Parse out the search modifiers: BRIEF(0), FULL(1). */
   searchmod[0] = searchmod[1] = 0;
   while (parse_fields (rest, 1, ss_qt2, modstr, 0, &fieldnum, data))
      searchmod[fieldnum] = 1;

   /*** If SEARCH PERSON and no names given, prompt for them. */
   if (choice == 5  &&  EMPTYCHX(person)) {
      if (mdprompter ("sch_Pperson", nullchix, 0, "sch_Hperson",
                        80, person, nullchix) < -1) FAIL;
   }

   /*** If "FOR" was not specified, prompt for the words. */
   if (EMPTYCHX (sfor)  &&
       mdprompter ("sch_Pwords", nullchix, 0, "sch_Hwords", 80, 
                                 sfor, nullchix) < -1)  FAIL;
   while (chxalter (sfor, 0L, chxquick(", ",0), chxquick(" ",1))) ;
   forlist = nlnames (sfor, nullchix);

   /*** Is this an AND or OR search?  Determine how many words from
   /    FORLIST are NEEDED to make match.  Ask user if necessary. */
   size = nlsize (forlist);
   if      (size == 1)    needed = -1;
   else if (anyof)        needed = -1;
   else if (allof)        needed = -size;
   else {
      mdtable  ("fnd_Aanyall", &anyall);
      chxtoken (all, nullchix, 1, anyall);
      chxtoken (any, nullchix, 2, anyall);
      if (mdprompter ("fnd_Panyall", anyall, 2, "fnd_Hanyall",
                         10, temp, all) < 0)  FAIL;
      jixreduce (temp);
      needed = (tablematch (anyall, temp) ? -1 : -nlsize (forlist));
   }

   /*** IF SAIDBY was specified, turn the name(s) into a namelist.
   /    (SAIDBY "*" matches everyone.)  SEARCH PERSON skips SAIDBY.
   /    Otherwise, call whosaidit() to prompt the user. */
   if      (choice == 5)                 saidlist = nlnode(0);
   else if (chxeq   (saidby, wildcard))  saidlist = nlnode(0);
   else if (EMPTYCHX(saidby)) {
      if ( (saidlist = whosaidit()) == null.nl)  FAIL;
   }
   else {
      names = nlnames (saidby, nullchix);
      matchnames (XSND, XSNN, 0, nullchix, names, AND, &saidlist, NOPUNCT);
      nlfree (names);
      if (nlsize (saidlist) == 0)   FAIL;
   }


   /*** Now that all the search-specific fields have been parsed off
   /    and/or filled in, actually go do the appropriate type of search. */
   switch (choice) {
      case (-1):
      case ( 0):
      case ( 1): 
         searchitem (&red, rest, forlist, needed, saidlist, searchmod[1]);
         break;

      case ( 2):
         /* searchmess   (&red, rest, forlist, needed); */
         mdwrite (XWER, "sea_Enomess", null.md);
         break;

      case ( 5):
         searchperson (&red, person, forlist, needed, confnum, searchmod[0]);
         break;
    }

done:
   nlfree (forlist);
   nlfree (saidlist);
   if (red.locked) {
      unit_close (red.unit);
      unit_unlk  (red.unit);
   }

   chxfree ( any );
   chxfree ( all );
   chxfree ( temp );
   chxfree ( rest );
   chxfree ( wildcard );
   chxfree ( person );
   chxfree ( saidby );
   chxfree ( sfor );
   chxfree ( data );
   RETURN  (success);
}
