/*** STATUS.  General information display and user's current status.
/
/    status (comline)
/
/  Parameters:
/     Chix   comline   (the entire command line)
/
/  Purpose:
/    STATUS tells the user:
/      what conference pern's on
/      who  the organizer is
/      who  the user is!
/      what the date and time are
/      what the <CANCEL> key is
/      what new items, new responses, new messages, new participants,
/         and unseen items the user has.  
/
/  How it works:
/
/  Returns:  1.
/
/  Error Conditions:
/
/  Side Effects:
/
/  Related to:
/
/  Called by:  interpret ()
/
/  Home:  cmd2/status.c
*/

/*** Copyright (C) 1985-1992 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:40 Source code master starting point */
/*: CR 11/11/88 21:35 Change sysversion() call. */
/*: CR  1/31/91 13:41 Use person_format() to display organizer name. */
/*: CR  6/25/91 15:27 Add 'confnum' arg to srvldmas() calls. */
/*: CR  7/16/91 17:08 Add HOST argument to person_format() calls. */
/*: CR  7/30/91 18:14 Add 3rd arg to loadreg(). */
/*: JX  2/22/92 17:00 Chixify. */
/*: DE  5/26/92 15:48 Fix chixification */
/*: JX  5/28/92 13:52 More Chix fix. */
/*: JV  5/05/93 12:10 Add CHN of original host for linked conferences. */
/*: CP  8/30/93 15:00 Replace parse_output() with parse_rout(). */

#include <stdio.h>
#include "caucus.h"
#include "chixuse.h"
#include "redirect.h"

extern struct master_template  master;
extern struct partic_t         thisconf;
extern union  null_t           null;
extern Userreg          thisuser;
extern short            confnum;
extern Chix             confname;

FUNCTION  status (comline)
   Chix   comline;
{
   Userreg org, make_userreg();
   Chix    result;
   Chix    syscancel(),  person_format();
   long    systime();
   Chix    format, date, day, time;
   Chix    lname, tname, orighost;
   static  Chix s_canis;
   Redirect red;

   ENTRY ("status", "");

   day    = chxalloc (32L, THIN, "status day");
   time   = chxalloc (32L, THIN, "status time");
   date   = chxalloc (16L, THIN, "status date");
   lname  = chxalloc (32L, THIN, "status lname");
   tname  = chxalloc (32L, THIN, "status lname");
   format = chxalloc (100L, THIN, "status format");
   orighost = chxalloc (32L, THIN, "status orighost");

   org = make_userreg();

   parse_rout     (&red, comline);
   apply_redirect (&red);
   unit_write (red.unit, NEWCPAGE);

   sysversion (red.unit, 1);
   mdwrite    (red.unit, "sta_Tconf", null.md);
   unit_write (red.unit, confname);

   get_trueconf (master.cnum, lname, tname, orighost);
   if (chxlen (orighost) > 0) {
      mdwrite      (red.unit, "sta_Torighost", null.md);
      unit_write   (red.unit, orighost);
   }

   loadreg (master.organizerid, org, NULLSITE);
   mdstr   (format, "sta_Forg", null.md);
   result = person_format (format, org->name, org->id, null.chx,
                           org->phone, org->briefs);
   unit_write (red.unit, result);
   chxfree (result);

   mdstr (format, "sta_Fyouare", null.md);
   result = person_format (format, thisuser->name,  thisuser->id, null.chx,
                                   thisuser->phone, thisuser->briefs);
   unit_write (red.unit, result);
   chxfree (result);

   sysdaytime (day, time, 0, systime());
   chxformat  (date, CQ("%s %s"), 0L, 0L, day, time);
   mdwrite    (red.unit, "sta_Ttime", null.md);
   unit_write (red.unit, date);
   unit_write (red.unit, CQ("\n"));

   mdwrite    (red.unit, "sta_Tcancel", &s_canis);
   unit_write (red.unit, syscancel());
   unit_write (red.unit, CQ("\n"));

   srvldmas (1, &master, confnum);
   show_new (red.unit, &thisconf);

   if (red.locked) {
      unit_close (red.unit);
      unit_unlk  (red.unit);
   }

   free_userreg (org);

   chxfree (day);
   chxfree (time);
   chxfree (date);
   chxfree (format);
   chxfree (lname);
   chxfree (tname);
   chxfree (orighost);
   RETURN (1);
}
