/*** WHOSAIDIT.   "Who said the words you're looking for?"
/
/   struct namlist_t *whosaidit()
/
/  Parameters: none.
/
/  Purpose:
/    FIND calls WHOSAIDIT in an attempt to reduce the amount of text
/    to search by asking the user who said the words we're about to
/    search for.  WHOSAIDIT prompts the user for a name, prints the
/    list of matching names, and asks the user to confirm that these
/    are the names desired.
/
/    WHOSAIDIT returns a namelist of the matching names.  If the
/    user doesn't know "who said it", the namelist is empty.  If the
/    user interrupts with <CANCEL>, it returns NULL. 
/
/  How it works:
/
/  Returns:
/
/  Error Conditions:
/
/  Side Effects:
/
/  Related to:
/
/  Called by: 
/
/  Home: 
*/

/*** Copyright (C) 1987-1992 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:40 Source code master starting point */
/*: JV  9/06/89 12:51 Added #ing of people in display, choice for exact. */
/*: CR 10/09/89 11:30 Use XW unit codes. */
/*: CR 10/11/89 10:17 Cleanup bugs surrounding use of nlunique(). */
/*: CR 10/31/89 18:18 Don't automatically add \n to fnd_Tdontknow. */
/*: CR  3/29/91 17:28 Use AND/OR code for matchnames() call. */
/*: CR  8/02/91 14:00 Add punctuation argument to matchnames() call. */
/*: JV 10/31/91 15:09 Declare sprintf */
/*: CR 11/07/91 14:18 Add confnum arg to matchnames() call. */
/*: JX  3/04/92 14:24 Chixify, add header. */
/*: DE  5/26/92 15:24 Fix chixification */
/*: CI 10/07/92 15:56 Chix Integration. */

#include <stdio.h>
#include "caucus.h"
#include "chixuse.h"

extern Chix                  yesnotable;
extern union null_t          null;

FUNCTION  struct namlist_t *whosaidit()
{
   struct namlist_t *names, *matches, *p, *nlunique(),
                    *nlnode(), *nlnames(), *nlexact, *retval;
   short  number, ok, i;
   Chix   authorstr, dontknow, format, id, answer, message, yes;

   ENTRY ("whosaidit", "");

   id        = chxalloc (40L,  THIN, "id");
   yes       = chxalloc (40L,  THIN, "yes");
   answer    = chxalloc (20L,  THIN, "answer");
   format    = chxalloc (20L,  THIN, "format");
   message   = chxalloc (200L, THIN, "message");
   dontknow  = chxalloc (20L,  THIN, "dontknow");
   authorstr = chxalloc (80L,  THIN, "authorstr");

   /*** Loop until we've got a definitive answer as to who said what the
   /    user is looking for. */
   mdstr   (dontknow, "fnd_Tdontknow", null.md);
   while (1) {
      if (mdprompter ("fnd_Pwhosaid", nullchix, 0, "fnd_Hwhosaid", 60, 
                       authorstr, dontknow) < -1)  {
         retval = null.nl;
         goto done;
      }
      if (chxeq (authorstr, dontknow))  {
         retval = nlnode(1);
         goto done;
      }

      names = nlnames (authorstr, null.chx);
      matchnames (XSND, XSNN, 0, null.chx, names, AND, &matches, NOPUNCT);
      nlfree (names);
/*    inthisconf (&matches); */  /* Person might *have been* a member! */
      number = nlsize (matches);

      if (number == 0) {                      /* No such participant. */
         mdstr      (format, "fnd_Fnopart", null.md);
         chxformat  (message, format, 0L, 0L, authorstr, null.chx);
         unit_write (XWTX, message);
         nlfree (matches);
         continue;
      }

      /*** Display the list of names that matched AUTHORSTR. */
      for (p=matches->next, i=1;   p!=null.nl;   p = p->next) {
         chxtoken  (id,   null.chx, 2,  p->str);
         person_is (XWTX, id, 0, null.par, i++);
      }

      /*** Are these the people the user had in mind? */
      chxtoken (yes, null.chx, 2, yesnotable);

      ok = mdprompter ( (number==1 ? "fnd_Pperson" : "fnd_Ppeople"), 
                      yesnotable, 2, "fnd_Hpeople", 10, answer, yes);

      if (ok     > 0)  { retval = matches; goto done; }
      if (number > 1) {
         nlexact = nlunique (authorstr, matches, "fnd_Pexact");
         if (nlexact != null.nl) {
            nlfree (matches);
            retval = nlexact;     /* This namelist has only one person */
            goto done;
         }
      }    
      nlfree (matches);               /* At this point, user has not */
      retval = null.nl;               /* selected anyone. */
      goto done;
   }

done:
   chxfree (id);
   chxfree (yes);
   chxfree (answer);
   chxfree (format);
   chxfree (message);
   chxfree (dontknow);
   chxfree (authorstr);
   RETURN  (retval);
}
