
/*** CHIXUSE.H.  Include file for applications that use chix functions. */

/*** Copyright (C) 1990 Camber-Roth.  All rights reserved. */

/*: CX 12/10/90 23:31 New file. */
/*: CR  4/30/91 23:40 Split off from old chix.h. */
/*: CR  6/19/91 13:10 Move nullchix def from chixdef.h to chixuse.h */
/*: CR  7/01/91 18:07 Add #ifndef... */
/*: CR  7/10/91 13:09 Make chxresize() a public function. */
/*: CR  7/11/91 12:05 Define EMPTYCHX. */
/*: CR  1/03/92 13:19 Raise ALLCHARS to 32760. */
/*: CR  3/20/92 15:17 Add CHXSUB macro. */
/*: CX  5/21/92 14:23 Add ascquick() decl. */
/*: JX  5/26/92 17:31 Move CQ, NEWLINE here from caucus.h, add NEWCPAGE. */
/*: JX  6/09/92 09:16 Add nullchix to EMPTYCHX. */
/*: JX  7/01/92 15:28 Raise ALLCHARS to 65530. */
/*: CR 11/15/92 23:09 Lower ALLCHARS back to 32760, damn it. */
/*: CR 12/10/92 14:58 Make maxlen member, some return values, long. */
/*: CR 12/11/92 16:22 ALLCHARS is now -1L, has special meaning. */
/*: CR 12/15/92 15:12 ALLCHARS is now -1000L, since -1 may be meaningful. */
/*: CR 12/21/92 14:11 Rearrange so ischix is contiguous with name. */
/*: CR  1/13/93 15:01 Move SIZE from chixdef.h. */
/*: CP  8/01/93 16:02 Add MEM_DEBUG macros. */

#ifndef CHIXUSE
#define CHIXUSE

/*** Currently, we only allow for two sizes of Chix: 1 and 2 bytes. */
#define THIN      1   /* Value must be equal to # of bytes. */
#define WIDE      2   /* Value must be equal to # of bytes. */

/*** A chix is actually a pointer to a structure.  The structure
/    contains a pointer to a dynamically-allocated block of memory,
/    which contains the "string".  The structure also contains other
/    control information about the string. */
struct chix_t {
   long    ischix;    /* Magic number to help confirm this is a chix! */
   char    name[16];  /* Name of this string, for debugging. */
   long    maxlen;    /* Current max length of string data, in characters. */
   char   *p;         /* Pointer to string data. */
   long    actlen;    /* Actual length, used only in chxcat(). */
   short   dic;       /* Dictionary number, if relevant. */
   short   resize;    /* Count of number of resizes of this string. */
   char    type;      /* # of bytes per character in this string. */
   char    expand;    /* Reserved for expansion. */
};

typedef struct chix_t *   Chix;

#define  nullchix     ((Chix) NULL)

/*** Collating stuff. */
#define  CLEAR_CT      0
#define  ADDRULE_CT    1
#define  APPLY_CT      2

/*** Special character stuff. */
#define  SC_GET        0
#define  SC_SET        1
#define  SC_BLANK      2

/*** Practically infinite number of characters; used when calling
/    chxcatsub(). */
#define  ALLCHARS     -1000L

/*** Shorter form of chxalsub() when we always want a substring from
/    position I through the end. */
#define  CHXSUB(a,i)   chxalsub(a,i,ALLCHARS)

/*** "Quickies" */
#define CQ(x)        chxquick (x,      0)
#define NEWLINE      chxquick ("\n",   9)
#define NEWCPAGE     chxquick ("\001", 8)

/*** MEM_DEBUG macros. */
#define MEM_ALLOC    1000
#define MEM_FREE     1001

/*** Is a chix empty, i.e. cleared? */
#define  EMPTYCHX(x)   (x == nullchix || chxvalue (x, 0L) == 0L)

/*** Long version of sizeof(). */
#define  SIZE(x)      ( (long) sizeof(x) )

/*** Declaration of return values of all "public" (i.e., meant to be used
/    by an application) chix functions. */
char *ascquick();
long  ascofchx();
Chix  chxalloc();
Chix  chxalsub();
int   chxalter();
int   chxcat();
int   chxcatval();
int   chxclear();
long  chxcollate();
int   chxcompare();
int   chxconc();
int   chxcpy();
int   chxeq();
int   chxfree();
long  chxindex();
long  chxlen();
long  chxlong();
int   chxofascii();
int   chxcatsub();
Chix  chxquick();
int   chxresize();
long  chxrevdex();
int   chxspecial();
int   chxtoken();
int   chxtype();
long  chxvalue();

#endif
