
/*** SERVER.H.   Include file for Caucus server routines. */

/*: CR 11/16/88  0:17 Initial release of server code. */
/*: CR  4/17/89 12:10 Add (missing?!) srv_stat macros. */
/*: JV  6/22/89 18:28 Added srv_stat stuff. */
/*: JV  7/19/89 11:32 Added CA to file typedef. */
/*: JV  8/02/89 15:35 Added handicap.h include file. */
/*: CW  7/20/89 16:03 Add TERMAFTER. */
/*: JV  2/28/90 16:43 Remove def of SHUTDOWN.  Noone used it. */
/*: JV  2/21/91 17:00 Add systype.h, MAXCLIENTS. */
/*: CR  7/01/91 20:37 Include handicap.h, systype.h w/o conditionals. */
/*: CR  7/01/91 20:42 Move server status values to srvstat.h. */

#include "handicap.h"
#include "systype.h"
#include "srvstat.h"
#include "errchk.h"

/*** Definition of type "file". */
#if CX | CVM | CA
typedef FILE   *file;
#define NOFILE  ( (file) NULL)
#endif

/*** There are two main data structures in the Caucus server program.
/    The first is CLITAB, the table of connected clients.  It keeps
/    track of what each client is doing, i.e. how many and which
/    files it has open and locked, etc.  
/
/    The second data structure is FILTAB.  It keeps track of the
/    actual name of a file that is in use.  FILTAB and CLITAB
/    point into each other.  For example, each client in CLITAB
/    has an array of open files ("array of units"), and each of
/    them point into FILTAB.  On the flip side, FILTAB keeps a
/    queue of clients desiring to lock a given file.
/
/    Since clients will come and go in an unorderly fashion,
/    the "alive" field = 1 for client slots that are being used,
/    and = 0 for client slots that are not being used.  */

struct clitab_t {
   short  message;          /* Message we're waiting on. */
   short  locks;            /* Number of files we have locked. */
   short  waiting;          /* Waiting for a lock? */
   short  dicnum;           /* Current dictionary number. */
   int    alive;            /* 0=slot not used, 1=slot used. */
   long   dicpos;           /* Position in current dictionary. */
   long   lastreply;        /* date of last reply sent to client. */
   struct unit_t *unit;     /* Array of units. */
};

/*** For each client, the server must keep track of the files, indexed by 
/    Caucus unit number, that it has locked or open.  That's what the
/    array of units is for. */
struct unit_t {
   file   f;                /* File opened for this unit. */
   short  n;                /* Index to table of file names. */
   char   write;            /* True if open for writing. */
};

/*** In FILTAB, each entry corresponds to one file, and contains a NAME
/    and a QUEUE of clients that desire to lock this file.  The first 
/    entry in a queue is the client that has the file locked now. */

struct file_t {
   char   name[80];
   struct queue_t *queue;
   long   lockdate;
   short  u;
};

struct queue_t {
   short  client;
   struct queue_t *next;
};

/*** A related data structure is the queue of messages that are ready
/    to be executed by the server.  For example, suppose a file is
/    locked, and has another client waiting for a lock.  When the file
/    is unlocked, the 2nd lock request is put on the ready-to-execute
/    queue. */
struct ready_t {
   short  mess;
   struct ready_t *next;
};

/*---------------Miscellaneous stuff-----------------------------*/

/*** For a client to be dead, it must have been "gone" for
/    at least DEADSEC seconds. */
#define  DEADSEC 180 

/*** Maximum number of clients at one time. */
#define  MAXCLIENTS 200

/*** Dictionary stuff peculiar to the server. */
struct diction_t {
   short  fd;         /* File descriptor dictionary is open on. */
   short  clients;    /* Number of clients that have it open. */
};
