
/*** SYSTYPE.H.  Define the type of this system.  The known
/    definitions are: 
/
/   SS | SN  (together known as SU)
/   A3 | AU | BU | FU | MU | MV | PU | SX | TX | HU | IX | SV | RO |
/   U3 | LA | DY | UL | AX
/            (or UNIX)
/   IM | PN | PR | VV
/   CX | CA | CVM     (or SRV)
/   PX | PS           (panel emulation under Xenix or SunOS)
/ */

/*: AA  7/29/88 17:47 Source code master starting point */
/*: CR  9/12/88 16:50 Add Data General systems*/
/*: CR  9/19/88 12:31 Add "CX" type for Client Xenix model. */
/*: CR  9/21/88 15:14 Add "HU" type for HP/UX. */
/*: JV  9/22/88 12:21 Changed the DG system definitions.*/
/*: CR  9/24/88  0:25 Add "IX" system type code. */
/*: CR 10/19/88 12:42 Remove old VM file i/o emulation stuff. */
/*: JV 12/06/88 14:34 Added AOS client code.*/
/*: CR  3/06/89 10:09 Add CVM to SRV definition. */
/*: CR  3/14/89 15:16 Add CVM back in to SRV def (it got removed...) */
/*: CR  4/29/89 13:57 Break SU into SN and SS. */
/*: CR  6/09/89 11:24 Add "SV" type for Sperry 5000. */
/*: CW  5/08/89 14:49 Add PX system type for panel-emulation Xenix code. */
/*: CR 10/12/89 15:28 Add DU for DG/UX. */
/*: CR  1/09/90 10:44 Add PR19 definition. */
/*: CR  4/30/90 20:45 Add more to header comments. */
/*: JV  7/06/90 16:01 Add RO for MIPS RISC/os. */
/*: CR  9/20/90 16:56 Add PS system type. */
/*: CR  9/27/90 12:09 Add U3,AU system types. */
/*: CJ 11/26/90 21:48 Add LA system type. */
/*: JV  1/13/91 17:08 Add CEO. */
/*: CR  3/04/91 18:39 Add DY system type. */
/*: CR  3/11/91 14:01 Add UL & AX system types. */
/*: JV  2/21/91 20:16 Add SYSTYPE marker. Fix HU comment. */
/*: CR  4/06/91 17:26 Integrate recent changes. */
/*: SB  5/16/91 10:13 Add DP for Dynix/PTX*/
/*: CR  1/22/92 16:28 Add comments about Decstation. */
/*: CR  5/05/92 12:12 DY = DYP | DY3, due to very slight differences. */
/*: JV  7/17/92 15:14 Add U4 for Unix Sys V Rel 4. */
/*: CR  7/21/92 17:40 Remove DP (in favor of DYP). */
/*: CR  8/20/92 15:03 MV becomes Coherent. */
/*: CR 10/22/92 15:12 Add U4 to UNIX. */
/*: CR 12/11/92 15:44 Define SMALL_MEMORY. */
/*: CR  2/16/94 19:42 Define DX. */

#ifndef SYSTYPE      /* #endif at end of file */
#define SYSTYPE U4

/*** Set SMALL_MEMORY 1 for systems with 16 bit limit on malloc(). */
#define  SMALL_MEMORY 0

/*** Unix systems. */
#define  A3   0    /* AT&T 3B System V Unix */
#define  AU   0    /* Apple Unix AUX II. */
#define  AX   0    /* IBM AIX. */
#define  BU   0    /* Berkeley 4.2 Unix */
#define  DX   0    /* DG/UX.  (Based on RO, with small changes.) */
#define  DY3  0    /* Sequent Symmetry Dynix 3.0. */
#define  DYP  0    /* Sequent Symmetry Dynix/ptx. */
#define  DY   DY3 | DYP
#define  FU   0    /* Fortune For:pro Unix. */
#define  HU   0    /* Hewlett-Packard HP/UX. */
#define  IX   0    /* Interactive Systems Unix 286/386. */
#define  LA   0    /* Omron LUNA system.  A3 with systemid() nodename. */
#define  MU   0    /* Masscomp RT Unix */
#define  MV   0    /* Coherent (yes, MV is a silly name...) */
#define  PS   0    /* Panel-emulation SunOS. */
#define  PU   0    /* Plexus System V Unix */
#define  PX   0    /* Panel-emulation Xenix. */
#define  RO   0    /* MIPS Computers RISC/os */
#define  SX   0    /* SCO Xenix */
#define  SV   0    /* Sperry 5000 Unix System V.  (Now Unisys) */
#define  SN   0    /* SunOS, for a network. */
#define  SS   0    /* SunOS, for a single processor. */
#define  TX   0    /* Tandy 6000 Xenix */
#define  U3   0    /* Unix 386 "standard" - like SX with IX systemid(). */
#define  U4   1    /* Unix SVR4.  Works for PC.  Others? */
#define  UL   0    /* DEC Ultrix (vaxstation). */
                   /* For Ultrix Decstation, use RO, GUARD 1, and */
                   /* compile & link with -YSYSTEM_FIVE. */

#if DX
#undef   RO
#define  RO   1
#endif

#define  SU    SS | SN
#define  UNIX_1  A3 | AU | BU | FU | MU | MV | PU | SU | SX | TX | HU
#define  UNIX_2  IX | SV | RO | U3 | LA | DY | UL | U4
#define  UNIX    UNIX_1 | UNIX_2

/* If GUARD is 1, forces use of guardian lock-files for system V unix
/  systems (even though they ought to be able to use lockf().) */
#define  GUARD  0

/*** Set of Data General systems. */
/*  AOS/VS systems now use "CA", as it uses a server. */
#define  AV   0    /* AOS/VS (Advanced Operating System/ Virtual Storage) */
#define  AR   0    /* AOS/RT32 (Real-Time version of AOS/VS) */
#define  AOS  AV | AR   /* In all cases so far, there is no difference... */
#define  CEO  1    /* Include CEO Integration Functionality */

/*** Most other systems. */
#define  IM   0    /* IBM PC MSDOS */
#define  JM   0    /* Japanese(?) IBM PC MSDOS */
#define  PN   0    /* IBM PC Network - DOS 3.x */
#define  PR   0    /* All versions of Prime PRIMOS.  Standard is rev 22. */
#define  PR19 0    /* Also make this 1 if using Primos rev 19. */
#define  VV   0    /* DEC Vax/VMS */

/*** Client-Server models.  SRV is logical OR of all client models. */
#define  CX   0    /* Client Xenix: for testing client/server models. */
#define  CA   0
#define  CVM  0
#define  SRV  CX | CA | CVM

#define  VMU  0    /* Obsolete, but kept for the moment. */
#define  VMR  0    /* Ditto. */
#define  VM   0    /* Ditto. */


/* If Vax, define some Vax-specific macros */
#if VV
   /*** Definitions of resource locking symbols, stolen directly
   /    out of SYS$LIBRARY:STARLET.REQ.  */
#define  LCK$K_EXMODE    5
#define  LCK$M_SYSTEM   16
#define  PSL$C_KERNEL    0
#define  PSL$C_EXEC      1
#define  PSL$C_SUPER     2
#define  PSL$C_USER      3
#endif

/*** If emulation of panel library stuff, undefine some symbols
/    that otherwise conflict with curses. */
#if PX | PS
#undef   CA
#undef   IM
#endif

#endif      /* For SYSTYPE */
