
/*** VC.H.   VC include file. */

/*: AA  9/19/88 12:00 First official version of VC code. */
/*: CR  9/20/88 14:59 Add new VCERNUM error codes. */
/*: JV  9/27/88 10:09 Added AOS stuff.*/
/*: JV  9/27/88 10:26 Separated out lint satisfier. AOS doesn't like it.*/
/*: CR 11/13/88 20:07 Integrate CX and CA vc changes. */
/*: CR 12/13/88 14:58 Integrate CVM code. */
/*: JV 11/07/88 14:07 Added new AOS defs, changed struct ipcbuffer. */
/*: JV  2/11/89 16:12 Changed SENDBUFLEN & RECVBUFLEN to 3000.*/
/*: JV  3/03/89 11:11 Added vcflag def. */
/*: CR  3/21/89 12:54 Fix syntax error in vcflag_template defn. */
/*: JV  3/16/89 17:25 Changed TERMINATE to SHUTDOWN. Used elsewhere. */
/*: JV  3/29/89 15:48 Added vcmethod #define's. */
/*: JV  4/12/89 20:50 Changed MESSAGE to VCMESSAGE. Former used in caucus.h */
/*: JV  4/12/89 20:51 Fixed syntax error in struct ipcbuffer. */
/*: JV  7/06/89 17:30 Added AOS/VS memxfer slot struct definition. */
/*: JV  8/02/89 15:32 Added handicap.h include file. */
/*: JV 10/05/89 15:15 Add timestamp field to memslot_t. */
/*: JV  2/28/90 16:45 Added srv_stat value defs. */
/*: JV  2/05/91 15:50 Added SPEAKINT. */
/*: JV  2/21/91 20:15 Added systype.h and GABBY and docs, rm MAXCLIENTS. */
/*: JV  3/12/91  2:35 Make ms_template.class unsigned. */
/*: CR  7/01/91 20:42 Move server status values to srvstat.h. */

#include "handicap.h"
#include "systype.h"
#include "srvstat.h"

/*** List of VCERNUM values, for all systems. */

#define  VCE_NOPEND    0    /* There were no pending requests of that type */
#define  VCE_NOMEM    10    /* out of dynamic memory! */
#define  VCE_NOINIT   11    /* VCINIT never called! */
#define  VCE_NOMETH   12    /* No valid VC method specified. */
#define  VCE_FULL     20    /* client or message table full */
#define  VCE_ISCON    30    /* server already connected, or there are still
                               connected clients */
#define  VCE_NOCON    40    /* server or client not connected */
#define  VCE_TIME     50    /* timeout waiting for an event */
#define  VCE_RANGE    60    /* message number out of range */
#define  VCE_NOMESS   70    /* no such message */
#define  VCE_RLEN     80    /* reply length too long */
#define  VCE_GHOST    90    /* The specified client is not one */
#define  VCE_VMCMS   100    /* VM/CMS specific error - see VCERVM */
#define  VCE_RETRY   110    /* Some resource is being used. VC suggests
                               that you retry */
#define  VCE_NOSRV   120    /* Couldn't create server */
#define  VCE_TWOSRV  130    /* There already is a server for this application*/
#define  VCE_FAILCON 140    /* Failed connection attempt */
#define  VCE_PACKLEN 150    /* A packet was too long to send */
#define  VCE_NEWCLI  160    /* IPCRECV got a new client to be cleared */
#define  VCE_UNKNOWN 200    /* An 'unknown' error has occurred. This usually
                               implies that processing shouldn't continue */

/* Values for method of vc communications. Each vc conection should
/  have its own vcflag structure to define it. */
#define  IPC   0    /* Whatever the OS defines as 'IPC', if it does */
#define  MEM   1    /* Direct memory transfer */
#define  STR   2    /* Unix Streams */
#define  SOC   3    /* Unix Sockets */
#define  X25   4    /* X.25 Network Connection */
#define  TCP   5    /* TCP/IP Network Connection */
#define  DSK   6    /* Any other method which uses disk files
                       for communication */

struct vcflag_template {
   short loctimeout;
   short remtimeout;
   short method;
};

/*** System-dependent definitions begin here... */

/* GABBY clients are only loosely coupled to their servers, so
   the server won't know if a client dies unexpectedly.  Therefore
   the server will remove the entry for any client that hasn't sent
   a message to the server in SPEAKINT seconds (SPEAK INTerval).
   This only works because the client sends a NULL message to the server
   if it hasn't sent us a real message in SPEAKINT/2 seconds. If the
   server *does* remove a client entry, it'll wait 2*SPEAKINT secs before
   checking again, to give other clients a chance to get up to speed. */
#if CA
#define GABBY 1
#else
#define GABBY 0
#endif

#if CX
/*** Define common system routines to make lint happy. */
char    *strcpy(), *strcat(), *sprintf(), *vcmem();

#define  SIGBASE    20 

/*** Names of files used for communication. */
#define  CONLCK  "/tmp/vc_l"
#define  CONNAM  "/tmp/vc_n"
#define  CONDON  "/tmp/vc_d"

struct vm_template {
   short pid;        /* PID of the client */
   short usesig;     /* Unix signal number to use in signalling the server. */
   FILE *fdsen;      /* File descr for Unix pipe to send on. */
   int   fdrec;      /* File ptr for Unix pipe to receive on. */
   char  name[12];   /* OS-dependent field. */
   short sigset;     /* Another Unix signalling gadget. */
};

struct ms_template {
   long  count;
   long  meslen;
   long  replen;
   unsigned short class;
   short machine;
   char *data;
   short pid;
   char  found;
};
#endif


/*---------------------------AOS/VS-------------------------*/
#if CA 
#define  READY         17
#define  DEAD          18
#define  FIRST         19

#define  LOCALTO      999    /* Length of time (sec) for client to 
                             wait for local server */
#define REMOTETO     9999    /* Length of time (sec) for client to
                             wait for remote server */

#define SHUTDOWN      250    /* Message from daemon killer: "Die!" */
#define DISCONNECT    100    /* Highest priority message class */
#define CONNECT        75    /* Connect to a server            */
#define VCMESSAGE      50    /* 'Normal' request class         */

/*** AOS client must talk to the server within
/    SPEAKINT (SPEAK INTerval) seconds or be cancelled.  */
#define  SPEAKINT 60

#define LOCCLIPRT      28    /* 'Local' Clients's local port number */
#define LOCSRVPRT      28    /* 'Local' Server's  local port number */
#define LOCCONPRT      55    /* Server's 'Local' Connection Port    */
#define LOCDISPRT      74    /* Server's 'Local' Disconnectino Port */
#define LOCSHDPRT      87    /* Server's 'Local' ShutDown Port      */
#define REMCLIPRT      82    /* 'Remote' Client's local port number */
#define REMSRVPRT      82    /* 'Remote' Server's local port number */
#define REMCONPRT      61    /* Server's 'Remote' Connection Port   */
#define REMDISPRT      17    /* Server's 'Remote' Disconnection Port*/
#define REMSHDPRT      98    /* Server's 'Remote' ShutDown Port     */

/* The following two macros MUST be EVEN integers.       */
#define SENDBUFLEN    3000    /* Size of IPC send buffer     */
#define RECVBUFLEN    3000    /* Size of IPC recv buffer     */

/* The following macro must calculate the size of the
/* 'ipcbuffer' header, defined below */ 
#define IPCHEADLEN    (4*sizeof (long) + sizeof (short) + sizeof (int) + 6)

#define MESSBUFLEN    (SENDBUFLEN - IPCHEADLEN)
#define VMNAMELEN     12

struct ipcbuffer {
   long count;              /* Unique Message Number           */
   long meslen;             /* Length of my message            */
   long replen;             /* Length of buffer for your reply */
   long machine;            /* Node number                     */
   int cgp;                 /* The client's global port number */
   short class;             /* Class of message                */
   char passw[6];           /* Password field                  */
   char mess [MESSBUFLEN];  /* Text of mess. */
};

struct vm_template {
   int   pid;
   int   ring;
   int   gportnum;      /* 'communication port' for this pid  */
   int   gdiscpnum;     /* 'disconnect port' for this pid */
   short usesig;
   char  name[VMNAMELEN];
};

struct ms_template {
   long  count;
   long  meslen;
   long  replen;
   long  machine;
   short class;
   short pid;
   short passw[6];
   char *data;
   char  found;
};

int port, pid, ring;  /* elements of gportnum */

/* The following defs are for the MEM version of AOS/VS VC stuff */
$shared $align(10) struct memslot_t {    /* Form of communication. */
                      short semasmores,  /* Condition of this slot */
		            clientnum,   /* Client that owns this slot or 0 */
			    pid;         /* Client's pid # */
                      long  timestamp;   /* Time last used (0=unused!) */
                      char  data[2038];  /* Data for this slot */
                   };

#define MESSSLOT    10            /* Number of memory xfer message slots. */
#define NUMSLOTS    MESSSLOT+3    /* Total number of memory xfer slots. */
#define CONNSLT     MESSSLOT+1    /* The Connection slot. */
#define DISCSLT     MESSSLOT+2    /* The Disconnection slot. */
#define SRVPIDSLT   0             /* Where the server puts its pid. */
#define HIGHSLT     NUMSLOTS-1    /* Highest reused slot. */

/* The next 4 are used in the semaphore locking scheme. */
#define CLIINUSE  1    /* Tells the world that this slot is in use. */
#define CLIDONE   2    /* Tells the server that the slot is ready.  */
#define SRVINUSE  4    /* Tells the client that this slot is being proc'd. */
#define SRVDONE   8  /* Tells the client that this request has been handled. */

#endif


/*--------VM/CMS dependent definitions.-------------*/
#if CVM
#define  EX_CONPEND  1
#define  EX_CONCOMP  2
#define  EX_PTHSEV   3
#define  EX_PTHQUI   4
#define  EX_PTHRES   5
#define  EX_REPLY_P  6
#define  EX_REPLY    7
#define  EX_MESSG_P  8
#define  EX_MESSG    9

#define  IPFGMCL     1
#define  IPFGPID     2
#define  IPFGMID     4

union  vc40    {      /* Miscellaneous double-word aligned storage block. */
   double aligner;
   long buf[10];
};

struct exin_pc {      /* Ext interrupt, Pending Connection. */
   short ippathid;
   char  ipflags1;
   char  iptype;
   short ipmsglim;
   char  ipfcncd;
   char  j0;
   char  ipvmid[8];
   long  ipuser[4];
   char  j1[8];
};

struct iucv_send {
   short  ippathid;
   char   ipflags1;
   char   iprcode;
   long   ipmsgid;
   short  class;       short machine;   /* Formerly iptrgcls */
   char  *ipbfadr1;
   long   ipbfln1f;
   long   ipsrccls;
   long   ipmsgtag;
   char  *ipbfadr2;
   long   ipbfln2f;
   long   j0;
};

struct iucv_tstcmp {
   short  ippathid;
   char   ipflags1;
   char   iprcode;
   long   ipmsgid;
   char   ipaudit[3];
   char   j0;
   long   iprmmsg1;
   long   iprmmsg2;
   long   ipsrccls;
   long   ipmsgtag;
   long   j1;
   long   ipbfln2f;
   long   j2;
};

struct iucv_dclbfr {
   char  j0[3];
   char  iprcode;
   long  j1;
   long  j2;
   long *ipbfadr1;
   long  j3[6];
};

struct vccon_template {
   short pathid;
   char  flags1;
   char  rcode;
   short msglim;
   char  fcncd;
   char  j0;

   char  vmid[8];
   char  user[16];
};

struct vm_template {
   short ippathid;
   char  name[10];
};

struct ms_template {
   long  count;
   long  ipmsgid;
   char *data;
   int   meslen;
   int   replen;
   short class;
   short machine;
   short ippathid;
   char  found;
};
#endif
