
/*** MDTABLE.  Get a table called NAME from the Master Dictionary.
/
/    ok = mdtable (name, table);
/
/    Parameters:
/       int     ok;      (success?)
/       char   *name;    (name of table to fetch)
/       Chix   *table;   (place to put the table)
/
/    Purpose:
/       Look in the current dictionary for a table called NAME.  Put
/       it in TABLE.
/
/    How it works:
/       Notes:
/         1. TABLE is a *pointer* to a Chix.
/         2. Chix handed to mdtable() are usually (always?) static.
/
/       IF...
/         TABLE has a value of nullchix, OR
/         TABLE contains a chix with a dictionary number different
/               than the current dictionary
/
/       ...THEN mdtable() allocates TABLE (if necessary), and loads table
/       NAME into it.
/
/       Otherwise, mdtable() just returns.
/
/    Returns: 1 on success, 0 if no table NAME in the dictionary.
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:
/
/    Operating system dependencies:
/
/    Known bugs:      none
/
/    Home:  inout/mdtable.c
/ */

/*** Copyright (C) 1987 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:41 Source code master starting point*/
/*: AM  2/07/89 10:27 Jcaucus changes*/
/*: CR 10/22/89 19:50 Use 2 bytes to save dic number in string. */
/*: JV 11/03/89 12:59 Changed mdbuffer to use MDBUFSIZE. */
/*: CR 11/22/89 15:10 Truncate mdbuffer[] properly. */
/*: CR  8/12/90 19:18 Cast sysmem() call. */
/*: JV  7/22/90 18:07 Cast arg to sysmem to shut lint up. */
/*: CX 10/18/91 16:23 Chixify, redo header comments. */
/*: CX  5/09/92 21:17 Change to use sysmdstr(). */
/*: CL 12/11/92 15:52 Long sysmem arg. */

#include <stdio.h>
#include "caucus.h"

extern union  null_t    null;
extern Userreg          thisuser;

FUNCTION  mdtable (name, table)
   char  *name;
   Chix  *table;
{
   int    status;
   char   temp[100];

   ENTRY ("mdtable", "");

   if (*table != null.chx) {
      if ( (*table)->dic == thisuser->dict)  RETURN (1);
      chxfree (*table);
      *table = null.chx;
   }

   *table = chxalloc (30L, THIN, "mdtable table");
   if (NOT sysmdstr (name, thisuser->dict, *table, &status)) {
      sprintf (temp, "(no table '%s')", name);
      chxofascii (*table, temp);
      RETURN (0);
   }
   (*table)->dic = thisuser->dict;

   RETURN (1);
}
