/*** CAUCUS.   Actual use of Caucus by a user.
/
/    When we get to this function, we have already identified
/    the person's userid.  Proceed to register the user (if needed).
/
/    The rest of this function handles the "JOIN which conference?"
/    prompt, and events that happen around it.  When the user
/    actually joins a conference, caucus() calls join(), which only
/    returns when the user leaves that conference.
/
/    Returns 1 normally, 0 if the user is aborting from Caucus for
/    some reason (such as pressing <CANCEL> during registration.)
/ */

/*** Copyright (C) 1985, 1987 Camber-Roth.  All rights reserved.  */

/*: AA  7/29/88 17:47 Source code master starting point */
/*: CR  1/17/89 14:07 Call expwarn(). */
/*: CR  2/11/89 15:52 Change call to log_user(). */
/*: JV  8/13/89 15:20 Removed call to globals(). Already in main! */
/*: JV  8/21/90 18:53 Check for ambiguous confnames. */
/*: CW  5/18/89 14:53 Add XW unit codes. */
/*: CR 10/07/89 13:02 Integrate & rearrange logic. */
/*: CR 10/09/89 15:06 Add 2nd arg to listconfs(). */
/*: JV 11/03/89 15:52 Added documentation. */
/*: CR 10/30/89 12:00 Remove result. */
/*: CR 11/13/89 16:55 Add _over_caucus AFM. */
/*: JV  1/06/90  0:35 Add mail importing code. */
/*: JV  2/21/90 05:36 Moved calls to sysoneuser() here from join(). */
/*: CR  7/17/90 12:11 Use new allowed_on() and primary_org() calls. */
/*: CR  8/12/90 19:05 Call fix_name() to enforce "(userid)" in full name. */
/*: JV  8/21/90 18:34 Add globals() after setdecode() to fix dicti. */
/*: JV 12/13/90 17:04 Add _begin_caucus AFM. */
/*: JV  1/13/91 11:30 Add comments about fixing begin_caucus. */
/*: CR  1/27/91 16:36 Make _begin_caucus handle JOIN command; add comments.*/
/*: CR  4/08/91 18:49 Add new unit_lock argument. */
/*: CR  7/02/91 16:25 globals() is now mainglobals(). */
/*: JV 10/28/91 14:50 Added 2nd arg to cnumber().*/
/*: CR 10/21/91 18:27 Use new form of allowed_on(). */
/*: WC 11/29/91 21:14 Add 3rd arg to cnumber(). */
/*: UL 12/11/91 18:43 Make unit_lock args 3-6 long.*/
/*: CR  1/07/92 15:18 Remove setdecode(). */
/*: CX  3/17/92 15:44 Replace FIRE_AFM with fire_afm(). */
/*: DE  3/18/92 13:56 Finished Chixifying */
/*: DE  3/23/92 12:35 Chixified */
/*: JX  5/29/92 16:29 Localized FORMAT. */
/*: JX  8/10/92 11:12 Flush allowed_on's ulist cache. */
/*: CR  7/17/92 13:50 Add _exit_caucus AFM. */
/*: CR  8/06/92 14:48 Add _exit_cantjoin AFM. */
/*: CR  9/21/92 11:24 Add _exit_noconf, _exit_ambconf handling. */
/*: CI 10/05/92 15:06 Chix Integration. */
/*: CR 10/08/92 23:22 Replace sysdiskfull() with diskfull(). */
/*: CR 10/11/92 21:57 Remove CQ from (md)prompter() call. */
/*: JV 12/04/92 13:42 Add sitevar arg to storereg(). */
/*: JV  4/23/93 14:58 Add CHECK at Join Which Donut? */
/*: JD  7/26/93 10:25 Add flag.diskfull. */
/*: CK 10/13/93 17:50 Check liense.confmax. */

#include <stdio.h>
#include "caucus.h"

extern struct flag_template    flag;
extern struct license_t        license;
extern Userreg                 thisuser;
extern Chix                    confid, confname;
extern short                   confnum;
extern union  null_t           null;
extern Chix                    ss_dotnl;
extern Chix                    afm_name;
extern Chix                    sitevar[];

FUNCTION  caucus()
{
   static Chix jointab;
   static char  first_time;
   struct namlist_t *groups, *nlnode();
   Chix   nextconf, newstr;
   int    v, success;
   short  found, may_join;
   long   systime();

   ENTRY ("caucus", "");

   newstr     = chxalloc (80L, THIN, "caucus newstr");
   nextconf   = chxalloc (40L, THIN, "caucus nextconf");
   success    = 0;

   if (NOT EMPTYCHX(flag.logfile)) log_user  (XSLG, 1);

   /*** See if the user is registered.  If not, register him/her.  If
   /    the user does not want to register, or registration aborts for
   /    some reason, go home. */
   if (NOT is_registered (thisuser)) {
      mdwrite (XWTX, "mai_Tbadreg", null.md);
      manager_text (flag.exitfile);
      if (NOT EMPTYCHX(flag.logfile)) log_user (XSLG, 0);
      FAIL;
   }

   /*** Check for one user per id at a time. */
   if (NOT sysoneuser("on")) sysexit (1);

   /*** Make sure user's full name includes "(userid)". */
   fix_name (thisuser);

   /*** User is registered.  Save current registration info, storereg()
   /    to log "user-entrance", and proceed to join a conference!*/
   storereg   (thisuser, 1, sitevar);
   mainglobals();
   mdtable    ("mai_Ajoin", &jointab);
   expwarn    ();

   /*** First time through the loop is special; it fires _over_caucus and
   /    begin_caucus AFMs. */
   first_time = 1;

   /*** Loop over each attempt to join a conference... */
   while (1) {

      /*** While no conference name given, ask the user for one.  We also
      /    allow STOP (to leave CAUCUS), LIST (to see list of
      /    available conferences) and CHECK (to check status). */

      while (NOT first_time  &&  EMPTYCHX (confname)) {
         found = mdprompter ("mai_Pjoin", jointab, 0, "mai_Hjoin",
                                          20, confname, nullchix);
         if (found <= -3)  chxtoken (confname, nullchix, 1, jointab);
      }

      /*** Fire off _begin_caucus on the first time through. */
      if (first_time) {
         fire_afm ("_begin_caucus", nullchix, nextconf);
         if (NOT EMPTYCHX (nextconf)) chxcpy (confname, nextconf);
      }

      /*** Fire off _over_caucus on the first time through, and on
      /    every time thereafter that the user chose to STOP from Caucus.
      /    If _over_caucus performs a "JOIN" command, set up CONFNAME
      /    so we don't ask "JOIN which conference" but join CONFNAME
      /    directly. */
      if (first_time  ||  tablefind (jointab, confname) == 0) {
         if (fire_afm ("_over_caucus", nullchix, nextconf) == 0) {
            if (NOT first_time  ||  EMPTYCHX(confname)  ||
                NOT EMPTYCHX(nextconf))  chxcpy (confname, nextconf);
         }
      }
      first_time = 0;
      if (EMPTYCHX(confname))  continue;

      jixreduce (confname);
      found = tablefind (jointab, confname);

      if (found==0)   break;              /* STOP */

      if (found==1) {                     /* LIST */
         unit_write (XWTX, NEWCPAGE);
         listconfs  (XWTX, 0);
         chxclear (confname);
         continue;
      }

      if (found==2) {                     /* CHECK */
         unit_write (XWTX, NEWCPAGE);
         unit_write (XWTX, NEWLINE);
         check(CQ("check"));
         chxclear (confname);
         continue;
      }

      /*** Figure the conference number, and get the full CONFNAME. */
      confnum = cnumber (confname, LOCALCONF, &v);

      /*** Does this conference exist? */
      if (confnum == -1) {
         fire_afm ("_exit_noconf", nullchix, nextconf);
         chxclear (confname);
         if (NOT EMPTYCHX (nextconf)) chxcpy (confname, nextconf);
      }

      /*** No unique match for supplied conference name found. */
      else if (confnum == 0) {
         fire_afm ("_exit_ambconf", nullchix, nextconf);
         chxclear (confname);
         if (NOT EMPTYCHX (nextconf)) chxcpy (confname, nextconf);
      }

      /*** Is this conference beyond max allowed by license? */
      else if (PLAIN(license.confmax) > 0  && 
               PLAIN(license.confmax) < confnum) {
         mdwrite (XWER, "lic_Econfmax", null.md);
         chxclear (confname);
      }

      /*** From here on, we have a unique match for the conference.
      /    Check all remaining circumstances that might keep the
      /    user from joining the conference (not allowed, disk full,
      /    etc. */

      else if (flag.diskfull && diskfull (confid)) {
         mdwrite (XWER, "mai_Ediskfull", null.md);
         break;
      }

      else if (NOT limitusers (1)) {
         mdwrite (XWER, "mai_Hmaxusr", null.md);
         break;
      }

      /*** Otherwise, see if we can join the conference!  Remember that
      /    the organizer is always allowed on, even if the XCUS file
      /    says otherwise. */
      else {
         groups   = nlnode (1);
         allowed_on (nullchix, null.nl, -1);  /* Flush Ulist cache. */
         may_join = allowed_on (thisuser->id, groups, confnum);
         thisuser->read_only = thisuser->is_organizer = 0;
         if (may_join == 1)   thisuser->read_only    = 1;
         if (may_join == 3)   thisuser->is_organizer = 1;
         may_join = (may_join > 0);
         nlfree (groups);

         if (NOT may_join) {
            may_join = primary_org (thisuser->id);
            if (may_join)  mdwrite (XWER, "mai_Torgallow", null.md);
         }

         if (may_join)  join (confname);
         else {
            mdwrite (XWER, "mai_Eexcluded", null.md);
            fire_afm ("_exit_cantjoin", nullchix, nextconf);
            chxclear (confname);
            if (NOT EMPTYCHX (nextconf)) chxcpy (confname, nextconf);
         }
      }
   }

   unit_lock (XILM, WRITE, 0L, 0L, 0L, 0L, thisuser->id);
   unit_kill (XILM);
   unit_unlk (XILM);

   storereg (thisuser, 0, sitevar);
   nlfree (thisuser->briefs);
   thisuser->briefs = null.nl;
   chxclear (confname);
   limitusers (0);

   manager_text (flag.exitfile);
   sysoneuser ("off");
   if (NOT EMPTYCHX(flag.logfile)) log_user (XSLG, 0);
   fire_afm ("_exit_caucus", nullchix, nextconf);
   SUCCEED;

 done:

   chxfree ( nextconf );
   chxfree ( newstr );

   RETURN  ( success );
}
