/*** CHKPRV.  Check to make sure that the Caucus executable has
/             sufficient priviliges to run.
/
/    ok = chkprv (path, num)
/
/    Parameters:
/       int    ok;     // 0=has sufficient priv's
/       char  *path;   // Full pathname of executable
/       int    num;    // Unused for now
/
/    Purpose:
/       Before CHKPRV, if the Caucus executable didn't have sufficient
/       priv's, it would make a complaint that had nothing to do with
/       priv's, e.g. "the dictionary is unavailable."
/       With CHKPRV, we will be able to report what is *really* wrong.
/
/    How it works:
/       Calls syschkprv(), complains appropriately.
/
/    Returns: 0 if sufficient priv's, 1 otherwise.
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  main()
/
/    Operating system dependencies: 
/
/    Known bugs:
/
/    Home:  main/chkprv.c
/ */

/*** Copyright (C) 1993 Camber-Roth.  All rights reserved. */

/*: JV  4/20/93 14:10 Create this. */
/*: JV  6/08/93 20:21 Change path to char ptr. */
/*: CK  7/28/93 15:12 Fix break's and spelling. */

#include <stdio.h>
#include "handicap.h"
#include "syserror.h"
#include "chixuse.h"

FUNCTION  chkprv (path, num)
   char  *path;
   int    num;
{
   int ok;

   ENTRY ("chkprv", "");

   ok = syschkprv (path, num);

   switch (ok) {

   case 0: break;

   case 1:  printf ("Priv error 1: Caucus has not been installed properly.\n");
            printf ("Caucus must run with its setuid bit turned on.\n");
            printf ("Please report this to the system manager.\n");
            break;

   case 9: /* Other Unix */
            printf ("Priv error 9: Caucus could not run properly because it does not \n");
            printf ("have sufficient privileges.\n");
            printf ("Please report this to the system manager.\n");
            break;

   case 11: /* VMS */
   case 20:
   default: printf ("Priv error ?: Caucus could not run properly because it does not \n");
            printf ("have sufficient privileges.\n");
            printf ("Please report this to the system manager.\n");
   }

   RETURN (ok);
}
