
/*** DISKFULL.   Is the disk containing PATH too full?
/
/    full = diskfull (path);
/
/    Parameters:
/       int    full;        (1 => disk full, 0 otherwise)
/       Chix   path;        (path name on disk)
/
/    Purpose:
/       Sometimes Caucus needs to know if there's "enough" room on the
/       disk (or disk partition) containing PATH.  Further, since most
/       of the Distributed Caucus programs have the potential to
/       temporarily use *alot* of disk, and because they run unattended
/       they need to frequently check disk space.
/
/    How it works:
/       Calls sysavail() to get actual available disk space, then compares
/       against some pretty arbitrary values.
/
/    Returns: 1 if "too full", 0 otherwise.
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  application.
/
/    Operating system dependencies: considerable!
/
/    Known bugs:
/
/    Home:  main/diskfull.c
/ */

/*** Copyright (C) 1992 Camber-Roth.  All rights reserved. */

/*: CR 10/08/92 23:13 "New" function. */
/*: CL 12/10/92 17:06 Long ascofchx args. */

#include <stdio.h>
#include "caucus.h"
#include "syserror.h"

#define  MIN_FILES      100
#define  MIN_BYTES   100000

FUNCTION  diskfull (path)
   Chix   path;
{
   char   pathstr[200];
   long   bytes, files;
   int    error, found;

   ENTRY ("diskfull", "");

   ascofchx (pathstr, path, 0L, L(200));
   found = sysavail (pathstr, &bytes, &files, &error);

   if (NOT found) {
      if (error==SE_BADDIR)  RETURN (1);
      if (error==SE_BADDEV)  RETURN (1);
      if (error==SE_NOTIMP)  RETURN (0);
      RETURN (1);
   }

   if (files != -1  &&  files < MIN_FILES)   RETURN (1);
   if (                 bytes < MIN_BYTES)   RETURN (1);
   RETURN (0);
}
