
/*** INPUT_REDIRECT.   Parse Caucus command and data input redirection.
/
/    found = input_redirect (comline, redstr, kind, unit);
/
/    Parameters:
/       int    found;       (Was REDSTR redirection found?)
/       Chix   comline;     (command line to parse)
/       Chix   redstr;      (choice of redirection type: "<" or "<<")
/       long   kind;        ("kind" of input: IC_COMM or IC_DATA)
/       int    unit;        (unit # for input: XICF or XIAL)
/
/    Purpose:
/       Parse a Caucus command line, and look for a command or data
/       input redirection symbol.  If found, cause the appropriate
/       redirection to occur.
/
/    How it works:
/       Interpret() calls input_redirect() to check for either
/       "<file" command redirection, or "<<file" command and data
/       redirection.
/
/       The redirection string (REDSTR) must occur at the beginning
/       of COMLINE to be considered "found".
/
/    Returns: 1 if a redirection string was found and acted upon.
/             0 if no redirection string was found.
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions: input_control()
/
/    Called by:  interpret()
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  main/inputred.c
/ */

/*** Copyright (C) 1992 Camber-Roth.  All rights reserved. */

/*: TP 10/20/92 12:53 New function. */
/*: CL 12/10/92 14:52 Long chxalter args. */
/*: CL 12/10/92 15:22 Long chxindex args. */

#include <stdio.h>
#include "caucus.h"
#include "inputcon.h"

extern struct flag_template   flag;
extern Chix                   ss_dotnl;
extern union  null_t          null;

FUNCTION  input_redirect (comline, redstr, kind, unit)
   Chix   comline, redstr;
   long   kind;
   int    unit;
{
   int    u_comm, u_keyb;

   ENTRY ("input_redirect", "");

   /*** Does COMLINE begin with the redirection symbol. */
   if (chxindex (comline, 0L, redstr) != 0)  RETURN (0);

   /*** Is redirection allowed? */
   u_comm = input_control (IC_ISUNIT | kind,    nullchix);
   u_keyb = input_control (IC_ISUNIT | IC_SESS, nullchix);
   if (NOT (flag.redirect  ||  u_comm!=u_keyb  ||
              (flag.in_macro && NOT flag.restrict_user))) {
      mdwrite (XWER, "err_Tinred", null.md);
      RETURN (1);
   }

   /*** Attempt to process the redirection.   If there is already
   /    a redirection file open on this KIND/UNIT, close it first. */
   chxalter    (comline, 0L, redstr, CQ(""));
   chxsimplify (comline);
   if (u_comm == unit) {
      input_control (IC_CLOSE | kind, nullchix);
      u_comm = input_control (IC_ISUNIT | kind, nullchix);
   }

   if (EMPTYCHX(comline))                                    RETURN (1);
   if (NOT unit_lock (unit, READ, 0L, 0L, 0L, 0L, comline))  RETURN (1);

   if (unit_view (unit))   input_control (IC_RED | kind | unit, nullchix);
   else {
      mdwrite    (XWER, "err_Tnofile", null.md);
      unit_write (XWER, comline);
      unit_write (XWER, ss_dotnl);
      unit_unlk  (unit);
   }

   RETURN (1);
}
