/*** JOIN.  Join a conference CNAME, and process all the user commands.
/
/   join (cname);
/
/   Parameters:
/      Chix   cname;  (lowercase complete name of conference)
/
/   Purpose: actually "put the user in" a conference, and provide the main
/      command-line loop which carries out user commands.
/
/   How it works:
/      Assumes that the global CONFNUM has been set before JOIN is called.
/
/   Returns: 1.
/
/   Error conditions:
/
/   Related functions:
/
/   Called by: caucus()
/
/   Operating system dependencies:
/
/   Known bugs:      none
/
/   Home: main/join.c
/ */

/*** Copyright (C) 1985 Camber-Roth.  All rights reserved.  */

/*: AA  7/29/88 17:47 Source code master starting point */
/*: CR  2/11/89 15:56 Call log_user for -u option. */
/*: CR  7/05/89 14:15 Execute macro _enter_conference on joining conf. */
/*: JV  9/18/89 15:06 Add AFM _exit_conference on leaving conf. */
/*: CW  5/18/89 14:53 Add XW unit codes. */
/*: CR  9/07/89 14:10 Use a fresh copy of _enter_conference each time. */
/*: CR  9/29/89 13:29 Add STARTMENU handling. */
/*: CR 10/11/89 14:56 Interpret(copy of thisuser.startmenu). */
/*: CR 10/17/89 15:17 Replace 2nd arg of FIRE_ARM with "". */
/*: CR 11/13/89 17:23 Add _enter_caucus AFM. */
/*: CR 11/21/89 12:34 If sta_Tcancel is empty, don't mention <CANCEL> key.*/
/*: JV  1/21/90 13:34 Added import_mail code. */
/*: CR  1/28/90  0:47 Fix unitkill/unitunlk on XITX, XITY. */
/*: JV  2/21/90 05:36 Moved calls to sysoneuser() to caucus(). */
/*: JV  3/23/90 11:49 Added store_user() call when leaving conf. */
/*: CR  3/09/90 14:13 Move welcome(), show_new() calls to _enter_conf. */
/*: CR  3/20/90 14:33 Add is_member checking back in! */
/*: CR  3/21/90 17:16 Sysprompt() maps pf keys to ascii. */
/*: CR  5/18/90 15:37 Always load mess dir.; write XUPA only if it exists.*/
/*: CR  6/06/90  1:10 Trim comments for ^*(%$! VM. */
/*: CR  6/11/90 13:45 When leaving conf, also kill XITY. */
/*: CR  6/21/90 12:25 Add LCNF argument to unit_wipe() call. */
/*: CR  7/13/90 14:19 Set thisuser.is_organizer for primary organizer! */
/*: CR  9/26/90 10:16 Make Lint happy about char. comparisons. */
/*: CJ 11/21/90 17:57 Only import mail if FM_CMI license bit is on. */
/*: CR  4/08/91 18:49 Add new unit_lock argument. */
/*: CR  4/08/91 19:18 Use new confnum argument to unit_name(). */
/*: CR  6/19/91 14:30 Add clear_master() call. */
/*: CR  6/20/91 17:29 Use new equal_id() function. */
/*: CR  6/25/91 15:27 Add 'confnum' arg to srvldmas() calls. */
/*: CR  7/02/91  0:00 Include "unitwipe.h". */
/*: CR  7/09/91 16:59 Remove XITX lock/unlock, not needed. */
/*: CR  7/23/91 15:00 Add 3rd argument to equal_id() call. */
/*: UL 12/11/91 18:43 Make unit_lock/name args 3-6 long.*/
/*: CX  3/17/92 15:44 Replace FIRE_AFM with fire_afm(). */
/*: DE  3/20/92 17:25 Chixified */
/*: JX  5/28/92 13:55 Fix tbuf. */
/*: CX  6/05/92 11:56 Replace com_unit with u_com, u_keybd. */
/*: CX  6/05/92 15:22 Replace sysprompt() with readprompt(). */
/*: CR  2/27/92 15:12 Do not clear used.lines; sysprompt() does it. */
/*: CR  8/06/92 14:47 Add _exit_nojoin AFM. */
/*: CR  9/04/92 18:03 Add logfeature() call. */
/*: CI 10/05/92 15:51 Chix Integration. */
/*: CR 10/08/92 23:22 Replace sysdiskfull() with diskfull(). */
/*: CR 10/11/92 21:46 Remove CQ from equal_id() call. */
/*: CR 10/11/92 22:08 Remove CQ from interpret() calls. */
/*: TP 10/19/92 13:38 Replace unit_read() with input_control(). */
/*: CR 11/10/92 11:08 Move close on IC_SESS to main(). */
/*: CR  1/26/93 15:49 Use PLAIN and strplain() to decrypt license info. */
/*: JD  7/26/93 10:26 flag.diskfull. */
/*: JJ  7/21/93 11:00 Reset thisitem.rnum. */
/*: CP  8/01/93 23:02 Remove CMI code to do_cmi(). */
/*: CP  9/02/93 10:32 Force startmenu to be an entry in 'macronames'. */
/*: CK  9/20/93 14:09 Run_prompt_macro() for AND NOW prompt. */
/*: CK  9/21/93 23:08 Add 'eot' arg to readprompt() call. */
/*: JZ  9/29/93 10:31 Add header comments, support for file libraries. */
/*: CP 11/19/93 13:58 Replace erroneous 'enter_lib' with 'exit_lib'. */

#include <stdio.h>
#include "caucus.h"
#include "inputcon.h"
#include "systype.h"
#include "units.h"
#include "unitwipe.h"
#include "api.h"

extern struct unit_template   units[];
extern Textbuf                tbuf;
extern struct screen_template used;
extern struct master_template master;
extern struct message_t       mess;
extern Userreg                thisuser;
extern struct fresh_template  fresh;
extern struct flag_template   flag;
extern struct this_template   thisitem;
extern struct partic_t        thisconf;
extern union  null_t          null;
extern Chix                   confid;
extern Chix                   afm_name;
extern Chix                   ss_dotnl;
extern short                  confnum;

FUNCTION  join (cname)
   Chix   cname;
{
   static Chix  s_andnow;
   static Chix  s_andhelp;
   static Chix  mactable;
   static char  first_time = 1;
   long   starttime, systime();
   int    u_comm, u_keyb;
   short  is_member, i, user_loop, success;
   Chix   comstream, nextconf, userid, afm_conf, newstr;
   Chix   syscancel();
   Conf   conf;

   ENTRY ("join", "");

   comstream  = chxalloc (204L, THIN, "join comstream");
   nextconf   = chxalloc (40L,  THIN, "join nextconf");
   userid     = chxalloc (40L,  THIN, "join userid");
   afm_conf   = chxalloc (40L,  THIN, "join afm_conf");
   newstr     = chxalloc (80L,  THIN, "join newstr");


   if (NOT chxtype(s_andnow))
      s_andnow   = chxalloc (80L,  THIN, "join s_andnow");
   if (NOT chxtype(s_andhelp))
      s_andhelp  = chxalloc (80L,  THIN, "join s_andhelp");
   success    = 0;

   /*** Clear out the text buffer structure. */
   tbuf->item      = 0;
   thisitem.number = thisitem.rnum = 0;

   /*** Load the table of items (& userid of organizer), and message dir. */
   starttime = systime();
   clear_master (&master);
   srvldmas  (1, &master, confnum);
   touch_master();
   if (unit_lock (XUMD, READ, 0L, 0L, 0L, 0L, thisuser->id)) {
      mess.highest   = loadmdir (0, mess.num);
      mess.high_seen = mess.highest;
      unit_unlk (XUMD);
   }

   /*** Load this user's participation file for this conference. */
   sysuserid (userid);
   if (equal_id (userid, master.organizerid, nullchix))
      thisuser->is_organizer = 1;
   unit_lock (XUPA, WRITE, L(confnum), 0L, 0L, 0L, userid);
   is_member = load_user (&thisconf);
   if (is_member == 0)  is_member = new_user (&thisconf);
   unit_unlk (XUPA);

   /*** If registration failed, go back to JOIN WHICH CONFERENCE? */
   if (is_member <= 0) {
      unit_write (XWTX, NEWCPAGE);
      chxclear (cname);
      fire_afm ("_exit_nojoin", nullchix, afm_conf);
      if (NOT EMPTYCHX (afm_conf)) chxcpy (cname, afm_conf);
      SUCCEED;
   }

   /*** We're GO for this conference.  Record FRESH items & responses. */
   for (i=0;   i<=thisconf.items;   ++i)  {
      fresh.responses[i] = thisconf.responses[i];
      fresh.backresp [i] = thisconf.backresp [i];
   }
   fresh.items = thisconf.items;

   if (flag.logbyconf)  log_user (XCLG, 1);

   if (first_time) {
      first_time = 0;
      user_loop  = fire_afm ("_enter_caucus", cname, nextconf);
   }
   else user_loop = 1;

   conf = a_mak_conf ("join");
   conf->cnum = confnum;
   a_get_conf (conf, P_FLIB, A_WAIT);

   if (user_loop) {

      /*** I'm not sure why this is here... */
      mdstr    (nextconf, "mai_Kjoinstop", null.md);
      chxtoken (nextconf, nullchix, 2, nextconf);
  
      /*** On entry to a conference, immediately execute either the
      /    _enter_conference AFM or the _enter_library AFM. */
      if (conf->library)
         chxcpy (comstream, CQ ("_enter_library"));
      else
         chxcpy (comstream, CQ ("_enter_conference"));
      user_loop = interpret (comstream, cname, nextconf, nullchix);
   }

   /*** If a StartMenu has been set, execute it. */
   if (user_loop) {
      mdtable  ("macronames", &mactable);
      if (tablematch (mactable, thisuser->startmenu) >= 0) {
         chxcpy (comstream, thisuser->startmenu);
         user_loop = interpret (comstream, cname, nextconf, nullchix);
      }
   }


   /*** Main command-line loop. */
   while (user_loop) {

      /*** Make sure the disk isn't too full. */
      if (flag.diskfull  &&  diskfull (confid)) {
         mdwrite (XWER, "mai_Ediskfull", null.md);
         break;
      }

      /*** About to prompt user with "AND NOW"?  But first, run prompt 
      /    macro, if any. */
      run_prompt_macro (thisuser->verbose ? "mai_Pandhelp" : "mai_Pandnow",
                        thisuser->dict, cname);

      /*** Ask the user for a command. */
/*    used.lines = used.width = 0; */
      sysbrkclear();
      sysflushin ();
      u_comm = input_control (IC_ISUNIT | IC_COMM, nullchix);
      u_keyb = input_control (IC_ISUNIT | IC_SESS, nullchix);
      if (u_comm != u_keyb  &&
                 NOT input_control (IC_GETEOF | IC_COMM, comstream))
                     u_comm = input_control (IC_ISUNIT | IC_COMM, nullchix);

      if (u_comm == u_keyb) {
         if (thisuser->verbose)  mdstr (newstr, "mai_Pandhelp", &s_andnow);
         else                    mdstr (newstr, "mai_Pandnow",  &s_andhelp);
         if (NOT readprompt (IC_COMM, newstr, nullchix, comstream, 1, 
                                                        thisuser->eot))  break;
      }

      if (sysbrktest()) continue;
      chxcat  (comstream, CQ(";"));
      tbuf->type = TB_IS_EMPTY;

      /*** Interpret the command (or command stream) */
/*    flag.in_macro = 0; */
      if (NOT interpret (comstream, cname, nextconf, nullchix))  break;

      /*** If something new JUST happened, tell the user about it. */
      if (sysbrand())  brandnew (&thisconf);
   }

   /*** Now fire off the closing AFM.  Copy afm_conf in case
   /    AFM caused a JOIN. (Check this out...) */
   if (conf->library) {
      fire_afm   ("_exit_library",   nullchix, afm_conf);
      logfeature (CQ("library"), cname, 0L, starttime, 0L, 0L);
   }
   else {
      fire_afm   ("_exit_conference", nullchix, afm_conf);
      logfeature (CQ("conf"),    cname, 0L, starttime, 0L, 0L);
   }
   if (NOT EMPTYCHX(afm_conf)) chxcpy (nextconf, afm_conf);
   chxcpy (cname, nextconf);

   a_fre_conf (conf);

   /*** Finish up with this user on this conference by deleting the
   /    misc text files, printing anything the user requested, and
   /    marking the user "off" this conference. */
   if (flag.logbyconf)  log_user (XCLG, 0);
   if (unit_lock (XUPA, WRITE, L(confnum), 0L, 0L, 0L, thisuser->id)) {
      if (unit_check (XUPA))  store_user (&thisconf);
      unit_unlk  (XUPA);
   }
   if (unit_lock (XITX, WRITE, 0L, 0L, 0L, 0L, nullchix)) {
      unit_kill  (XITX);
      unit_unlk  (XITX);
   }
   if (unit_lock (XITY, WRITE, 0L, 0L, 0L, 0L, nullchix)) {
      unit_kill  (XITY);
      unit_unlk  (XITY);
   }

   if (flag.print)  print_it();

   /*** Wipe any conference-related locks. */
   unit_wipe (LCNF);
   SUCCEED;

 done:
   chxfree (comstream);
   chxfree (nextconf);
   chxfree (userid);
   chxfree (afm_conf);
   chxfree (newstr);

   RETURN (success);
}
