
/*** LOGINSHELL.  Provide a login shell for systems without one.
/
/    LOGINSHELL is invoked by the CAUCUS main program to make sure
/    the user is "logged in".  For systems that don't provide
/    login id's, such as the IBM PC, LOGINSHELL prompts for a user
/    name and password before letting the user on.  Systems with
/    login id's may wish to dedicate one for Caucus use, rather
/    than handing out id's for people who only want to use Caucus.
/    In that case, the system manager can tell Caucus to use its
/    own login shell.
/    
/    New users may register a name and password by logging in
/    as NEWUSER, a la Unix.
/
/    Calls to SYSUSERID will return the user name that was
/    requested and saved by LOGINSHELL.
/
/    Returns 1 on success, or 0 if the user interrupted the
/    login in process with <CANCEL>.  Exits with status LOGIN_EXIT
/    if "-v" autologin and a bad name:password combination given.
/ */

/*** Copyright (C) 1987 Camber-Roth.  All rights reserved.  */

/*: AA  7/29/88 17:41 Source code master starting point*/
/*: CR 10/14/88 15:28 Lowercase NAME after reading it. */
/*: CR 10/18/88 14:55 Raise max size of password to 20 chars. */
/*: CR  1/20/89 16:03 Map passwords into all lower-case. */
/*: CW  5/18/89 14:25 Add XW unitcodes. */
/*: CR 12/14/89 13:56 Exit if end-of-file and local. */
/*: JV  2/23/90 05:00 Changed sysecho() to turn_echo(). */
/*: CR  4/09/90 17:29 Replace turn_echo() with systurn(). */
/*: CR  9/11/90 15:24 Use maximum password length MAXPASS. */
/*: CJ 11/22/90 22:35 Use new form of systurn() call. */
/*: CJ 11/26/90 17:12 Add new OS_ECHO systurn() stuff. */
/*: CR  2/09/91 21:40 Rearrange mdprompter() lines that were too long for VM. */
/*: CR  3/02/91 18:53 Add autologin code. */
/*: CR  7/01/91 23:49 Rearrange include files. */
/*: CR  8/02/91 12:44 Add NOPUNCT as new last arg to modnames() call. */
/*: CR 11/18/91 17:20 Add CNUM argument to modnames() call. */
/*: DE  3/23/92 12:32 Chixified*/
/*: JX  5/19/92 19:51 Fix Chix. */
/*: JX  6/24/92 16:23 chxformat() uses %d, not %ld. */
/*: CL 12/10/92 17:06 Long ascofchx args. */
/*: CP  5/06/92 12:41 Add 0L as 'item' arg to modnames() call. */

#include <stdio.h>
#include "caucus.h"
#include "sysexit.h"

extern struct flag_template flag;
extern struct termstate_t   term_in_caucus;
extern union  null_t        null;
extern int                  debug;
extern Chix   confid;

FUNCTION  loginshell()
{
   struct namlist_t *add, *del, *nlnode();
   Chix   name, name2, pw1, pw2, pwstr;
   long   mask, pass, mycrypt();
   short  choice, exists, success;
   Chix   newuser, newstr;

   ENTRY ("loginshell", "");

   name       = chxalloc (30L,          THIN, "loginshe name");
   name2      = chxalloc (30L,          THIN, "loginshe name2");
   pw1        = chxalloc (L(MAXPASS+2), THIN, "loginshe pw1");
   pw2        = chxalloc (L(MAXPASS+2), THIN, "loginshe pw2");
   pwstr      = chxalloc (50L,          THIN, "loginshe pwstr");
   newuser    = chxalloc (30L,          THIN, "loginshe newuser");
   newstr     = chxalloc (80L,          THIN, "loginshe newstr"); 
   success    = 0;

   mdstr (newuser, "log_Tnewuser", null.md);

   /*** If autologin, check the "-v name:password" arguments.
   /    If they are a correct name and password pair, set the
   /    userid and return 1.  Otherwise, exit immediately with
   /    the exit status code EXIT_LOGIN. */
   if (NOT EMPTYCHX(flag.autologin)) {
      chxbreak  (flag.autologin, name, pwstr, CQ(":"));
      chxcpy    (name2, name);
      sysdfname (name);

      if (debug) {
         chxclear(newstr);
         chxformat (newstr, CQ("Autologin: name='%s', name2='%s'"), 
                     0L, 0L, name, name2 );
         chxformat (newstr, CQ(" pw='%s'\n"), 0L, 0L, pwstr, null.chx);
         unit_write (XWDB, newstr);
      }

      if (NOT  pwfind (name, &pass)  &&
          NOT (chxcpy (name, name2), pwfind (name, &pass)) ) 
               sysexit (EXIT_LOGIN);
 
      /*** Check password; if it (or its lower-case form) mycrypt's
      /    into the same as PASS, it's good. */
      if (mycrypt  (pwstr) == pass  ||
         (jixreduce (pwstr), mycrypt (pwstr) == pass))  {
         strcpy (flag.userid, ascquick(name));
         SUCCEED;
      }
      sysexit (EXIT_LOGIN);
   }

   /*** Ask the user who s/he is, and keep trying till ok. */
   while (1) {

      /*** Prompt for the user's login name.  If no carrier, loop,
      /    waiting for carrier detect AND a carriage RETURN. */
      choice = mdprompter ( (flag.newuser ? "log_Plogin0" : "log_Plogin1"),
                            null.chx, 0, "log_Hlogin", 20, name, null.chx);

      /*** Exit if ^C, hangup/end-of-file/catastrophe AND in local mode.
      /    Loop if just <RETURN> or hangup on remote line. */
      if (choice <= -3  &&  NOT flag.baud)  FAIL;

      if (choice == -4) {
         syspcinit();
         chxclear(name);
         continue;
      }
      jixreduce (name);

      unit_write (XWTX, CQ("\n"));
      if (choice == -2)  continue;

      /*** NEWUSER who wants a user id. */
      if (chxeq (name, newuser)  &&  flag.newuser) {
         if (mdprompter ("log_Plastname", null.chx, 0, "log_Hlastname", 20, 
                          name, null.chx) < -1  ||  bad_name (name))  continue;
         sysdfname (name);

         if (pwfind (name, &pass)) {
            mdwrite (XWER, "log_Tinuse", null.md);
            continue;
         }

         /*** Get the password twice and make sure they match. */
         mdwrite (XWTX, "log_Tgetpass", null.md);
         systurn (&term_in_caucus, CAUCUS_ECHO + OS_ECHO, OFF);
         choice = mdprompter ("log_Pgetp1", null.chx, 0, "log_Hgetp1", 
                               MAXPASS, pw1, null.chx);
         choice = mdprompter ("log_Pgetp2", null.chx, 0, "log_Hgetp2", 
                               MAXPASS, pw2, null.chx);
         systurn (&term_in_caucus, CAUCUS_ECHO + OS_ECHO, ON);
         if (choice < -2)  continue;
         jixreduce (pw1);
         jixreduce (pw2);
         if (NOT chxeq (pw1, pw2)) {
            mdwrite (XWER, "log_Tnomatch", null.md);
            continue;
         }

         /*** Ok.  We've got a good password.  Add this person's login name
         /    and encrypted password to the system password file. */
         chxformat (pwstr, CQ("%s %d"), mycrypt(pw1), 0L, name, null.chx);
         add = nlnode(1);
         del = nlnode(1);
         nladd (add, pwstr, 0);
         modnames (XSPD, XSPN, null.chx, 0, 0L, add, del, NOPUNCT);
         nlfree (add);
         nlfree (del);
         break;
      }

      else {

         /*** Ask for a password, and then compute its sum-code.  If
         /    this person really exists, AND they gave the right password,
         /    let them log in. */
         chxcpy (name2, name);
         sysdfname (name);

         exists = pwfind (name, &pass);
         if (NOT exists)  {
            chxcpy (name, name2);
            exists = pwfind (name, &pass);
         }
 
         if (exists  &&  pass==0)  break;
         systurn (&term_in_caucus, CAUCUS_ECHO + OS_ECHO, OFF);
         choice = mdprompter ("log_Ppassword", null.chx, 0, "log_Hpassword", 
                               MAXPASS, pw1, null.chx);
         unit_write (XWTX, CQ("\n"));
         systurn (&term_in_caucus, CAUCUS_ECHO + OS_ECHO, ON);
         if (choice < -2)  continue;

         /*** Check password; if it (or its lower-case form) mycrypt's
         /    into the same as PASS, it's good. */
         mask = mycrypt (pw1);
         if (exists  &&  mask==pass)  break;
         jixreduce (pw1);
         mask = mycrypt (pw1);
         if (exists  &&  mask==pass)  break;

         mdwrite (XWER, "log_Tfailed", null.md);
      }
   }
   
   /*** User is logged in.  Set USERID properly so SYSUSERID call
   /    will work. */
   ascofchx (flag.userid, name, 0L, 30L);

   SUCCEED;

done:

   
   chxfree( name );
   chxfree( name2 );
   chxfree( pw1 );
   chxfree( pw2 );
   chxfree( pwstr );
   chxfree( newuser );
   chxfree( newstr );

   RETURN ( success );
}




