/*** ADD_CAUMESS.  Send a Caucus message to a user on this host.
/
/    ok = add_caumess (author, authorid, authorchn, subject,
/                      messtype, date, time, prevtext, prevparts,
/                      retrecsize, recipid, newtext, attach, error);
/
/    Parameters:
/       int    ok;          (Success?)
/       Chix   author;      (full name of message author)
/       Chix   authorid;    (userid of message author)
/       Chix   authorchn;   (author lives on this host)  (may be nullchix!)
/       Chix   subject;     (one-line subject of message)
/       int    messtype;    (type of message, from mess.h)
/       Chix   date;        (date message was created)
/       Chix   time;        (time message was created)
/       Chix   prevtext;    (text of "previous" parts of message)
/       int    prevparts;   (number of parts in prevtext)
/       int    retrecsize;  (size of desired return receipt)
/       Chix   recipid;     (userid of recipient)
/       Chix   newtext;     (text of new part of message)
/       Attachment attach;      (File Attachment)
/       int   *error;       (return error code)
/
/    Purpose:
/       Use add_caumess() to send a Caucus message to someone on the
/       local host.
/
/    How it works:
/       Add_caumess() looks in the recipient's register file to see
/       if they have "SET EXPORT".  If not, the mail goes to the user's Caucus
/       mailbox (in which case it calls add_dmess(), which has exactly
/       the same argument list).  If so, it exports the mail to the user's
/       system mailbox (in which case it calls add_cmimess()).
/
/       Retrecsize of -1 means no return receipt was requested.  Otherwise,
/       it is the number of lines in the original message that should
/       accompany the return receipt.
/
/       PREVTEXT and NEWTEXT must be allocated Chixen.
/
/       Messtype must be MSEND, MFORWARD, or MREPLY, from mess.h.
/
/    Returns: 1 on success, sets ERROR to 0.
/             0 on any error condition, see below.
/
/    Error Conditions: sets value of ERROR accordingly:
/       DNOMEM       ran out of dynamic memory
/       DNOUSER      userid recipid does not exist
/       DBOXFULL     this user's mailbox is full
/       0            could not send external system mail
/  
/    Side effects: may write in text buffer structures block/line/text.
/                  may write in partfile directory pdir.
/
/    Related functions: add_dmess(), add_cmimess()
/
/    Called by:  Caucus functions
/
/    Operating system dependencies:
/
/    Known bugs:      none
/
/    Home:  mess/addcaume.c
/ */

/*** Copyright (C) 1993 Camber-Roth.  All rights reserved. */

/*: CR  2/26/93 17:00 New function. */
/*: CK  3/18/93 12:56 Call add_dmess(), not add_caumess()!! */
/*: JV  5/28/93 20:24 Add attach arg. */

#include <stdio.h>
#include "caucus.h"
#include "derr.h"

extern Chix newuseropts;

FUNCTION  add_caumess (author, authorid, authorchn, subject,
                       messtype, date, time, prevtext, prevparts,
                       retrecsize, recipid, newtext, attach, error)
   Chix    author, authorid, authorchn, recipid;
   Chix    subject, date, time;
   Chix    prevtext, newtext;
   int     messtype, prevparts, retrecsize, *error;
   Attachment  attach;
{
   Userreg recip, make_userreg();
   int     exists, export, success;

   ENTRY ("add_caumess", "");

  *error = 0;

   /*** Find out if the user has EXPORT set. */
   recip  = make_userreg();
   exists = loadreg (recipid, recip, NULLSITE);
   decode_setopts   (recip, recip->setopts, nullchix, newuseropts);
   export = recip->export_mail;
   free_userreg (recip);
   if (NOT exists) { *error = DNOUSER;   RETURN(0); }

   if (export) {
      success = add_cmimess (recipid, author, authorid, messtype,
                             subject, newtext, prevtext);
   }
   else {
      success = add_dmess   (author, authorid, authorchn, subject,
                             messtype, date, time, prevtext, prevparts,
                             retrecsize, recipid, newtext, attach, error);
   }

   RETURN (success);
}
