
/*** GETMLINE.  Get a line of a message.
/
/    mline = getmline (owner, message, linenum, lockxumd);
/
/    Parameters:
/       Chix   mline;     (returned chix containing text of a message line)
/       Chix   owner;     (userid of owner of message)
/       int    message;   (message number)
/       int    linenum;   (number of desired line)
/       int    lockxumd;  (should getmline lock/unlock XUMD?)
/
/    Purpose:
/       Return a chix that contains a particular line of a Caucus message.
/
/    How it works:
/       Getmline() looks in global text buffer TBUF to see if it already
/       contains this message; if not, it loads the message into TBUF with
/       loadtext().  In the latter case, getmline() locks/unlocks XUMD
/       itself if LOCKXUMD is false; otherwise, it assumes the caller
/       does the locking and unlocking.
/
/    Returns:
/       If the desired OWNER/MESSAGE/LINENUM combination does not exit,
/       getmline() returns NULLCHIX.
/
/       If it does exist, getmline() returns the chix that contains the
/       desired line, from TBUF.
/
/       WARNING: the caller *must* *not* chxfree the returned value from
/       getmline().
/       
/    Error Conditions:
/  
/    Side effects: uses/modifies contents of external text buffer 'tbuf'.
/
/    Related functions: loadtext()
/
/    Called by:
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  mess/getmline.c
/ */

/*: AA  7/29/88 17:48 Source code master starting point */
/*: CR  9/02/88 15:05 Add LOCKXUMD argument to GETMLINE. */
/*: CR  4/08/91 18:49 Add new unit_lock argument. */
/*: CR  8/07/91 14:25 Add 'confnum' arg to syspdir() call. */
/*: UL 12/11/91 18:43 Make unit_lock args 3-6 long.*/
/*: JV 12/16/91 16:39 Change pdir.lnum to long. */
/*: JX  5/15/92 11:24 Chixify. */
/*: CR 10/08/92 12:06 Put in header comments. */
/*: CR 12/07/92 13:39 Add 0 (CNUM) arg to loadtext(). */

#include <stdio.h>
#include "caucus.h"
#include "chixuse.h"

extern struct pdir_t         pdir;
extern union  null_t         null;
extern Textbuf               tbuf;

FUNCTION  Chix getmline (owner, message, linenum, lockxumd)
   Chix   owner;
   int    message, linenum, lockxumd;
{
   short  is_mess, after_first, beyond_last, p, b, m;

   ENTRY ("getmline", "");

   /*** Is the desired message attached to the right item (the one we
   /    have loaded in the text buffer)?  Is it at or after the first
   /    message in the buffer?  Is it beyond the last message in the
   /    buffer? */

   /*** Does the text buffer contain a message partfile?  Is the desired
   /    message at or after the first message in the buffer?  Is it beyond
   /    the last message in the buffer? */
   is_mess     = (tbuf->type == XUMF);

   after_first = (message >  tbuf->r_first) ||
                 (message == tbuf->r_first  &&  linenum >=
          tbuf->firstline);

   beyond_last = (message >  tbuf->r_last)  ||
                 (message == tbuf->r_last   &&  linenum >  tbuf->lastline);

   /*** If the desired message & line is outside the current text buffer
   /    structure, figure out what partfile the message is in (if any),
   /    and load it. */
   if (NOT is_mess  ||  NOT after_first  ||  beyond_last) {

      /*** If the message/linenum appears to be beyond this partfile, and
      /    yet is not in the next one, then it doesn't exist. */
      if (is_mess  &&  after_first  &&  beyond_last  &&  pdir.type==XUMF) {
         p = partnum (&pdir, message, linenum);
         if (pdir.resp[p] == tbuf->r_first  &&
             pdir.lnum[p] == tbuf->firstline)      RETURN (nullchix);
      }

      /*** If the message is before this partfile, and this is the FIRST
      /    partfile, then the message doesn't exist. */
      if (is_mess  &&  NOT after_first) {
         p = partnum (&pdir, message, linenum);
         if (p==0  &&  message < pdir.resp[0])    RETURN (nullchix);
      }

      /*** Lock the part file directory, and load it. */
      if (lockxumd) unit_lock (XUMD, READ, 0L, 0L, 0L, 0L, owner);
      syspdir   (&pdir, XUMF, 0, 0, owner);
      if (pdir.parts == 0)
         { if (lockxumd) unit_unlk (XUMD);   RETURN(nullchix); }
      p = partnum (&pdir, message, linenum);
      if (p==0  &&  message < pdir.resp[0])
         { if (lockxumd) unit_unlk (XUMD);   RETURN(nullchix); }

      /*** Lock the desired part file.  (Once we've got it locked, no one
      /    else can change it, so we can let go the part file dir.) */
      unit_lock (XUMF, READ, 0L, 0L, L(pdir.resp[p]), pdir.lnum[p],
          owner);
      if (lockxumd) unit_unlk (XUMD);
      loadtext  (XUMF, 0, 0, pdir.resp[p], pdir.lnum[p], tbuf);
      unit_unlk (XUMF);
   }

   b =  message -  tbuf->r_first;
   if (b < 0  ||  message > tbuf->r_last)  RETURN (nullchix);
   m = (message == tbuf->r_first ? linenum - tbuf->firstline : linenum);
   p = tbuf->r_start[b] + m;
   if (p > tbuf->r_stop[b])  RETURN (nullchix);

   RETURN (tbuf->line[p]);
}
