/*** MESSDEL.   Delete messages marked for delete.
/
/    Scan through the table of messages in FLAGS, and actually delete
/    the messages marked with MSG_DEL.
/
/    If a message partfile contains nothing but deleted messages,
/    delete the entire partfile.  
/
/    MESSDEL returns the new value of HIGHEST, the highest numbered
/    existing message. */

/*** Copyright (C) 1987 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:48 Source code master starting point */
/*: CR  9/02/88 15:10 Change call to getmline() to 4 arguments. */
/*: CR  4/08/91 18:49 Add new unit_lock argument. */
/*: UL 12/11/91 18:43 Make unit_lock args 3-6 long.*/
/*: JV 12/16/91 16:40 Change pdir.lnum to long. */
/*: CR 12/26/91 11:58 Last arg to loadtext must be long. */
/*: JX  5/15/92 11:30 Finish chixifying. */
/*: CR 12/07/92 13:39 Add 0 (CNUM) arg to loadtext(). */
/*: CN  5/15/93 21:02 Make 'flags' short. */
/*: JV  9/20/93 12:02 Empty lines and bytes fields of tbuf. */

#include <stdio.h>
#include "caucus.h"
#include "chixuse.h"

extern Userreg   thisuser;
extern Textbuf   tbuf;

FUNCTION  messdel (pdir, highest, flags)
   struct pdir_t  *pdir;
   int    highest;
   short  flags[];
{
   short  p0, p, m, b, h;
   Chix   getmline();

   ENTRY ("messdel", "");

   /*** P0 is the index to the partfile for the previous messsage. */
   p0 = -1;

   /*** Find the index H of the highest possible message left after
   /    deletions are done. */
   for (h=highest;   h>0  &&  (flags[h] & MSG_DEL);   --h) ;

   /*** For each message M to be deleted... */
   for (m=1;   m<=highest;   ++m)  if (flags[m] & MSG_DEL) {
      p = partnum (pdir, m, 0);

      /*** If M is in a different partfile from the previous deleted
      /    message, delete the previous partfile if completely empty,
      /    or else rewrite it. */
      if (p != p0  &&  p0 != -1  &&  NOT delempty (p0, pdir) ) {
         storetext (XUMF, 0, tbuf->r_used-1, tbuf);
         unit_unlk (XUMF);
      }

      /*** If M is in a different partfile from the previous message,
      /    load the partfile containing M. */
      if (p != p0) {
         unit_lock (XUMF, WRITE, 0L, 0L, L(pdir->resp[p]), 0L, thisuser->id);
         loadtext  (XUMF, 0, 0, pdir->resp[p], 0L, tbuf);
         p0 = p;
      }

      /*** Delete M by truncating all of its text. */
      b = m - tbuf->r_first;
      tbuf->r_stop[b] = tbuf->r_start[b];
      tbuf->lines[b]  = tbuf->bytes[b]   = 0;
      flags[m] = 0;
   }

   /*** The loop is over, finish with the last partfile. */
   if (p0 != -1  &&  NOT delempty (p, pdir) ) {
      storetext (XUMF, 0, tbuf->r_used-1, tbuf);
      unit_unlk (XUMF);
   }

   /*** Finally, find & return the actual highest numbered message remaining.
          */
   for (;   h > 0  &&  (getmline (thisuser->id, h, 1, 0) == nullchix);  --h) ;
   RETURN (h);
}



/*** DELEMPTY deletes an entire message partfile if it is empty.   It
/    returns 1 if it was empty, else 0. */

FUNCTION  delempty (p, pdir)
   int    p;
   struct pdir_t *pdir;
{
   short  empty, i, b;

   ENTRY ("delempty", "");

   /*** Is this partfile empty? */
   for (empty=1, b=0;   b<tbuf->r_used;   ++b)
      if (tbuf->r_stop[b] > tbuf->r_start[b])  empty = 0;

   if (NOT empty)  RETURN (0);

   /*** Delete the whole partfile. */
   unit_kill (XUMF);
   unit_unlk (XUMF);

   /*** If this message covered several partfiles (i.e., it was a long
   /    message), delete all the continuation partfiles as well. */
   for (i=p+1;   pdir->resp[i] == pdir->resp[p];   ++i) {
      unit_lock (XUMF, WRITE, 0L, 0L, L(pdir->resp[i]), pdir->lnum[i],
          thisuser-> id);
      unit_kill (XUMF);
      unit_unlk (XUMF);
   }
   RETURN (1);
}

