/*** MESS_SEND.  Do all of the preparation for sending a message,
/    whether it's an original message, a reply, a forwarded message,
/    a message sent AGAIN, or a message from a file (e.g, "SEND <file").
/
/    MESSTYPE is the type of message (MSEND, MREPLY, MFORWARD, MAGAIN,
/    MFILE), and MESSNUM is the number of the "original" message for types
/    MREPLY and MFORWARD.
/
/    If ADDRESS contains a userid or CMI recipient, MESS_SEND sends the
/    message to that person (and then prompts for other recipients, as
/    usual).  If ADDRESS is empty, MESS_SEND prompts for the
/    original recipient as well.  (Exception: type MREPLY always picks
/    address out of message being replied to.)
/
/    MESS_SEND prompts the user for subject, message text, editing
/    of message text, cutting earlier exchanges, name(s) of recipients,
/    and confirmation-to-send.  Some of these are skipped where
/    inappropriate (e.g. cutting earlier exchanges on an original
/    message).
/
/    Once the message is complete and the recipient is identified,
/    MESS_SEND calls addxmess() or add_dmess() to prepare and transmit
/    the message.
/
/    New for Caucus Mailer Interface:
/      if recipient's name begins with a '=',  (actually ss_cmiesc)
/         call sysmsprep() and sysmstrans() instead of
/              messprep()  and messtrans().
/
/    Known bugs:
/       The "No such user" chxformat string should really come from
/       the dictionary.  Because this is a 2.4 "backfit" for Wigwam,
/       it is hardcoded for now.
/   */

/*** Copyright (C) 1987 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:49 Source code master starting point */
/*: CR  9/02/88 15:10 Change call to getmline() to 4 arguments. */
/*: CW  5/18/89 15:07 Add XW unit codes. */
/*: CR  8/31/89 15:27 Use HOW to pass return-receipt info. */
/*: JV  1/01/90 21:58 Added mailer interface stuff. */
/*: JV  1/05/90 22:30 Biggered preface[]. */
/*: CR 12/29/89 16:56 Add more comments, unlk XILM if SEND AGAIN fails. */
/*: CR  1/08/90 17:01 XLIM -> XILM. */
/*: JV  1/27/90 17:59 Fixed Caucus Mailer Interface.*/
/*: JV  2/06/90 17:21 Fixed interprep preface. */
/*: JV  3/09/90 12:12 Increased size of TO_ID, cuz CMI address often BIG. */
/*: JV  3/16/90 18:49 Added USERID to argument list. */
/*: CR  3/04/90 22:45 Add context argument to textenter(). */
/*: CR  4/24/90 21:36 Change definition and use of 3rd argument. */
/*: CR  4/25/90 17:04 Reformat, add comments. */
/*: CR  8/12/90 20:21 Cast textenter/textedit arg to make Lint happy. */
/*: CJ 11/21/90 18:12 Allow CMI only if function_mask bit set. */
/*: CR  1/31/91 18:25 Use person_format() to format person's name. */
/*: CR  2/08/91 17:06 Force author and type of message to 35 & 12 char max.*/
/*: CR  4/08/91 18:49 Add new unit_lock argument. */
/*: CR  4/26/91 13:50 Expand buffers str, to_id. */
/*: CR  6/20/91 13:59 Add 4th arg to removeid(). */
/*: CR  7/05/91 17:44 Change to use add_dmess(). */
/*: CR  7/08/91 16:25 More changes re add_dmess(). */
/*: CR  7/28/91 15:48 chxalloc() first argument is long. */
/*: CR 10/10/91 22:49 Use author name in "OK to send..?" for REPLY. */
/*: UL 12/11/91 18:43 Make unit_lock args 3-6 long.*/
/*: CR  1/10/92 21:57 Hard-code old mes_Atype string. */
/*: DE  3/27/92 15:58 Chixified */
/*: DE  4/21/92 15:00 Fix missing and inconsistant args */
/*: JX  5/27/92 16:54 Cast arg9 to add_dmess(), declare make_address(). */
/*: CX  6/01/92 12:04 Make Address use consistent. */
/*: CR  7/24/92 11:54 Use parse_preface() to get author name from reply. */
/*: CR  9/08/92 16:46 Add logfeature() call for outgoing messages.  */
/*: CR  9/08/92 17:47 Add logfeature() call for ret. rec's requested. */
/*: CI 10/08/92 14:20 Chix Integration. */
/*: JX 10/09/92 17:52 Chixify call to sysfsize. */
/*: JV 10/19/92 10:43 Preface Reply subjects with mes_Treplypref. */
/*: TP 10/21/92 11:19 Only write mes_Tenterit if IC_DATA not session. */
/*: CL 12/10/92 14:52 Long chxalter args. */
/*: CL 12/10/92 15:22 Long chxindex args. */
/*: CR  1/26/93 15:49 Use PLAIN and strplain() to decrypt license info. */
/*: CR  2/25/93 20:12 Remove code to add_cmimess(). */
/*: JV  4/05/93 18:19 Add attach arg to add_dmess(). */
/*: CT  5/28/93 12:25 For outgoing CMI, use address->id when possible. */
/*: CW  7/06/93 13:10 Handle new return of how == -1 from sendwho(). */
/*: CK  9/19/93 22:44 Take whole cmi address as address->id on reply. */

#include <stdio.h>
#include "caucus.h"
#include "inputcon.h"
#include "derr.h"

extern struct license_t  license;
extern Userreg           thisuser;
extern union  null_t     null;
extern int               debug;
extern Chix              yesnotable, ss_cmiesc;

FUNCTION    mess_send (messnum, messtype, address)
   int      messnum, messtype;
   Address  address;
{
   static   Chix  editoktab, meseditab, s_rep_pref=nullchix;
   Chix     query, str, localhost, editor, subject, preface, nodename;
   Chix     date, time, temp, replypref;
   Chix     misc, no, yes, authorname, authorline, prevtext, newtext;
   long     parts;
   int      error, success, rr_requests, export, text_loaded, was_sent;
   short    messfile, allofit, sentonce, how;
   char     prompt[40];
   char    *logmesstype;
   Chix     getmline(), person_format();
   long     systime();

   ENTRY ("mess_send", "");

   no         = chxalloc ( 20L, THIN, "mess_send no");
   yes        = chxalloc ( 20L, THIN, "mess_send yes");
   str        = chxalloc (200L, THIN, "mess_send str");
   date       = chxalloc ( 40L, THIN, "mess_send date");
   temp       = chxalloc ( 40L, THIN, "mess_send temp");
   time       = chxalloc ( 40L, THIN, "mess_send time");
   misc       = chxalloc ( 80L, THIN, "mess_send misc");
   query      = chxalloc ( 40L, THIN, "mess_send query");
   editor     = chxalloc ( 60L, THIN, "mess_send editor");
   subject    = chxalloc (100L, THIN, "mess_send subject");
   preface    = chxalloc (160L, THIN, "mess_send preface");
   nodename   = chxalloc ( 20L, THIN, "mess_send nodename");
   localhost  = chxalloc (100L, THIN, "mess_send localhost");
   replypref  = chxalloc ( 16L, THIN, "mess_send replypref");
   authorname = chxalloc ( 80L, THIN, "mess_send authorname");
   prevtext   = chxalloc (200L, THIN, "mess_send prevtext");
   newtext    = chxalloc (200L, THIN, "mess_send newtext");
   success    = 0;

   getmyhost (localhost);

   logmesstype = (messtype==MREPLY ? "mailrepl" : "mailsent");
  
   /*** For Send or Reply, get text of message from user. */
   unit_lock (XILM, WRITE, 0L, 0L, 0L, 0L, thisuser->id);
   if      (messtype == MSEND  ||  messtype == MREPLY) {
      unit_kill  (XILM);
      if (input_control (IC_ISUNIT | IC_DATA, nullchix) ==
          input_control (IC_ISUNIT | IC_SESS, nullchix))
          mdwrite (XWTX, "mes_Tenterit", null.md);
      if (NOT textenter (XILM, (int) thisuser->truncate, 0, 0,
          messtype==MREPLY))           { unit_unlk (XILM);   FAIL; }
      messfile = XILM;
   }
   else if (messtype == MAGAIN  &&  NOT unit_check (XILM)) {
      unit_unlk (XILM);
      mdwrite (XWER, "mess_Tnolast", null.md);
      FAIL;
   }
   else if (messtype == MAGAIN)  {
      mdwrite (XWTX, "mes_Tuselast", null.md);
      messfile = XILM;
   }
   else if (messtype == MFILE)     messfile = XILM;
   else if (messtype == MFORWARD)  messfile = 0;


   /*** If REPLY, get the recipient ID and SUBJECT from the message header.
   /    Otherwise, get the subject from the user. */
   if (messtype == MREPLY) {
      chxtoken (misc, null.chx, 2, getmline (thisuser->id, messnum, 0, 1));
      if (chxvalue (misc, 0L) == '=')  chxcpy (address->id, misc);
      else   chxbreak (misc, address->id, address->host, CQ("@"));
      if ( (authorline = getmline (thisuser->id, messnum, 1, 1)) != NULL)
         parse_preface (authorline, address->name, date, time, temp);
      chxcpy   (address->name, address->id);
      getmsubj (thisuser->id, messnum, subject);
      mdstr (replypref, "mes_Treplypref", &s_rep_pref);
      if (chxindex (subject, 0L, replypref) != 0)
         chxconc (subject, replypref, subject);
   }
   else {
      if (mdprompter ("mes_Psubj", null.chx, 0, "mes_Hsubj", 66, subject,
                       null.chx) <= -3) { unit_unlk (XILM);   FAIL; }
   }

   /*** Edit the message text once entered? */
   mdtable  ("mes_Aedit", &meseditab);
   chxtoken (no, null.chx, 1, meseditab);
   if ( messtype != MFORWARD  &&  mdprompter ("mes_Pedit", meseditab,
                                  2, "mes_Hedit", 40, query, no) > 0) {
      mdtable ("mes_Aeok", &editoktab);
      if (iseditreq (query, editoktab, thisuser->editor, editor))
         textedit (XILM, editor, (int) thisuser->truncate);
   }

   /*** Cut earlier exchanges, if there were any? */
   allofit = 1;
   if (messtype == MREPLY) {
      chxtoken (str, null.chx, 3, getmline (thisuser->id, messnum, 0, 1));
      chxnum   (str, &parts);
      chxtoken (yes, null.chx, 2, yesnotable);
      if (parts >= 2)  allofit = mdprompter ("mes_Pcut", yesnotable, 2,
                                   "mes_Hcut", 10, query, yes) < 1;
   }


   /*** Master loop to send message to one or more recipients, and handle
   /    bad names or multiple matches to a name. */
   strcpy (prompt, "mes_Psendwho1");
   sentonce =   0;
   rr_requests = 0;
   text_loaded = 0;   /* Have we loaded any "previous message" text into mem? */

   /*** May be sending id, but may be getting it. */
   if ( (how = sendwho (prompt, address, &export))) {
/*    unit_lock (XITX, WRITE, 0L, 0L, 0L, 0L, null.chx); */
      chxclear (prevtext);
      chxclear (newtext);
      chxtoken (misc, null.chx, messtype+1,
        CQ("MESSAGE_from FORWARDED_by REPLY___from MESSAGE_from MESSAGE_from"));
      while    (chxalter (misc, 0L, chxquick("_",0), chxquick(" ",1))) ;
      sysdaytime  (date, time, 0, systime());
      strcpy (prompt, "mes_Psendwho2");
      do {

         /*** First time through, prepare chix containing "previous"
         /    message (message being replied to or forwarded), and
         /    current message, in case we send this message multiple times. */
         if (NOT text_loaded) {
            parts = prevmess (messnum, thisuser->id, allofit, -1, prevtext);
            chxclear (newtext);
            if (messfile > 0) chixfile (messfile, newtext, " ");
            text_loaded = 1;
         }

         /*** Now that we are actually sending the message, there are
         /    four possible cases:
         /      (a) local user who has EXPORT on, becomes case (b).
         /      (b) CMI interface: address is "=user@net_address".
         /      (c) address points to local user.
         /      (d) address points to remote (CaucusNet) user. */

         if (export) chxconc (address->name, CQ("="), address->id);

         /*** Handle messages sent via Caucus Mailer Interface.  */
         if (chxindex (address->name, 0L, ss_cmiesc) == 0  &&
               (PLAIN(license.function_mask) & FM_CMI)) {

            /*** Use prevtext & newtext. */
            if (add_cmimess (address->name, thisuser->name, thisuser->id,
                     messtype, subject, newtext, prevtext)) sentonce = 1;
            else   mdwrite (XWER, "mes_Tnounsnd", null.md);
            into_address (null.chx, address);
            continue;
         }

         /*** From here, we have regular Caucus mail, going either to a
         /    local user or a remote (CaucusNet) user. */

         sysdaytime  (date, time, 0, systime());

         /*** Handle mail sent to local user. */
         if (EMPTYCHX(address->host)  ||  chxeq (address->host, localhost)) {
            if (how == -1) {
               chxformat (temp, CQ("No such user: '%s'"), 0L, 0L,
                                address->id, nullchix);
               was_sent = add_dmess (thisuser->name, thisuser->id, null.chx,
                              temp, messtype, date, time, prevtext, 
                              (int) parts, -1, thisuser->id, newtext, 
                              (Attachment) NULL, &error);
            }
            else was_sent =
                 add_dmess (thisuser->name, thisuser->id, null.chx, subject,
                           messtype, date, time, prevtext, (int) parts,
                           (int) (how==2 ? thisuser->receiptsize : -1),
                            address->id, newtext, (Attachment) NULL, &error);
            if (was_sent) {
               logfeature (CQ(logmesstype), thisuser->id, 0L, 0L, 0L,
                             L(chxlen(prevtext) + chxlen(newtext)) );
               if (how == 2)  ++rr_requests;
               mdwrite (XWTX, "mes_Tsentmess", null.md);
               sentonce = 1;
            }
            else if (error == DBOXFULL)
               mdwrite (XWER, "mes_Trecipfull", null.md);
         }

         /*** Handle mail sent to a remote user. */
         else {
            if (addxmessage (thisuser->name, thisuser->id, subject, messtype,
                             prevtext, (int) parts,
                             (how==2 ? thisuser->receiptsize : -1),
                             0, address->name, address->id, address->host,
                             newtext, date, time, &error)) {
               logfeature (CQ(logmesstype), thisuser->id, 0L, 0L, 0L,
                             L(chxlen(prevtext) + chxlen(newtext)) );
               if (how == 2)  ++rr_requests;
               mdwrite (XWTX, "mes_Tsentmess", null.md);
               sentonce = 1;
            }
            /* else? Do something with ERROR? */
         }
         into_address (null.chx, address);

      } while ( (how = sendwho (prompt, address, &export)));
  
/*    unit_kill (XITX); */
/*    unit_unlk (XITX); */
    }

   /*** Log the number of return receipt messages requested. */
   if (rr_requests > 0)
      logfeature (CQ("mailrrct"), null.chx, 0L, 0L, L(rr_requests), 0L);

   /*** If the user entered a message and didn't send it to ANYBODY,
   /    remind user that it's still sendable via SEND AGAIN. */
   if (NOT sentonce  &&
      (messtype==MSEND || messtype==MREPLY || messtype==MFILE))
          mdwrite (XWTX, "mes_Tsendagin", null.md);

   unit_unlk (XILM);
   SUCCEED;

done:
   chxfree (no);
   chxfree (yes);
   chxfree (str);
   chxfree (date);
   chxfree (time);
   chxfree (temp);
   chxfree (misc);
   chxfree (query);
   chxfree (editor);
   chxfree (subject);
   chxfree (preface);
   chxfree (nodename);
   chxfree (localhost);
   chxfree (replypref);
   chxfree (authorname);
   chxfree (prevtext);
   chxfree (newtext);

   RETURN  (success);
}
