
/*** PARSE_PREFACE.   Pluck out the various parts of a message preface.
/
/    ok = parse_preface (preface, author, date, time, keyword);
/
/    Parameters:
/       int    ok;          (success?)
/       Chix   preface;     (input: message header or "preface" line)
/       Chix   author;      (resulting author name)
/       Chix   date;        (resulting date)
/       Chix   time;        (resulting time)
/       Chix   keyword;     (resulting message type keyword)
/
/    Purpose:
/       Given a message "preface" or "header" line, pluck out
/       the important parts: author, date, time, and message
/       keyword ("MESSAGE", "REPLY, "FORWARD").
/
/    How it works:
/
/    Returns: 1 on success
/             0 if one of the parts could not be found
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  mess/parsepre.c
/ */

/*** Copyright (C) 1992 Camber-Roth.  All rights reserved. */

/*: CR  7/24/92 11:14 New function. */
/*: CR 10/08/92 13:18 Chixify. */
/*: CL 12/11/92 11:17 Long Chxcatsub args. */

#include <stdio.h>
#include "caucus.h"

extern union  null_t     null;

FUNCTION  parse_preface (preface, author, date, time, keyword)
   Chix   preface, author, date, time, keyword;
{
   int    success, word;
   long   pos;
   Chix   temp;

   ENTRY ("parse_preface", "");

   temp = chxalloc (100L, THIN, "parse_pre temp");

   chxclear (date);
   chxclear (time);

   /*** The DATE and TIME are the last two words in PREFACE.  Pluck them
   /    off, and remove them from our copy of PREFACE.  Then words 4 
   /    through the end make up the AUTHOR.  */
   for (word=3;   chxtoken (author, null.chx, word, preface) >= 0;   ++word) ;
   word = word - 2;
   if (word < 1)         { chxclear (author);   FAIL; }

   pos = chxtoken (date, null.chx, word, preface);
   chxtoken  (time, null.chx, word+1, preface);
   chxcatsub (temp, preface, 0L, pos);

   if (chxtoken (null.chx, author, 4, temp) == -1)  FAIL;
   chxtrim  (author);
   chxtoken (keyword, null.chx, 2, temp);
   SUCCEED;

done:
   chxfree (temp);
   RETURN  (success);
}
