/* ADD_NAYBOR:   Create the files necessary to define a neighbor.
/
/  int n = add_naybor ()
/
/  Purpose:
/     The process of creating a new neighbor:
/     0) Prompt for necessary info
/     1) Create the Hnnn directory
/     2) Create the hostinfo file (XHI1) for the neighbor
/     3) Find out the shipping number, copy the template from ???
/
/  How it works:
/
/  Returns:
/    0: Failed to perform any action due to input problems.
/    1: Carried out mgr's wishes as much as possible.
/
/  Error conditions:
/
/  Related functions: 
/
/  Called by: donaybor()
/
/  Home: netmaint/addnaybo.c
/
/  Dictionary strings:
/     net_Tnayintro: You have chosen to add a new neighbor to your CaucusLink
                     system.
      net_Pnunaychn: What is the Caucus Host Name of this neighbor? $
      net_Hnunay:    (The Usual drivel.)
      net_Tnotchn:   That is not a valid Caucus Host Name.  Please try again
                     or press Ctrl-C.
      net_Pnunaymgr: What is the Caucus ID of the Caucus Manager on the
                     neighbor? $
      net_Tnotmgr:   That is not a valid name for a Caucus ID.  Please try
                     again or press Ctrl-C.
      net_Pnunayaddr: What is the e-mail address of the Caucus account on the
                      neighbor? $
      net_Tnotaddr:   That is not a valid e-mail address.  Please try again
                      or press Ctrl-C.
      net_Tnunaydone: This host has been added as a neighbor.  You must use
                      NetMaint to Attach this neighbor before the two
                      computers will exchange any Caucus data.
*/

/*: JV 12/10/91 16:47 Creat (sic) this. */
/*: UL 12/11/91 18:43 Make unit_lock args 3-6 long.*/
/*: CR 12/11/91 23:55 Fix hnumber() call. */
/*: JV 12/21/91 14:07 Check to see if host already exists. */
/*: JV  1/03/92 16:37 Change names of some dictionary strings. */
/*: JV  1/15/92 10:45 Change initial toss time to 7days. */
/*: JX  3/23/92 13:19 Chixify. */
/*: JX  6/09/92 17:40 chxtoken returns -1 on fail. */
/*: JX  8/03/92 11:48 Fixify. */
/*: CP  5/06/92 12:41 Add 0L as 'item' arg to modnames() call. */

#include <stdio.h>
#include "handicap.h"
#include "derr.h"
#include "netmaint.h"
#include "unitcode.h"
#include "common.h"
#include "caucus.h"

extern union null_t null;

FUNCTION add_naybor()
{
   int   hostnum, c, success;
   Chix  chn, newline, mgr, old, addr, new, str;
   struct namlist_t *add, *del, *nlnode();

   ENTRY ("add_naybor", "");

   chn  = chxalloc (80L, THIN, "add chn");
   mgr  = chxalloc (80L, THIN, "add mgr");
   new  = chxalloc (80L, THIN, "add new");
   old  = chxalloc (80L, THIN, "add old");
   str  = chxalloc (80L, THIN, "add str");
   addr = chxalloc (80L, THIN, "add addr");
   newline = chxalloc (6L, THIN, "newline");

   /*** Tell the manager what's going on. */
   unit_write (XWTX, NEWCPAGE);
   mdwrite    (XWTX, "net_Tnayintro", null.md);

   /*** Prompt for the CHN, LHN, and MGRID of the neighbor. */
   while (1) {
      c = mdprompter ("net_Pnunaychn", nullchix, 0, "net_Hnunay", 80,
                      chn, nullchix);
      if (c <=  -2)  FAIL;
      if (hnumber (chn) > 0) {
         mdwrite (XWTX, "net_Ehostexist", null.md); continue; }
      if (chxtoken (str, nullchix, 2, chn) == -1)  break;
 
      mdwrite (XWTX, "net_Tnotchn", null.md);
   }
   while (1) {
      c = mdprompter ("net_Pnunaymgr", nullchix, 0, "net_Hnunay", 80,
                      mgr, nullchix);
      if (c <  -2)  FAIL;
      if (c == -2)  continue;
      if (chxtoken (str, nullchix, 2, mgr) == -1)  break;
 
      mdwrite (XWTX, "net_Tnotmgr", null.md);
   }
   while (1) {
      c = mdprompter ("net_Pnunayaddr", nullchix, 0, "net_Hnunay", 80,
                      addr, nullchix);
      if (c <  -2)  FAIL;
      if (c == -2)  continue;
      if (chxtoken (str, nullchix, 2, addr) == -1)  break;
 
      mdwrite (XWTX, "net_Tnotaddr", null.md);
   }

   /*** Create the new conference name and number. */
   chxcpy (str, CQ("!hosts"));
   hostnum = hnumber (str) + 1;
   if (hostnum < 1) {
      mdwrite (XWER, "net_Ebaddb", null.md);
      FAIL;
   }
   chxformat (old, CQ("!hosts #%03d"), L(hostnum-1), 0L, nullchix, nullchix);
   chxformat (new, CQ("!hosts #%03d"), L(hostnum), 0L, nullchix, nullchix);
   chxclear  (str);
   chxformat (str, CQ("%s #%03d"), L(hostnum), 0L, chn,  nullchix);
   add = nlnode(1);
   nladd (add, new, 0);
   nladd (add, str, 0);
   del = nlnode(1);
   nladd (del, old, 0);
   modnames (XSHD, XSHN, nullchix, hostnum, 0L, add, del, PUNCT);
   nlfree (add); nlfree (del);

   /*** First try and make the conference directory... */
   if (NOT sysmkhost (hostnum)) {
      mdwrite (XWER, "net_Ebaddb", null.md);
      FAIL;
   }

   /*** Now write the hostinfo file. */
   unit_lock (XHI1, WRITE, L(hostnum), 0L, 0L, 0L, nullchix);
   unit_make (XHI1);

   chxofascii (newline, "\n");
   unit_write (XHI1, chn);    unit_write (XHI1, newline);
   unit_write (XHI1, chn);    unit_write (XHI1, newline);
   unit_write (XHI1, mgr);    unit_write (XHI1, newline);

   unit_write (XHI1, CQ("1 604800 86400 2\n"));
   unit_write (XHI1, CQ("604800\n"));
   unit_write (XHI1, CQ("1\n"));
   unit_write (XHI1, CQ("0 0 1\n"));
   unit_write (XHI1, CQ("1 1\n"));
   unit_write (XHI1, CQ("1 1\n"));

   unit_close (XHI1);
   unit_unlk  (XHI1);

   /*** Now write the shipinfo file. */
   unit_lock (XHSV, WRITE, L(hostnum), L(1), 0L, 0L, nullchix);
   unit_make (XHSV);
   unit_write (XHSV, CQ("10000 10000 10000\n"));

   unit_write (XHSV, addr);   unit_write (XHSV, newline);

   unit_close (XHSV); unit_unlk  (XHSV);

   mdwrite (XWTX, "net_Tnunaydone", null.md);

   SUCCEED;

done:
   chxfree (chn);
   chxfree (mgr);
   chxfree (new);
   chxfree (old);
   chxfree (str);
   chxfree (addr);
   chxfree (newline);

   RETURN (success);
}
