/* ADD_PERSON:   Add people interactively or from a file.
/
/  int n = add_person ()
/
/  Purpose:
/     The process of creating a new neighbor:
/     1) Batch or interactive?
/     B-a) Prompt for filename
/     B-b) Call reg_parse_file
/     I-a) Prompt for info
/     I-b) Call reg_user
/
/  How it works:
/
/  Returns:
/    0: Failed to perform any action due to input problems.
/    1: Carried out mgr's wishes as much as possible.
/
/  Error conditions:
/
/  Related functions: 
/
/  Called by: doperson()
/
/  Home: netmaint/addperso.c
/
/  Dictionary strings:
/     net_Tperintro: You have chosen to add one or more people to your
                     Caucus system.
      net_Pnuperson: What is the Full Name of this person? $
      net_Hnuper:    (The Usual drivel.)
      net_Tnotper:   That is not a valid Caucus Full Name.  Please try again
                     or press <CANCEL>.
      net_Pnulast:   What name should this person be sorted by? $
      net_Hnulast:   (More drivel)
      net_Tnotlast:  That is not a valid name.  Please try again or press
                     <CANCEL>.
      net_Pnuid:     What is the Caucus ID of this person? $
      net_Hnuid:     (less drivel)
      net_Tnotid:    That is not a valid Caucus ID.  Please try again or press
                     <CANCEL>.
      net_Tidexist:  That Caucus ID already exists.  Please try again or press
                     <CANCEL>.
      net_Pnuphone:  What is the telephone number of this person? $
      net_Hnuphone:  (guess)
      net_Tnotphone: That is not a valid telephone number.  Please try again
                     or press <CANCEL>.
      net_Piscapt:   Should this person be a captive user? (no) $
      net_Piscapt:   Should this person be a captive user? (no) $
      net_Ppasswd:   What should this user's captive password be? $
      net_Tnuperdone: This person has been added as a Caucus user.


Question: why is second name automatically accepted if there are only two?
(Look carefully for this!)
*/

/*: JV  8/07/92 11:54 Creat (sic) this. */
/*: CR 10/09/92 17:15 Alloc lastword; remove add, del. */

#include <stdio.h>
#include "handicap.h"
#include "derr.h"
#include "netmaint.h"
#include "unitcode.h"
#include "common.h"
#include "caucus.h"

extern struct flag_template flag;
extern union null_t null;
extern Chix         yesnotable;

FUNCTION add_person()
{
   int     c, success, i;
   Chix    newline, old, new, str, lastword, yes, answer, passwd, word;
   Userreg reg, reg2, make_userreg();
   Flag    iscapt;

   ENTRY ("add_person", "");

   new      = chxalloc (80L, THIN, "addp new");
   old      = chxalloc (80L, THIN, "addp old");
   str      = chxalloc (80L, THIN, "addp str");
   yes      = chxalloc (20L, THIN, "addp yes");
   word     = chxalloc (16L, THIN, "addp word");
   answer   = chxalloc ( 6L, THIN, "addp answer");
   passwd   = chxalloc ( 8L, THIN, "addp passwd");
   newline  = chxalloc ( 6L, THIN, "addp newline");
   lastword = chxalloc (60L, THIN, "addp lastword");

   reg  = make_userreg();
   reg2 = make_userreg();

   /*** Tell the manager what's going on. */
   unit_write (XWTX, NEWCPAGE);
   mdwrite    (XWTX, "net_Tperintro", null.md);

   /* Check if captive.  If so, prompt for pw and suffix underscore to id. */
   chxtoken (yes, null.chx, 2, yesnotable);

   c = mdprompter ("net_Piscapt", yesnotable, 2, "net_Hnuper", 80,
                      answer, yes);

   if (c > 0) iscapt = 1;
   else       iscapt = 0;

   /*** Prompt for the FullName, SortName, and ID of the new user. */
   while (1) {
      c = mdprompter ("net_Pnuperson", nullchix, 0, "net_Hnuper", 80,
                      reg->name, nullchix);
      if (c <= -3) FAIL;
      if (c == -2) continue;
 
      /*** Remove additional blanks, and make sure the result isn't null. */
      chxsimplify (reg->name);
      if (NOT EMPTYCHX (reg->name))  break;

      mdwrite (XWTX, "net_Tnotper", null.md);
   }

   /*** Get the name to sort by ("last" name). */
   if (flag.japan) chxtoken (lastword, nullchix, 1, reg->name);
   else
      for (i=1;   chxtoken (word, nullchix, i, reg->name) != -1;   ++i)
         chxcpy (lastword, word);
 
   jixreduce (lastword);
   if  (i==2)  { chxcpy (reg->lastname, lastword);   SUCCEED; }

   while (1) {
      c = mdprompter ("net_Pnulast", nullchix, 0, "net_Hnulast", 80,
                      reg->lastname, lastword);
      if (c <  -2)  FAIL;
      if (c == -2)  continue;
      if (NOT EMPTYCHX (reg->lastname))  break;

      /*** Remove blanks, check non-null. */
      chxsimplify (reg->lastname);
      if (NOT EMPTYCHX (reg->lastname))  break;
 
      mdwrite (XWTX, "net_Tnotlast", null.md);
   }

   /*** Get ID. */
   while (1) {
      c = mdprompter ("net_Pnuid", nullchix, 0, "net_Hnuid", 80,
                      reg->id, nullchix);
      if (c <  -2)  FAIL;
      if (c == -2)  continue;

      if (iscapt) chxcat (reg->id, CQ("_"));

      /* Check if a valid id. */
      if (bad_name (reg->id)) {
         mdwrite (XWTX, "net_Tnotid", null.md);
         continue;
      }

      /* Check if person exists already. */
      if (NOT get_person (reg->id, reg2)) break;
      mdwrite (XWTX, "net_Tidexist", null.md);
   }

   if (iscapt) {
      c = mdprompter ("net_Pnupw", nullchix, 0, "net_Hnupw", MAXPASS,
                      passwd, nullchix);
      if (c <  -2) FAIL;

      jixreduce (passwd);
   }
      
   /*** Add person. */
   reg_user (reg, iscapt, passwd);

   SUCCEED;

done:
   chxfree (new);
   chxfree (old);
   chxfree (str);
   chxfree (word);
   chxfree (answer);
   chxfree (passwd);
   chxfree (newline);
   chxfree (lastword);

   free_userreg (reg);
   free_userreg (reg2);
   RETURN (success);
}
