/* ATTACH:   Change the network DB and send an ATTREQ xaction.
/
/  int n = attach ()
/
/  Purpose:
/     After a neighbor has been defined an initial xaction must be sent to it.
/     This must happen from one side only, even both sides must define each
/     other.
/
/  How it works:
/     It asks which neighbor, then sends the xaction.
/
/     Later, when we distribute people information, that will have to happen
/     here.
/
/  Returns:
/    0: Failed to perform any action due to input problems.
/    1: Carried out mgr's wishes as much as possible.
/
/  Error conditions:
/
/  Related functions: 
/
/  Called by: donaybor()
/
/  Home: netmaint/attach.c
/
/  Dictionary strings:
/     net_Tattintro: You have chosen to attach to a neighbor.
/     net_Tattnonay: Unfortunately, you have not defined any neighbors yet.
/                    Please see the CaucusLink Network Manager's Guide for
/                    more information on Defining Neighbors.
/     net_Tattnone:  Unfortunately, you have attached to all of your
/         neighbors.
/     net_Tlsnay:    This is a list of all your neighbors:
/     net_Flschnstt: %3d) %40s %s\n
/                    (Format string for displaying CHN's and status)
/     net_Tattdone:  An Attachment Request has been sent to this neighbor.
/                    CaucusLink will send a message to your Caucus mailbox
/                    when the attachment is complete.
/  */

/*: JV 12/11/91 13:35 Creat (sic) this. */
/*: UL 12/11/91 18:43 Make unit_lock args 3-6 long.*/
/*: CR 12/12/91 21:28 fix sysfree() args, header. */
/*: JV 12/19/91 18:14 Actually set lhnum before locking XHI1. */
/*: JV  1/03/92 16:45 Clean up header. */
/*: JX  3/23/92 14:48 Chixify */
/*: JX  6/09/92 17:42 chxtoken returns -1 on fail. */
/*: JX  8/03/92 11:53 Fixify. */
/*: JX  9/01/92 13:51 Change empty_xact to make_xaction(). */
/*: CR 12/14/92 15:15 Long chxvalue arg. */
/*: CP  8/01/93 14:32 Add 'name' arg to make_textbuf() call. */

#include <stdio.h>
#include "handicap.h"
#include "derr.h"
#include "netmaint.h"
#include "unitcode.h"
#include "common.h"
#include "xfp.h"
#include "xaction.h"
#include "caucus.h"

extern union null_t null;
extern Chix         naystat;

FUNCTION attach ()
{
   int    ok, success, highest;
   long   i, lhnum, which;
   char  *sysmem();
   Chix   id, left, number, format, status, line, str;
   struct namlist_t *namdir, *this, *nlnode();

   Textbuf  npfbuf, make_textbuf();
   HOST     hosthead, *hostptr, *nextptr, *myptr, *make_host();
   Hinfoptr make_hinfo();
   Xactionptr  xaction, make_xaction();
   Chix carray[2];

   ENTRY ("attach", "");

   id    = chxalloc (40L, THIN, "id");
   str   = chxalloc (60L, THIN, "str");
   left  = chxalloc (40L, THIN, "left");
   line  = chxalloc (85L, THIN, "line");
   number = chxalloc (10L, THIN, "number");
   format = chxalloc (200L, THIN, "format");
   status = chxalloc (40L, THIN, "status");

   npfbuf  = make_textbuf (TB_RESPS, TB_LINES, "attach npfbuf");
   xaction = make_xaction();

   hosthead.next = (HOST *) NULL;

   /*** Tell the manager what's going on. */
   unit_write (XWTX, NEWCPAGE);
   mdwrite    (XWTX, "net_Tattintro", null.md);

   /*** First see if there any hosts.  If there are, build a linked list
   /    of them and display the list.  Remember to free the list after the
   /    user chooses one. */
   if (NOT unit_lock (XSHD, READ, 0L, 0L, 0L, 0L, nullchix))  {
      bug ("ATTACH: can't lock XSHD.", 0);   FAIL; }
   namdir = nlnode(4);
   ok = loadnmdir (XSHD, namdir);
   unit_unlk (XSHD);
   if (NOT ok)     {
      nlfree (namdir);
      bug ("ATTACH: can't load XSHD.", 0);     FAIL; }

   /*** Load each namepart file in turn and store host data. */
   for (this=namdir->next;   this != null.nl;   this = this->next) {
      if (this->num <= 0)  continue;
      if (NOT unit_lock (XSHN, READ, 0L, L(this->num), 0L, 0L, nullchix)) 
          continue;
      if (NOT loadnpart (XSHN, this->num, npfbuf))
         { unit_unlk (XSHN);  continue; }
      unit_unlk (XSHN);

      /* Aaah! Finally we find some host data and load each one.
      /  LH# is word 2 in LINE.  Then get the host with get_neighbor()
      /  and add the host to the end of the linked list. */
      for (i=0, nextptr = &hosthead;   i<npfbuf->l_used;   ++i) {
         if (chxtoken (id, nullchix, 2, npfbuf->line[i]) == -1)
            continue;
         if (chxvalue (npfbuf->line[i], 0L) == '!')
            continue;                             /* Ignore "!host nnn" */
         chxbreak (id, left, number, CQ("#"));
         if (NOT chxnum (number, &lhnum)) {
            bug ("ATTACH: error loading LH#'s.", 0); FAIL; }
         hostptr = make_host();
         hostptr->info = make_hinfo();
         unit_lock (XHI1, READ, lhnum, 0L, 0L, 0L, nullchix);
         get_neighbor ((int) lhnum, hostptr, XHI1);
         unit_unlk (XHI1);
         nextptr->next = hostptr;
         nextptr = hostptr;
         hostptr->next = (HOSTPTR) NULL;
      }
   }  /* for each partfile */
   nlfree (namdir);
 
   /*** Now that we have the linked list, display it and ask for the
   /    number.  Then follow that number thru the linked list to get
   /    the LHNUM of the host. */
   mdwrite (XWTX, "net_Tlsnay", null.md);
   mdstr (format, "net_Flschnstt", null.md);
   for (i=1, hostptr = hosthead.next; hostptr != (HOSTPTR) NULL;
        i++, hostptr = hostptr->next) {
      chxtoken  (status, nullchix, (int) hostptr->info->status+1, naystat);
      chxclear  (line);
      chxformat (line, format, i, 0L, hostptr->chn, status);
      unit_write (XWTX, line);
   }
   highest = i-1;

   while (1) {
      if (mdprompter ("net_Pselchn", nullchix, 0, "net_Hselchn",
          60, str, nullchix) < -1)    FAIL;
      chxnum (str, &which);
      if (which < 1 || which > highest) {
         mdwrite (XWTX, "net_Enonum", null.md); FAIL; }
      for (i=1, hostptr = hosthead.next;
           hostptr != (HOSTPTR) NULL && i<which;
           i++, hostptr = hostptr->next) ;
      if (hostptr->info->status == UNATTACHED ||
          hostptr->info->status == DETACHED)  break;
      mdwrite (XWTX, "net_Eattached", null.md);
   }

   for (i=1, hostptr = hosthead.next;
        hostptr != (HOSTPTR) NULL && i<which;
        i++, hostptr = hostptr->next) ;

   /*** Now change the status to PENDING, send the ATTREQ xaction,
   /    and report to mgr. */
   hostptr->info->status = PENDING;
   lhnum = hostptr->lhnumber;
   if (NOT unit_lock (XHI1, WRITE, (long) lhnum, 0L, 0L, 0L, nullchix)) {
      bug ("ATTACH: couldn't lock XHI1 for change.", (int) lhnum);
      FAIL; }
   set_neighbor (hostptr, XHI1);
   unit_unlk (XHI1);

   xaction->typenum = TT_HOST;
   xaction->actnum  = XT_ATTREQ;
   unit_lock (XSNF, READ, 0L, 0L, 0L, 0L, nullchix);
   myptr = make_host();
   myptr->info = make_hinfo();
   get_neighbor  (0, myptr, XSNF);
   unit_unlk (XSNF);
   chxformat   (xaction->uid, CQ("%s@%s"), 0L, 0L,
                                          myptr->info->mgrid, myptr->chn);
   unit_lock (XHML, WRITE, L(lhnum), 0L, 0L, 0L, nullchix);
   unit_append (XHML);
   carray[0] = carray[1] = chxalloc (0L, THIN, "attach carray");
   putnextxact (XHML, xaction, carray);
   unit_close  (XHML);  unit_unlk   (XHML);
   free_host   (myptr);
   chxfree     (carray[0]);

   mdwrite (XWTX, "net_Tattdone", null.md);
   success = 1;

done:
   /*** Now free up memory and go home. */
   for (hostptr = hosthead.next; hostptr != (HOSTPTR) NULL; ) {
      nextptr = hostptr->next;
      free_host (hostptr);
      hostptr = nextptr;
   }
   chxfree (id);
   chxfree (str);
   chxfree (left);
   chxfree (line);
   chxfree (number);
   chxfree (format);
   chxfree (status);
   free_xaction (xaction);
   RETURN (success);
}
