/*** CHEKLINK: command line interface to routines which check the 
/    CaucusLink database.
/
/   Usage:
/      cheklink_x confid [ -f ] | [ -r # [ -f ]] | [ -s CHN [ -f ]]
/
/ v----Feature has not been implemented.
/
/   Options:
/      without any options, cheklink_x checks the ParseBay (XSPB) and
/      ReParseBay (XSRP) against the sysinfo (XSNF) file.
/      It validates that each value in XSNF is within its range.
/      It warns about any transport files in either of these bays which
/      *should* be there, but aren't.
/ *    It warns about files in either of these bays which *should not*
/      be there.
/ *    If the -f option is used alone, cheklink_x attempts to fix any
/      inconsistencies by:
/ *       if (XSNF:seq_from != earliest XSPF file):
/            change XSNF:seq_from to the seq # of that XSPB file.
/
/ *       if (XSNF:seq_ack != latest XSPF file + 1)
/            change XSNF:seq_ack to (seq# of that XSPB file plus 1).
/            
/ *       if (XSNF:seq_thrulo != earliest XSRP file):
/            change XSNF:seq_thrulo to the seq # of that XSRP file.
/
/ *       if (XSNF:seq_thruhi != latest XSRP file + 1)
/            change XSNF:seq_thruhi to (seq# of that XSRP file plus 1).
/
/ *    -r # [ -f ]
/         # must be a receiving method number.  With this option, cheklink_x
/         checks the files in that Receive bay against info in the Receive
/         database (XRCC, XRCH) and the appropriate XHI1 files.
/ *       It comments on the existence of a raw file, any valid chunks, and
/         transport files in the receive bay.
/ *       It warns about invalid chunks and transport files.
/ *       If the -f option is used in conjunction with -r, cheklink_x
/         attempts to fix any inconsistencies by:
/ *          If there are any chunks higher than XRCC, change XRCC to 
/               be one greater than the highest numbered chunk.
/ *          If there are any chunks not listed in XRCH, delete them.
/ *          If there are any entries in XRCH with no chunks, delete the
/               entry.
/ *          If there are RERECV files with seq# lower than XHI1:seq_from,
/ *?Does this   (within a sanity limit) set seq_from to the file's seq# so
/  make sense?  that it will be processed next time through RECEIVE.
/
/ *    -s CHN [ -f ]
/         CHN must be a CHN for a neighbor.  With this option, cheklink_x
/         checks the files in the Shipping Bay which is used to send files
/         to this neighbor.
/ *       It warns if the status != ATTACHED (3).
/ *       It warns about any (XPXH, XPXT, XHXA) which should exist, but don't.
/ *       It warns about any invalid (XPXH, XPXT, XHXA) which exist.
/ *       If the -f option is used in conjunction with -s, cheklink_x
/         attempts to fix any inconsistencies by:
/ *          if (XHI1:to_low >= any ship file && shipfile # is w/in sanity):
/               decrease to_low to be one less than the lowest shipfile #
/ *          if (XHI1:to_hi < the highest ship file):
/               increase to_hi to the highest numbered ship file.
/ */

/*: JV 11/05/93 17:39 Create this. */
/*: CP 12/10/93 20:26 Make debug extern. */
/*: CR  3/24/94 13:35 Add xfpglobals() call; won't link w/o it on some sys. */

#include <stdio.h>
#include "handicap.h"
#include "chixuse.h"
#include "caucus.h"

static char usage_str[] = 
   "Usage: %s confid [-f] | [-r # [-f]] | [-s CHN [-f]]\n";

#define Usage { printf(usage_str, argv[0]); exit(1); }
#define SYSTEM  1
#define RECEIVE 2
#define SHIP    3

Chix confid; 
extern int  debug;
extern struct screen_template used, screen;

main (argc, argv)
int argc; char *argv[];
{
   char fix=0, type=0;
   int  rec_meth, errors;
   Chix chn;

   xfpglobals();

   chn = chxalloc (20L, THIN, "main chn");
   confid = chxalloc (20L, THIN, "main confid");

   if (argc == 1) Usage;

   chxofascii (confid, argv[1]);

   screen.lines=23;  used.lines=0;
   screen.width=80;  used.width=0;

   if (argc < 4) {

      /*** Just check XSNF and ParseBays. */
      type = SYSTEM;
      if (argc == 3) 
         if (NOT streq (argv[2], "-f")) Usage
         else fix=TRUE;
   } else {
      /*** Just check a Receive bay. */
      if (streq (argv[4], "-f")) fix=TRUE;
      if (streq (argv[2], "-r")) {
         type = RECEIVE;
         if ((rec_meth = atoi (argv[3])) < 1) Usage
      }
      if (streq (argv[2], "-s")) {
         /*** Just check shipping bay for a host. */
         type = SHIP;
         chxofascii (chn, argv[3]);
      }
   }

   switch (type) {
      case SYSTEM:  errors=chk_system  (confid,           fix);  break;
      case RECEIVE: errors=chk_receive (confid, rec_meth, fix);  break;
      case SHIP:    errors=chk_ship    (confid, chn,      fix);  break;
      default:      Usage
   }

   if (NOT errors) printf ("No errors were detected.\n");
}
