/* DELET_CONF: User interface for removing conferences.
/
/  int n = delet_conf()
/
/  Purpose:
/    NetMaint handles various management tasks for the manager.  One of
/    these is removing conferences.  There is another utility, cv2remov,
/    which was created for a similar purpose for Caucus Version 2.  This
/    function also takes CaucusLink into consideration.
/
/  How it works:
/    Q.E.D.
/
/  Returns:
/    1 on success (conf removed or user changed their mind)
/    0 on failure (user hit break, DB error, or rmconf() failed)
/
/  Error conditions:
/
/  Related functions: disp_conf(), publish_conf(), subscr_conf()
/
/  Called by: do_conf()
/
/  Home: netmaint/deletcon.c
/
/  Dictionary strings:
/     net_Tdelconintro: (You're about to delete a conference.)
/     net_Pconfname: Which conference would you like to delete?
/     net_Hconfname: (Help drivel)
*/

/*: JV  1/14/92 14:03 Creat (sic) this. */
/*: JX  3/23/91 15:01 Chixify. */
/*: JX  5/16/92 15:05 Remove unnecessary OK. */
/*: JX  6/09/92 17:43 chxtoken returns -1 on fail. */
/*: JX  8/03/92 11:56 Fixify. */
/*: CK  3/20/93 19:28 "no" response should fail & not delete conf! */

#include <stdio.h>
#include "handicap.h"
#include "netmaint.h"
#include "unitcode.h"
#include "chixuse.h"
#include "done.h"
#include "null.h"
#include "conf.h"

extern union null_t null;
extern Chix  yesnotable;

FUNCTION delet_conf()
{
   Chix  cname, no, format, message, answer, str; 
   int   v, is_local, confnum, success;
   short c;

   ENTRY ("delet_conf", "");

   no    = chxalloc (20L, THIN, "no");
   str   = chxalloc (20L, THIN, "str");
   cname = chxalloc (80L, THIN, "cname");
   answer  = chxalloc (20L, THIN, "answer");
   format  = chxalloc (1024L, THIN, "format");
   message = chxalloc (1024L, THIN, "message");

   /*** Tell the user what we're about. */
   unit_write (XWTX, NEWCPAGE);
   mdwrite    (XWTX, "net_Tdelconintro", null.md);

   /*** List the existing conferences. */
   listconfs (XWTX, 1);
   mdwrite   (XWTX, "rem_Tborder", null.md);

   /*** Get the conference name. */
   while (1) {
      c = mdprompter ("net_Pconfname", nullchix, 0, "net_Hconfname", 35,
                      cname, nullchix);
      if (c <  -2)  FAIL;
      if (c == -2)  continue;
 
      confnum = cnumber (cname, LOCALCONF, &v);

      /*** No conferences matched, complain and try again. */
      if (confnum < 0) {
         mdwrite (XWER, "rem_Enoconf", null.md);
         continue;
      } else if (confnum == 0) {
         mdwrite (XWER, "rem_Eambconf", null.md);
         continue;
      }

      if (chxtoken (str, nullchix, 2, cname) == -1)  break;
      mdwrite (XWTX, "net_Tnotname", null.md);
   }

   /*** If it's a published conference, disallow. */
   if (NOT unit_lock (XCNB, READ, L(confnum), 0L, 0L, 0L, nullchix)) FAIL;
   is_local = NOT unit_check (XCNB);
   unit_unlk (XCNB);
   if (NOT is_local) {
      mdwrite (XWTX, "net_Tremnotlocal", null.md);
      FAIL;
   }

   /*** Request confirmation before removing conference. */
   chxtoken (no, nullchix, 1, yesnotable);
   mdstr    (format, "rem_Fconfirm", null.md);
   chxformat  (message, format, 0L, 0L, cname, nullchix);
   c = prompter (message, yesnotable, 2, "rem_Hconfirm", 20, answer, no);
   if (c <  1)  FAIL;

   success = rmconf (confnum);
    
   chxupper   (cname);
   unit_write (XWTX, CQ("\n"));
   unit_write (XWTX, cname);
   mdwrite    (XWTX, success ? "rem_Tdone" : "rem_Tbad", null.md);

done:
   chxfree (no);
   chxfree (str);
   chxfree (cname); 
   chxfree (answer);
   chxfree (format);
   chxfree (message);
   RETURN (success);
}
