/* DISP_CONF:   Display general information about a conference.
/
/  int n = disp_conf()
/
/  Purpose:
/     NetMaint helps the manager to understand what conferences exist,
/     who's in charge of them, whether or not they are distributed,
/     and how big they are.
/
/  How it works:
/
/  Returns:
/    0: Failed to perform any action due to input problems.
/    1: Carried out mgr's wishes as much as possible.
/
/  Error conditions:
/
/  Related functions: 
/
/  Called by: doconf()
/
/  Home: netmaint/dispconf.c
/
/  Dictionary strings:
/     net_Ebaddb:  There is an inconsistency within the Caucus database.
/                  Please report this to the Camber-Roth Support staff.
*/

/*: JV 12/10/91 10:27 Create this. */
/*: UL 12/11/91 18:43 Make unit_lock args 3-6 long.*/
/*: CR 12/11/91 23:55 Fix header to match actual function definition! */
/*: JV  1/03/92 14:48 Remove unused variable 'error'. */
/*: JX  3/23/92 16:32 Chixify */
/*: CX  5/28/92 18:37 Replace putnewline() with unit_write(...\n). */
/*: JX  8/03/92 11:58 Fixify. */
/*: JV  3/28/93 12:15 Chxclear before second chxformat. */
/*: JV  3/28/93 16:10 Display conf size. */
/*: CK  7/27/93 17:29 Declare long confsize(). */

#include <stdio.h>
#include "handicap.h"
#include "chixuse.h"
#include "done.h"
#include "derr.h"
#include "netmaint.h"
#include "unitcode.h"
#include "common.h"
#include "master.h"
#include "conf.h"
#include "maxparms.h"

extern union null_t null;

FUNCTION disp_conf()
{
   struct master_template master;
   Chix   confname, locname, tname, orighost, line;
   int    cnum, openval, dist=0, success;
   long   confsize();

   ENTRY ("disp_conf", "");

   line     = chxalloc (100L, THIN, "line");
   tname    = chxalloc (40L, THIN, "tname");
   locname  = chxalloc (40L, THIN, "locname");
   confname = chxalloc (80L, THIN, "confname");
   orighost = chxalloc (80L, THIN, "orighost");

   while (1) {
      if (mdprompter ("net_Plistconf", nullchix, 0, "net_Hlistconf", 80,
                       confname, nullchix) < -1)   FAIL;
 
      if ((cnum = cnumber (confname, LOCALCONF, &openval)) <= 0) {
         mdwrite (XITT, "rem_Enoconf", null.md); continue; }
      break;
   }

   /*** Get Primary Organizer ID. */
   if (NOT unit_lock (XCMA, READ, L(cnum), 0L, 0L, 0L, nullchix)) {
      mdwrite (XWER, "net_Ebaddb", null.md);
      FAIL;
   }

   make_master (&master, MAXITEMS);
   if (NOT srvldmas (0, &master, cnum)) {
      unit_unlk (XCMA);
      mdwrite (XWER, "net_Ebaddb", null.md);
      FAIL;
   }

   unit_unlk (XCMA);

   if (is_distributed (cnum)) {
      dist = 1;
      get_trueconf (cnum, locname, tname, orighost);
   }

   mdwrite (XITT, "net_Tdislconf", null.md);
   unit_write (XITT, confname);
   unit_write (XITT, CQ("\n"));

   if (dist) {
      mdwrite (XITT, "net_Tdistconf", null.md);
      chxformat (line, CQ("%s@%s\n"), 0L, 0L, tname, orighost);
      unit_write (XITT, line);
   }

   mdwrite (XITT, "net_Tdisporg", null.md);
   unit_write (XITT, master.organizerid);
   unit_write (XITT, CQ("\n"));

   mdwrite (XITT, "net_Tdisitems", null.md);
   chxformat (line, CQ("%d\n"), L(master.items), 0L, nullchix, nullchix);
   unit_write (XITT, line);

   mdwrite (XITT, "net_Tdiscsize", null.md);
   chxclear  (line);
   chxformat (line, CQ("%d\n"), confsize (cnum), 0L, nullchix, nullchix);
   unit_write (XITT, line);

   mdwrite (XITT, "net_Tdiscnum", null.md);
   chxclear  (line);
   chxformat (line, CQ("%d\n"), L(cnum), 0L, nullchix, nullchix);
   unit_write (XITT, line);

   unit_write (XITT, CQ("\n"));

   SUCCEED;

done:
   chxfree (line);
   chxfree (tname);
   chxfree (locname);
   chxfree (confname);
   chxfree (orighost);
   RETURN (success);
}
