/* DISP_NAYBOR:  List all of the neighbors, prompt for choice,
/                  display info about that neighbor.
/
/  disp_naybor ()
/
/  Purpose:
/    The CaucusLink Mgr needs to alter the neighbor DB.  To alter it,
/    he needs to look at it.  This function provides that.
/
/  How it works:
/
/  Returns: 0 on DB error, input error,
/           1 on success.
/
/  Bugs: This code assumes that all hosts from 001 to HIGHEST exist.
/        It should get fixed to be like ATTACH.  Further, it has the
/        wrong mapping from number to name.
/
/  Error conditions:
/    Nothing can go wrong...
/
/  Called by: do_naybor.
/
/  Home: netmaint/dispnayb.c
/
/  Dictionary strings:
/     net_Tlsnay: This is a list of all of your neighbors:
/     net_Flschn: %3d) %s\n
/     net_Pselchn: Please choose a number from the list.
/     net_Hselchn: (Help text for previous)
/     net_Enonum:  That was not a number from the list.
/
/     net_Fnaydisp1:  Caucus Host Name: %s
/     net_Fnaydisp2:   Local Host Name: %s
/     net_Fnaydisp3: Local Host Number: %4d
/     net_Fnaydisp4: Connection Status: %s
/     net_Fnaydisp5: Manager Caucus ID: %s
/     net_Fnaydisp6:   Shipping Method: %d %s
/     net_Fnaydisp7:   ReShipment Time: %s
/     net_Fnaydisp8:         Toss Time: %s
/
/     net_Kstatunat: Not Attached
/     net_Kstatpend: Attachment Pending
/     net_Kstatatt:  Attached
/     net_Kstatdet:  Detached
/
/     net_Kminute:   m
/     net_Khour:     h
/     net_kday:      d
/  */

/*: JV 11/16/91 12:47 Creat (sic) this. */
/*: JV 11/17/91 17:42 Fix bug. */
/*: JV 11/18/91 17:47 Make first arg of time_simplify long. */
/*: CR 11/25/91 14:08 Declare strtoken(). */
/*: JV 12/11/91 14:37 Add bug warning. */
/*: UL 12/11/91 18:43 Make unit_lock args 3-6 long.*/
/*: JV  1/15/92 10:00 Use seconds instead of minutes for times. */
/*: JX  3/23/92 16:42 Chixify. */
/*: JX  7/31/92 16:21 Add make_host(), fixify. */
/*: JV  9/15/92 12:25 Test neighbor existence by locking XHDD, not XHI1. */
/*: CI 10/06/92 15:47 Chix Integration. */
/*: CR 12/14/92 15:15 Long chxvalue arg. */
/*: CR  2/22/93 13:53 Make sure XHDD exists. */

#include <stdio.h>
#include "handicap.h"
#include "chixuse.h"
#include "done.h"
#include "netmaint.h"
#include "xfp.h"
#include "unitcode.h"
#include "null.h"

extern union null_t null;

FUNCTION disp_naybor ()
{
   Chix  chn, str, line, word, word1, format, method, status,
         time, minstr, hourstr, daystr;
   int   i, ok=0, days=0, hours=0, minutes=0, success, exists;
   long  highest=0L, which=0L, methnum=0;
   HOSTPTR  host, make_host();
   Hinfoptr make_hinfo();

   ENTRY ("disp_naybor", "");

   chn  = chxalloc (100L, THIN, "dispn chn");
   str  = chxalloc (200L, THIN, "dispn str");
   line = chxalloc (90L, THIN, "dispn line");
   word = chxalloc (20L, THIN, "dispn word");
   word1   = chxalloc (20L, THIN, "dispn word1");
   format  = chxalloc (200L, THIN, "dispn format");
   method  = chxalloc (80L, THIN, "dispn method");
   status  = chxalloc (80L, THIN, "dispn status");
   time    = chxalloc (20L, THIN, "time");
   daystr  = chxalloc (20L, THIN, "daystr");
   hourstr = chxalloc (20L, THIN, "hourstr");
   minstr  = chxalloc (20L, THIN, "minstr");

   host = make_host();
   host->info = make_hinfo();

   mdwrite (XWTX, "net_Tlsnay", null.md);

   /*** Get number of highest-numbered host. */
   unit_lock (XSHN, READ, 0L, 1L, 0L, 0L, nullchix);
   unit_view (XSHN);
   unit_read (XSHN, line, 0L);  unit_read (XSHN, line, 0L);
   unit_close (XSHN);  unit_unlk (XSHN);

   if (chxvalue (line, 0L) != '!') {
      bug ("DISP_NAYBOR: EBADDB", 0);  FAIL; }
   chxtoken (word, nullchix, 2, line);
   word1 = chxalsub (word, 1L, 1000L);
   chxnum (word1, &highest);
   if (highest < 1) {
      bug ("DISP_NAYBOR: EBADDB", 0);  FAIL; }

   /*** Now list all that exist. */
   mdstr (format, "net_Flschn", null.md);
   for (i=1; i<=highest; i++) {                                /* Exists? */
      if (NOT unit_lock (XHDD, READ, L(i), 0L, 0L, 0L, nullchix)) continue;
      exists = unit_check (XHDD);
      unit_unlk (XHDD);
      if (NOT exists)  continue;

      if (NOT unit_lock (XHI1, READ, L(i), 0L, 0L, 0L, nullchix)) continue;
      exists = unit_check (XHDD);
      unit_unlk (XHI1);
      if (NOT exists)  continue;

      get_chn (chn, XHI1, i);
      chxclear  (line);
      chxformat (line, format, L(i), 0L, chn, nullchix);
      unit_write (XWTX, line);
   }

   /*** Which one? */
   if (mdprompter ("net_Pselchn", nullchix, 0, "net_Hselchn", 60,
                   str, nullchix) < -2)
      FAIL;
   chxnum (str, &which);
   if (which < 1) {
      mdwrite (XWTX, "net_Enonum", null.md); FAIL; }
   if (NOT unit_lock (XHDD, READ, L(which), 0L, 0L, 0L, nullchix)) {
      mdwrite (XWTX, "net_Enonum", null.md); FAIL; } /* Exists? */
   unit_unlk (XHDD);
   unit_lock (XHI1, READ, L(which), 0L, 0L, 0L, nullchix);
   if (NOT unit_check (XHI1)) {
      mdwrite (XWTX, "net_Enonum", null.md);
      unit_unlk (XHI1); FAIL; }

   /*** Finally display it. */
   ok = get_neighbor ((int) which, host, XHI1);
   unit_unlk (XHI1);
   if (NOT ok) {
      bug ("DISP_NAYBOR: EBADDB", 0); FAIL; }

   mdstr (format, "net_Fnaydisp1", null.md);
   chxclear  (line);
   chxformat (line, format, 0L, 0L, host->chn, nullchix);
   unit_write (XWTX, line);
   mdstr (format, "net_Fnaydisp2", null.md);
   chxclear  (line);
   chxformat (line, format, 0L, 0L, host->lhname, nullchix);
   unit_write (XWTX, line);
   mdstr (format, "net_Fnaydisp3", null.md);
   chxclear  (line);
   chxformat (line, format, L(host->lhnumber), 0L, nullchix, nullchix);
   unit_write (XWTX, line);
   mdstr (format, "net_Fnaydisp4", null.md);
 
   /*** Translate the status number. */
   switch (host->info->status) {
      case (1): mdstr (status, "net_Kstatunat", null.md); break;
      case (2): mdstr (status, "net_Kstatpend", null.md); break;
      case (3): mdstr (status, "net_Kstatatt",  null.md); break;
      case (4): mdstr (status, "net_Kstatdet",  null.md); break;
      default: bug ("DISP_NAYBOR: EBADDB", 0); FAIL;
   }
   chxclear  (line);
   chxformat (line, format, 0L, 0L, status, nullchix);
   unit_write (XWTX, line);

   mdstr (format, "net_Fnaydisp5", null.md);
   chxclear  (line);
   chxformat (line, format, 0L, 0L, host->info->mgrid, nullchix);
   unit_write (XWTX, line);

   /*** Must find the name of the shipping method. */
   unit_lock (XSSM, READ, 0L, 0L, 0L, 0L, nullchix);
   if (NOT unit_view (XSSM)) {
      unit_unlk (XSSM);
      bug ("DISP_NAYBOR: EBADDB: XSSM.", 0); FAIL; }
   while (unit_read (XSSM, line, 0L)) {
      chxtoken (word, nullchix, 1, line);
      chxnum   (word, &methnum);
      if (methnum == host->info->shipmeth) break;
   }
   unit_close (XSSM); unit_unlk (XSSM);
   if (methnum == 0) FAIL;
   chxtoken (method, nullchix, 2, line);

   mdstr (format, "net_Fnaydisp6", null.md);
   chxclear  (line);
   chxformat (line, format, L(host->info->shipmeth), 0L, method, nullchix);
   unit_write (XWTX, line);

   /*** Simplify time measurements before displaying. */
   mdstr (minstr,  "net_Kminute", null.md);
   mdstr (hourstr, "net_Khour",   null.md);
   mdstr (daystr,  "net_Kday",    null.md);
   time_simplify (host->info->reshiptime, &days, &hours, &minutes);

   if (days > 0)
      chxformat (time, CQ("%d%s"), L(days), 0L, daystr, nullchix);
   else if (hours > 0)
      chxformat (time, CQ("%d%s"), L(hours), 0L, hourstr, nullchix);
   else
      chxformat (time, CQ("%d%s"), L(minutes), 0L, minstr, nullchix);

   mdstr (format, "net_Fnaydisp7", null.md);
   chxclear  (line);
   chxformat (line, format, 0L, 0L, time, nullchix);
   unit_write (XWTX, line);

   days = hours = minutes = 0;
   time_simplify (host->info->tosstime, &days, &hours, &minutes);

   chxclear  (time);
   if (days > 0)
      chxformat (time, CQ("%d%s"), L(days), 0L, daystr, nullchix);
   else if (hours > 0)
      chxformat (time, CQ("%d%s"), L(hours), 0L, hourstr, nullchix);
   else
      chxformat (time, CQ("%d%s"), L(minutes), 0L, minstr, nullchix);

   mdstr (format, "net_Fnaydisp8", null.md);
   chxclear  (line);
   chxformat (line, format, 0L, 0L, time, nullchix);
   unit_write (XWTX, line);

   SUCCEED;

done:
   chxfree (chn);    chxfree (str);
   chxfree (line);   chxfree (word);
   chxfree (word1);  chxfree (format);
   chxfree (method); chxfree (status);
   chxfree (time);
   chxfree (daystr);
   chxfree (hourstr);
   chxfree (minstr);
   RETURN (success);
}

/*** Times are stored in XHI* in seconds.  The user (Caucus Mgr)
/    enters the time values in one of these formats:
/    3m; 8h; 4d; corresponding to minutes, hours, days.
/
/    We need to display the times in the "simplest" format.
/    Always round seconds to integer minutes for display.
/    If the number of minutes in XHI1 is an integer number of days,
/    we use that format.  If it's not, but is an integer number of hours,
/    we use that format.  If it's not anything else, display it in minutes.
/
/    TIME_SIMPLIFY takes an integer in TIME and stores the result in one
/    of DAYS, HOURS, MINUTES.
/
/    TIME_SIMPLIFY returns 1. */

int time_simplify (time, days, hours, minutes)
   long time;
   int *days, *hours, *minutes;
{
   int temptime, tempmin;

   tempmin = time / 60;
   temptime = tempmin / 1440;           /* Minutes in a day */
   if (temptime * 1440 == tempmin) {
      *days = temptime; return (1); }

   temptime = tempmin / 60;
   if (temptime * 60 == tempmin) {
      *hours = temptime; return (1); }

   *minutes = tempmin; return (1);
}
