/* DO_CONF:   Print out the menu of things you can do to a conference
/      and then do the bidding of the manager.
/
/  int n = do_conf ()
/
/  Purpose:
/    NetMaint handles various management tasks for the manager.  One of
/    these is managing conferences.  The manager can create, destroy,
/    publish, subscribe to, and unsubscribe from conferences.
/    This function prints out the menu of the actions that can be performed
/    on a conf, and takes an appropriate action depending on the choice.
/
/  How it works:
/    Q.E.D.
/
/  Returns:
/    0: Failed to perform any action due to input problems.
/    1: Carried out mgr's wished as much as possible.
/
/  Error conditions:
/    Each "action function" that do_conf calls takes care of reporting
/    problems.
/
/  Related functions: do_naybor(), do_people().
/
/  Called by: netmaint (main).
/
/  Home: netmaint/doconf.c
/
/  Dictionary strings:
/    net_Fbadchoice:
       "You must choose a number from the menu.
        %d is not one of those choices."
*/

/*: JV 10/27/91 17:46 Creat (sic) this. */
/*: JV 11/18/91 11:55 Initialize choice. */
/*: CR 11/25/91 14:09 Fix mdwrite() call. */
/*: JV 12/10/91 10:23 Add Display Conference. */
/*: CR 12/11/91 23:55 Fix disp_conf() call. */
/*: JV  1/14/92 14:04 Add delet_conf() call. */
/*: DE  4/09/92 13:56 Chixified */
/*: JX  8/09/92 16:16 Fixified. */
/*: CR 10/11/92 13:11 Check return value from chxnum(). */

#include <stdio.h>
#include "handicap.h"
#include "netmaint.h"
#include "unitcode.h"
#include "common.h"
#include "done.h"

extern union null_t null;

FUNCTION do_conf()
{
   long  choice;
   int    ok;
   short finished=0;
   Chix  str, str2, format;
   int   success;

   ENTRY ("do_conf", "");

   str       = chxalloc (200L, THIN, "doconf str");
   str2      = chxalloc (200L, THIN, "doconf str2");
   format    = chxalloc (200L, THIN, "doconf format");
   success   = 0;

   while (NOT finished) {
      print_menu (NM_CONF);
      ok = mdprompter ("net_Pchoose", nullchix, 0, "net_Hchoose", 60, str,
                        nullchix);
      if (ok < -2)  FAIL;
      if (ok == 0) {mdwrite (XWTX, "net_Hchoose", null.md); continue; }

      if (NOT chxnum (str, &choice))  choice = -1L;

      switch (choice) {
         case 0:                           finished=1; break;
         case 1: listconfs (XITT, TRUE);           break;
         case 2: disp_conf ();                     break;
         case 3: if (costart(str2))        finished=1; break;
         case 4: if (delet_conf())         finished=1; break;
         case 5: if (publish_conf()) {
                    mdwrite (XWTX, "net_Tpubdone", null.md);  finished=1; }
                 else 
                    mdwrite (XWTX, "net_Tpubfail", null.md);
                 break;
         case 6: if (subscr_conf())        finished=1; break;
/*       case 7: if (unsubscr_conf())      finished=1; break;  */
         default:
            mdstr (format, "net_Fbadchoice", null.md);
            chxformat (str2, format, 0L, 0L, str, nullchix);
            unit_write (XWTX, str2);
      }
   }
   SUCCEED;

done:

   chxfree ( str );
   chxfree ( str2 );
   chxfree ( format );

   RETURN ( success );
}
