/* DO_NAYBOR: Print out the menu of things you can do to a neighbor,
/      and then do the bidding of the manager.
/
/  int n = do_naybor ()
/
/  Purpose:
/    NetMaint handles various management tasks for the manager.  One of
/    these is managing neighbors.  The manager can define, display,
/    attach to, detach from, synchronize with, and delete definitions
/    of neighbors.
/
/    This function prints out the menu of the actions that can be performed
/    on a neighbor, and takes an appropriate action depending on the choice.
/
/  How it works:
/    Q.E.D.
/
/  Returns:
/    0: Failed to perform any action due to input problems.
/    1: Carried out mgr's wished as much as possible.
/
/  Error conditions:
/    Each "action function" that do_naybor calls takes care of reporting
/    problems.
/
/  Related functions: do_conf(), do_people().
/
/  Called by: netmaint (main).
/
/  Home: netmaint/donaybor.c
/
/  Dictionary strings:
/    net_Fbadchoice:
       "You must choose a number from the menu.
        %d is not one of those choices."
*/

/*: JV 10/27/91 17:03 Creat (sic) this. */
/*: JV 11/19/91 16:33 Help user input. */
/*: CR 11/25/91 14:09 Fix mdwrite() call. */
/*: JV 12/10/91 16:41 Add add_naybor(), attach(). */
/*: DE  4/09/92 14:40 Chixified */
/*: JX  8/09/92 16:17 Fixified. */
/*: CR 10/11/92 13:12 Check return value from chxnum. */

#include <stdio.h>
#include "handicap.h"
#include "unitcode.h"
#include "netmaint.h"
#include "caucus.h"

extern union null_t null;

FUNCTION do_naybor()
{
   long  choice;
   int   ok;
   short finished=0;
   Chix  str, str2, format;
   int   success;

   ENTRY ("do_naybor", "");

   str      = chxalloc (200L, THIN, "donaybor str");
   str2     = chxalloc (200L, THIN, "donaybor str2");
   format   = chxalloc (200L, THIN, "donaybor format");
   success  = 0;

   while (NOT finished) {
      print_menu (NM_NAYBOR);
      ok = mdprompter ("net_Pchoose", nullchix, 0, "net_Hchoose", 60, str, 
                       nullchix);
      if (ok < -2)  FAIL;
      if (ok == 0) {mdwrite (XWTX, "net_Hchoose", null.md); continue; }

      if (NOT chxnum (str, &choice))  choice = -1L;

      switch (choice) {
         case 0: finished=1;              break;
         case 1: if (add_naybor())    break; else continue;
/*       case 2: if (delete_naybor()) break; else continue; */
         case 3: if (disp_naybor())   break; else continue;
         case 4: if (attach()) break; else continue;
/*       case 5: if (detach_naybor()) break; else continue;
         case 6: if (sync_naybor())   break; else continue;
*/       default:
            mdstr (format, "net_Fbadchoice", null.md);
            chxformat (str2, format, 0L, 0L, str, nullchix);
            unit_write (XWTX, str2);
      }
   }

   SUCCEED;

 done:

   chxfree ( str );
   chxfree ( str2 );
   chxfree ( format );

   RETURN ( success );
}
