/* NETMAINT.  This is the program that eases the Caucus Mgr's task
/        of managing CaucusLink.  It provides a menu-driven interface
/        similar to Sys V's "sysadm".  It enables the CaucusLink Mgr
/        to Define & Attach new neigbors, Publish local conf's &
/        Subscribe to remote conf's, and control various other features,
/        all without using an editor!
/
/  How it works:
/    It uses print_menu() to display the choices available..
/    The action taken depends on the manager's choice.
/
/    Here is a complete list of what we intend to allow the manager to
/    do with NetMaint:
/
/    Neighbors:
/      1) Add (Define) a new neighbor
/      2) Delete a definition of a neighbor
/      3) Display a neighbor that's been added
/      4) Attach to a defined neighbor
/      5) Detach from a neighbor
/      6) Synchronize with an attached neighbor
/    Conferences:
/      7) Add a conference
/      8) Delete a conference
/      9) Publish a conference
/     10) Subscribe to a conference on an attached neighbor
/    People:
/     11) Add a person
/     12) Delete a person
/     13) Change a captive person's password
/
/    Here is a complete list of what the manager can do, as of 27-OCT-91:
/
/    0) None of the above.
/
/  Dictionary strings:
/     net_Twelcome:
        "Welcome to the Caucus NetMaint facility.
         To use these menus, press the number of the action
         you'd like to perform, then press the <RETURN> key."
/     net_Tleaving:
        "You are leaving NetMaint."
*/

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved.  */

char copyright[] = 
"Copyright (C) 1991 Camber-Roth.  All rights reserved.";

char *trade[] = {
"This program belongs to Camber-Roth.  It is considered a trade secret",
"and is not to be divulged or used by parties who have not received",
"written authorization from the owner."};

/*: JV  5/20/91 18:10 Begin NetMaint. */
/*: JV 11/19/91 16:31 Help user input. */
/*: CR 11/25/91 14:10 Fix mdwrite() calls. */
/*: CR 12/26/91 13:18 Add used, screen & initialize them. */
/*: JV  1/15/92 10:46 Set debug to -1 if -d. */
/*: CR  2/03/92 18:02 Add systurn, sysexit, screen.lines initialization. */
/*: DE  4/09/92 14:58 Chixified */
/*: JX  8/09/92 16:19 Fixified. */
/*: CR 10/11/92 13:13 Check return value from chxnum(). */
/*: CP  3/04/93 23:16 Make 'debug' extern. */

#include <stdio.h>
#include "caucus.h"
#include "goodlint.h"
#include "unitcode.h"
#include "xaction.h"
#include "xfp.h"
#include "sequence.h"
#include "netmaint.h"

Userreg                        thisuser;
extern struct termstate_t      term_in_caucus, term_original;
extern int                     debug;
extern struct screen_template  used, screen;
extern union  null_t           null;
extern Chix                    confid, confname;

FUNCTION main (argc, argv)
int argc; char *argv[];
{
   short finished=0;
   long  choice;
   int   ok;
   Chix  str, str2, format;
   int   success;

   screen.lines = 23;    used.lines = 0;
   screen.width = 80;    used.width = 0;

   if (argc < 2) {printf ("Usage: netmaint_x confid [-d [#]]\n"); exit(0); }

   str       = chxalloc (200L, THIN, "netmaint str");
   str2      = chxalloc (200L, THIN, "netmaint str2");
   confid    = chxalloc (40L,  THIN, "netmaint confid");
   format    = chxalloc (200L, THIN, "netmaint format");
   confname  = chxalloc (20L,  THIN, "netmaint confname");
   success   = 0;

   chxcpy (confid, CQ(argv[1]));

   systurn (&term_in_caucus, GET_TERM, 0);
   systurn (&term_original,  GET_TERM, 0);
   sysexit (-1);
   utilglobals();
   if (NOT mainglobals()) exit (1);
  
   if (streq (argv[2], "-d")) debug = -1;
   unitglobals();
   xfpglobals();
   sysglobals();

   screen.lines = 23;
   mdwrite (XITT, "net_Twelcome", null.md);

   while (NOT finished) {
      print_menu (NM_MAIN);
      ok = mdprompter ("net_Pchoose", nullchix, 0, "net_Hchoose", 60, str,
                        nullchix);
      if (ok < -2)  FAIL;
      if (ok == 0) {mdwrite (XWTX, "net_Hchoose", null.md); continue; }

      if (NOT chxnum (str, &choice)) choice = -1L;

      switch (choice) {
         case 0: finished=1;      break;
         case 1: do_naybor(); break;
         case 2: do_conf();   break;
         case 3: do_people(); break;
         default:
            mdstr (format, "net_Fbadchoice", null.md);
            chxformat (str2, format, 0L, 0L, str, nullchix);
            unit_write (XWTX, str2);
      }
   }

   mdwrite (XITT, "net_Tleaving", null.md);

   /***de This preserves the original exit technique */

   chxfree (str);
   chxfree (str2);
   chxfree (format);

   sysexit (0);

 done:

   chxfree ( str );
   chxfree ( str2 );
   chxfree ( format );

   RETURN  ( success ); 
}
