
/*** LOGFEATURE.   Log use of a "feature" in XULG logging file.
/
/    logfeature (feature, value, starttime, count, size);
/
/    Parameters:
/       Chix    feature;     (name of feature)
/       Chix    value;       (value, e.g. conf name)
/       long    ivalue;      (numeric value, e.g. item number)
/       long    starttime;   (time use of feature started)
/       long    count;       (number of uses of feature)
/       long    size;        (size of 'thing' used or did)
/       
/    Purpose:
/       Write a record to the current user's XULG feature logging file
/       recording the use of a particular feature.
/
/    How it works:
/       Note: FEATURE may be of the form CQ("xxxx").  (This is unusual,
/       but desireable in this case.)
/
/    Returns:
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  Caucus functions
/
/    Operating system dependencies:
/
/    Known bugs:      none
/
/    Home:  people/logfeat.c
/ */

/*** Copyright (C) 1992 Camber-Roth.   All rights reserved. */

/*: CR  9/04/92 16:20 New function. */
/*: CR  9/09/92 14:20 Add ivalue arg.  */
/*: CI 10/07/92 17:24 Chixify. */
/*: CR 10/13/92 14:55 Remove extra \n from NUMBERS. */

#include <stdio.h>
#include "caucus.h"

extern struct flag_template flag;

FUNCTION  logfeature (feature, value, ivalue, starttime, count, size)
   Chix   feature, value;
   long   ivalue, starttime, count, size;
{
   Chix   userid, temp, fcopy;
   char   numbers[100];
   int    year, month, day, hour, min, sec, success;
   long   now, duration, systime();

   ENTRY  ("logfeature", "");

   if (NOT flag.logfeature)  RETURN (1);

   userid = chxalloc (20L, THIN, "logfeat userid");
   temp   = chxalloc (90L, THIN, "logfeat temp");
   fcopy  = chxalloc (20L, THIN, "logfeat fcopy");
   chxcpy (fcopy, feature);

   sysuserid (userid);
   if (NOT unit_lock   (XULG, WRITE, 0L, 0L, 0L, 0L, userid))  FAIL;
   if (NOT unit_append (XULG))           { unit_unlk (XULG);   FAIL; }

   now = systime();
   duration = (starttime > 0L ? now - starttime : 0L);
   if (duration > 48L * 3600L)  duration = 0L;
   sysymd  (now, &year, &month, &day, &hour, &min, &sec);

   sprintf (numbers, "%5ld %02d%02d%02d %02d%02d%02d %6ld %3ld %7ld",
                     ivalue, year, month, day, hour, min, sec,
                     duration, count, size);
   chxformat  (temp, CQ("%-12s %-8s "), 0L, 0L, userid, fcopy);
   chxformat  (temp, CQ("%-12s %s\n"),  0L, 0L, value, chxquick(numbers,1));
   unit_write (XULG, temp);
   unit_close (XULG);
   unit_unlk  (XULG);
   SUCCEED;

done:
   chxfree (userid);
   chxfree (temp);
   chxfree (fcopy);
   RETURN  (success);
}
