/*** MAKE_TEXTBUF.   Allocate and initialize a textbuffer.
/
/    t = make_textbuf (resps, lines, name);
/
/    Parameters:
/       Textbuf t;       ( Resulting text buffer.)
/       int     resps;   ( Max # of responses or messages per text buffer.)
/       int     lines;   ( Max # of lines, total, in text buffer.)
/       char   *name;    ( name, for debugging)
/
/    Purpose:
/       Create a new, empty text buffer.
/
/    How it works:
/      Calls to make_textbuf usually use the TB_RESPS and TB_LINES constants
/      defined in textbuf.h as the args to make_textbuf().
/
/    Returns: a new Textbuf on success
/             NULL on error
/
/    Error Conditions:
/       Insufficient memory, or the heap is screwed up.
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:
/
/    Operating system dependencies:
/
/    Known bugs:      none
/
/    Home:  resp/maketext.c
/ */

/*** Copyright (C) 1990 Camber-Roth.  All rights reserved. */

/*: CR 10/15/91 14:17 New function. */
/*: CR 12/11/92 15:56 Long sysmem arg. */
/*: CR  2/28/93 21:32 Initialize new 'lastused' member. */
/*: JV  3/09/93 11:38 Add File Attachment. */
/*: JV  6/06/93 11:52 Add text length info. */
/*: CP  8/01/93 14:22 Add 'name' argument. */
/*: CP  8/12/93 13:08 Make t->attach alloc. array of Attachments, make NULL. */
/*: CK  9/22/93 20:30 Raise initial textbuf line alloc to 92. */
/*: JZ 10/18/93 11:03 Declare strncpy(). */

#include <stdio.h>
#include "caucus.h"

extern int debug;

FUNCTION   Textbuf make_textbuf (resps, lines, name)
   int     resps, lines;
   char   *name;
{
   Textbuf t;
   int     i;
   char   *sysmem(), *strncpy();

   ENTRY ("make_textbuf", "");

   if (resps <  20)  resps =  20;
   if (lines < 100)  lines = 100;

   t = (Textbuf) sysmem (L(sizeof (struct textbuf_t)));
   if (t == nulltext)  RETURN (t);

   t->type     = TB_IS_EMPTY;
   strncpy (t->dname, name, 15);  t->dname[15] = '\0';
   t->l_max    = lines;
   t->r_max    = resps;
   t->r_used   = 0;
   t->r_start  = (short *) sysmem ( (sizeof(short)) * L(resps + 1) );
   t->r_stop   = (short *) sysmem ( (sizeof(short)) * L(resps + 1) );
   t->origresp = (short *) sysmem ( (sizeof(short)) * L(resps + 1) );
   t->orighost = (Chix *)  sysmem ( (sizeof(Chix))  * L(resps + 1) );
   t->line     = (Chix *)  sysmem ( (sizeof(Chix))  * L(lines + 1) );
   t->lines    = (long *)  sysmem ( (sizeof(long))  * L(resps + 1) );
   t->bytes    = (long *)  sysmem ( (sizeof(long))  * L(resps + 1) );
   t->lastused = 0;
   t->attach   = (Attachment *) sysmem ((sizeof(Attachment)) *
                                       L(resps+1));

   if (t->orighost == (Chix *)       NULL)   RETURN (nulltext);
   if (t->attach   == (Attachment *) NULL)   RETURN (nulltext);
   for (i=0;   i<resps;   ++i) {
      t->orighost[i] = nullchix;
      t->attach  [i] = (Attachment) NULL;
   }

   if (t->line     == (Chix *) NULL)   RETURN (nulltext);
   for (i=0;   i<lines;   ++i)
      t->line[i] = chxalloc (92L, THIN, "make_text line");

   if (t->r_start  == (short *) NULL  ||
       t->r_stop   == (short *) NULL  ||
       t->origresp == (short *) NULL  ||
       t->orighost == (Chix *)  NULL  ||
       t->lines    == (long *)  NULL  ||
       t->bytes    == (long *)  NULL  ||
       t->attach   == (Attachment *) NULL ||
       t->line     == (Chix *)  NULL)      t = nulltext;

   if (debug  &&  t!=nulltext)  mem_debug (MEM_ALLOC, t->dname, "tbuf", 999L);

   RETURN (t);
}
