/*** SYSACCESS.  Do we have access to the directory containing file NAME?
/
/    i = sysaccess (name);
/
/    Parameters:
/       int   i;
/       char *name;
/
/    Purpose:
/       Determine if we can read and write files in the directory that
/       contains file NAME.
/
/       Sysaccess() is called by unit_view(), unitmkseq(), and unit_append,
/       just before they start working with a particular file.
/
/    How it works:
/       For most systems, sysaccess() just returns 1.
/       For Ultrix, sysaccess() checks the usegid flag set by sysprv().
/       If usegid is off, we call access() to see if we can access
/       the directory with the real userid and group-id.
/
/    Returns: 1 on success (we can read/write in the directory)
/             0 otherwise
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions: sysprv()
/
/    Called by:  unit_view(), unitmkseq(), unit_append()
/
/    Operating system dependencies:
/       meaningful only for ultrix
/
/    Known bugs:
/
/    Home:  sys/sysacces.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: CR  4/23/91 15:45 New function. */
/*: DE  6/01/92 15:19 Unchixified ( Reverted ) */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"

#if UL
#include <unistd.h>
extern int usegid;
#endif

extern int debug;

FUNCTION  sysaccess (name)
   char  *name;
{
   ENTRY ("sysaccess", "");

#if UL
   {  char *s, dirname[200];
      int      n;
      char *bugtell();

      /*** Only check for access by real group-id when usegid is OFF. */
      if (usegid)  RETURN (1);

      /*** Find last slash in file name.  Truncate to make directory name.
      /    (If no slash, user is writing in current directory, which is OK. */
      strcpy (dirname, name);
      for (s=dirname;   *s;   ++s);
      for (;           (1);   --s) {
         if (*s == '/')      break;
         if ( s == dirname)  RETURN (1);
      }
      *s = '\0';

      /*** Do we (via real group-id) have write access to directory? */
      n = access (dirname, W_OK);
      if (debug) {
         bugtell ("sysaccess", n, 0L, name);
      }
      RETURN (n >= 0);
   }
#endif

   RETURN (1);
}
