/*** SYSALARM.  This file contains the alarm initialization routine,
/    the alarm handler, and the alarm setter routines.  The purpose
/    of the alarm timer is operating system dependent.
/
/    For AOS, we need to send a NULL message to the server every
/    SPEAKINT/2 seconds, because the Server forgets about any
/    client who it hasn't heard from in SPEAKINT seconds.
/    SPEAKINT is generally set to about 10 minutes, though the
/    actual value isn't important.  It should be low enough to
/    not put a burden on the IPC mechnaism.
/
/    sysalminit() sets up the alarm handler the first time around.
/
*/

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: JV  2/05/91 12:10 Created sysalarm(). */
/*: CR  4/25/91  0:51 Add include files so this compiles on Unix! */
/*: DE  5/27/92 15:12 Chixified */
/*: JX 10/08/92 11:00 Fixify. */
/*: CR 10/09/92 17:52 Comment out unused sysalarminit(), etc. */
/*: JV  3/29/93 16:20 Comment in  unused code so ranlib will run. */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"

#if CA

#include <signal.h>
#include "vc.h"
#include "errchk.h"
#include "actions.h"

extern union null_t null;
extern int   vcernum;       /* Global error number. */
extern char *vcernam;       /* Global error name. */
extern int   vcdebug;
extern char  alarmflag;
void         (*oldhandler)();  /* Ptr to previous SIGALRM handler. */
unsigned int oldsec;        /* Amount of time left on previous alarm call */

FUNCTION  sysalinit()
{
   int sysalhandler();
   
   if (vcdebug) printf ("In sysalinit.\n");
/* signal (SIGALRM, sysalhandler); */
}

FUNCTION  sysalhandler()
{
   extern Chix  servername;
   
   if (vcdebug) printf ("In sysalhandler.  About to vcsend.\n");
   if (NOT vcsend (servername, ANULL, null.str, 0, null.str, 0)) {
      ERRCHK ("sysalhandler");
      unit_write (XWER, CQ("Caucus Server has terminated.\n"));
      unit_write (XWER, CQ("Caucus is now aborting.\n"));
      sysexit (0);
   }
   if (vcdebug) printf ("In sysalhandler.  Back from vcsend.\n");
   alarmflag = 1;
   signal (SIGALRM, oldhandler);   /* Reset to previous handler. */
}

FUNCTION  sysalstart()   /* This clears the alarm and tells */
{                        /* it to go off in SPEAKINT secs  */
   alarmflag = 0;
   oldhandler = signal (SIGALRM, sysalhandler);  /* Save previous handler. */
   if (vcdebug) printf ("Starting the timer...\n");
   oldsec = alarm (SPEAKINT/2);
}

FUNCTION sysalstop()                 /* This resets the alarm clock to */
{                                    /* its previous state. */
   signal (SIGALRM, oldhandler);     /* Restore the handler. */
   alarm (oldsec);                   /* Restore the time on the clock. */
}

FUNCTION sysalclear ()
{ alarmflag = 0; }

FUNCTION sysalset ()
{ alarmflag = 1; }

FUNCTION sysaltest ()
{
   return (alarmflag);
}
#endif

#if CVM | UNIX | SU | IM | PC | PR | VV | CX | PX | PS

/*** Define stubs for all others, even though only sysalarmset()
/    is called for now (in sys/sysread.c). */

FUNCTION  sysalarminit()
{}

FUNCTION  sysalarm_handler()
{}

FUNCTION  sysalarmset()
{}

#endif




