/*** SYSCHKPRV. Check to make sure that the Caucus executable has
/               sufficient priviliges to run.
/
/   ok = syschkprv (path, num)
/
/   Parameters:
/     int   ok;     // Return value, 0=yes
/     char *path;   // Full path of executable
/     int   num;    // Unused for now
/
/   Purpose:
/      See main/chkprv.c
/
/   How it works:
/      Unix: check setuid bit.
/      VMS:
/
/   Returns: 0:  success
/            1:  Unix setuid bit missing
/            9:  Generic Unix problem
/            11: VMS SYSPRV missing
/            12: VMS SYSLCK missing
/            20: Unidentified error
/
/   Errors:
/
/   Related functions:  main/chkprv()
/
/   Called by:
/
/   Operating System Dependencies:
/
/   Home: sys/syschkpr.c
/ */

/*** Copyright (C) 1993 Camber-Roth.  All rights reserved. */

/*: JV  4/20/93 14:43 Create this. */
/*: JV  6/08/93 20:23 Changt path to char ptr. */
/*: CP  8/09/93 13:15 Put compiler error in VV code as flag. */
/*: CK 10/05/93 12:55 Add 1st draft of VV code. */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "handicap.h"
#include "systype.h"
#include "chixuse.h"

FUNCTION  syschkprv (path, num)
   char *path;
   int   num;
{
/*---------------------Unix-----------------------------------------*/
#if UNIX
   int    found;
   struct stat buf;

   found = stat (path, &buf);

   if (found == -1)            return (9);
   if (buf.st_mode & S_ISUID)  return (0);
   else                        return (1);
#endif

/*---------------------Vax/VMS--------------------------------------*/
#if VV
   static char enable     =   1;
   static char prmflg     =   0;
   static long zenmask[2] = { 0, 0 };
   static long oldmask[2] = { 0, 0 };
   long   bit, word;
   int    PRV$V_SYSPRV();
 
   /*** Set the bit mask according to which bit number PRV$V_SYSPRV is. */
/*
   bit = &PRV$V_SYSPRV;
   word = bit / 32;
   bit  = bit % 32;
   newmask[word] = 1 << bit;
*/

   sys$setprv (enable, zenmask, prmflg, oldmask);
   printf ("syschkprv: %08x %08x\n", oldmask[0], oldmask[1]);
   return (1);
#endif
}
