/*** SYSCLKCHK.   Clock Check.  Has the system clock been set back?
/
/    SYSCLKCHK(0) checks the current time (NOW) against the time stored in
/    the binary XSCK file (THEN).  If THEN is later than NOW + 7200, the
          system
/    clock was set back, and we return 0.  Otherwise, everything's OK and
/    we return 1.  (The 7200--two hours--is to allow for time zone changes
/    and the vagaries of individual PCs connected to a network.)
/
/    SYSCLKCHK(1) resets the value in the XSCK file to NOW and returns 1. */

/*** Copyright (C) 1988 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:44 Source code master starting point */
/*: CR  8/03/88 16:47 initialize 'good', integrate AOS & VM */
/*: JV  9/13/88 16:30 Changed AOS code to create file with correct ACL.*/
/*: CR 10/06/88 14:55 Change IM to use QuickC; (char *) cast read/write args.*/
/*: CR 11/02/88 16:50 Add SRV code, separate SRV from rest of code. */
/*: CR 11/09/88 14:45 Fix SRV bugs. */
/*: CR  3/21/89 15:15 Add use of servername to #if SRV code. */
/*: CW  5/08/89 15:17 Add PX system code. */
/*: JV 10/04/89 17:53 Stripped out AOS non-server code. */
/*: JV 10/05/89 11:02 SRV now exits if vcsend() fails. */
/*: CR 11/14/89 18:08 Allow two hours leeway. */
/*: CR  9/20/90 17:03 Add PS system type. */
/*: CR  4/08/91 18:49 Add new unit_lock argument. */
/*: CR  4/08/91 19:18 Use new confnum argument to unit_name(). */
/*: UL 12/11/91 18:44 Make unit_lock/name args 3-6 long.*/
/*: JV  8/27/92 13:33 Fix VV code. */
/*: DE  5/28/92 12:54 Chixified */
/*: CI 10/07/92 13:55 Chix Integration. */
/*: CV  4/07/93 15:05 Remove duplicate VV in "fd = creat/open" call. */
/*: CR 12/10/93 15:47 Use sysbrkignore()/init() around creat(). */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"

#if VV
#include <unixio>
#include <file>
#endif

#if IM | PN
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <io.h>
#endif

FUNCTION  sysclkchk (reset)
   int    reset;
{
#if UNIX | VV | PR | IM | PN | PX | PS
   char   name[80];
   int    fd, good;
   long   then, now;
   long   systime();

   ENTRY ("sysclkchk", "");

   /*** Lock the XSCK file and prepare to check NOW against THEN. */
   if (NOT unit_lock (XSCK, WRITE, 0L, 0L, 0L, 0L, nullchix))  RETURN (0);
   unit_name (name, XSCK, 0L, 0L, 0L, 0L, nullchix);
   now  = systime();
   then = now;
   good = 1;

#if UNIX | PX | PS
   sysbrkignore();
   fd = (reset ? creat (name, 0700) : open (name, 2));
   sysbrkinit();
#endif
#if VV
   fd = (reset ? creat (name, 0700) : open (name, O_RDWR, 0700));
#endif

#if IM | PN
   fd = open (name, O_BINARY | O_RDWR | (reset ? O_CREAT : 0),
              S_IREAD | S_IWRITE);
#endif

#if PR
   fd = open (name, 2);
#endif

   /*** Read THEN; if ok, rewrite XSCK with NOW. */
   if (fd >= 0) {
      if (NOT reset) {
         read  (fd, (char *) &then, 4);
         lseek (fd, 0L, 0);
      }
      if (now >= then)  write (fd, (char *) &now, 4);
      close (fd);
   }

   good = (fd >= 0)  &&  (now >= then - 7200);

   unit_unlk (XSCK);

   RETURN (good);
#endif

/*-------------------------------Server-based systems-------------------*/
#if SRV
#include "actions.h"
#include "errchk.h"
   extern int   vcernum;
   extern char *vcernam;
   extern Chix  confid;
   extern Chix  servername;
   short  class;
   Flag   in, out;

   ENTRY ("sysclkchk", "");

   class = ACLCH + XSCK;
   in    = reset;

   if (NOT vcsend (servername, class, &in, 1, &out, 1)) {
      ERRCHK("sysclkchk");
      unit_write (XITT, CQ("Caucus Server has terminated.\n"));
      unit_write (XITT, CQ("Caucus is now aborting.\n"));
      sysexit (0);
   }
   RETURN ( (int) out);
#endif

}
