
/*** SYSCLRSCN.  Clear the screen for systems that support it. 
/
/    SYSCLRSCN is used only by "echo -c", which is used in turn
/    by the Caucus menus for some system to clear the screen before
/    displaying the menu.
/
/    Supported systems:
/
/      Vax/VMS: all ansi-style terminals (vt100, etc.), and vt52.
/               Uses VMS's knowledge of terminal type.
/
/      MSDOS:   just sends a ^L, seems to work OK on console.
/
/      UNIX:    uses Unix standard "clear" command.
/
/      AOS/VS:  sends ^L, Jeff V. says it works.
/  */

/*** Copyright (C) 1989 Camber-Roth.  All rights reserved. */

/*: CR 10/25/89 15:21 New function. */
/*: JV  3/10/90 19:50 Added CEO stuff for AOS. */
/*: JV  3/16/90 15:53 Charlie invokes guest programmer laws. */
/*: CR  3/20/90 17:31 Removed most CA code to sysstatus(). */
/*: CR  6/05/90 21:53 CVM, PX: Check for flag.fullscreen. */
/*: CR  9/20/90 17:03 Add PS system type. */
/*: CR  9/27/90 13:45 Add VV code, new header comments. */
/*: CR 10/09/90 19:10 Unix code now uses output from "clear" command. */
/*: CR 11/10/90 18:10 Remove extra ENTRY. */
/*: CR  4/08/91 18:49 Add new unit_lock argument. */
/*: CR  4/08/91 19:18 Use new confnum argument to unit_name(). */
/*: UL 12/11/91 18:44 Make unit_lock/name args 3-6 long.*/
/*: CR  1/08/92 18:09 Unix: try clear, then tput clear, then ^L. */
/*: DE  5/28/92 13:18 Chixified */
/*: JV  3/18/93 12:37 Add WAIT arg to syscmd. */

#include <stdio.h>
#include "caucus.h"
#include "sys.h"
#include "systype.h"

extern struct flag_template flag;

FUNCTION  sysclrscn()
{

#if IM | PN | PR | CA
   ENTRY ("sysclrscn", "");
   sysputstr ("\014");
   RETURN (1);
}
#endif

#if UNIX
#define  MAXCLEAR   200
   static char clear[MAXCLEAR];
   Chix        userid;
   char        clrcom[200], clrfile[160];

   ENTRY ("sysclrscn", "");

   userid   = chxalloc (60L, THIN, "sysclrsc userid");

   /*** On first call, try to get the character sequence that clears
   /    the screen and save it permanently in CLEAR. */
   if (NULLSTR(clear)) {

      /*** Try to put the sequence in temp file XITY. */
      sysuserid (userid);
      unit_name (clrfile, XITY, 0L, 0L, 0L, 0L, userid);

      /*** First, try the Unix 'clear' command with output redirection. */
      strconc   (clrcom, "clear >", clrfile);
      syscmd    (clrcom, SYS_WAIT);
      sysldclr  (clear, userid);

      /*** If that fails (e.g., SCO Unix), try 'tput clear'. */
      if (NULLSTR (clear)) {
         strconc  (clrcom, "tput clear >", clrfile);
         syscmd   (clrcom, SYS_WAIT);
         sysldclr (clear, userid);
      }
  
      /*** If that also fails, give up and guess ^L. */
      if (NULLSTR(clear))  strcpy (clear, "\014");
   }

   sysputstr (clear);

   chxfree ( userid );

   RETURN (1);
}

FUNCTION  sysldclr (clear, id)
   char  *clear;
   Chix   id;
{
   Chix  line;

   line   = chxalloc (200L, THIN, "sysclrsc line");

   clear[0] = '\0';
   if (unit_lock (XITY, WRITE, 0L, 0L, 0L, 0L, id)) {
     if (unit_view (XITY)) {
        while (unit_read (XITY, line, 0L)) {
           if (NOT NULLSTR(clear)) strcat (clear, "\n");
           strcat (clear, ascquick(line));
           if (strlen (clear) >  MAXCLEAR - 80)  break;
        }
        unit_close (XITY);
     }
     unit_kill (XITY);
     unit_unlk (XITY);
  }

  chxfree ( line );
}
#endif


#if PX | CVM | PS
   ENTRY ("sysclrscn", "");
   if (flag.fullscreen)  unit_write (XWTX, NEWCPAGE);
   else                  unit_write (XWTX, CQ("\n"));
   RETURN (1);
}
#endif


#if VV
#include <rmsdef>
#include <ssdef>
#include <descrip>
#include <iodef>
#include <dvidef>

#define  EFN          0
#define  CHAN         0
#define  IOSB         0
#define  ASTADR       0
#define  ASTPRM       0
#define  IS_VT100     2
#define  IS_VT52      1
#define  IS_OTHER     0
#define  NOT_SET     -1

   static int terminal_type = -1;
   struct dsc$descriptor_s devnam;
   long   RETURN_length, answer;

   struct {
      short buffer_length;
      short item_code;
      long *buffer_ptr;
      long *RETURN_ptr;
      long  zero;
   } itemlist;

   ENTRY ("sysclrscn", "");

   /*** On the first call, determine the terminal type. */
   if (terminal_type == NOT_SET) {

      /*** Set up the string descriptor DEVNAM for the terminal. */
      devnam.dsc$w_length  = 11;
      devnam.dsc$a_pointer = "sys$command";
      devnam.dsc$b_class   = DSC$K_CLASS_S;
      devnam.dsc$b_dtype   = DSC$K_DTYPE_T;
   
      itemlist.buffer_length = 4;
      itemlist.item_code     = DVI$_TT_ANSICRT;
      itemlist.buffer_ptr    = &answer;
      itemlist.RETURN_ptr    = &RETURN_length;
      itemlist.zero          = 0L;
   
      answer = 0;
      sys$getdviw (EFN, CHAN, &devnam, &itemlist, IOSB, ASTADR, ASTPRM, 0);

      /*** If the Vax thinks the terminal is an ansi crt, we'll call it a
      /    vt100.  Otherwise, see if it's a VT52. */
      if (answer)  terminal_type = IS_VT100;
      else {
         itemlist.item_code = DVI$_DEVTYPE;
         sys$getdviw (EFN, CHAN, &devnam, &itemlist, IOSB, ASTADR, ASTPRM,
          0);
         if (answer = 0x40)  terminal_type = IS_VT52;
         else                terminal_type = IS_OTHER;
      }
   }


   if (terminal_type == IS_VT100)  sysputstr ("\033[1;1H\033[2J");
   if (terminal_type == IS_VT52)   sysputstr ("\033H\033J");

   RETURN (1);
}
#endif




