/*** SYSCOPY.   Fast copy of (any) file A to file B.
/
/    ok = syscopy (a, b);
/
/    Parameters:
/       int    ok;          (Success?)
/       char  *a;           (name of source file)
/       char  *b;           (name of destination file)
/
/    Purpose:
/       Copy file A to file B.
/
/    How it works:
/
/    Returns: 1 on success
/             0 can't open file A or B.
/
/    Error Conditions:
/ 
/    Related functions:
/
/    Called by:  Caucus functions
/
/    Operating system dependencies: heavy
/
/    Known bugs:      none
/
/    Home:  sys1/syscopy.c
/ */

/*** Copyright (C) 1984 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:44 Source code master starting point */
/*: JV  9/02/88 15:02 Changed ACL of AOS open call.*/
/*: CR 10/06/88 15:06 Change IM to use QuickC. */
/*: CR 11/02/88 17:06 Change VM -> SRV. */
/*: CR 12/16/88 13:03 PN code must use file-sharing. */
/*: CW  5/08/89 15:17 Add PX system code. */
/*: CW  5/19/89 10:54 Add XW unit codes. */
/*: JV 10/04/89 17:56 Stripped out AOS non-server code. */
/*: CR  9/20/90 17:03 Add PS system type. */
/*: CJ 11/26/90 21:50 Add LA system type code. */
/*: JV  3/03/93 10:35 Add comment. */
/*: CP  8/19/93 12:37 New header comments; must copy *any* kind of file. */
/*: CR 12/10/93 15:48 Use sysbrkignore()/init() around creat(). */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"

#if IM | PN
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <share.h>
#include <io.h>
#endif

FUNCTION  syscopy (a, b)
   char  a[], b[];
{
#if SRV
   ENTRY ("syscopy", "");
   unit_write (XWER, "ERROR: Syscopy called.\n");
#endif

#if UNIX | IM | JM | PN | PX | PS
   int   fda, fdb, len;
   char  buf[1024];

   ENTRY ("syscopy", "");

/*---------------------------------------FORTUNE & MASSCOMP & A3 | LA---------*/
#if UNIX | PX | PS
   sysbrkignore();
   fda = open  (a,    0);
   fdb = creat (b, 0600);
   sysbrkinit();
#endif
 
/*---------------------------------------IBM PC--------------------------*/
#if IM
   fda = open (a, O_BINARY | O_RDONLY);
   fdb = open (b, O_BINARY | O_RDWR | O_CREAT, S_IREAD | S_IWRITE);
#endif

#if PN
   fda = sopen (a, O_BINARY | O_RDONLY,         SH_DENYNO);
   fdb = sopen (b, O_BINARY | O_RDWR | O_CREAT, SH_DENYNO, S_IREAD | S_IWRITE);
#endif

   if (fda < 0) { close (fdb);  RETURN (0); }
   if (fdb < 0) { close (fda);  RETURN (0); }

   for (len=1024;   (len = read (fda, buf, len)) > 0;)  write (fdb, buf, len);

   close (fda);
   close (fdb);
#endif

/*---------------------------------------VAX-----------------------------*/
#if VV
   5 = ;
/* Old, bad, code. 
   fda = open  (a, 0);
   delete (b);
   fdb = creat (b, 0700, "rfm=var", "rat=cr");

   while ( (len = read (fda, buf, 1024)) > 0)  write (fdb, buf, len);
*/
#endif

/*---------------------------------------PRIME---------------------------*/
#if PR
   ENTRY  ("syscopy", "");
   delete (   b);
   copy   (a, b);
#endif

   RETURN (1);
}
