/*** SYSDIRCHK.  Directory Check.  Makes sure that the user has 
/    the USERS and TEXTS directories needed for the participation
/    files and text-editing files, respectively. 
/
/    Returns 1 on success, 0 if failure (which is probably
/    catastrophic.) */

/*** Copyright (C) 1984 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:44 Source code master starting point */
/*: CR 11/02/88 17:16 Add SRV code. */
/*: CR 11/09/88 15:30 Fix SRV bugs. */
/*: CR  3/21/89 15:15 Add use of servername to #if SRV code. */
/*: CR  7/03/89 16:48 UNIX: replace open() with sysopen(). */
/*: CR  8/08/89 13:26 Add sysudnum() call for Unix. */
/*: CW  5/08/89 15:17 Add PX system code. */
/*: JV 10/04/89 17:59 Stripped out AOS non-server code. */
/*: JV 10/05/89 11:03 SRV now exits if vcsend() fails. */
/*: CR 10/24/89 13:23 Add 3rd diagnostic argument to sysudnum(). */
/*: CR  9/20/90 17:03 Add PS system type. */
/*: CJ 11/26/90 21:50 Add LA system type code. */
/*: DE  5/28/92 15:09 Chixified */
/*: CL 12/10/92 17:06 Long ascofchx args. */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"

extern Chix confid;

FUNCTION  sysdirchk()
{

/*----------------------------------FORTUNE & MASSCOMP & A3 | LA--------------*/
#if UNIX | PX | PS
   Chix  thisuser;
   char  thisuser_str[40], confid_str[200];
   char  dir[80];
   int   fd, udnum;

   ENTRY ("sysdirchk", "");

   thisuser  = chxalloc (40L, THIN, "sysdirch thisuser");
   sysuserid (thisuser);

   /*** Handle the USERS directory, which must remain accesible only
   /    to the caucus id. */
   udnum = sysudnum (thisuser, 1, 0);
   if (udnum <= 0)  RETURN (0);

   ascofchx  (thisuser_str, thisuser, 0L,  40L);
   ascofchx  (confid_str,   confid,   0L, 200L);

   /*** Handle the TEXTS directory, which everybody can conceivably 
   /    access. */
   sprintf (dir, "%s/TEXT%03d/%s", confid_str, udnum, thisuser_str);
   if ( (fd = sysopen (dir, 0, "sysdirch")) >= 0)  close  (fd);
   else                                        sysmkdir (dir, 0);

   chxfree ( thisuser );
#endif

/*---------------------------------------IBM PC--------------------------*/
#if IM | JM | PN
   Chix  thisuser;
   char  thisuser_str[40], confid_str[200];
   char  dir[80];

   ENTRY ("sysdirchk", "");

   thisuser = chxalloc (40L, THIN, "sysdirch thisuser");
   sysuserid (thisuser);
   ascofchx  (thisuser_str, thisuser, 0L,  40L);
   ascofchx  (confid_str,   confid,   0L, 200L);

   /*** Try to create the USER directory.  Fail => it was already there.*/
   sprintf  (dir, "%s\\USER\\%s", confid_str, thisuser_str);
   sysmkdir (dir, 0);

   chxfree ( thisuser );
#endif

/*---------------------------------------VAX-----------------------------*/
#if VV
   Chix  thisuser;
   char  thisuser_str[40], confid_str[200];
   char  dir[80];

   ENTRY ("sysdirchk", "");

   thisuser = chxalloc (40L, THIN, "sysdirch thisuser");
   sysuserid (thisuser);
   ascofchx  (thisuser_str, thisuser, 0L,  40L);
   ascofchx  (confid_str,   confid,   0L, 200L);

   /*** Handle the USER directory, which must remain accesible only
   /    to the caucus id. */
   sprintf  (dir, "%s.USER.%s", confid_str, thisuser_str);
   sysmkdir (dir, 1);

   chxfree ( thisuser );
#endif

/*---------------------------------------PRIME---------------------------*/
#if PR
   Chix   thisuser;
   extern char primepass[];
   char   thisuser_str[40], confid_str[200];
   char   dir[80];

   ENTRY  ("sysdirchk", "");
   
   thisuser  = chxalloc (40L, THIN, "sysdirch thisuser");
   sysuserid (thisuser);
   ascofchx  (thisuser_str, thisuser, 0L,  40L);
   ascofchx  (confid_str,   confid,   0L, 200L);

   /*** Handle the USERS directory, which must remain accessible only
   /    to the Caucus id. */
   sprintf (dir, "%s>USER %s>%s", confid_str, primepass, thisuser_str);
   if (access (dir, 0) < 0)   sysmkdir (dir, 1);

   chxfree ( thisuser );
#endif

/*-------------------------------Server-based systems-------------------*/
#if SRV
#include "actions.h"
#include "errchk.h"
   extern int   vcernum;
   extern char *vcernam;
   extern Chix  servername;
   Chix  thisuser;

   ENTRY ("sysdirchk", "");

   thisuser  = chxalloc (40L, THIN, "sysdirch thisuser");
   sysuserid (thisuser);

   if (NOT vcsend (servername, ADRCH, thisuser, 40, NULL, 0)) {
      ERRCHK("sysdirchk");
      unit_write (XITT, CQ("Caucus Server has terminated.\n"));
      unit_write (XITT, CQ("Caucus is now aborting.\n"));
      sysexit (0);
   }
   chxfree ( thisuser );
#endif

   RETURN (1);
}
