
/*** SYSEDITOR.  Invoke user-selected text editor on a file.
/
/    ok = syseditor (file, editstr);
/
/    Parameters:
/       int    ok;          (Success?)
/       int    file;        (unit number)
/       Chix   editstr;     (command line or macro expression to run editor)
/
/    Purpose:
/       SYSEDITOR invokes the editor selected in EDITSTR on the file
/       numbered FILE.
/
/    How it works:
/       If EDITSTR contains a macro expression, SYSEDITOR substitutes
/       the file name in for argument "$2".  Otherwise, it just tacks
/       the file name onto the end of EDITSTR.
/
/    Returns: 2 if the file was successfully edited.
/             1 if the editor exists but the file was not changed.
/             0 if the editor could not be accessed.
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  editfile()
/
/    Operating system dependencies:
/
/    Known bugs:      none
/
/    Home:  sys/sysedito.c
/ */

/*** Copyright (C) 1984 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:44 Source code master starting point */
/*: CR  8/04/88 14:37 VV must not put vers # on edited file name */
/*: CR  9/21/88 15:25 Add "HU" system type for HP/UX. */
/*: CR  9/23/88 13:00 Add "IX" system type. */
/*: CR 11/02/88 17:31 Add CX, CVM, CA types. */
/*: CR  6/09/89 12:16 Add "SV" system type code. */
/*: CR  6/28/89 12:56 Change Unix child proc status mask to 0xFD00. */
/*: CW  5/08/89 15:17 Add PX system code. */
/*: JV 10/04/89 18:03 Stripped out AOS non-server code. */
/*: CR 10/12/89 15:35 Add DU system type code. */
/*: CR  2/26/90 13:57 PX: use new vwin functions. */
/*: CR  3/07/90 16:56 PX, CVM: editor "window" means call vmiedit(). */
/*: CR  3/22/90 12:34 New return code: 2, 1, or 0. */
/*: CR  3/24/90 14:30 PX/CVM: before vmiedit(), clear XWTX. */
/*: CR  5/01/90 14:17 UNIX: change file permissions only if not writable. */
/*: CR  5/05/90  1:05 VV must handle file.ext.ver */
/*: CR  6/05/90 13:17 CVM, PX: Check flag.fullscreen. */
/*: CR  6/20/90 12:28 PX: use vwinreset() instead of vmistart/stop. */
/*: CR  6/28/90 15:19 Expand size of command[] string. */
/*: JV  7/06/90 20:41 Add MIPS RISC/os (RO). */
/*: JV  7/22/90 16:27 Add getuid decl to Sys V. */
/*: CR  8/09/90  1:21 IX: vi is in /usr/bin/vi, not /bin/vi. */
/*: CR  9/20/90 17:03 Add PS system type. */
/*: CR  9/26/90 11:40 Unix: Remove special treatment of 'vi' and 'ed'. */
/*: CR  9/27/90 12:19 Add U3, AU system types. */
/*: CJ 11/22/90 22:17 Save/restore terminal state around editor call. */
/*: CJ 11/26/90 21:50 Add LA system type code. */
/*: CR  1/28/91 20:07 Detect hangups during wait for child. */
/*: CR  2/01/91 16:39 Replace global hangup with syshngtest(). */
/*: SB 11/29/90 14:04 Add DY system code*/
/*: CR  3/11/91 16:20 Add AX & UL system type codes. */
/*: JV  1/06/92 17:26 Update return from syscmd(). */
/*: CR  1/06/92 19:18 Add DP system type. */
/*: JV  7/16/92 15:35 Add U4. */
/*: CR  7/21/92 17:39 Remove DP (in favor of DYP). */
/*: JV  9/25/92 13:42 Fix return values for non-Unix. */
/*: CR  8/21/92 16:28 Use 0xFE00 mask for MV. */
/*: DE  5/28/92 17:04 Chixified */
/*: CI 10/07/92 14:11 Chix Integration. */
/*: CL 12/10/92 15:22 Long chxindex args. */
/*: CL 12/10/92 17:06 Long ascofchx args. */
/*: CB  6/02/93 16:50 Major rewrite of VV code. */
/*: CV  6/02/93 17:18 Fix bugs in rewrite. */
/*: JV  3/18/93 12:38 Add WAIT arg to syscmd. */

#include <stdio.h>
#include "caucus.h"
#include "sys.h"
#include "systype.h"
#include "units.h"

extern struct unit_template units[];
extern struct termstate_t   term_in_caucus, term_original;
extern struct flag_template flag;
extern int                  debug;

FUNCTION  syseditor (file, editstr)
   int    file;
   Chix   editstr;
{
#define   CSIZE   300L
   char   command[CSIZE];

/*----------------------------------FORTUNE & MASSCOMP & A3--------------*/
#define SYSV1  A3 | LA | SV | HU | FU | MU | MV | PU | SX | TX | IX | CX
#define SYSV2  PX | DU | RO | PS | U3 | AU | DY | AX | U4
#if SYSV1 | SYSV2
   char  filename[80];
   int   status, child, writable, mask;
   unsigned short getuid();

   ENTRY ("syseditor", "");

   strcpy (filename, units[file].nameis);
   writable = (access (filename, 6) == 0);
   if (NOT writable)  chmod (filename, 0777);

#if PX | PS
   if (flag.fullscreen  &&  chxeq (editstr, CQ("window"))) {
      unit_write (XWTX, NEWCPAGE);
      RETURN (vmiedit (file, 0) + 1);
   }
#endif

   systurn (&term_in_caucus, GET_TERM, 0);
   systurn (&term_original,  SET_TERM, 0);
   child  = fork();
   if      (child == -1) {
      systurn (&term_in_caucus, SET_TERM, 0);
      RETURN (0);
   }

   /*** The parent task waits for the child to terminate. */
   else if (child >   0)
      while (wait (&status) != child)  { if (syshngtest()) break; }
 
   /*** The child task reverts to the Real uid, and exec's the
   /    selected editor. */
   else {
      setuid (getuid());
      if (chxindex (editstr, 0L, CQ("$2")) > 0) {
         ascofchx (command,  editstr, 0L, CSIZE);
         while  (stralter (command, "$2", filename)) ;
      }
      else
         sprintf (command, "%s %s", ascquick(editstr), filename);
      execl ("/bin/sh", "sh", "-c", command, NULL);
      _exit (1);
   }

   /*** If the child process exited with a status other than 0, we
   /    couldn't find the desired editor. */
   systurn (&term_in_caucus, SET_TERM, 0);
#if PX | PS
   if (flag.fullscreen)  vwinreset();
#endif

   if (NOT writable) chmod (filename, 0700);

   mask = 0xFD00;
#if MV
   mask = 0xFE00;
#endif
   RETURN ( (status & mask) ? 0 : 2);
#endif


/*---------------------Berkeley Unix----------------------------------*/
#if BU | SU | UL
   char  filename[80];
   int   status, child, writable;

   ENTRY ("syseditor", "");

   strcpy (filename, units[file].nameis);
   writable = (access (filename, 6) == 0);
   if (NOT writable)  chmod (filename, 0777);

   systurn (&term_in_caucus, GET_TERM, 0);
   systurn (&term_original,  SET_TERM, 0);
   child  = vfork();
   if      (child == -1)  RETURN (0)

   /*** The parent task waits for the child to terminate. */
   else if (child >   0)
      while (wait (&status) != child)  { if (syshngtest()) break; }
 
   /*** The child task reverts to the Real uid, and exec's the
   /    selected editor. */
   else {
      setreuid (getuid(), getuid());
      if (chxindex (editstr, 0L, CQ("$2")) > 0) {
         ascofchx (command, editstr, 0L, CSIZE);
         while  (stralter (command, "$2", filename)) ;
      }
      else
         sprintf (command, "%s %s", ascquick(editstr), filename);
      execl ("/bin/sh", "sh", "-c", command, NULL);
      _exit (1);
   }

   /*** If the child process exited with a status other than 0, we
   /    couldn't find the desired editor. */
   systurn (&term_in_caucus, SET_TERM, 0);
   if (NOT writable) chmod (filename, 0700);
   RETURN ( (status & 0xFF00) ? 0 : 2);
#endif

/*---------------------------------------IBM PC, VM/CMS------------------*/
#if IM | JM | PN | CVM | CA
   ENTRY ("syseditor", "");

#if CVM
   if (flag.fullscreen  &&  streq (editstr, "window"))
      RETURN (vmiedit (file, 0) + 1);
#endif

   if (chxindex (editstr, 0L, CQ("$2")) > 0) {
      ascofchx (command, editstr, 0L, CSIZE);
      while  (stralter (command, "$2", units[file].nameis)) ;
   }
   else
      sprintf (command, "%s %s", ascquick(editstr), units[file].nameis);

   RETURN ((syscmd (command, SYS_WAIT) > 0) ? 2 : 0);
#endif

/*------------------------------------PRIME-----------------------------*/
#if PR
   ENTRY  ("syseditor", "");

   if (chxindex (editstr, 0L, CQ("$2")) > 0) {
      ascofchx (command, editstr, 0L, CSIZE);
      while  (stralter (command, "$2", units[file].nameis)) ;
   }
   else     sprintf (command, "%s %s", ascquick(editstr), units[file].nameis);

   RETURN ((syscmd (command, SYS_WAIT) > 0) ? 2 : 0);
#endif

/*---------------------------------------VAX-----------------------------*/
#if VV
#include <rmsdef>
#include <ssdef>
#include <descrip>
   struct dsc$descriptor_s f_desc, r_desc;
   char   pattern[200], result[254];
   long   context;
   char  *f;
   char   oldvers[200], newvers[200], base[200];
   int    status, i, hi_ver, lo_ver, version, code;

   ENTRY ("syseditor", "");

   /*** Get the BASE name of the file, w/o version number. */
   strcpy   (base, units[file].nameis);
   stralter (base, ";1", "");
   stralter (base, ".1", "");

   /*** Expand/interpret the EDITSTR editing command. */
   if (chxindex (editstr, 0L, CQ("$2")) > 0) {
      ascofchx (command, editstr, 0L, CSIZE);
      while  (stralter (command, "$2", base)) ;
   }
   else     sprintf (command, "%s %s", ascquick(editstr), base);

   /*** Run the editor. */
   status = syscmd (command, SYS_WAIT);


   /*** Look for all the versions of this file (the editor may have
   /    created many).  HI_VER is the highest version number, LO_VER
   /    is the lowest version number. */
   strconc (pattern, base, ";*");
   hi_ver =    0;
   lo_ver = 9999;
   f_desc.dsc$w_length  = strlen (pattern);
   f_desc.dsc$a_pointer = pattern;
   f_desc.dsc$b_class   = DSC$K_CLASS_S;
   f_desc.dsc$b_dtype   = DSC$K_DTYPE_T;
   r_desc.dsc$w_length  = 254;
   r_desc.dsc$a_pointer = result;
   r_desc.dsc$b_class   = DSC$K_CLASS_S;
   r_desc.dsc$b_dtype   = DSC$K_DTYPE_T;
   context     = 0L;

   f = result;
   while (lib$find_file (&f_desc, &r_desc, &context, 0, 0, 0, 0)
             == RMS$_NORMAL) {
      if (*f != ';')   for (f=result;   *f != ';';   ++f);
      sscanf (f+1, "%d", &version);
      if (version < lo_ver)  lo_ver = version;
      if (version > hi_ver)  hi_ver = version;
   }
   lib$find_file_end (&context);

   if (lo_ver > hi_ver)  RETURN (0);

   /*** Delete all but the highest version number. */
   for (i=lo_ver;   i<hi_ver;   ++i) {
      sprintf (oldvers, "%s;%d", base, i);
      code = delete  (oldvers);
   }

   /*** Rename the file back to version 1. */
   sprintf   (oldvers, "%s;%d", base, hi_ver);
   sprintf   (newvers, "%s;1",  base);
   sysrename (oldvers, units[file].nameis);

   RETURN (status>0 ? 2 : 0);
#endif
}
