/*** SYSEXIT.  System-dependent exit to operating system.
/
/    SYSEXIT is called with one argument, the error STATUS.
/    Zero means "normal exit"; all other values have some meaning.
/
/   (If STATUS is -1, it has the special meaning "initialize".
/    This special call is made from MAIN when the program begins,
/    and initializes some special condition and then returns.)
/
/    SYSEXIT must undo any special conditions set up by Caucus when
/    it began exiting, such as Unix umasks, or VM/CMS IUCV connection
/    to the server machine. */

/*** Copyright (C) 1987 Camber-Roth..  All rights reserved. */

/*: AA  7/29/88 17:44 Source code master starting point */
/*: CR 10/07/88 13:52 Change IM to use cbreset(). */
/*: CR 11/02/88 17:36 Add SRV, remove old VM code. */
/*: CR 11/21/88 17:05 version 2 PN changes. */
/*: CR 12/08/88 11:26 Set umask(077) for VV. */
/*: CR 12/12/88 12:25 Add 3rd arg to #if SRV invocation of vcinit(). */
/*: JV  1/19/89 14:00 Fixed vcinit call to use 3rd param. */
/*: JV  2/02/89 14:11 Added SERVER param to vcinit() call. */
/*: JV  2/13/89 18:26 Added ioctl to CA.*/
/*: JV  2/14/89 16:41 Added vcflag to vcinit call. */
/*: JV  2/17/89 11:08 Added startlog to CA. */
/*: JV  2/21/89  9:49 Removed startlog(). */
/*: CR  3/14/89 16:29 CX & CVM must have vcflag arg for vcinit() call. */
/*: CR  3/14/89 16:29 CX & CVM need vcflag; ioctl for CA only! */
/*: CR  3/21/89 15:02 Add use of servername in #if SRV code. */
/*: JV  3/03/89 11:55 Changed CA to always exit with 0. */
/*: JV  3/29/89 16:50 Added debug param to vcinit() call. */
/*: JV  4/04/89 17:25 Added assignments to vcflag. */
/*: JV  4/12/89 20:48 Added debug declaration to SRV code. */
/*: JV  6/26/89 16:25 Added pathname param to vcconnect call. */
/*: JV  7/20/89 15:33 Made CVM a special case. */
/*: JV  8/04/89 12:12 Adjustments for optchk(). */
/*: CR  7/27/89 17:07 Changed VV include syntax. */
/*: CW  5/08/89 15:17 Add PX system code. */
/*: JV 10/04/89 18:04 Stripped out AOS non-server code. */
/*: JV 10/06/89 14:13 Only print debug mess after vcconn/init/sever if debug.*/
/*: JV  2/21/90 15:07 Call sysecho() and syscook() at beginning and end. */
/*: JV  2/26/90 08:55 Change AOS terminal set-up. */
/*: JV  3/06/90 10:38 Changed exit code for AOS. */
/*: JV  3/10/90 19:00 Added CEO stuff to AOS. */
/*: JV  4/04/90 18:00 More CEO integration. */
/*: JV  4/14/90 03:00 ...and still more... */
/*: JV  3/23/90 12:08 Added realuid and caucusuid to Unix. */
/*: CR  3/23/90 13:88 Removed realuid and caucusuid from sysexit. */
/*: JV  2/23/90 05:00 Changed sysecho() to turn_echo(). */
/*: CR  2/19/90 14:58 CVM, PX: use vmi functions in place of cwin. */
/*: CR  4/09/90 17:40 Replace turn_echo() with systurn(). */
/*: CR  5/18/90 16:06 Make UNIX umask(077). */
/*: CR  6/05/90 13:18 CVM, PX: Check flag.fullscreen. */
/*: CR  9/20/90 17:03 Add PS system type. */
/*: JV 11/13/90 18:25 CA: Make oldbuf and termbuf extern's (sysglobals) */
/*: JV  1/13/91 19:08 Made CEO code a conditional compile. */
/*: CJ 11/22/90 22:35 Use new form of systurn() call. */
/*: CR  4/23/91 11:22 Add ultrix set-gid fixes. */
/*: JV  1/17/91 16:53 Fix def of umaskval for SU, add umask's include file. */
/*: CX  5/09/92 19:42 Remove global mdfd, replace with sysmdstr() call. */
/*: DE  5/28/92 17:39 Chixified */
/*: JX  6/04/92 13:41 SU: umask: mode_t (SysV) -> int. */
/*: CR  9/05/92 12:28 Make MV like UL. */
/*: CL 12/11/92 15:52 Long sysmem arg. */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"

#if SU
#include <sys/stat.h>
#endif

#if CA
#include <it_lib.h>
#include <termio.h>
#endif

#if SRV
#include "vc.h"
#endif

extern struct flag_template flag;
extern struct termstate_t   term_original;
extern Chix   servername;
extern union  null_t null;

FUNCTION  sysexit (status)
   int    status;
{
   int    junk;

#if PR
   if (status >= 0) { 
      systurn (&term_original, SET_TERM, 0);
      sysmdstr ("", -1, nullchix, &junk);
      exit (status);
   }
#endif


#if IM | PN
   if (status >= 0) {
      systurn (&term_original, SET_TERM, 0);
      sysmdstr ("", -1, nullchix, &junk);
      cbreset ();
      exit (status);
   }
#endif


#if UNIX | PX | PS
   static int    umaskval = 99;

   if (status == -1) {
      umaskval = umask(077);
#if UL | MV
      umask (007);
#endif
#if PX | PS
      if (flag.fullscreen)  vmistart();
#endif
   }
   else {
#if PX | PS
      if (flag.fullscreen)  vmistop();
#endif
      if (umaskval != 99) umask (umaskval);
      sysmdstr ("", -1, nullchix, &junk);
      systurn (&term_original, SET_TERM, 0);
      exit    (status);
   }
#endif


#if VV
#include <ssdef>
   static int  ctrly_status = 99;
   static int  umaskval     = 99;
   Chix   newstr;

   newstr = chxalloc (160L, THIN, "sysexit newstr");

   if (status == -1) {
      ctrly_status = syscyset(  0);
      umaskval     = umask   (077);
      return;
   }

   if (ctrly_status != 99) syscyset (ctrly_status);
   if (umaskval     != 99) umask    (umaskval);
   if (status !=  0) {
      chxformat (newstr, CQ("Exit status: %d\n"), L(status), 0L, 
                 null.chx, null.chx);
      unit_write (XWTX, newstr);
   }
   sysmdstr ("", -1, nullchix, &junk);
   systurn (&term_original, SET_TERM, 0);

   chxfree ( newstr );

   exit (SS$_NORMAL);
#endif


#if SRV
   extern int   debug;
   extern int   vcernum;
   extern char *vcernam;
   extern Chix  confid;
   Chix         userid;
   struct vcflag_template vcflag;
#if CA
   extern short under_ceo;  /* In CEO? */        /* Terminal settings in */
   extern struct termio termbuf, oldbuf;    /* <-----  and out of Caucus */
   int ioctl(), ioerr;
   short fnctn=0, ceo_error=0, aos_error=0, prnt_flag=0, sum_flag=0,
         task=$NO_TASKING, user_cmds=$EAT_USER_CMDS;
   char  *sum_ptr = null.str, *sysmem();
   it_start_struct *it_ptr;
#endif

   userid  = chxalloc (40L, THIN, "sysexit userid");

   /*** If Client Caucus initialization, we (a) initialize VC communications,
   /   and (b) connect to the Caucus server machine. If an error occurs,
   /   exit immediately with a diagnostic. */
   if (status < 0) {
      sysuserid (userid);

#if CA
#if CEO
      under_ceo = IT2_UNDER_CEO ();
      if (under_ceo) {        /*  If we're a son of CEO, init this stuff. */
         it_ptr = (it_start_struct *) sysmem (L(sizeof (it_start_struct)));
         if (it_ptr == NULL) {
            printf ("\n%s\n(err=%d, name=%s)\n",
             "Cannot initialize Caucus<->CEO communications", vcernum, vcernam);
            exit (1); }
         IT2_START (&it_ptr);        
         sysputstr ("\022");    /* Enable scrolling (CEO turns it off). */
      }
#endif  /* CEO */
      ioerr = ioctl (0, TCGETA, &oldbuf);        /* Modify keyboard to  */
      ioerr = ioctl (0, TCGETA, &termbuf);       /* get every keystroke */
      termbuf.c_lflag = termbuf.c_lflag & (~ECHO);         /* Turn off echo */
      termbuf.c_lflag = termbuf.c_lflag & (~ICANON);    /* Turn off cooking */
      termbuf.c_lflag = termbuf.c_lflag | CBREAK;     /* Don't buffer input */
      termbuf.c_iflag = termbuf.c_iflag | ICRNL;                 /* CR-->NL */
      ioerr = ioctl (0, TCSETA, &termbuf);
      if (ioerr) printf ("ioctl SETA error:%d\n", ioerr);
      vcflag.loctimeout = flag.loctimeout;
      vcflag.remtimeout = flag.remtimeout;
      vcflag.method     = flag.vcmethod;
#endif

      chxfree ( userid );

      if (NOT vcinit (1, userid, confid, 0, &vcflag, debug)) {
         printf ("\n%s\n(err=%d, name=%s)\n",
               "Cannot initialize VC communications", vcernum, vcernam);
         exit (0);
      }

      /*** VM picks up server name from command line.  The rest
      /    always use "caucus" as the server name. */
#if CA | CX
      chxcpy (servername, CQ("caucus"));
#endif

      if (NOT vcconnect (confid, servername)) {
         printf ("%s\n(err=%d, name=%s)\n",
                 "Cannot connect to Caucus server.", vcernum, vcernam);
         exit (0);
      }
#if CVM
      if (flag.fullscreen)  vmistart();
#endif
   }

   /*** On exit from Caucus, sever the VC communication link with the
   /    server, and clean up. */
   else {
#if CVM
      if (flag.fullscreen)  vmistop();
#endif
      if (NOT vcsever (servername))
         printf ("%s\n(err=%d, name=%s)\n",
           "Error disconnecting from Caucus server.", vcernum, vcernam);
      vcdone();
#if CA
      ioerr = ioctl (0, TCSETA, &oldbuf); /* Put back keyboard */
      if (under_ceo) {
         IT2_TERMINATE (&fnctn, &ceo_error, &aos_error, 
                        &prnt_flag, &sum_flag, &sum_ptr); 
         sysputstr ("\023");      /* Disable scroll, cuz that's how CEO is. */
      }
      exit (0);    /* Otherwise it *always* prints a stupid msg to screen */
#endif
      exit (status);
   }
#endif

}
