/*** SYSFSIZE.   Return the size of a file, in bytes.
/
/    ok = sysfsize (fname, bytes, error);
/
/    Parameters:
/       int    ok;          (success?)
/       char  *fname;       (full pathname of the file)
/       long  *bytes;       (returned size of file)
/       int   *error;       (error code if failure)
/
/    Purpose:
/       Determine the size of a file.
/
/    How it works:
/
/    Returns: 1 on success, 0 on failure.  Sets ERROR on failure.
/
/    Error Conditions:  (see syserror.h.)
/       SE_NOTIMP     Function not implemented for this system.
/       SE_BADFIL     File FNAME does not exist.
/       SE_NOINFO     Cannot get information for unknown reason.
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  application.
/
/    Operating system dependencies: considerable!
/
/    Known bugs:      none
/
/    Home:  sys/sysfsize.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: CR  8/08/91 12:45 New function. */
/*: CR  6/23/91 13:59 Make RO system V function. */
/*: JV  7/16/92 15:38 Add U4, move include files. */
/*: JV  8/18/92 10:13 Add VMS code. */
/*: JV  8/27/92 13:13 Fix VMS code. */
/*: JV  9/04/92 15:11 Move DYP to SYSV. */
/*: CR  9/08/92 15:57 Add MV to SYSV.  */
/*: CR  3/28/93 15:48 Add HU to SYSV group. */
/*: CR  3/15/94 23:50 BU acts like SYSV. */

#include <stdio.h>
#include "handicap.h"
#include "systype.h"
#include "entryex.h"
#include "syserror.h"

#define SYSV A3 | SX | TX | SU | U3 | RO | U4 | DYP | MV | HU | BU
#if SYSV
#include <sys/types.h>
#include <sys/stat.h>
#endif

#if VV
#include <stat>
#endif

FUNCTION  sysfsize (fname, bytes, error)
   char  *fname;
   long  *bytes;
   int   *error;
{
/*--------------------------System V Unix--------------------------------*/
#if SYSV | VV
#if SYSV
   struct stat  statbuf;
#endif
#if VV
   stat_t statbuf;
#endif

   ENTRY  ("sysfsize", "");

   if (stat (fname, &statbuf))      { *error = SE_BADFIL;    RETURN (0); }
   if (statbuf.st_size < 0)         { *error = SE_NOINFO;    RETURN (0); }
   *error = 0;
   *bytes = statbuf.st_size;
   RETURN (1);
#endif

#define NONE1  AU | DU  | FU | MU | PU | IX | SV
#define NONE2  LA | DY3 | UL | IM | PN | PR | CX | CA | CVM
#if NONE1 | NONE2
   ENTRY  ("sysfsize", "");
   5 = ;
   *error = SE_NOTIMP;
   RETURN (0);
#endif

}
