/*** SYSGETNODE.  Get node name. 
/
/    Gets node name into NODENAME.  This is the name
/    **as known to other computers**.
/    
/    Nodenames are limited to 20 characters.
/    Returns 1 on success, 0 on failure.
/    */

/*** Copyright (C) 1990 Camber-Roth.  All rights reserved.  */

/*: JV  1/03/90 11:57 Function created. */
/*: CR  4/27/90 12:44 VV: add #include <syidef>. */
/*: CR  4/30/90 20:51 Use common RETURN (success). */
/*: CR  6/08/90 15:57 VV: rewrite to get translation of SYS$NODE. */
/*: JV  7/06/90 20:20 Add MIPS RISC/os (RO). */
/*: CR  9/20/90 17:03 Add PS system type. */
/*: CR  9/27/90 12:19 Add U3, AU system types. */
/*: CJ 11/22/90 15:56 SV uses nodename! */
/*: CJ 11/26/90 21:50 Add LA system type code. */
/*: SB 11/29/90 14:05 Add DY system code*/
/*: CR  3/11/91 16:20 Add AX & UL system type codes. */
/*: CR  4/02/91 23:55 Correct HU entry. */
/*: SB  5/16/91 10:38 Add DP system type. */
/*: JV  7/16/92 15:28 Add U4, move include files. */
/*: CR  7/21/92 17:39 Remove DP (in favor of DYP). */
/*: CI 10/07/92 14:36 Chix Integration. */
/*: JX 10/09/92 17:47 Chixify arg. */
/*: CL 12/11/92 11:18 Long Chxcatsub args. */
/*: CV  4/07/93 15:16 Remove extra '{' in VV code. */
/*: CP  2/08/94 15:56 Fix printf(...nodename.p). */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"

#define SYSV1 A3 | AU | AX | CX | DU | DY | HU | IX
#define SYSV2 LA | PU | PX | RO | SV | SX | U3 | U4
#if SYSV1 | SYSV2
#include <sys/utsname.h>
#endif

extern int debug;

FUNCTION  sysgetnode (nodename)
   Chix   nodename;
{
   int    success = 0;

/*------------------------------AOS--------------------------------------*/
#if CA
   ENTRY ("sysgetnode", "");
#endif

/*------------------------Unix-------------------------------------------*/
#if SYSV1 | SYSV2
   struct  utsname u;

   ENTRY   ("sysgetnode", "");
   uname   (&u);
   chxcpy  (nodename, CQ(u.nodename));
   if (debug) printf ("Node: %s\n", nodename->p);
   success = 1;
#endif

/*-----------------------BERKELEY Unix------------------------------*/
#if BU | SU | PS | UL
   char hostname[120];

   ENTRY ("sysgetnode", "");

   gethostname (hostname, 100);
   chxcatsub   (nodename, CQ(hostname), 0L, L(20));
   jixreduce   (nodename);
   success = 1;
#endif

/*---------------------------------------VAX-----------------------------*/
#if VV
#include <ssdef>
#include <descrip>
#include <iodef>
#include <syidef>
#include <lnmdef>
#define   ATTR    0
#define   ACMODE  0

   struct {
      short buffer_length;
      short item_code;
      long *buffer_ptr;
      long *return_ptr;
      long  zero;
   } itemlist;

   static Chix thisnode;
   static long code;
   struct dsc$descriptor_s tabname, logname;
   char   buffer[32];
   long   return_length;

   if (NOT chxtype (thisnode))
      thisnode = chxalloc (32L, THIN, "sysgetno thisnode");

   if (EMPTYCHX (thisnode)) {

      /*** TABNAME is the name of the equivalence table to be searched. */
      tabname.dsc$a_pointer  = "LNM$SYSTEM_TABLE";
      tabname.dsc$w_length   = 16;
      tabname.dsc$b_class    = DSC$K_CLASS_S;
      tabname.dsc$b_dtype    = DSC$K_DTYPE_T;
   
      /*** LOGNAME is the logical symbol to be translated.  ("sys$node") */
      logname.dsc$a_pointer  = "SYS$NODE";
      logname.dsc$w_length   = 8;
      logname.dsc$b_class    = DSC$K_CLASS_S;
      logname.dsc$b_dtype    = DSC$K_DTYPE_T;
   
      /*** ITEMLIST tells VMS where to put the result. */
      itemlist.buffer_length = 30;
      itemlist.item_code     = LNM$_STRING;
      itemlist.buffer_ptr    = &buffer;
      itemlist.return_ptr    = &return_length;
      itemlist.zero          = 0L;
      return_length          = 0L;
   
      code = sys$trnlnm (ATTR, &tabname, &logname, ACMODE, &itemlist);

   
      if (code != SS$_NORMAL  ||  return_length == 0) 
         chxcpy (thisnode, CQ("???"));
      else {
         buffer[return_length]='\0';
         chxcpy   (thisnode, CQ(buffer));
         chxalter (thisnode, 0L, chxquick("::",0), chxquick("",1));
      }
   }

   chxcpy    (nodename, thisnode);
   success = (code == SS$_NORMAL);
#endif


/*---------------------IBM PC----------------------------------------*/
#if IM | JM | PN
   ENTRY  ("sysgetnode", "");
#endif


/*---------------------PRIME-----------------------------------------*/
#if PR
   ENTRY  ("sysgetnode", "");
#endif


/*---------------------VM/CMS----------------------------------------*/
#if CVM
   ENTRY  ("sysgetnode", "");
#endif


/*---------------------Machines without unique system identifiers------*/
#if FU | MU | TX | MV
   ENTRY  ("sysgetnode", "");
#endif

   RETURN (success);
}
