
/*** SYSMKDIR.  Make a directory DIR. 
/
/    Create the directory specified by the full path name in DIR.
/    Make it accessible to the CAUCUS task if PROTECT is true;
/    otherwise, it should be accessible to everyone.
/
/    Returns 1 if successful, 0 on failure. */

/*** Copyright (C) 1984 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:45 Source code master starting point */
/*: CR  9/21/88 15:25 Add "HU" system type for HP/UX. */
/*: CR  9/23/88 13:00 Add "IX" system type. */
/*: JV  9/13/88 16:30 Changed AOS code to create dir with correct ACL.*/
/*: CR 10/06/88 15:44 Change IM to use QuickC; same as PR! */
/*: JV 11/22/88 14:13 Fixed AOS code to leave ACL's alone.*/
/*: JV 11/22/88 16:54 Added debug declaration.*/
/*: CR  1/06/89 16:12 CVM changes only. */
/*: CR  6/09/89 12:16 Add "SV" system type code. */
/*: CR  8/09/89 12:25 Sys V: mkdir 2>/dev/null, fix ownership & protect. */
/*: CW  5/08/89 15:17 Add PX system code. */
/*: CR  9/12/89 16:53 Declare bugtell(). */
/*: JV 10/04/89 18:15 Stripped out AOS non-server code. */
/*: CR 10/12/89 15:35 Add DU system type code. */
/*: CR  6/21/90 16:30 BU/SU: use chmod() to force permission mask. */
/*: JV  7/06/90 20:47 Add MIPS RISC/os (RO). */
/*: JV  7/22/90 16:28 Add getuid, geteuid, getgid decl's for Sys V. */
/*: CR  9/03/90 16:28 Move HU into BU/SU camp. */
/*: CR  9/27/90 12:19 Add U3, AU system types. */
/*: CJ 11/26/90 21:50 Add LA system type code. */
/*: CR  1/28/91 20:48 Detect hangup while wait()ing for child. */
/*: CR  2/01/91 16:43 Replace global hangup with syshngtest(). */
/*: SB 11/29/90 14:05 Add DY system code*/
/*: CR  3/11/91 16:43 Add AX & UL system type codes. */
/*: CR  4/08/91 18:49 Add new unit_lock argument. */
/*: CR  4/24/91 16:40 Fix Ultrix permissions. */
/*: UL 12/11/91 18:44 Make unit_lock args 3-6 long.*/
/*: CR  1/06/92 18:22 Add DP system type. */
/*: SB  5/16/91 10:41 Add DP system type. */
/*: CR 12/05/91 11:15 SysV: skip steps if not a set-uid program. */
/*: CR  1/22/92 15:50 SYSV: check actual ownership of new DIR. */
/*: DE  5/29/92 10:34 Chixified */
/*: JV  7/16/92 15:29 Add U4. */
/*: CR  7/21/92 17:39 Remove DP (in favor of DYP). */
/*: CR  9/05/92 12:22 Make MV like UL. */
/*: CI 10/07/92 14:47 Chix Integration. */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"

#define SYSV1  A3 | LA | SV | FU | MU | PU | SX | TX | IX | PX | DU
#define SYSV2  RO | U3 | AU | AX | U4 | DY3

#if SYSV1 | SYSV2
#include <sys/types.h>
#include <sys/stat.h>
#endif

extern int  debug;

FUNCTION  sysmkdir (dir, protect)
   char   dir[];
   int    protect;
{
   char  *bugtell();

#if BU | SU | HU | UL | DYP | MV
   int    success;
   ENTRY  ("sysmkdir", "");
#if BU | SU | HU | DYP
   success = (mkdir (dir, 0700) == 0);
#endif
#if UL | MV
   success = (mkdir (dir, 0770) == 0);
#endif
#if MV
   setuid (getuid());   /* Coherent mkdir bug! */
#endif
   if (NOT protect)  chmod (dir, 0777);
   RETURN (success);
#endif

#if CVM
   RETURN (1);
#endif

/*----------------------------------FORTUNE & MASSCOMP & A3--------------*/
#if SYSV1 | SYSV2
   extern  int   errno;
   struct  stat  father, dirstat;
   int     status, child, p, eff_id, act_id, group, code, is_setuid;
   char    parent[80];
   FILE   *freopen();
   unsigned short getuid(), geteuid(), getgid();

   ENTRY ("sysmkdir", "");

   /*** The Unix version is *complicated*.  It follows these steps:
   /    1. Figure out the name of the parent directory of DIR.
   /    2. Lock XSMK -- to avoid multiple simultaneous sysmkdir()s.
   /    3. Get the permission mask for the parent.
   /    4. Set the permission mask for the parent to 777.
   /    5. Fork a task that exec's 'mkdir' on DIR.
   /    6. Check to see if the directory was created.
   /    7. Change the ownership of the directory to the effective userid.
   /    8. If PROTECT, change permission to 0700.
   /    9. Set the permission mask on the parent to its original value.
   /   10. Unlock XSMK. 
   /   Note: steps 3, 4, 7, & 9 are skipped if we are NOT running
   /   a set-uid program. */

   strcpy (parent, dir);
   p = strlen (parent) - 1;
   while  (parent[p] != '/')  --p;
   parent[p] = '\0';
   
   /*** Are we running as set-uid? */
   eff_id = geteuid();
   act_id = getuid ();
   is_setuid = (eff_id != act_id);

   if (NOT unit_lock (XSMK, READ, 0L, 0L, 0L, 0L, nullchix))  RETURN (0);
   if (is_setuid) {
      stat   (parent, &father);
      chmod  (parent, 0777);
   }
   
   child  = fork();
   if      (child == -1) { unit_unlk (XSMK);   RETURN (0); }

   /*** The parent task waits for the child to terminate. */
   else if (child >   0) 
      while (wait (&status) != child)  { if (syshngtest()) break; }
 
   else {   /* Child process to make directory. */
      umask   (0);
      freopen ("/dev/null",  "w", stderr);  /* Warnings --> bit bucket. */
      execl   ("/bin/mkdir", "mkdir", dir, NULL);
   }

   /*** Did the mkdir succeed?  Test by trying to access the new directory. */
   if (access (dir, 0) < 0) {
      if (is_setuid) chmod (parent, father.st_mode & 0777);
      unit_unlk (XSMK);
      RETURN (0);
   }

   /*** Make the new directory owned by Caucus.  This involves changing
   /    our effective userid to the real userid, changing the directory
   /    ownership, and then changing the effective userid back again. */
   if (is_setuid) {
      group  = getgid ();
      setuid (act_id);
      code   = chown (dir, eff_id, group);

      if (setuid (eff_id) < 0) {
         bugtell ("SYSMKDIR: setuid failure", act_id, (long) eff_id, "");
         sysexit (1);
      }

      /*** See if the chown failed, and if DIR is not owned by the
      /    effective id.  (Some mkdir's make the directory owned by
      /    eff id--figure that one!) */
      if (code < 0) {
         stat (dir, &dirstat);
         if (dirstat.st_uid != eff_id)
            buglist (bugtell ("SYSMKDIR: chown failure",  errno, 0L, dir));
      }
   }
   if (protect)  chmod (dir, 0700);

   if (is_setuid) chmod (parent, father.st_mode & 0777);
   unit_unlk (XSMK);

   RETURN (1);
#endif


/*---------------------------------------VAX-----------------------------*/
#if  VV
#include <ssdef>
#include <descrip>

   struct dsc$descriptor_s desc;
   int    status;
   char   fulldir[80];

   ENTRY ("sysmkdir", "");
   strconc (fulldir, dir, "]");

   desc.dsc$w_length  = strlen (fulldir);
   desc.dsc$a_pointer = fulldir;
   desc.dsc$b_class   = DSC$K_CLASS_S;
   desc.dsc$b_dtype   = DSC$K_DTYPE_T;

   status = lib$create_dir (&desc);
   if (status==SS$_CREATED  ||  status==SS$_NORMAL)  RETURN (1);
   RETURN (0);
#endif

/*---------------------------------------PRIME---------------------------*/
#if PR | IM | PN
   extern int  debug, errno;
   int         status;

   ENTRY ("sysmkdir", "");

   status = mkdir (dir);
   if (status < 0  &&  debug) {
      bugtell ("SYSMKDIR failed:", errno, 0L, dir);
      perror ("");
   }
   RETURN (status == 0);
#endif
}
