
/*** SYSONEUSER.  Allow only one user per userid on Caucus at a time.
/
/    SYSONEUSER is called with the argument "on" when a user first gets
/    on a conference.  It attempts to lock the XUOU file.  If successful,
/    it returns 1.
/
/    If XUOU is not locked, SYSONEUSER returns immediately.
/    If XUOU is already locked, the action taken by SYSONEUSER depends
/    on the setting of the ONEUSER option.
/    If ONEUSER is ON, Caucus exits immediately.
/    If ONEUSER is OFF, the user is informed that someone else
/    is using that id.  SYSONEUSER prompts to see if the
/    user *really* wants to join anyway.  If the user responds YES,
/    SYSONEUSER returns 1, otherwise it returns 0.
/
/    SYSONEUSER is called with the argument "off" when a user is exiting
/    CAUCUS; this unlocks XUOU. */

/*** Copyright (C) 1984 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:45 Source code master starting point */
/*: CR 10/06/88 15:46 IM change: must enforce XUOU locking! */
/*: CR 11/03/88 15:51 Add SRV to UNIX list, remove VM code. */
/*: CR 11/22/88 17:43 New PN code. */
/*: CW  5/08/89 15:17 Add PX system code. */
/*: JV 10/04/89 18:16 Stripped out AOS non-server code. */
/*: CR 12/02/89 14:56 Change "#if ... SRV" to CA | CX. */
/*: JV  2/20/90 07:47 Add use of flag.oneuser. */
/*: CR  9/20/90 17:03 Add PS system type. */
/*: CR  4/08/91 18:49 Add new unit_lock argument. */
/*: UL 12/11/91 18:44 Make unit_lock args 3-6 long.*/
/*: JX  5/15/96 21:37 Chixify. */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"

extern Userreg    thisuser;
extern struct flag_template flag;
extern union  null_t     null;
extern Chix              confid;
extern Chix              yesnotable;

FUNCTION  sysoneuser (on_or_off)
   char   on_or_off[];
{
   int  success;
   Chix no, mtu, format;
   Chix  reply;

   ENTRY ("sysoneuser", "");

   no    = chxalloc (20L, THIN, "sysoneu no");  
   mtu   = chxalloc (200L, THIN, "sysoneu mtu");  
   format= chxalloc (80L, THIN, "sysoneu format");  

/*-----------------------------Most UNIXes-------------------------------*/
#if UNIX | PR | VV | IM | CX | CA | PN | PX | PS
   reply = chxalloc (20L, THIN, "sysoneu reply");
   success=1;

   if      (streq (on_or_off, "off"))      unit_unlk (XUOU);

   else if (unit_lock (XUOU, TLOCK, 0L, 0L, 0L, 0L, thisuser->id)) ;

   else {
      if (flag.oneuser) {
         /* Tell person why we're quitting.  */
         mdwrite (XWER, "sys_Etwoyou", null.md);
         sysexit (1);
      }
      else {
         chxtoken (no, null.chx, 1, yesnotable);
         mdstr    (format, "sys_Ttwoyou", null.md);
         chxformat  (mtu, CQ("\n%s %s"), 0L, 0L, thisuser->name, format);
         success = prompter (mtu, yesnotable, 2, "sys_Htwoyou", 10, reply, no)>0;
      }
   }
   chxfree (reply);
#endif

   chxfree (no);
   chxfree (mtu);
   chxfree (format);

   RETURN (success);
}
