
/*** SYSPCINIT.   (Re)-Initialize PC communications port. 
/
/    If this is a PC in remote mode, SYSPCINIT initializes the PC 
/    communications port, and returns 1.  Otherwise, it returns 0.
/
/    SYSPCINIT is called once at the beginning of Caucus, and then
/    again (from "loginshell") whenever the carrier-detect signal
/    drops (i.e. sysgetch returns -1, or prompter returns -4).  */

/*: AA  7/29/88 17:45 Source code master starting point */
/*: CR 09/27/88 14:55 Move manager_text() to after 1st sysbaud(). */
/*: CR 10/06/88 16:41 Remove IM "-b 999" option. */
/*: CR 10/14/88 15:28 IM changes only. */
/*: DE  5/29/92 11:49 Chixified */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"

extern struct flag_template flag;
extern union  null_t        null;

FUNCTION  syspcinit()
{

#if IM | JM | PN
   extern int  debug;
   static char firstcall = 1;
   Chix   newstr;
   Chix   dstr;
   int    success;
   short  c, baud, rate, state, cr;

   ENTRY ("syspcinit", "");

   /*** If we're talking to the console, no need to init the comm port. */
   if (NOT flag.baud)  RETURN (0);

   newstr   = chxalloc (160L, THIN, "syspcini newstr");
   dstr     = chxalloc (80L,  THIN, "syspcini dstr");
   success  = 0;

   /*** If we're talking remote, there are several possibilities, selected
   /    by the "-b" command line argument, stored in FLAG.BAUD.  In all
   /    cases, we want to see at least 2 CR's before proceeding, to
   /    eliminate confusion caused by noise during modem connect.
   /        300, 1200, 2400, 4800, 9600   use selected baud rate
   /        666   try 3 chars at a time, alternating speeds.  Recognizing 
   /              2 CR's locks in that speed.  */

   chxclear (dstr);
   state   = -1;
   rate    =  1;
   cr      =  0;

   while (1) {
      switch (flag.baud) {
         case (9600):
         case (4800):
         case (2400):
         case (1200):
         case ( 300):   baud =  flag.baud;                            break;

         case ( 666):   baud = (rate                 ? 1200 : 300);   break;
         default:       baud = 1200;                                  break;
      }
      sysbaud (baud, 3);

      /*** On first call, tell the console that we've gone remote, and
      /    send the "-a" text to the serial port.  */
      if (firstcall) {
         mdstr (newstr, "sys_Tremote", null.md);
         sysconsole (newstr);
         sysconsole (CQ("\015\012"));
         manager_text (flag.entrfile);
         firstcall = 0;
      }

      c = sysgetch ();
      if (c  <      0 )      { state = -1;   cr = 0; }
      if (c  == '\015')        ++cr;

      chxclear  (newstr);
      chxformat (newstr, CQ("PCINIT: flag=%4d  state=%2d  "), flag.baud, 
                 state, null.chx, null.chx);
      chxformat (newstr, CQ("baud=%4d  c=%3o  "), baud, c & 0377, 
                 null.chx, null.chx);
      chxformat (newstr, CQ("cr=%d\r\n"), cr, 0L, null.chx, null.chx);
      if (debug  &&  (c>0 || NOT chxeq (newstr, dstr)))  sysconsole (newstr);
      chxcpy (dstr, newstr);

      if (cr >= 2)  SUCCEED;

      /*** Keep count of the number of actual characters we've seen.  If
      /    we get to three (in 666 mode), change to the next speed and
      /    try again.  */
      switch (state) {
         case (-1):   if (c > 0 )    state = 1;                          break;

         case ( 0):
         case ( 1):   if (c > 0 )    state++;                            break;

         case ( 2):   if (c > 0 )  { state = cr = 0;   rate = 1-rate; }  break;
      }
   }

done:

   chxfree ( newstr );
   chxfree ( dstr );

   RETURN ( success );

#else
   ENTRY  ("syspcinit", "");
   RETURN (0);
#endif
}

