
/*** SYSPDIR.  Get part file directory listing.
/
/    ok = syspdir (pdir, u, confnum, item, owner)
/
/    Parameters:
/       int    ok;             (success?)
/       struct pdir_t *pdir;   (resulting partfile directory list)
/       int    u;              (desired unit code number)
/       int    confnum;        (conference number)
/       int    item;           (item number, if relevant)
/       Chix   owner;          (owning userid, if relevant)
/
/    Purpose:
/       Get a list of the existing partfiles for a specific unit code.
/
/    How it works:
/       Examines the directory specified by unit code U, conference
/       CONFNUM, item ITEM, userid OWNER.  Builds a sorted list of the 
/       matching partfiles in PDIR.  Each entry in the list has the
/       response number (or equivalent) and line number.
/       
/    Returns: 1 on success
/             0 on error
/
/    Error Conditions: can't open directory to read
/  
/    Side effects: none
/
/    Related functions:
/
/    Called by:  getline(), getmline(), etc.
/
/    Operating system dependencies: heavy
/
/    Known bugs:
/       The server/client code is broken and out-of-date.
/       The PR (primos) code does not have the XARF stuff.
/
/    Home:  sys/syspdir.c
/ */

/*** Copyright (C) 1987 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:45 Source code master starting point */
/*: CR  8/23/88 14:03 New Vax/VMS code */
/*: CR  8/31/88 22:25 PR change only: "R%03d" -> "r%03d" */
/*: CR  9/21/88 15:25 Add "HU" system type for HP/UX. */
/*: CR  9/23/88 13:00 Add "IX" system type. */
/*: JV  9/23/88 14:07 Changed AOS code to avoid stat() call.*/
/*: CR 10/18/88 15:36 Add SRV client code. */
/*: CR 11/09/88 15:30 Fix SRV bugs. */
/*: CR 11/12/88 23:10 Fix more SRV bugs. */
/*: CR 11/17/88 14:48 Add PN to IM code. */
/*: JV 11/22/88 16:56 Squished bugs in AOS code.*/
/*: JV  2/11/89 17:22 Fixed vcsend call. */
/*: CR  3/21/89 15:15 Add use of servername to #if SRV code. */
/*: CR  6/09/89 12:16 Add "SV" system type code. */
/*: CR  8/08/89 13:31 UNIX: Call sysudnum() to get userid directory number. */
/*: CW  5/08/89 15:17 Add PX system code. */
/*: JV 10/04/89 18:19 Stripped out AOS non-server code. */
/*: JV 10/05/89 11:04 SRV now exits if vcsend() fails. */
/*: CR 10/12/89 15:35 Add DU system type code. */
/*: CR 10/24/89 13:23 Add 3rd diagnostic argument to sysudnum(). */
/*: JV  7/06/90 20:37 Add MIPS RISC/os (RO). */
/*: CR  9/20/90 17:03 Add PS system type. */
/*: CR  9/27/90 12:19 Add U3, AU system types. */
/*: CJ 11/26/90 21:50 Add LA system type code. */
/*: SB 11/29/90 14:05 Add DY system code*/
/*: CR  3/11/91 16:43 Add AX & UL system type codes. */
/*: CR  7/27/91 13:23 Make Lint happier. */
/*: CR  7/28/91 14:26 Make SU same as RO. */
/*: JV  8/13/91 16:30 Move UNIX includes around to compile. */
/*: CR  8/06/91 23:25 Add XPAK matching for Unix. */
/*: CR  8/07/91 14:20 Add explicit confnum argument. */
/*: CR  8/14/91 18:42 Add XRXC to Unix. */
/*: JV  8/25/91 10:39 Fix name of XPAK files. */
/*: JV 12/16/91 16:28 Allow base 36 for XPAK, XRXC. */
/*: CR 12/26/91 12:00 Declare str36toi()! */
/*: CR  1/06/92 18:20 Add DP system type. */
/*: CR  5/05/92 12:14 DY3 and DYP use different code. */
/*: JV  7/17/92 09:57 Add U4, move include files out of function. */
/*: CR  7/21/92 17:39 Remove DP (in favor of DYP). */
/*: JV  8/16/92 18:39 Add CaucusLink units to VMS. */
/*: JV  8/27/92 13:17 Remove extra <sys/stat.h>, etc. */
/*: DE  5/29/92 12:38 Chixified */
/*: JX  6/18/92 11:33 Handle owner==nullchix. */
/*: CI 10/07/92 15:36 Chix Integration. */
/*: JV 10/13/92 16:55 Increase MAXITEMS (&resp) to 10K. */
/*: JV 10/21/92 12:00 Cast 2nd arg to strito36(). */
/*: CL 12/10/92 17:06 Long ascofchx args. */
/*: CR  5/20/93 16:38 Add code for Vax, DOS, Prime (!!!) */
/*: CV  5/20/93 17:14 Declare VV nresp_str. */
/*: CV  6/01/93 17:54 More Vax/VMS fixes. */
/*: JV  5/03/93 13:20 Add XARF for Unix, VMS, DOS. */
/*: CR  6/08/93 16:17 Integrate, fix XARF for IM, VV. */
/*: CP  1/19/94 14:08 MV 4.2 now uses DIRENT. */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"
#include "units.h"

extern Chix    confid;
extern int     debug;

/*** Old style unixes, i.e. prior to new "dirent.h" */
#define  OLD2   BU | HU | DU | PS  | UL
#define  OLD3a  A3 | AU | AX | DY3 | FU | IX | LA | MU | PU | PX
#define  OLD3b  SV | SX | TX | U3

#define  DIRENT RO | SU | DYP | U4 | MV

#if OLD2 | OLD3a | OLD3b
#include <sys/types.h>
#include <sys/dir.h>
#include <sys/stat.h>
#endif

#if DIRENT
#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>
#endif

FUNCTION  syspdir (pdir, u, confnum, item, owner)
   struct pdir_t *pdir;
   int    u, confnum, item;
   Chix   owner;
{
   long   str36toi();
   char  *strito36();

/*------------------------------Most UNIXes------------------------------*/
#if UNIX
/*** Old style unixes, i.e. prior to new "dirent.h" */
#define  DIRECTORY  0040000

   char   match[20], searchdir[200], std[12], *file;
   char   confid_str[200], owner_str[80], nresp_str[8];
   int    nresp, nlnum, i, j, udnum;
   short  mlen, dup;
   struct stat  statbuf;

#if OLD3a | OLD3b
   struct direct entry;
   FILE  *fp, *fopen();
#endif
#if OLD2
   DIR           *dir, *opendir();
   struct direct *dp,  *readdir();
#endif
#if DIRENT
   DIR           *dir, *opendir();
   struct dirent *dp,  *readdir();
#endif

   ENTRY ("syspdir", "");

   pdir->type = u;

   /*** Defaults are for USER files. */
   mlen = 1;
   udnum = sysudnum (owner, 0, u + 1000);

   ascofchx (confid_str, confid, 0L, 200L);
   if (owner != nullchix) {
      ascofchx (owner_str,  owner,  0L, 80L);
      sprintf  (searchdir, "%s/USER%03d/%s", confid_str, udnum, owner_str);
   }

   switch (u) {
      case (XUSF):   strcpy  (match, "s");                        break;
      case (XUMF):   strcpy  (match, "m");                        break;
      case (XUPA):   strcpy  (match, "p");                        break;

      case (XCSF):   sprintf (searchdir, "%s/C%03d", confid_str, confnum);
                     strcpy  (match, "s");                        break;

      case (XCRF):   sprintf (searchdir, "%s/C%03d", confid_str, confnum);
                     if (item > 999) strito36 (match, 
                                               L(item + TEN_K_OFFSET), 3);
                     else            sprintf  (match, "%03d", item);
                     mlen = 3;   break;

      case (XARF):   sprintf (searchdir, "%s/ATTACH/C%03d", confid_str, confnum);
                     if (item > 999) strito36 (match, 
                                               L(item + TEN_K_OFFSET), 3);
                     else            sprintf  (match, "%03d", item);
                     mlen = 3;   break;

      case (XPAK):   sprintf (searchdir, "%s/SHIP%03d", confid_str, confnum);
                     strcpy  (match, "a");                        break;

      case (XRXC):   sprintf (searchdir, "%s/RECV%03d/CHUNK", confid_str,
                              confnum);
                     strcpy  (match, "x");                        break;
   }

   /*** Make sure the directory really exists. */
   pdir->parts = 0;
   if (stat (searchdir, &statbuf) != 0  ||
       (statbuf.st_mode & 0070000) != DIRECTORY)  RETURN (1);

   /*** Examine each entry in the directory.  If it matches MATCH,
   /    add it to the sorted list of partfiles. */

#if OLD3a | OLD3b
   if ( (fp = fopen (searchdir, "r")) == NULL)  RETURN (1);

   while (fread (&entry, sizeof(entry), 1, fp)) {
      if (entry.d_ino == 0)  continue;
      entry.d_name[13] = '\0';
      file = entry.d_name;
#endif

#if DIRENT | OLD2
   if ( (dir = opendir (searchdir)) == (DIR *) NULL)  RETURN (1);

#if DIRENT   /* RO SU DP DYP U4 */
   while ( (dp = readdir (dir)) != (struct dirent *) NULL) {
#endif
#if OLD2   /* BU HU DU PS UL */
   while ( (dp = readdir (dir)) != (struct direct *) NULL) {
#endif
      file = dp->d_name;
#endif

      if (match[0] != file[0])                     continue;

      if (mlen > 1  &&  (match[1] != file[1]  ||
                         match[2] != file[2]))     continue;

      switch (u) {
      case XPAK:
         sscanf (file+mlen, "%3d", &nresp);
         strsub (std, file, mlen+3, 4);
         nlnum = str36toi (std);
         break;
      case XRXC:
         strsub (std, file, mlen, 7);
         nlnum = str36toi (std);
         break;
      case XCRF:
         if (*(file+mlen) >= '0' && *(file+mlen) <= '9')
            sscanf (file+mlen, "%3d%4d", &nresp, &nlnum);
         else {                                  /* Respnum is > 999. */
            sscanf (file+mlen, "%3s%4d", nresp_str, &nlnum);
            nresp = str36toi (nresp_str);
            nresp -= TEN_K_OFFSET;
         }
         break;
      case XARF:
         if (*(file+mlen) >= '0' && *(file+mlen) <= '9')
            sscanf (file+mlen, "%3d", &nresp);
         else {                                  /* Respnum is > 999. */
            sscanf (file+mlen, "%3s", nresp_str);
            nresp = str36toi (nresp_str);
            nresp -= TEN_K_OFFSET;
         }
         break;
      default:
         sscanf (file+mlen, "%3d%4d", &nresp, &nlnum);
         break;
      }
            

      if (u == XPAK  &&  nresp != item)            continue;

      /*** Find where in PDIR this entry belongs.  Remove duplicates. */
      for (dup=0, i=pdir->parts-1;    i>=0;    --i) {
         if (       nresp <  pdir->resp[i])   continue;
         if (       nresp >  pdir->resp[i])   break;
         if (       nlnum >  pdir->lnum[i])   break;
         if (dup = (nlnum == pdir->lnum[i]))  break;
      }
      if (dup)  continue;

      /*** Slip this entry into PDIR. */
      for (j=pdir->parts-1;   j>i;    --j)  {
         pdir->resp[j+1] = pdir->resp[j];
         pdir->lnum[j+1] = pdir->lnum[j];
      }

      pdir->resp[i+1] = nresp;
      pdir->lnum[i+1] = nlnum;
      ++pdir->parts;
      if (pdir->parts >= PDIRMAX-3)  break;
   }

#if OLD3a | OLD3b
   fclose (fp);
#endif
#if DIRENT | OLD2
   closedir (dir);
#endif

#endif


/*------------------------------IBM PC----------------------------------*/
#if IM | PN
#include "sysdir.h"

   struct node_template *head, *node, *sysdir();
   char   match[20], searchdir[200], pattern[80], file[30];
   int    nresp, nlnum;
   short  i, j, mlen, dup;
   char   confid_str[200], owner_str[80];

   ENTRY ("syspdir", "");

   pdir->type = u;

   /*** Defaults are for USER files. */
   mlen = 1;
   ascofchx (confid_str, confid, 0L, 200L);
   if (owner != nullchix) {
      ascofchx (owner_str,  owner,  0L,  80L);
      sprintf  (searchdir, "%s\\USER\\%s", confid_str, owner_str);
   }

   switch (u) {
      case (XUSF):   strcpy  (match, "s");                        break;
      case (XUMF):   strcpy  (match, "m");                        break;
      case (XUPA):   strcpy  (match, "p");                        break;

      case (XCSF):   sprintf (searchdir, "%s\\C%03d", confid_str, confnum);
                     strcpy  (match, "s");                        break;

      case (XCRF):   sprintf (searchdir, "%s\\C%03d", confid_str, confnum);
                     if (item > 999) strito36 (match, 
                                               L(item + TEN_K_OFFSET), 3);
                     else            sprintf  (match, "%03d", item);
                     mlen = 3;                                    break;

      case (XPAK):   sprintf (searchdir, "%s\\SHIP%03d", confid_str, confnum);
                     strcpy  (match, "a");                        break;

      case (XRXC):   sprintf (searchdir, "%s\\RECV%03d\\CHUNK", confid_str,
                              confnum);
                     strcpy  (match, "x");                        break;

      case (XARF):   sprintf (searchdir, "%s\\ATTACH\\C%03d",
                                         confid_str, confnum);
                     if (item > 999) strito36 (match, 
                                               L(item + TEN_K_OFFSET), 3);
                     else            sprintf  (match, "%03d", item);
                     mlen = 3;   break;
   }
   sprintf (pattern, "%s\\%s*.*", searchdir, match);

   /*** Get the list of file names that match the PATTERN. */
   head = sysdir (pattern);

   /*** Examine each name in the list.  Pluck out the numeric parts
   /    and add them to the sorted PDIR. */
   pdir->parts = 0;
   for (node=head;   node != NULL;   node = node->next) {
      strcpy   (file, node->name);
      stralter (file, ".", "");
      nresp = nlnum = -1;

      switch (u) {
         case XPAK:
            sscanf (file+mlen, "%3d", &nresp);
            strsub (std, file, mlen+3, 4);
            nlnum = str36toi (std);
            break;
         case XRXC:
            strsub (std, file, mlen, 7);
            nlnum = str36toi (std);
            break;
         case XCRF:
            if (*(file+mlen) >= '0' && *(file+mlen) <= '9')
               sscanf (file+mlen, "%3d%4d", &nresp, &nlnum);
            else {                                  /* Respnum is > 999. */
               sscanf (file+mlen, "%3s%4d", nresp_str, &nlnum);
               nresp = str36toi (nresp_str) - TEN_K_OFFSET;
            }
            break;

         case XARF:
            if (*(file+mlen) >= '0' && *(file+mlen) <= '9')
               sscanf (file+mlen, "%3d", &nresp);
            else {                                  /* Respnum is > 999. */
               sscanf (file+mlen, "%3s", nresp_str);
               nresp = str36toi (nresp_str) - TEN_K_OFFSET;
            }
            break;

         default:
            sscanf (file+mlen, "%3d%4d", &nresp, &nlnum);
            break;
      }

      if (nresp < 0  ||  nlnum < 0)      continue;
      if (u == XPAK  &&  nresp != item)  continue;

      /*** Find where in PDIR this entry belongs.  Remove duplicates. */
      for (dup=0, i=pdir->parts-1;    i>=0;    --i) {
         if (       nresp <  pdir->resp[i])   continue;
         if (       nresp >  pdir->resp[i])   break;
         if (       nlnum >  pdir->lnum[i])   break;
         if (dup = (nlnum == pdir->lnum[i]))  break;
      }
      if (dup)  continue;

      for (j=pdir->parts-1;   j>i;    --j)  {
         pdir->resp[j+1] = pdir->resp[j];
         pdir->lnum[j+1] = pdir->lnum[j];
      }

      pdir->resp[i+1] = nresp;
      pdir->lnum[i+1] = nlnum;
      ++pdir->parts;
      if (pdir->parts >= PDIRMAX-3) break;
   }

   sysdirfree (head);
#endif


/*------------------------------Vax/VMS---------------------------------*/
#if VV
#include <rmsdef>
#include <ssdef>
#include <descrip>
   struct dsc$descriptor_s f_desc, r_desc;
   char   searchdir[200], match[20], pattern[80], result[254], std[12];
   char   confid_str[200], owner_str[80], nresp_str[20];
   long   context;
   int    nresp, nlnum, len;
   short  i, j, mlen, dup;
   char  *f, *ptr;

   ENTRY ("syspdir", "");

   mlen = 1;
   ascofchx (confid_str, confid, 0L, 200L);
   if (owner != nullchix) {
      ascofchx (owner_str,  owner,  0L,  80L);
      sprintf  (searchdir, "%s.USER.%s", confid_str, owner_str);
   }

   switch (u) {
      case (XUSF):   strcpy  (match, "s");                        break;
      case (XUMF):   strcpy  (match, "m");                        break;
      case (XUPA):   strcpy  (match, "p");                        break;

      case (XCSF):   sprintf (searchdir, "%s.C%03d", confid_str, confnum);
                     strcpy  (match, "s");                        break;

      case (XCRF):   sprintf (searchdir, "%s.C%03d", confid_str, confnum);
                     if (item > 999) strito36 (match, 
                                               L(item + TEN_K_OFFSET), 3);
                     else            sprintf  (match, "%03d", item);
                     mlen = 3;                                    break;
 
      case (XPAK):   sprintf (searchdir, "%s.ship%03d", confid_str, confnum);
                     strcpy  (match, "a");                        break;

      case (XRXC):   sprintf (searchdir, "%s.recv%03d.chunk", confid_str,
                              confnum);
                     strcpy  (match, "x");                        break;
      case (XARF):   sprintf (searchdir, "%s.ATTACH.C%03d", 
                                  confid_str, confnum);
                     if (item > 999) strito36 (match, 
                                               L(item + TEN_K_OFFSET), 3);
                     else            sprintf  (match, "%03d", item);
                     mlen = 3;   break;
   }
   sprintf (pattern, "%s]%s*.*", searchdir, match);
   if (debug) printf ("syspdir: pattern='%s'\n", pattern);

   /*** Initialize directory search. */
   f_desc.dsc$w_length  = strlen (pattern);
   f_desc.dsc$a_pointer = pattern;
   f_desc.dsc$b_class   = DSC$K_CLASS_S;
   f_desc.dsc$b_dtype   = DSC$K_DTYPE_T;
   r_desc.dsc$w_length  = 254;
   r_desc.dsc$a_pointer = result;
   r_desc.dsc$b_class   = DSC$K_CLASS_S;
   r_desc.dsc$b_dtype   = DSC$K_DTYPE_T;
   context     = 0L;
   pdir->parts = 0;
   pdir->type  = u;

   /*** Examine each matching name.  Pluck out the numeric parts
   /    and add them to the sorted PDIR. */
   f = result;
   while (lib$find_file (&f_desc, &r_desc, &context, 0, 0, 0, 0)
             == RMS$_NORMAL) {
      if (*f != ']')   for (f=result;   *f != ']';   ++f);
      nresp = nlnum = -1;

      if (debug) { 
         char *p;
         for (p=f;   *p && *p!=' ';   ++p) ;
        *p = '\0';
         printf ("\nsyspdir: '%s'", f);
      }

      switch (u) {
         case (XPAK):
            sscanf (f+1+mlen, "%3d", &nresp);
            strsub (std, f+1, mlen+3, 4);
            nlnum = str36toi (std);
            break;

         case (XRXC):
            strsub (std, f+1, mlen, 7);
            nlnum = str36toi (std);
            break;

         case (XCRF):
            if (*(f+1+mlen) >= '0' && *(f+1+mlen) <= '9')
               sscanf (f+1+mlen, "%3d%4d", &nresp, &nlnum);
            else {
               /*** Make sure this is really an XCRF file, syntax
               /    iiirrrllll, and not something like "alsub001"! */
               for (ptr=f+1+mlen, len=0;   *ptr && *ptr!='.';   ++ptr, ++len) ;
               if  (len < 7)  continue;

               /*** XCRF file, Resp # > 999. */
               sscanf (f+1+mlen, "%3s%4d", nresp_str, &nlnum);
               nresp = str36toi (nresp_str) - TEN_K_OFFSET;
            }
            break;

         case XARF:
            if (*(f+1+mlen) >= '0' && *(f+1+mlen) <= '9')
               sscanf (f+1+mlen, "%3dd", &nresp);
            else {                                  /* Respnum is > 999. */
               sscanf (f+1+mlen, "%3s", nresp_str);
               nresp = str36toi (nresp_str) - TEN_K_OFFSET;
            }
            break;

         default:
            sscanf (f+1+mlen, "%3d%4d", &nresp, &nlnum);
            break;
      }

      if (nresp < 0  ||  nlnum < 0)      continue;
      if (u == XPAK  &&  nresp != item)  continue;

      /*** Find where in PDIR this entry belongs.  Remove duplicates. */
      for (dup=0, i=pdir->parts-1;    i>=0;    --i) {
         if (       nresp <  pdir->resp[i])   continue;
         if (       nresp >  pdir->resp[i])   break;
         if (       nlnum >  pdir->lnum[i])   break;
         if (dup = (nlnum == pdir->lnum[i]))  break;
      }
      if (dup)  continue;

      for (j=pdir->parts-1;   j>i;    --j)  {
         pdir->resp[j+1] = pdir->resp[j];
         pdir->lnum[j+1] = pdir->lnum[j];
      }

      pdir->resp[i+1] = nresp;
      pdir->lnum[i+1] = nlnum;
      ++pdir->parts;
      if (pdir->parts >= PDIRMAX-3)  break;
   }

   lib$find_file_end (&context);
#endif

/*------------------------------PRIME------------------------------------*/
#if PR
   extern char primepass[];
   char   match[20], searchdir[200], file[80];
   char   confid_str[200], owner_str[80]
   char  *ptr, *lsdir();
   int    nresp, nlnum, i, j, fd;
   short  mlen, dup;

   ENTRY ("syspdir", "");

   pdir->type = u;

   /*** Defaults are for USER files. */
   mlen = 1;

   ascofchx (confid_str,    confid,  0L, 200L);
   if (owner != nullchix) {
      ascofchx (owner_str,  owner,   0L,  80L);
      sprintf  (searchdir, "%s>USER %s>%s", confid_str, primepass, owner_str);
   }

   switch (u) {
      case (XUSF):   strcpy  (match, "s");                        break;
      case (XUMF):   strcpy  (match, "m");                        break;
      case (XUPA):   strcpy  (match, "p");                        break;

      case (XCSF):
         sprintf (searchdir, "%s>C %s>C%03d", confid_str, primepass,
                  confnum);
         strcpy  (match, "s");                                    break;

      case (XCRF):
         sprintf (searchdir, "%s>C %s>C%03d", confid_str, primepass, confnum);
         if (item > 999) strito36 (match+1, L(item + TEN_K_OFFSET), 3);
         else            sprintf  (match+1, "%03d", item);
         match[0] = 'r';
         mlen = 4;   break;

      case (XPAK):   sprintf (searchdir, "%s>SHIP%03d %s", confid_str, 
                                         confnum, primepass);
                     strcpy  (match, "a");                        break;

      case (XRXC):   sprintf (searchdir, "%s>RECV%03d %s>CHUNK", confid_str,
                              confnum, primepass);
                     strcpy  (match, "x");                        break;
   }

   /*** Open the directory and prepare to search it. */
   pdir->parts = 0;
   if ( (fd = open (searchdir, 0)) < 0)  RETURN (0);

   /*** Examine each entry in the directory.  If it matches MATCH,
   /    add it to the sorted list of partfiles. */
   while ( (ptr = lsdir (fd)) != NULL) {
      strcpy   (file, ptr);
      strlower (file);

      if (match[0] != file[0])                     continue;

      if (mlen > 1  &&  (match[1] != file[1]  ||  match[2] != file[2]  ||
                         match[3] != file[3]))     continue;

      switch (u) {
         case XPAK:
            sscanf (file+mlen, "%3d", &nresp);
            strsub (std, file, mlen+3, 4);
            nlnum = str36toi (std);
            break;
         case XRXC:
            strsub (std, file, mlen, 7);
            nlnum = str36toi (std);
            break;
         case XCRF:
            if (*(file+mlen) >= '0' && *(file+mlen) <= '9')
               sscanf (file+mlen, "%3d%4d", &nresp, &nlnum);
            else {                                  /* Respnum is > 999. */
               sscanf (file+mlen, "%3s%4d", nresp_str, &nlnum);
               nresp = str36toi (nresp_str) - TEN_K_OFFSET;
            }
            break;
         default:
            sscanf (file+mlen, "%3d%4d", &nresp, &nlnum);
            break;
      }

      /*** Find where in PDIR this entry belongs.  Remove duplicates. */
      for (dup=0, i=pdir->parts-1;    i>=0;    --i) {
         if (       nresp <  pdir->resp[i])   continue;
         if (       nresp >  pdir->resp[i])   break;
         if (       nlnum >  pdir->lnum[i])   break;
         if (dup = (nlnum == pdir->lnum[i]))  break;
      }
      if (dup)  continue;

      /*** Slip this entry into PDIR. */
      for (j=pdir->parts-1;   j>i;    --j)  {
         pdir->resp[j+1] = pdir->resp[j];
         pdir->lnum[j+1] = pdir->lnum[j];
      }

      pdir->resp[i+1] = nresp;
      pdir->lnum[i+1] = nlnum;
      ++pdir->parts;
      if (pdir->parts >= PDIRMAX-3)  break;
   }

   close (fd);
#endif


/*------------------------------Server/client code-----------------------*/
#if SRV
#include "actions.h"
#include "errchk.h"
   extern int   vcernum;
   extern char *vcernam;
   extern Chix  servername;
   struct pinfo_t pinfo;
   short  class;

   ENTRY ("syspdir", "");

   pdir->parts   = 0;
   pinfo.item    = item;
   pinfo.confum  = confnum;
   ascofchx (pinfo.userid,  owner,   0L, 20L);

   class = APDIR + u;
   if (NOT vcsend (servername, class, (char *) &pinfo, sizeof(pinfo),
                                      (char *)  pdir,
                                      sizeof(struct pdir_t))) {
      ERRCHK("syspdir");
      unit_write (XITT, CQ("Caucus Server has terminated.\n"));
      unit_write (XITT, CQ("Caucus is now aborting.\n"));
      sysexit (0);
   }

#endif

   RETURN (1);
}
