
/*** SYSPLUCK.   Pluck a file name out from a redirection string.
/
/    ok = syspluck (comline, pos, file);
/
/    Parameters:
/       int  ok;            (success?)
/       Chix comline;       (command line containing redirection string)
/       int  pos;           (starting position in COMLINE of redirect string)
/       Chix file;          (put file name here)
/
/    Purpose:
/       Pluck a "redirection string" out of a command line, and return
/       the file name.
/
/    How it works:
/       Syspluck is handed the command line, and the position (in the
/       command line) at which the redirection string begins.  It removes
/       the entire redirection string from the command line, and places
/       just the file name in FILE.
/
/       Redirection strings look like:
/         "<file", ">file", or ">>file"      (most systems)
/
/         "<fname ftype [fmod]"              (vm/cms)
/         ">fname ftype [fmod]"
/         ">>fname ftype [fmod]"
/
/    Returns: 1 on success
/             0 on error
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  Caucus functions
/
/    Operating system dependencies:
/
/    Known bugs:      none
/
/    Home:  sys/syspluck.c
/ */

/*** Copyright (C) 1987 Camber-Roth.  All rights reserved.  */

/*: AA  7/29/88 17:45 Source code master starting point */
/*: CR  8/03/88 17:09 VM changes only */
/*: CR 11/03/88 16:08 Add CX, CA, CVM types. */
/*: CR  1/09/89 14:15 Null_t, is union, not struct! */
/*: CW  5/08/89 15:17 Add PX system code. */
/*: JC 10/04/89 18:21 Stripped out AOS non-server code. */
/*: CR  4/16/90 15:41 Expand header comments. */
/*: CR  9/20/90 17:03 Add PS system type. */
/*: DE  5/29/92 13:09 Chixified */
/*: JX  6/09/92 17:46 chxtoken returns -1 on fail. */
/*: JX  6/16/92 17:00 chxalter doesn't accept null.chx. */
/*: CR 10/15/92 12:14 Add POS arg and re-chixify! */
/*: CL 12/10/92 14:52 Long chxalter args. */
/*: CR 12/10/92 15:16 Change POS arg to long. */
/*: CL 12/10/92 15:22 Long chxindex args. */
/*: CL 12/11/92 13:48 Long Chxvalue N. */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"

extern Chix ss_inred, ss_outred, ss_appred;
extern union null_t null;

FUNCTION  syspluck (comline, pos, file)
   Chix   comline, file;
   long   pos;
{
   Chix  rest;

/*-------"Normal" systems: filename is one word.-------------------- */
#if UNIX | IM | JM | PN | PR | VV | CX | CA | PX | PS
   ENTRY ("syspluck", "");

   rest = chxalloc (80L, THIN, "syspluck rest");
   chxcatsub (rest, comline, pos, ALLCHARS);
   chxtoken  (file, null.chx, 1, rest);
   chxalter  (comline, pos, file, CQ(""));

   if      (chxindex (file, 0L, ss_appred) == 0)  
            chxalter (file, 0L, ss_appred, CQ("")) ;
   else if (chxindex (file, 0L, ss_outred) == 0)  
            chxalter (file, 0L, ss_outred, CQ("")) ;
   else if (chxindex (file, 0L, ss_inred)  == 0)  
            chxalter (file, 0L, ss_inred,  CQ(""))  ;

   chxfree (rest);
#endif


/*-------VM/CMS: filename is "filename [filetype [filemode] ]"------ */
#if CVM
#include <ctype.h>
   Chix   word;
   extern union null_t null;
   int    success;

   ENTRY ("syspluck", "");

   word = chxalloc (60L, THIN, "syspluck word");
   rest = chxalloc (80L, THIN, "syspluck rest");
   chxcatsub (rest, comline, pos, ALLCHARS);

   /*** Get the filename out of COMLINE. */
   chxtoken (file, null.chx, 1, rest);
   chxalter (comline, pos, file, CQ(""));
   while    (chxalter (file, 0L, ss_inred,  CQ(""))) ;
   while    (chxalter (file, 0L, ss_outred, CQ(""))) ;
   while    (chxalter (file, 0L, ss_appred, CQ(""))) ;

   /*** Get the filetype out of COMLINE, if it's there. */
   if (chxtoken (word, null.chx, 2, rest) == -1)  FAIL;
   chxalter (comline, pos, word, CQ(""));
   chxcat   (file, CQ(" "));
   chxcat   (file, word);
      
   /*** Get the filemode out of COMLINE, if it's there.  The filemode MUST
   /    be one letter. */
   if (chxtoken (word, null.chx, 3, rest) == -1)                     SUCCEED;
   if (chxlen   (word) > 1 || NOT isalpha ((int) chxvalue(word,0L)))  SUCCEED;
   chxalter (comline, pos, word, CQ(""));
   chxcat   (file, CQ(" "));
   chxcat   (file, word);
   SUCCEED;

 done:
   chxfree ( word );
   chxfree ( rest );
   RETURN  ( success );
  
#endif

   RETURN (1);
}
