/*** SYSPRV.  Turn access to Caucus data files on (1) or off (0).
/
/    ok = sysprv (choice);
/
/    Parameters:
/       int    ok;          (Success?)
/       int    choice;      (1=on, 0=off)
/
/    Purpose:
/       Turns privileged access to Caucus data files on (1) or off (0).
/
/    How it works:
/       This means different things for different operating systems.
/   
/       For Vax/VMS, access to the Caucus data files requires SYSPRV
/       priviledge.  The program starts with SYSPRV on due to the way
/       it is installed.  It is turned off briefly when using I/O
/       indirection or EDIT at AND NOW, to keep the user from accessing
/       any file on the system!
/   
/       For UNIX, access to the Caucus data files is done with the set
/       uid bit.  Normally Caucus runs as the "caucus" user.  For some
/       operations (again, i/o redirection, EDIT at AND NOW) Caucus
/       temporarily reverts to the real userid, and afterwards switches
/       back to the effective userid.
/   
/       (In the old Ultrix implementation, Caucus accesses its data files 
/        via the set-gid bit.  Sysprv simply toggles the value of the 
/        global usegid, which is then checked by unit_make(), unit_view(), 
/        and unit_append().)
/   
/   
/       For all other systems, SYSPRV does nothing and always returns 1. 
/
/
/    Returns: 1 on success
/             0 on error
/
/    Error Conditions:
/ 
/    Side effects:
/
/    Related functions:
/
/    Called by:  Caucus functions
/
/    Operating system dependencies:
/
/    Known bugs:      none
/
/    Notes:
/       MUST NOT USE CHIX!  Some chix debugging functions call this,
/       which could set up a infinite stack-devouring loop.
/
/    Home:  sys2/sysprv.c
/ */

/*** Copyright (C) 1986 Charles Roth.  All rights reserved. */

/*: AA  7/29/88 17:59 Source code master starting point */
/*: CR  1/25/90 15:35 Make part of the general SYS functions. */
/*: CR  3/23/90 15:24 Unix: Save RID, EID on first call regardless. */
/*: CR  6/21/90 16:30 SU, (maybe BU) uses seteuid() instead of setuid. */
/*: JV  7/06/90 20:49 Add MIPS RISC/os (RO). */
/*: JV  7/22/90 16:30 Add getuid, geteuid decl's to Sys V. */
/*: CR  9/20/90 17:03 Add PS system type. */
/*: CR  9/27/90 12:19 Add U3, AU system types. */
/*: CJ 11/26/90 21:50 Add LA system type code. */
/*: CR  2/09/91 21:44 Split long #if line in two (SYSV1, SYSV2). */
/*: SB 11/29/90 14:05 Add DY system code*/
/*: CR  3/11/91 16:44 Add AX & UL system type codes. */
/*: CR  4/23/91 11:30 Add ultrix set-gid fixes. */
/*: SB  5/16/91 10:45 Add DP system type. */
/*: JV  7/17/92 10:00 Add U4. */
/*: CR  7/21/92 17:39 Remove DP (in favor of DYP). */
/*: CR  9/02/92 11:11 Make MV like UL. */
/*: DE  5/29/92 14:01 Chixified */
/*: CI 10/07/92 15:47 Chix Integration. */
/*: CP  7/26/93 12:42 MV is now like regular SYSV. */
/*: CK  9/02/93 18:19 Remove all chix, add header comments. */
/*: JV  9/27/93 10:07 Remove ENTRY: entry_test calls sysprv! */
/*: CP 11/22/93 13:48 Test debug anded with DEB_PRV. */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"

extern int    debug;

FUNCTION sysprv (choice)
   int   choice;
{
#if VV
   static char disable   = 0;
   static char enable    = 1;
   static char temp      = 0;
   static long newmask[2], oldmask[2];
   long   bit, word;
   int    PRV$V_SYSPRV();
 
   /*** Set the bit mask according to which bit number PRV$V_SYSPRV is. */
   bit = &PRV$V_SYSPRV;
   word = bit / 32;
   bit  = bit % 32;
   newmask[word] = 1 << bit;

   sys$setprv ( (choice ? enable : disable), newmask, temp, oldmask);
   return (1);
#endif

#define SYSV1  A3 | LA | DU | FU | MU | MV | PU | SX | TX | HU | IX
#define SYSV2  SV | PX | RO | U3 | AU | DY | AX | U4
#if UNIX | PX | PS
#if SYSV1 | SYSV2
   unsigned short getuid(), geteuid();
#endif
   static int rid = -1, eid = -1;
   int    success;
  
   if (rid < 0) {
      rid = getuid ();
      eid = geteuid();
   }

#if SU | BU | PS
   /* Not sure about BU! */
   success = (choice==0 ? seteuid (rid) : seteuid (eid) ) >= 0;
#endif
#if SYSV1 | SYSV2
   success = (choice==0 ? setuid  (rid) : setuid  (eid) ) >= 0;
#endif
#if UL
   { extern usegid;
     usegid  = choice;
     success = 1;
   }
#endif
  
   if (debug & DEB_PRV) printf ("sysprv(%d), success=%d\n", choice, success);

   return (success);
#endif


#if AOS | IM | PN | PR | CA | CVM
   return (1);
#endif

}
