
/*** SYS SET.   Set values of system-dependent user parameters.
/
/    SYSSET acts identically to SET (in fact it is called from
/    SET), but works with the system-dependent user parameters only. 
/
/    If SYSSET is called with parameters, it displays the current
/    settings of all the system-dependent parameters.
/
/    If the parameter is "?", the help text for the system-dependent
/    parameters is displayed.  Otherwise, SET will set the value
/    of the selected parameter. */

/*** Copyright (C) 1987 Camber-Roth.  All rights reserved.  */

/*: AA  7/29/88 17:45 Source code master starting point */
/*: CR 11/04/88 13:48 Remove VM, add SRV. */
/*: CR  1/09/89 14:14 Remove braces from UNIX sysopt0. */
/*: CW  5/08/89 15:17 Add PX system code. */
/*: JV 10/04/89 18:24 Stripped out AOS non-server code. */
/*: CR  9/20/90 17:03 Add PS system type. */
/*: CR  7/24/91 21:58 Make Lint happier. */
/*: CR  1/10/92 20:55 Make more language independent. */
/*: DE  5/29/92 16:17 Chixified */
/*: CX  6/02/92 13:30 Handle systurn(...LINE_EDIT). */
/*: JX  6/09/92 17:47 chxtoken returns -1 on fail. */
/*: JV 12/04/92 13:45 Add sitevar arg to storereg(). */
/*: CL 12/11/92 13:48 Long Chxvalue N. */
/*: CV  4/07/93 15:26 Treat thisuser as a ptr; add Chix format. */
/*: CR  6/03/93 12:52 Fix some VV stuff. */
/*: CV  6/06/93 12:27 Only systurn LINE_EDIT if value has changed! */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"

extern Userreg            thisuser;
extern struct termstate_t term_in_caucus;
extern union  null_t      null;
extern Chix   sitevar[];

#if VV
static char  *helptab   [] = { "sys_Hlineedit" };
static char   restrictab[] = { 2 };
static Chix   answertab [3];
static int    first_time = 1;
#endif

FUNCTION  sysset (out, command)
   int    out;
   Chix   command;
{
#if UNIX | IM | JM | PN | PR | SRV | PX | PS
   ENTRY  ("sysset", "");
   RETURN (-1);
#endif

#if VV
   static Chix optiontab = nullchix;
   Chix   option, value, def, onofftable, format;
   Chix   newstr, tempchx;
   short  i, choice, c, supplied;
   char   means[20];
   int    success, newstate;

   ENTRY ("sysset", "");

   if (first_time) {
      first_time = 0;
      answertab[0] = chxalloc (80L, THIN, "sysset answertab[0]");
      answertab[1] = chxalloc (80L, THIN, "sysset answertab[1]");
      answertab[2] = chxalloc (80L, THIN, "sysset answertab[2]");
   }

   option     = chxalloc (80L,  THIN, "sysset option");
   value      = chxalloc (80L,  THIN, "sysset value");
   def        = chxalloc (40L,  THIN, "sysset def");
   onofftable = chxalloc (100L, THIN, "sysset onofftable");
   newstr     = chxalloc (160L, THIN, "sysset newstr");
   tempchx    = chxalloc (80L,  THIN, "sysset tempchx");
   format     = chxalloc (200L, THIN, "sysset format");
   success    = 0;

   mdtable ("sys_Avvset", &optiontab);
   mdstr   (onofftable, "onofftable", null.md);

   /*** A parameter of "?" means display the help text about the
   /    system-dependent SET options. */
   if (chxvalue(command,0L) == L('?')) {
      mdwrite (out, "sys_Hvvset", null.md);
      SUCCEED;
   }

   /*** The command SET without options displays current settings. */
   /*   (This code only knows about 1 option.  If we add more, this
   /    should become some kind of case/loop.) */
   if (chxtoken  (option, null.chx, 2, command) == -1) {
      chxtoken (option, null.chx, 1, optiontab);
      chxupper (option);
      chxtoken (value, null.chx, 2-thisuser->sysi1, onofftable);
      sprintf  (means, "sys_Tvvset%d", 0);
      mdstr    (format, means, null.md);
      chxformat  (newstr, CQ("%-12s %10s    "), 0L, 0L, option, value);
      chxformat  (newstr, CQ("%s\n"), 0L, 0L, format, null.chx);
      unit_write (out, newstr);
      SUCCEED;
   }

   /*** Here if we were supplied an option.  Find out which one,
   /    and complain appropriately if not a good option. */
   choice = tablematch (optiontab, option);
   if (choice < 0)  DONE (choice);

   /*** If a value was SUPPLIED for this option on the command line,
   /    then run it through PROMPTER.  Otherwise, ask the user for
   /    a new value. */

   chxtoken (value, nullchix, 3, command);
   supplied = (EMPTYCHX(value) ? 0 : 3);

   chxcpy (answertab[0], onofftable);

   sprintf  (means, "sys_Pvvset%d", choice);
   chxtoken (def, null.chx, 2-thisuser->sysi1, onofftable);
   c = mdprompter (means, answertab[choice], restrictab[choice] + supplied,
                          helptab[choice], 40, value, def);

   if (c < -1)  SUCCEED;  /* No change */
   chxtoken (newstr, null.chx, 1, value);

   /*** Put the new value into the user's SYSOPTS string. */
   thisuser->sysi1 = 1 - tablematch (onofftable, newstr);
   newstate = (thisuser->sysi1 ? ON : OFF);
   if (term_in_caucus.lineedit != newstate)
      systurn (&term_in_caucus, LINE_EDIT, newstate);

   /*** Write out the changed USER information. */
   storereg (thisuser, 1, sitevar);
   SUCCEED;

 done:
   chxfree ( option );
   chxfree ( value );
   chxfree ( def );
   chxfree ( onofftable );
   chxfree ( newstr );
   chxfree ( tempchx );
   chxfree ( format );
   RETURN  ( success );
#endif
}
