/*** SYSTMPFILE.  Build a full pathname for a temporary file.
/
/    ok = systmpfile (dir, file);
/
/    Parameters:
/       int     ok;        (success?)
/       Chix    dir;       (directory for file)
/       Chix    file;      (full pathname)
/
/    Purpose:
/       Build a full pathname for a new, temporary file.  Typically this
/       should be created in a large, publicly accessible pool of disk
/       space, even though we will create private files there.  DIR allows
/       the caller to specify a directory to put the file, or NULL to let
/       systmpfile() choose the directory.
/
/    How it works:
/       On systems which supply a system call to build such a pathname,
/       use it.  Otherwise, make one up.
/
/    Returns: 1 if a pathname was constructed correctly
/             0 otherwise
/
/    Error Conditions:
/ 
/    Side effects:
/
/    Related functions:
/
/    Called by: 
/
/    Operating system dependencies:
/       Plenty.

/    Known bugs:      none
/
/    Home:  sys/systmpfi.c
/ */

/*** Copyright (C) 1993 Camber-Roth.  All rights reserved. */

/*: JV  4/13/93 13:04 Created function. */
/*: CP  7/15/93 15:23 Add MV, compiler complaint if not implemented. */
/*: CK 10/13/93 16:45 Extend to all Unix, check err return from tempnam(). */

#include <stdio.h>
#include "handicap.h"
#include "chixuse.h"
#include "systype.h"

FUNCTION  systmpfile (dir, file)
   Chix  dir, file;
{
   char *tmpname, *tempnam();

#if UNIX
   if (dir==nullchix  ||  EMPTYCHX (dir)) 
        tmpname = tempnam ("/tmp",        (char *) NULL);
   else tmpname = tempnam (ascquick(dir), (char *) NULL);

   if (tmpname != NULL) {
      chxofascii (file, tmpname);  
      free (tmpname);
   }
   else chxofascii (file, "/tmp/caucus_err");

#else
   /*** Force compiler to complain; this means this function has not
   /    been implemented. */
   5 = ;
#endif

   return  (1);
}
