/*** SYSUSERID.  Copies the account name of the current user
/                over the string supplied. */

/*** Copyright (C) 1983 Camber-Roth.  All rights reserved.  */

/*: AA  7/29/88 17:46 Source code master starting point */
/*: CR 11/04/88 13:54 Remove VM, add CX, CA, CVM. */
/*: CR  6/26/89 14:31 UNIX now uses cuserid() instead of getlogin(). */
/*: JV  9/13/89 18:15 Take DG out of UNIX cuz it don't have cuserid. grrr*/
/*: CW  6/27/89 15:49 Add PX system type code. */
/*: CR 12/14/89 13:37 Remove all lines that copy str -> flag.userid. */
/*: CR  1/26/90 16:01 UNIX code: save userid once in static area. */
/*: JV  3/23/90 23:00 Fixed banner! */
/*: CR  5/01/90 16:17 Unix: switch temporarily to real id to get userid. */
/*: CR  6/21/90 16:30 Use sysprv() instead of calling setuid() directly. */
/*: CR  9/20/90 17:03 Add PS system type. */
/*: CR  9/26/90 10:55 PS & PX should also do lower-casing at end. */
/*: CR 10/01/90 21:49 SV must call getlogin(), not cuserid(). */
/*: CR 10/29/90 13:00 IM & PN use "caucus" as default userid if none given. */
/*: CJ 11/26/90 21:50 Add LA system type code. */
/*: CR  2/09/91 21:44 Split long #if line in two (UNIX1, UNIX2). */
/*: SB 11/29/90 14:05 Add DY system code*/
/*: CR  3/11/91 16:44 Add AX & UL system type codes. */
/*: SB  5/16/91 10:51 Add DP system type.*/
/*: CR  7/27/91 13:30 Make Lint happier. */
/*: CR 12/30/91 11:39 Unix: exit if no userid! */
/*: JV  7/17/92 10:03 Add U4. */
/*: CR  7/21/92 17:39 Remove DP (in favor of DYP). */
/*: CR  8/20/92 15:05 Make MV use same code as SV. */
/*: CX  5/07/92 18:38 Minor fixes. */
/*: DE  6/01/92 12:49 Chixified - alternate buffer converts output*/
/*: CI 10/07/92 16:15 Chix Integration. */
/*: JV  3/29/93 11:18 Change chxcpy to chxofascii to avoid CQ. */
/*: CP  5/10/93  0:48 Add new Coherent code. */
/*: CP  7/08/93 16:02 Integrate 2.5 changes. */
/*: CP 12/04/93 14:04 If flag.underscore, add "_" to userid. */
/*: CR  3/15/94 12:47 Move 'BU' to 'SV' section. */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"
#include "sysexit.h"

extern struct flag_template flag;

FUNCTION  sysuserid (str_chx)
   Chix     str_chx;
{
   char   *s;
   char    str[160];

/*-------------------------------VM/CMS----------------------------------*/
#if CVM
   char   *c, *cuserid();
   ENTRY ("sysuserid", "");

   for (s=str, c=cuserid(NULL);   *c && *c!=' ';   ++c)  *s++ = *c;
   *s = '\0';
#endif

/*---------------------------AOS/VS--------------------------------------*/
#if CA
   char *getlogin(void);
  
   ENTRY ("sysuserid", "");

   if      (NOT flag.login)         strcpy (str, getlogin());
   else                             strcpy (str, flag.userid);
#endif

/*-------------------------------All Unix besides SV---------------------*/
#define UNIX1  A3 | LA | AU | DU | FU | MU | PU | SU | SX | TX
#define UNIX2  HU | IX | RO | U3 | CX | PX | PS | DY | UL | AX | U4
#if UNIX1 | UNIX2
   static  char save_userid[40];
   char  *p, *cuserid();

   ENTRY ("sysuserid", "");
   if (flag.login)  strcpy (str, flag.userid);
   else {
      if (NULLSTR (save_userid)) {
         sysprv (0);
         p = cuserid( (char *) NULL);
         sysprv (1);
         if (p == NULL  ||  NULLSTR(p))  sysexit (EXIT_NOUID);

         strcpy (save_userid, p);
      }
      strcpy (str, save_userid);
   }
#endif

/*-------------------------------Unisys SV Unix--------------------------*/
#if SV | BU
   static char save_userid[40];
   char  *ptr, *getlogin();

   ENTRY ("sysuserid", "");

   /*** SV version of cuserid() invokes yellow-pages, sockets, and all
   /    kinds of crazy stuff!  So we use getlogin() here instead. */
   if (flag.login)  strcpy (str, flag.userid);
   else {
      if (NULLSTR (save_userid)) {
         sysprv (0);
         if ( (ptr = getlogin()) == NULL  ||  NULLSTR(ptr)) {
            printf ("Bad return from getlogin()!\n");
            exit   (9);
         }
         strcpy (save_userid, ptr);
         sysprv (1);
      }
      strcpy (str, save_userid);
   }
#endif

/*-------------------------------Coherent--------------------------------*/
#if MV
#include <pwd.h>
   static char save_userid[40];
   char  *ptr, *getlogin();
   int    ruid;
   struct passwd *pstp;

   ENTRY ("sysuserid", "");

   /*** First try getlogin().  If that fails, get our real uid, and
   /    call getpwuid() to go look through the *(%$! /etc/passwd file
   /    for our userid. */
   if (flag.login)  strcpy (str, flag.userid);
   else {
      if (NULLSTR (save_userid)) {
         sysprv (0);
         if ( (ptr = getlogin()) == NULL  ||  NULLSTR(ptr)) {
            setpwent();         /* Rewind file /etc/passwd */
            ruid = getuid();
            if ((pstp = getpwuid(ruid)) == NULL) {
               printf ("Bad return from getpwuid()!\n");
               exit   (9);
            }
            ptr = pstp->pw_name;
         }
         strcpy (save_userid, ptr);
         sysprv (1);
      }
      strcpy (str, save_userid);
   }
#endif

/*---------------------------------------IBM PC--------------------------*/
#if IM | JM | PN
   ENTRY ("sysuserid", "");

   if (NULLSTR (flag.userid))  strcpy (flag.userid, "caucus");
   strcpy (str, flag.userid);
#endif

/*---------------------------------------VAX-----------------------------*/
#if VV
   char  temp[80];

   ENTRY ("sysuserid", "");

   if (NOT flag.login) {
      cuserid (temp);
      temp[15] = '\0';
      strcpy (str, temp);
   }
   else   strcpy (str, flag.userid);
#endif

/*---------------------------------------PRIME---------------------------*/
#if PR
   char *cuserid();
   char  temp[80];

   ENTRY ("sysuserid", "");

   if (NOT flag.login) {
      strcpy (temp, cuserid(NULL));
      temp[15] = '\0';
      strcpy (str, temp);
   }
   else   strcpy (str, flag.userid);
#endif


#if UNIX | IM | PN | PR | VV | CX | CVM | PX | PS
   for (s=str;   *s;   ++s)    if (cf_upper(*s))    *s += ('a' - 'A');
#endif

   if (flag.underscore  &&  NOT flag.login)  strcat (str, "_");
   chxofascii (str_chx, str);

   RETURN (1);
}
