/*** ENTRY_TEST.  Function invoked by ENTRY macro in all Caucus code. */

/*: AA  7/29/88 17:54 Source code master starting point */
/*: CR  8/04/88 13:19 fix bugs in entry_test() */
/*: JV  8/01/88 14:19 Added declaration of 'i' and 'level'.*/
/*: CR  3/27/89 14:13 Added (temporary?) master.items bug test. */
/*: CR  4/08/91 18:49 Add new unit_lock argument. */
/*: CR  6/21/91 13:17 Add stackprint() function. */
/*: CR  7/02/91 17:38 Remove master. */
/*: JV  7/22/91 15:32 Check value of level before accessing data. */
/*: JV  7/23/91 13:19 Conditionally compile functions. */
/*: CR  7/24/91 21:59 Minor fixes. */
/*: CR  7/29/91 23:18 Revamp results when level goes out of bounds. */
/*: UL 12/11/91 18:44 Make unit_lock args 3-6 long.*/
/*: JV 10/14/92 10:37 Change sanity limit on master.items to MAXITEMS. */
/*: JV  9/08/93 09:59 Add ENTRYLOG handling. */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"
#include "units.h"

#define  MAXLEVEL   200    /* Max number of stack levels */
#define  MAXWIDE     16    /* Width of char string containing func. name */

extern struct unit_template   units[];
extern int                    debug;

static int  level = 0;
static char cstack[MAXLEVEL][MAXWIDE];

static FILE *entryfile = NULL;

FUNCTION  entry_test (ets, t)
   char   *ets, *t;
{
#if ENTRYEXIT
   char   temp[200];
   int    i;

#if ENTRYLOG
   if (entryfile == NULL) {
      sysprv (0);
      entryfile = fopen ("entry.log", "a");
      sysprv (1);
   }

   if (entryfile == NULL) printf ("Couldn't open entry log.\n");
   fprintf (entryfile, "%s\n", ets);
   fflush  (entryfile);
#endif

   if (level >= MAXLEVEL) {
      printf ("ENTRY_TEST: level has reached %d!\n", level);
      for (i=0;   i<MAXLEVEL;   ++i)
         printf (" %3d %s\n", i, cstack[i]);
      exit (1);
   }

   ++level;
   strcpy (temp, ets);
   temp[MAXWIDE] = '\0';
   strcpy (cstack[level], temp);
#endif

/* mtest  ("entering"); */
}


FUNCTION  exit_test()
{
/* mtest ("leaving"); */

#if ENTRYEXIT
   int  i;

   if (level < 0) {
      printf ("ENTRY_TEST: stack level < 0!\n");
      for (i=0;   i<20;   ++i)
         printf (" %3d %s\n", i, cstack[i]);
      exit (1);
   }

   --level;
#endif
}


#define  MTEST   0
#if MTEST

FUNCTION mtest (direction)
   char  *direction;
{
   char   str[200];
   int    i;

   if (master.items    < 0  ||  master.items    > MAXITEMS  ||
       master.nochange < 0  ||  master.nochange > 1     ||
       master.noenter  < 0  ||  master.noenter  > 1) {
      sprintf (str, "Change to master struct %s %s: %d %d %d\n",
                     direction, cstack[level], 
                     master.items, master.nochange, master.noenter);
      master.items = master.nochange = master.noenter = 0;
      buglist (str);
      unit_lock   (XSBG, WRITE, 0L, 0L, 0L, 0L, "");
      unit_append (XSBG);
      for (i=1;   i<=level;   ++i)  {
         unit_write (XSBG, cstack[i]);
         unit_write (XSBG, "\n");
      }
      unit_close  (XSBG);
      unit_unlk   (XSBG);
   }
}
#endif
