
/*** UNIT_APPEND.  Open the file of TYPE for write/append. */

/*** Copyright (C) 1984 Camber-Roth.  All rights reserved.  */

/*: AA  7/29/88 17:54 Source code master starting point */
/*: CR  9/30/88 15:44 Add server code in "#if SRV" section. */
/*: CR 11/09/88 15:43 Add #include "errchk.h" to SRV code. */
/*: JV  2/10/89 15:46 Casted vcsend parms in SRV.*/
/*: CR  3/21/89 15:26 Add use of servername to #if SRV code. */
/*: CR  3/22/89 12:51 Use MEMBUF for server data transfers. */
/*: JV  4/05/89 21:33 Added code to stop client if server is dead. */
/*: CR  4/20/89 17:50 CVM: if no file options for fopen(), add default. */
/*: CR  4/27/89 16:26 Add lock-consistency check code. */
/*: CW  5/08/89 14:56 Add PX system code. */
/*: CW  5/19/89 12:51 Add XW unit codes. */
/*: JV  3/23/90 12:12 Surround syscheck/fopen with sysprv(0/1) calls. */
/*: CR  4/24/90 21:09 Remove unused 'exists'. */
/*: CR  9/20/90 16:55 Add PS system type. */
/*: CR  3/02/91 19:10 Add XIBB code. */
/*: CR  4/23/91 16:32 Add sysaccess() call .*/
/*: JV  5/14/91 19:42 Add unittag use. */
/*: CR  6/21/91 17:38 Ensure unittag[n], n>=0 ! */
/*: DE  4/09/92 13:19 Chixified */
/*: JX 10/08/92 11:00 Fixify. */
/*: CL 12/11/92 15:52 Long sysmem arg. */
/*: CR 10/27/93 14:42 Set units[n].fd to -1, just like unitview/unitmkseq. */
/*: CR 12/10/93 15:00 Use debug & DEB_FILE. */

#include "caucus.h"
#include <stdio.h>
#include "systype.h"
#include "units.h"

#if SRV
#include "actions.h"
#include "errchk.h"
extern int   vcernum;
extern char *vcernam;
extern Flag  localfile[];
extern Chix  confid;
extern Chix  servername[];
#endif

extern struct unit_template  units[];
extern char                 *unittag[];
extern int                   debug;

FUNCTION  unit_append (n)
   int  n;
{
   FILE  *fopen();
   char  *bugtell();

   ENTRY ("unitappend", "");
   if (debug & DEB_FILE)  bugtell ("APPEND", n, 0L, (n>=0 ? unittag[n] : ""));

   /*** Bug check: is this file N already open? */
   if (units[n].access != LOCKED) {
      buglist (bugtell ("Unitappe: ", n, 0L, "file is not locked!\n"));
      RETURN  (0);
   }

   /*** We can always append to the bit bucket. */
   if (n == XIBB)  RETURN (1);

#if SRV
   if (localfile[n]) {
      char  filename[80];

      strcpy (filename, units[n].nameis);
#if CVM
      if (strindex (filename, "(") < 0)  
         strcat (filename, " (text lrecl 80 recfm v");
#endif
      if ( (units[n].fp = fopen (filename, "a")) == NULL)  RETURN(0);
   }

   else {
      char  *sysmem();
      short  stat, class;

      if ( (units[n].buffer = sysmem (L(MEMBUF))) == NULL)  RETURN(0);
      units[n].buflen = 0;
      units[n].p = units[n].buffer;
     *units[n].p = '\0';

      class = AAPND + n;
      if (NOT vcsend (servername, class, NULL, 0, (char *) &stat, 1)) {
         ERRCHK ("unitappend");
         unit_write (XWER, CQ("Caucus Server has terminated.\n"));
         unit_write (XWER, CQ("Caucus is now aborting.\n"));
         sysexit (0);
      }
      if (NOT stat)  RETURN(0);
   }

   units[n].access = WRITE;
   RETURN (1);
}
#endif


#if UNIX | IM | JM | PN | VV | PR | AOS | PX | PS

   /*** Ensure Caucus privilege is not used for uncontrolled file access. */
   if (n < XURG)  sysprv(0);
   if (NOT sysaccess (units[n].nameis))  {
      if (n < XURG)  sysprv(1);
      RETURN (0);
   }

   if (syscheck (units[n].nameis)) {

#if UNIX | IM | JM | PN | AOS | PX | VV | PS
      units[n].fp = fopen (units[n].nameis, "a");
      units[n].fd = -1;
      if (n < XURG)  sysprv(1);
      if (units[n].fp == NULL)   RETURN (0);
#endif

#if PR
      fortran  prwf$$(), tsrc$$();
      short    key, funit, chrpos[2], type, code, zero;
      short   *bufadr;
      long     eof;

      /*** First, open the file for writing. */
      key = KWRIT + KGETU + KNSAM;
      chrpos[0] = 0;
      chrpos[1] = strlen(units[n].nameis);
      tsrc$$ (&key, (char []) units[n].nameis, &funit, chrpos, &type, &code);
      units[n].fd = funit;
      if (code != 0)  RETURN (0);

      /*** Now move to the end of the file. */
      key    = KPOSN;
      bufadr = (short *) NULL;
      zero   = 0;
      eof    = 100000000L;
      prwf$$ (&key, &funit, bufadr, &zero, &eof, &zero, &code);
#endif

      units[n].access = WRITE;
      RETURN (1);
   }

   if (n < XURG)  sysprv(1);
   RETURN (unit_make (n));
}
#endif




