/*** UNIT_B_COPY.   Copy unit SRC to unit DEST.
/
/    ok = unit_b_copy (src, dest);
/
/    Parameters:
/      int  ok;    (success?)
/      int  src;   (source file unit number, locked by caller)
/      int  dest;  (destination file unit number, locked by caller)
/      
/    Purpose:
/      This function copies one file to another, retaining any
/      file format and record length attributes.   If there was
/      a pre-existing "destination" file, it is deleted.
/
/      It DOES:
/         copy binary data
/         retain file format and record length
/         copy from: Caucus file to:    Caucus file
/                    Caucus file        User file  (unit XIRE)
/                    User file (XIRE)   Caucus file
/                    
/      It does NOT:
/         allow variable length records (unless data can be moved
/            independently of record structure)
/
/    How it works:
/      Both units must already be locked.
/
/    Returns 1 on success, 0 on failure.
/
/    Error Conditions:
/
/    Side effects:
/
/    Related Functions:
/
/    Called by:
/
/    Operating System Dependencies:
/
/    Known Bugs: Vax/VMS code not written.
/
/    Home: unit/unitbcop.c
/
/ */

/*** Copyright (C) 1993 Camber-Roth.  All rights reserved. */

/*: JV  3/05/93 10:45 Create this. */
/*: CP  8/17/93 13:36 Rewrite and simplify. */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"
#include "units.h"

#if IM | PN
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <share.h>
#include <io.h>
#endif

#if UNIX | PX | PS
#include <signal.h>
#include <errno.h>
extern int errno;
#endif

extern struct unit_template  units[];

FUNCTION  unit_b_copy (src, dest)
   int src, dest;
{

/*----------------------------------Unix, DOS-------------------------------*/
#if UNIX | PX | PS | IM | PN
   char     buf[1024];
   int      len;

   if (NOT unit_b_view (src,  0L, 0))                          return (0);
   if (NOT unit_b_make (dest, 0L, 0, dest==XIRE ? 1 : 0))
                                        { unit_close (src);    return (0); }

   for (len=1024;   (len = read (units[src].fd, buf, len)) > 0;)
      write (units[dest].fd, buf, len);

   unit_close (src);
   unit_close (dest);
#endif

/*---------------------------------------VAX-----------------------------*/
#if VV
   5 = ;
#endif

/*---------------------------------------PRIME---------------------------*/
#if PR
   delete (units[dest].nameis);
   copy   (units[src].nameis, units[dest].nameis);
#endif

   return (1);
}
