/*** UNIT_B_MAKE.  Create a file for binary writing.
/
/    ok = unit_b_make (n, format, rec_len, user);
/
/    Parameters:
/       int  ok;       (success in creating file)
/       int  n;        (unit number of file)
/       long format;   (file format options)
/       int  rec_len;  (record length)
/       int  user;     (1=create as user, 0=create as Caucus)
/
/    Purpose:
/       Create and open a binary file.
/
/    How it works:
/    unit_b_make() creates and opens the binary file associated with
/    unit N.  If USER is 1, it tries to create the file as
/    though the user were writing it.  Otherwise, it creates the file
/    with Caucus' privileges. 
/
/    Returns: 1 on success
/             0 on failure (error)
/
/    Error Conditions:
/
/    Side effects:
/
/    Related functions:
/
/    Called by:  Application.
/
/    Operating system dependencies: 
/       Unix: uses open()
/       DOS:  open()
/       VV:   not implemented.
/
/    Known bugs:
/
/    Home:  unit/unitbmak.c
/ */

/*** Copyright (C) 1993 Camber-Roth.  All rights reserved. */

/*: JV  3/03/93 14:40 Create this. */
/*: CP  7/16/93 17:07 Add return code comments, compiler error for IM, VV. */
/*: CP  8/16/93 14:28 Delete file just before opening it. */
/*: CP  8/16/93 16:32 Clean up fd/fp usage; fix IM/PN code. */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"
#include "units.h"

#if SU
#include <sys/stat.h>
#endif

extern struct unit_template units[];
extern int                  debug;

FUNCTION  unit_b_make (n, format, rec_len, user)
   int    n, rec_len, user;
   long   format;
{

/*---------------------------------------IBM PC--------------------------*/
#if IM
   ENTRY ("unit_b_make", "");
   unlink (units[n].nameis);
   units[n].fd = open (b, O_BINARY | O_RDWR | O_CREAT, S_IREAD | S_IWRITE);
   units[n].fp = NULL;
   RETURN (units[n].fd >= 0);
#endif

#if PN
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <share.h>
   int    fd;
   ENTRY ("unit_b_make", "");

   /*** Create the sequential file with the attribute shareable. */
   unlink (units[n].nameis);
   if ( (fd = sopen (units[n].nameis, O_CREAT | O_RDWR, SH_DENYNO, 
                                      S_IREAD | S_IWRITE)) < 0)  RETURN (0);
   units[n].fd = fd;
   units[n].fp = NULL;
   RETURN (units[n].fd >= 0);
#endif


/*---------------------------------------VAX-----------------------------*/
#if VV
   FILE *fopen();
   ENTRY ("unitmkseq", "");

   5 = ;
   if (user)  sysprv (0);
   delete (units[n].nameis);
   units[n].fp = fopen (units[n].nameis, "w", "rfm=var", "rat=cr", "fop=sup");
   if (user)  sysprv (1);
   if (debug  &&  units[n].fp==NULL) {
      bugtell ("unitmkse: can't make.", n, 0L, units[n].nameis);
      perror  ("reason");
   }
   RETURN (units[n].fp != NULL);
#endif
 
/*-------------------------------FORTUNE & MASSCOMP & A3 | LA-----------------*/
#if UNIX | PX | PS
#include <fcntl.h>
   int    oldmask;

   ENTRY ("unit_b_make", "");

   if (user)  sysprv(0);
   if (NOT sysaccess (units[n].nameis))  units[n].fd = -1;
   else {
      if (n==XIRE)  oldmask = umask (0077);
      unlink (units[n].nameis);
      units[n].fd = open (units[n].nameis, O_WRONLY | O_CREAT, 0700);
      units[n].fp = NULL;
      if (n==XIRE)  umask (oldmask);
   }
   if (user)  sysprv(1);

   if (units[n].fd <= 0)  RETURN (0);
   units[n].access = WRITE;
   RETURN (1);
#endif

}
