/*** UNIT_B_WRITE.  Write some data to a file.
/
/    ok = unit_b_write (n, buf, size);
/
/    Parameters:
/       int   ok;      (success?)
/       int   n;       (unit number of open file)
/       char *buf;     (use data from here)
/       long  size;    (size of data to write)
/
/    Purpose:
/       Write a bufferful of data from BUF to the open file on unit N.
/       Use file format and/or record attribute info in units structure.
/
/       Unlike unit_write(), unit_b_write() does not adjust the data it
/       is writing in any way.
/
/    How it works:
/       
/    Returns: 1 on success
/             0 on end-of-file, interrupt, hangup, or error
/
/    Error Conditions:
/
/    Side effects:
/
/    Related functions:
/
/    Called by:  Application.
/
/    Operating system dependencies: lots
/
/    Known bugs:
/       We may wish to have unit_b_write() return some indication
/       of an error other than the return value 0.
/
/    Home:  unit/unitbwri.c
/ */

/*** Copyright (C) 1993 Camber-Roth.  All rights reserved.  */

/*: JV  3/05/93 13:39 Create this. */
/*: CP  7/16/93 17:09 Add compiler error for non-implented sys types. */

#include <stdio.h>
#include "handicap.h"
#include "systype.h"
#include "entryex.h"
#include "unitcode.h"
#include "units.h"

extern struct unit_template units[];

FUNCTION  unit_b_write  (n, buf, size)
   int   n;
   char *buf;
   long  size;
{
   char *bugtell();

   ENTRY ("unit_b_write", "");

   if (n <= XITT) RETURN (0);    /* No writing binary to terminal yet! */
   if (n == XIBB) RETURN (1);    /* "OK, I did it!!!" */

   if (units[n].access != WRITE)
      buglist (bugtell ("Unit_b_write: ", n, 0L,
               "writing to unopened file!\n"));

#if UNIX
   RETURN (write (units[n].fd, buf, (int)size));
#else
   5 = ;
#endif

}
