/*** UNIT_KILL.   Kill (delete) a given file.
/
/   n = unit_kill (unit);
/
/   Parameters:
/      int unit;
/
/   Purpose:
/      Delete the file currently locked by UNIT_LOCK of type UNIT.
/
/   How it works:
/      On most systems, it calls sysunlink() on the filename
/      associated with UNIT.  On server-based systems, it calls
/      sysunlink for local files, but for Caucus files it sends
/      a message to the server.
/
/   Note: the file must be locked before calling unit_kill, and should
/      be unlocked afterward.  The file should not be opened for
/      reading or writing.
/
/   Return:
/      Most systems always return 1.
/      Server-based systems return 1 on success, 0 on failure.
/
/   Related functions: unit_lock, unit_unlk.
/
/   Called by: lots and lots.
/
/   Operating system dependencies: See above.
/
/   Home: unit/unitkill.c
/ */

/*** Copyright (C) 1984 Camber-Roth.  All rights reserved.  */

/*: AA  7/29/88 17:54 Source code master starting point */
/*: CR  9/30/88 15:32 Add server code in "#if SRV" section. */
/*: CR 11/09/88 15:43 Add #include "errchk.h" to SRV code. */
/*: JV  2/10/89 15:46 Casted vcsend parms in SRV. */
/*: CR  3/21/89 15:26 Add use of servername to #if SRV code. */
/*: JV  4/05/89 21:36 Added code to stop client if server is dead. */
/*: CW  5/08/89 14:56 Add PX system code. */
/*: CW  5/19/89 12:51 Add XW unit codes. */
/*: CR  9/12/89 17:00 Declare bugtell(). */
/*: CR  3/23/90 16:01 Surround sysunlink() call with sysprv(0/1). */
/*: CR  9/20/90 16:55 Add PS system type. */
/*: JV  5/01/91 19:26 Add header docs. */
/*: CR  8/24/92 17:24 Fix "nested" comment. */
/*: DE  4/09/92 13:17 Chixified */
/*: CI 10/08/92  9:27 Chix Integration. */
/*: CP 11/22/93 13:50 Test debug anded with DEB_FILE. */

#include "caucus.h"
#include <stdio.h>
#include "systype.h"
#include "units.h"

#if SRV
#include "actions.h"
#include "errchk.h"
extern int   vcernum;
extern char *vcernam;
extern Flag  localfile[];
extern Chix  servername;
#endif

extern struct unit_template units[];
extern int                  debug;

FUNCTION  unit_kill (n)
   int  n;
{
   char *bugtell();

#if UNIX | IM | JM | PN | PR | VV | AOS | PX | PS
   ENTRY ("unitkill", "");
   if (debug & DEB_FILE)  bugtell ("KILL", n, 0L, units[n].nameis);
   if (n < XURG)  sysprv(0);
   sysunlink (units[n].nameis);
   if (n < XURG)  sysprv(1);
   RETURN (1);
#endif


#if SRV
   extern int debug;
   char   stat;
   short  class;

   ENTRY ("unitkill", "");

   if (debug & DEB_FILE)
      bugtell ("Unitkill", n, (long) localfile[n], units[n].nameis);

   if (localfile[n])  RETURN (sysunlink (units[n].nameis));

   class = AKILL + n;
   if (NOT vcsend (servername, class, NULL, 0, (char *) &stat, 1)) {
      ERRCHK("unitkill");
      unit_write (XWER, CQ("Caucus Server has terminated.\n"));
      unit_write (XWER, CQ("Caucus is now aborting.\n"));
      sysexit (0);
   }

   RETURN (stat);
#endif

}
