
/*** UNIT_MAKE.  Open the file of TYPE for write/create. */

/*** Copyright (C) 1984 Camber-Roth.  All rights reserved.  */

/*: AA  7/29/88 17:54 Source code master starting point */
/*: CR  8/04/88 12:57 VM changes only */
/*: CR  9/30/88 15:22 Add server code in "#if SRV" section. */
/*: CR 11/09/88 15:43 Add #include "errchk.h" to SRV code. */
/*: JV  1/26/89 10:22 Fixed vcsend call in SRV. */
/*: CR  3/21/89 15:26 Add use of servername to #if SRV code. */
/*: CR  3/22/89 12:24 Use MEMBUF for server data transfer buffering. */
/*: JV  4/05/89 21:38 Added code to stop client if server is dead. */
/*: CR  4/20/89 17:50 CVM: if no file options for fopen(), add defaults. */
/*: CR  4/27/89 16:24 Add lock-consistency check code. */
/*: CW  5/08/89 14:56 Add PX system code. */
/*: CW  5/19/89 12:51 Add XW unit codes. */
/*: JV  3/23/90 12:12 Use n < XURG instead of ==XIPT, ==XIRE. */
/*: CR  9/20/90 16:55 Add PS system type. */
/*: CR  9/26/90 12:15 Make Lint happy when not SRV. */
/*: CR  3/02/91 19:10 Add XIBB code. */
/*: JV  8/13/91 09:36 Add debug.h code to Unix only */
/*: DE  4/09/92 13:16 Chixified */
/*: JX 10/08/92 11:00 Fixify. */
/*: CL 12/11/92 15:52 Long sysmem arg. */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"
#include "units.h"

#if SRV
#include "actions.h"
#include "errchk.h"
extern int   vcernum;
extern char *vcernam;
extern Flag  localfile[];
extern Chix  confid;
extern Chix  servername;
#endif

extern struct unit_template  units[];
extern int                   debug;

FUNCTION  unit_make (n)
   int  n;
{
   char   backup[130];
   char   format[160];
   char  *bugtell();
   Chix   newstr;
   int    success;

   ENTRY ("unitmake", "");
   if (debug & DEB_FILE)  bugtell ("MAKE", n, 0L, "");

   newstr    = chxalloc (160L, THIN, "unitmake newstr");
   success   = 0;

   /*** Bug check: is this file N already open? */
   if (units[n].access != LOCKED) {
      buglist (bugtell ("Unitmake: ", n, 0L, "file is not locked!\n"));
      FAIL;
   }

   /*** We can always write to the bit bucket. */
   if (n == XIBB)  SUCCEED;

#if SRV
   if (localfile[n]) {
      char   filename[80];

      strcpy (filename, units[n].nameis);
#if CVM
      if (strindex (filename, "(") < 0)  
         strcat (filename, " (text lrecl 80 recfm v");
#endif

      if ( (units[n].fp = fopen (filename, "w")) == NULL)  FAIL;
   }

   else {
      /*** Allocate space for text to send to server. */
      short  class, stat;
      char  *sysmem();

      if ( (units[n].buffer = sysmem (MEMBUF)) == NULL)  FAIL;
      units[n].buflen = 0;
      units[n].p  = units[n].buffer;
      *units[n].p = '\0';

      class = AMAKE + n;
      if (NOT vcsend (servername, class, NULL, 0, (char *)&stat, 1)) {
         ERRCHK ("unitmake");
         unit_write (XWER, CQ("Caucus Server has terminated.\n"));
         unit_write (XWER, CQ("Caucus is now aborting.\n"));
         sysexit (0);
      }
      if (NOT stat)  FAIL;
   }

#endif

#if UNIX | IM | JM | PN | PR | VV | AOS | PX | PS
   /*** Make up name of BACKUP file. */
   sysbackup (backup, units[n].nameis);

   if (n < XURG) {
      if (NOT unitmkseq (n, 1))   FAIL;
   }

   else {
      /*** For normal files, we're going to rewrite the entire file anyway,
      /    so just rename the original to the backup file. */
      if (syscheck (backup)) {
         sprintf (format, "%s\n  %d %s\n",
                 "Unitmake: seq: abort found, use backup.", 
                  n, units[n].nameis);
         buglist (format);
      }
      else if (syscheck (units[n].nameis) )
         sysrename (units[n].nameis, backup);

      if (NOT unitmkseq (n, 0))  FAIL;
   }
#endif

   units[n].access = WRITE;

   SUCCEED;

 done:

   chxfree ( newstr );

   RETURN ( success );
}
