
/*** UNITMKSEQ.  Create a sequential file.
/
/    UNITMKSEQ creates and opens the sequential file associated with
/    unit N.  If USER is 1, UNITMKSEQ tries to create the file as
/    though the user were writing it.  Otherwise, it creates the file
/    with Caucus' privileges. */

/*** Copyright (C) 1986 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:54 Source code master starting point */
/*: JV  9/02/88 11:23 Added debug code to AOS, simpfld creation code.*/
/*: JV  9/13/88 15:38 Changed AOS to create file with correct ACL's.*/
/*: CR  9/21/88 15:42 Add "HU" system type for HP/UX. */
/*: CR  9/23/88 12:59 Add "IX" system type. */
/*: JV 11/22/88 14:21 Changed AOS file 'open'ing to use DEFACL.*/
/*: CR 12/16/88 13:13 PN code: make new files shareable. */
/*: CR  6/09/89 12:21 Add "SV" system type code. */
/*: CW  5/08/89 14:56 Add PX system code. */
/*: CR 10/12/89 15:43 Add DU system type code. */
/*: CR  3/23/90 17:47 Use sysprv to handle UNIX protection. */
/*: CR  4/24/90 21:10 Remove unused UNIX fd, status, child. */
/*: CR  6/06/90 14:10 UNIX: play with umask() if output redirection file. */
/*: CR  8/22/90 14:18 VV: fix fopen() call to supersede existing file. */
/*: CR  9/20/90 16:55 Add PS system type. */
/*: CJ 11/26/90 21:46 Add LA system type. */
/*: CR  4/23/91 16:26 Add sysaccess() call. */
/*: JV  1/17/92 17:05 Fix SU umask usage. */
/*: JX  5/28/92 13:50 Make SU *not* use SysV compatibility. */
/*: CP  8/16/93 15:58 Set units[n].fd to -1. */
/*: CK 10/15/93 13:20 Comment out UNIX umask() tinkering.  Why is it there? */

#include "caucus.h"
#include <stdio.h>
#include "systype.h"
#include "units.h"

#if SU
#include <sys/stat.h>
#endif

#if AOS
extern char * strcat();
#include <paru.h>
#include <errno.h>
#endif

extern struct unit_template units[];
extern int                  debug;

FUNCTION  unitmkseq (n, user)
   int    n, user;
{

/*---------------------------------------IBM PC--------------------------*/
#if IM
   FILE *fopen();
   ENTRY ("unitmkseq", "");
   units[n].fp = fopen (units[n].nameis, "w");
   units[n].fd = -1;
   RETURN (units[n].fp != NULL);
#endif

#if PN
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <share.h>
   FILE  *fopen();
   int    fd;
   ENTRY ("unitmkseq", "");

   /*** Create the sequential file with the attribute shareable. */
   if ( (fd = sopen (units[n].nameis, O_CREAT | O_RDWR, SH_DENYNO, 
                                      S_IREAD | S_IWRITE)) < 0)  RETURN (0);
   close (fd);
   
   units[n].fp = fopen (units[n].nameis, "w");
   units[n].fd = -1;
   RETURN (units[n].fp != NULL);
#endif


/*---------------------------------------NEW PRIME-----------------------*/
#if PR
   fortran tsrc$$();
   short   key, funit, type, code, chrpos[2];

   ENTRY ("unitmkseq", "");

   key = KWRIT + KGETU + KNSAM;
   chrpos[0] = 0;
   chrpos[1] = strlen(units[n].nameis);
   tsrc$$ (&key, (char []) units[n].nameis, &funit, chrpos, &type, &code);
   units[n].fd = funit;
   RETURN (code==0);
#endif

/*---------------------------------------VAX-----------------------------*/
#if VV
   FILE *fopen();
   ENTRY ("unitmkseq", "");

   if (user)  sysprv (0);
   units[n].fp = fopen (units[n].nameis, "w", "rfm=var", "rat=cr", "fop=sup");
   units[n].fd = -1;
   if (user)  sysprv (1);
   if (debug  &&  units[n].fp==NULL) {
      bugtell ("unitmkse: can't make.", n, 0L, units[n].nameis);
      perror  ("reason");
   }
   RETURN (units[n].fp != NULL);
#endif
 
/*-------------------------------FORTUNE & MASSCOMP & A3 | LA-----------------*/
#if UNIX | PX | PS
   FILE *fopen();
   int    oldmask;

   ENTRY ("unitmkseq", "");

   if (user)  sysprv(0);
   if (NOT sysaccess (units[n].nameis))  units[n].fp = NULL;
   else {
/*    if (n==XIRE)  oldmask = umask (0);  */
      units[n].fp = fopen (units[n].nameis, "w");
      units[n].fd = -1;
/*    if (n==XIRE)  umask (oldmask);      */
   }
   if (user)  sysprv(1);

   RETURN (units[n].fp != NULL);
#endif

/*----------------------------AOS/VS, MV/UX----------------------------*/
#if AOS
   FILE *fopen();
   int dadpid, sumode, privword, forp;
   char username[$MXUN], command[60];  
   ENTRY ("unitmkseq", "");

   if      (NOT user)  close (creat (units[n].nameis, 0707)); /* 0707 works! */
   else {                                  /* If a 'user' file, create it in */
       forp=fopen (units[n].nameis, "w");
       if (debug && forp <0) {
         perror("unitmkseq");printf("%d\n", errno); }
       fclose (forp);
   }
 
   units[n].fp = fopen (units[n].nameis, "w");
   units[n].fd = -1;
   if (debug && units[n].fp == NULL) {
     perror("unitmkseq2");printf("%d\n", errno); }
    
RETURN (units[n].fp != NULL);

#endif
}
