
/*** UNIT_NAME.   Figure the name of a conference file.
/
/    1 = unitname (name, n, cnum, item, resp, lnum, word)
/
/    Parameters:
/       int  n;       // Unit number
/       long cnum,    // Conference number
/            item,    // Item number
/            resp,    // Resp number (or message number)
/            lnum;    // Line number
/       char name[];  // Place to store name
/       Chix word;    // Textual input info
/
/    Purpose:
/       UNIT_NAME takes a unit number N, and returns the system-dependent
/       filename in NAME.  In many cases, the precise name of the file is
/       further specified by the numbers CNUM, ITEM, RESP, and LNUM, and the
/       string WORD.
/
/    How it works:
/       For a detailed list of the arguments required for each file unit
/       number, see the header documentation for unit_lock.
/
/    Returns: 1
/
/    Error Conditions:
/
/    Side effects:
/
/    Related functions:
/
/    Called by:
/
/    Operating system dependencies
/
/    Known bugs:
/       XSAM and file attachments have not been implemented for Primos.
/
/       AOS and Client sections are considered obsolete code and are
/       not up-to-date in many areas.
/
/    Home: unit/unitname.c
/ */

/*** Copyright (C) 1987 Camber-Roth.  All rights reserved.  */

/*: AA  7/29/88 17:55 Source code master starting point */
/*: CR  8/04/88 13:02 Add VM code (no other changes) */
/*: JV  9/08/88 14:24 Added AOS XSCK unit name. */
/*: CR  9/30/88 16:42 Server code: SRV, VM, CX. */
/*: CR 11/22/88 17:11 IM and PN changes. */
/*: JV 12/07/88 11:04 Added CA code. */
/*: CR  1/06/89 16:29 Change VM -> CVM. */
/*: CR  1/31/89 14:42 Add XCLG file for ALL systems. */
/*: JV  1/23/89 10:06 Removed AOS code, folded CA into CX.*/
/*: CR  3/03/89 14:47 CVM files must have filemode "a". */
/*: CR  3/27/89 12:09 PR code: temp files now begin with "t$" prefix. */
/*: CR  4/20/89 16:58 CVM: Make sure files have filemode. */
/*: CR  7/17/89 15:12 Add default case for switch(es). */
/*: CR  8/08/89 13:54 UNIX: call sysudnum() to get userid directory number. */
/*: CR  8/09/89 15:20 Sys V only: add XSMK. */
/*: CW  5/08/89 14:56 Add PX system code. */
/*: CW  7/20/89 11:40 Add XSPV file. */
/*: CR 10/24/89 13:23 Add 3rd diagnostic argument to sysudnum(). */
/*: CR 11/14/89 17:33 Add XSMO file. */
/*: CR  1/26/90 15:53 Made UNIX XICE file TEXT instead of USER. */
/*: JV  1/18/90 09:12 Changed AOS to ignore confid when getting dicti. */
/*: JV  2/26/90 15:54 Added XSMD and XSMO to CA. */
/*: CR  3/24/90 13:07 Comments in header point to unitlock() doc. */
/*: CR  5/05/90  1:03 VV: TEXT() macro uses file.name.version. */
/*: CR  6/11/90 15:11 CVM: always uppercase filenames! */
/*: CR  7/16/90  8:06 Add XSUG code. */
/*: CR  9/20/90 16:55 Add PS system type. */
/*: CR  1/22/91 20:23 Make CA code use userid as part of local file names. */
/*: CJ 11/22/90 16:35 Sysudnum complains only if debug on. */
/*: CR  3/03/91 13:25 Add XIBB code. */
/*: CR  4/08/91 19:07 Add new cnum argument. */
/*: JV  4/12/91 11:21 Add CaucusNet units XUXF/T XCXT XCNB. */
/*: JV  4/12/91 12:05 Add CaucusNet units XSPB XSHD/N SNF SSM SRM SPL SPU. */
/*: JV  4/12/91 12:39 Add CaucusNet units XH** XP** XR**. */
/*: JV  5/23/91 17:37 Add CaucusNet unit XCTC, chg XHST->XHI1, XHI2. */
/*: JV  6/16/91 11:35 Fix Net codes. */
/*: JV  7/10/91 17:18 Fix HOST macro. */
/*: JV  7/15/91 18:58 Add XSRP, remove XRRP. */
/*: CR  7/31/91 17:58 Add XHSV, XSSS to Unix. */
/*: CR  8/01/91 12:06 Add XPXJ. */
/*: CR  8/02/91 16:22 Add XPRH, XPRT for Unix. */
/*: CR  8/02/91 17:37 Add XHSL. */
/*: CR  8/07/91 14:14 Expand XPAK name syntax. */
/*: CR  8/08/91 22:18 XPNO -> XPNH, XPNT. */
/*: CR  8/14/91 18:38 XRXC changes. */
/*: CR  8/19/91 22:22 Add XRCC, XRCH, XRRX, XRRL to Unix. */
/*: CR  8/22/91 17:39 Trim XPAK syntax to 8 chars. */
/*: JV  8/23/91 09:30 Change filename of XRRX. */
/*: CR  9/06/91 13:10 Remove XPXC, XPXR, XRXF. */
/*: CR  9/12/91 16:04 XSSS, XSRS are in SCRIPTS directory. */
/*: JV 10/08/91 12:00 Add XSRD/XSRN to Unix. */
/*: JV 10/14/91 12:13 Add XSR2, 2nd XSRP file */
/*: JV 10/17/91 13:55 Add Audit files: XAxxx */
/*: JV 12/10/91 18:22 Add XHDD. */
/*: UL 12/11/91 20:47 Expands args 3-6 to longs. */
/*: JV 12/16/91 14:40 XHXA XPX[HTJ] XPR[HT] XPN[HT] XPAK XRXC XRRX --> STD */
/*: CR 12/27/91 18:00 Unix: XCRF, cnum needs %ld. */
/*: JV  1/20/92 11:36 Add XCUX. */
/*: DE  4/09/92 13:16 Chixified */
/*: JX  5/28/92 14:47 Strupper->chxupper. */
/*: JX  6/05/92 23:35 Fixify. */
/*: CX  6/07/92 19:03 Simplify & rationalize chix -> char stuff. */
/*: JX  6/09/92 09:56 Some code needs both userid & useridstr. */
/*: JX  6/16/92 13:37 Use WORDSTR in all CASEs, not just some. */
/*: JX  8/05/92 16:08 Add XSUP. */
/*: CX  9/17/92 12:10 Fix confid -> confidstr oops. */
/*: CI 10/08/92 17:29 Chix Integration. */
/*: CR 10/13/92 11:12 Finish adding XIMF to all system types. */
/*: CR 10/13/92 11:56 Add XISF. */
/*: JV 10/13/92 16:29 Increase MAXITEMS (& responses) to 10K for Unix, VMS. */
/*: TP 10/14/92 17:38 Add XIAL. */
/*: CR 10/30/92 14:47 Add XIOF. */
/*: JV 10/26/92 14:28 Add XHRV. */
/*: CL 12/10/92 17:07 Long ascofchx args. */
/*: CR 12/16/92 11:59 Use ascquick(flag.logfile). */
/*: CR  1/08/93 15:01 Expand uidstr to same size as wordstr. */
/*: JV  2/26/93 22:41 Add Attachment units to Unix, VMS, PC/Novell. */
/*: JV  2/26/93 22:45 Move AOS/VS near end. */
/*: JV  3/03/93 17:10 Add XSAM to Unix, VMS, PC/Novell. */
/*: JV  3/26/93 15:45 Add XUDD to Unix, VMS, PC/Novell. */
/*: JV  4/08/93 10:56 Add header. */
/*: JV  5/24/93 14:45 Add XART, XAMT, for Unix, VMS, PC/Novell. */
/*: CR  5/20/93 15:56 Clean up VV code for 10K items/resps. */
/*: CV  6/02/93 14:39 VV: Make all file names use "name.ext;ver". */
/*: CV  6/06/93 21:21 VV code, revert TEXT files back to name.ext.ver. */
/*: CR  6/08/93 16:38 Integrate XARF & VV changes. */
/*: CP  5/06/92 11:49 Add XCED,XCEN, XCFD,XCFN. */
/*: CP  7/08/93 16:58 Integrate 2.5 changes. */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"
#include "units.h"

extern struct unit_template units[];
extern struct flag_template flag;
extern Chix   confid;
extern int    debug;

FUNCTION  unit_name (name, n, cnum, item, resp, lnum, word)
   int    n;
   long   cnum, item, resp, lnum;
   char   name[];
   Chix   word;
{
   static char first_xulg = 1;
   static char xulgname[160];
   char   uidstr[200], std[16], std2[16], *strito36();
   char   cidstr[200], wordstr[200], item_str[8], resp_str[8];
   Chix   uid;
   int    year, month, day, hour, minute, sec;
   char  *bugtell();
   long   systime();

   ENTRY ("unitname", "");

   uid = chxalloc (16L, THIN, "unitname uid");

   /*** Get the regular ascii form of WORD. */
   wordstr[0] = '\0';
   if (word!=nullchix) ascofchx (wordstr, word, 0L, L(200));

   /*** Get the regular ascii form of CONFID. */
   ascofchx (cidstr, confid, 0L, L(200));

   /*** If WORD is supplied, use it for the uid as well.  Otherwise,
   /    ask the operating system for the current uid. */
   if (NOT NULLSTR(wordstr)) {
      strcpy (uidstr, wordstr);
      chxcpy (uid, CQ(wordstr));
   } else {
      sysuserid (uid);
      ascofchx  (uidstr, uid, 0L, L(40));
   }

  

/*============================Unixes=====================================*/
#if UNIX | PX | PS

#define USTUFF        cidstr, udnum, uidstr
#define USERNUM(q,n)  sprintf (name, "%s/USER%03d/%s/%s%03ld",    USTUFF, q, n)
#define USER(q)       sprintf (name, "%s/USER%03d/%s/%s",         USTUFF, q)
#define TEXT(q)       sprintf (name, "%s/TEXT%03d/%s/%s",         USTUFF, q)
#define CONFNUM(q,n)  sprintf (name, "%s/C%03ld/%s%03ld", cidstr, cnum,q, n)
#define CONF(q)       sprintf (name, "%s/C%03ld/%s",      cidstr, cnum,q)
#define SYSTNUM(q,n)  sprintf (name, "%s/MISC/%s%03ld",   cidstr,      q, n)
#define SYST(q)       sprintf (name, "%s/MISC/%s",        cidstr,      q)
#define HOST(q)       sprintf (name, "%s/H%03ld/%s",      cidstr, cnum,q)

  {

   int  udnum;

   /*** Get user directory number for those files that need it. */
   if (n==XITX  ||  n==XITY  ||  n==XICE  ||  n==XILM  ||
                (n>=XURG  &&  n< XCRD) || (n>=XAUCO && n<=XAUMO))
      udnum = sysudnum (uid, (debug ? 0 : -1), n+2000);

   switch (n) {

   /*-----------I/O files--------------------------------------------*/
      case (XIRE):
      case (XICF):
      case (XIMF):
      case (XISF):
      case (XIAL):
      case (XIOF):
      case (XIPT):  strcpy  (name, wordstr);    break;
      case (XIBB):  strcpy  (name, "xxx");      break;

      case (XITX):  TEXT    ("textmsc1");       break;
      case (XITY):  TEXT    ("textmsc2");       break;
      case (XICE):  TEXT    ("caucused");       break;  /* Used to be USER */
      case (XILM):  TEXT    ("lastmess");       break;

   /*-----------User files-------------------------------------------*/
      case (XURG):  USER    ("register");       break;
      case (XUOU):  USER    ("onlyuser");       break;
      case (XUND):  USER    ("nmsubddd");       break;
      case (XUNN):  USERNUM ("nmsub", item);    break;
      case (XUOD):  USER    ("alsubddd");       break;
      case (XUON):  USERNUM ("alsub", item);    break;
      case (XUXF):  USER    ("xactions");       break;
      case (XUXT):  USER    ("tmpxact");        break;
      case (XAUCO): USER    ("confnet.au");     break;
      case (XAUMO): USER    ("mailout.au");     break;
      case (XAUMI): USER    ("mailin.au");      break;

      case (XUDD):
         sprintf (name, "%s/USER%03d/%s", cidstr, udnum, wordstr);
         break;

      case (XUPA):
         sprintf (name, "%s/USER%03d/%s/p%03ld0000",
                        cidstr, udnum, wordstr, cnum);
         break;

      case (XUSD):  USER    ("dirsubjs");       break;
      case (XUSF):
         sprintf (name, "%s/USER%03d/%s/s%03ld%04ld",
                        cidstr, udnum, wordstr, resp, lnum);
         break;

      case (XUMD):  USER    ("dirmessg");       break;
      case (XUMF):
         sprintf (name, "%s/USER%03d/%s/m%03ld%04ld",
                        cidstr, udnum, wordstr, resp, lnum);
         break;

      case (XULG):
         if (first_xulg) {
            sysymd  (systime(), &year, &month, &day, &hour, &minute, &sec);
            sprintf (xulgname, "%s/USER%03d/%s/lg%02d%02d%02d",
                               cidstr, udnum, wordstr, year, month, day);
         }
         strcpy (name, xulgname);
         first_xulg = 0;
         break;

   /*-----------Conference files-------------------------------------*/
      case (XCMD):  CONF    ("membrddd");       break;
      case (XCMN):  CONFNUM ("membr", item);    break;
      case (XCTD):  CONF    ("titleddd");       break;
      case (XCTN):  CONFNUM ("title", item);    break;
      case (XCAD):  CONF    ("authrddd");       break;
      case (XCAN):  CONFNUM ("authr", item);    break;
      case (XCND):  CONF    ("nmsubddd");       break;
      case (XCNN):  CONFNUM ("nmsub", item);    break;
      case (XCOD):  CONF    ("alsubddd");       break;
      case (XCON):  CONFNUM ("alsub", item);    break;

      case (XCMA):  CONF    ("masteres");       break;
      case (XCIN):  CONF    ("introduc");       break;
      case (XCGR):  CONF    ("greeting");       break;
      case (XCUS):  CONF    ("userlist");       break;
      case (XCXT):  CONF    ("xaction");        break;
      case (XCUX):  CONF    ("userxact");       break;
      case (XCNB):  CONF    ("neighbor");       break;
      case (XCTC):  CONF    ("trueconf");       break;
      case (XACCI): CONF    ("netin.au");       break;
      case (XACCO): CONF    ("netout.au");      break;

      case (XCED):  CONF    ("itimeddd");       break;
      case (XCEN):  CONFNUM ("itime", item);    break;

      case (XCFD):
         if (item > 999) strito36 (item_str, item + TEN_K_OFFSET, 3);
         else            sprintf  (item_str, "%03ld", item);
         sprintf (name, "%s/C%03ld/rt%sddd", cidstr, cnum, item_str);
         break;

      case (XCFN):
         if (resp > 999) strito36 (item_str, resp + TEN_K_OFFSET, 3);
         else            sprintf  (item_str, "%03ld", resp);
         sprintf (name, "%s/C%03ld/rt%s%03ld", cidstr, cnum, item_str, item);
         break;

      case (XCRD):  CONFNUM ("direc", item);    break;
      case (XCRF):
         /*** Handle 10K Items and responses in the following way (each):
         /    Name is in pre-2.4 format (all decimal digits, iiirrrllll)
         /    Item & Response parts are in STD (base36), from
         /    A00 (12960) to GXZ (21959). */
         if (item > 999) strito36 (item_str, item + TEN_K_OFFSET, 3);
         else            sprintf  (item_str, "%03ld", item);
 
         if (resp > 999) strito36 (resp_str, resp + TEN_K_OFFSET, 3);
         else            sprintf  (resp_str, "%03ld", resp);
 
         sprintf (name, "%s/C%03ld/%s%s%04ld", cidstr, cnum,
                  item_str, resp_str, lnum);    break;

      case (XCSD):  CONF    ("dirsubjs");       break;
      case (XCSF):
         sprintf (name, "%s/C%03ld/s%03ld%04ld", cidstr, cnum, resp, lnum);
         break;

      case (XCDD): sprintf (name, "%s/C%03ld",   cidstr, cnum);   break;
      case (XCLG): sprintf (name, "%s/log%03ld", cidstr, cnum);   break;

   /*-----------System files-----------------------------------------*/
      case (XSMD):  SYSTNUM ("dicti", item);    break;
      case (XSMU):  SYSTNUM ("mxusr", item);    break;
      case (XSPC):  SYST    ("printcod");       break;
      case (XSBG):  SYST    ("bugslist");       break;
      case (XSLG):  SYST    (ascquick(flag.logfile));   break;
      case (XSCK):  SYST    ("confsmsk");       break;
      case (XSMK):  SYST    ("mkdirlk");        break;

      case (XSND):  SYST    ("namesddd");       break;
      case (XSNN):  SYSTNUM ("names", item);    break;
      case (XSPD):  SYST    ("passwddd");       break;
      case (XSPN):  SYSTNUM ("passw", item);    break;
      case (XSCD):  SYST    ("confsddd");       break;
      case (XSCN):  SYSTNUM ("confs", item);    break;
      case (XSPV):  SYST    ("privlist");       break;
      case (XSMO): sprintf (name, "%s/%s",        cidstr, wordstr);   break;
      case (XSUP): sprintf (name, "%s/%s",        cidstr, wordstr);   break;
      case (XSUG): sprintf (name, "%s/GROUPS/%s", cidstr, wordstr);   break;
      case (XSPB): sprintf (name, "%s/PARSEBAY/x%s", cidstr,
                                   strito36 (std, cnum, 7));       break;
      case (XSRP): sprintf (name, "%s/REPARSE/x%s",  cidstr,
                                   strito36 (std, cnum, 7));       break;
      case (XSR2): sprintf (name, "%s/REPARSE/x%s",  cidstr,
                                   strito36 (std, cnum, 7));       break;
      case (XSSS): sprintf (name, "%s/SCRIPTS/shipm%03ld", cidstr, cnum);
                   break;
      case (XSRS): sprintf (name, "%s/SCRIPTS/recvm%03ld", cidstr, cnum);
                   break;
      case (XSHD):  SYST    ("hostddd");        break;
      case (XSHN):  SYSTNUM ("host", item);     break;
      case (XSNF):  SYST    ("sysinfo");        break;
      case (XSSM):  SYST    ("shipmeth");       break;
      case (XSRM):  SYST    ("recvmeth");       break;
      case (XSPL):  SYST    ("xfplock");        break;
      case (XSUL):  SYST    ("xfulock");        break;
      case (XSRD):  SYST    ("routeddd");       break;
      case (XSRN):  SYSTNUM ("route", item);    break;
      case (XSAM):  SYST    (wordstr);          break;

   /*-----------Host files-----------------------------------------*/
      case (XHI1):  HOST    ("hostinfo");       break;
      case (XHI2):  HOST    ("hostinfo");       break;
      case (XHML):  HOST    ("outmail");        break;
      case (XHSL):  HOST    ("shipper");        break;
      case (XAHMI): HOST    ("mailin.au");      break;
      case (XAHMO): HOST    ("mailout.au");     break;
      case (XAHCI): HOST    ("confin.au");      break;
      case (XAHCO): HOST    ("confout.au");     break;
      case (XAHPI): HOST    ("persin.au");      break;
      case (XAHPO): HOST    ("persout.au");     break;
      case (XHDD):  sprintf (name, "%s/H%03ld",   cidstr, cnum);   break;
      case (XHSV):  sprintf (name, "%s/H%03ld/shipv%03ld", cidstr, cnum, item);
                    break;
      case (XHRV):  sprintf (name, "%s/H%03ld/recvv%03ld", cidstr, cnum, item);
                    break;
      case (XHXA):  sprintf (name, "%s/H%03ld/xact%s",  cidstr, cnum,
                                    strito36 (std, item, 4));
                    break;

   /*-----------Other CaucusLink units-------------------------------*/
      case (XPXH): sprintf (name, "%s/SHIP%03ld/H%03ld%s", cidstr,
                                   cnum, item, strito36 (std, resp, 4));
                   break;
      case (XPXT): sprintf (name, "%s/SHIP%03ld/T%03ld%s", cidstr,
                                   cnum, item, strito36 (std, resp, 4));
                   break;
      case (XPXJ): sprintf (name, "%s/SHIP%03ld/J%03ld%s.%s",
                            cidstr, cnum, item, strito36 (std,  resp, 4),
                                                strito36 (std2, lnum, 3));
                   break;
      case (XPRH): sprintf (name, "%s/SHIP%03ld/RESHIP/H%03ld%s",
                            cidstr, cnum, item, strito36 (std, resp, 4));
                   break;
      case (XPRT): sprintf (name, "%s/SHIP%03ld/RESHIP/T%03ld%s",
                            cidstr, cnum, item, strito36 (std, resp, 4));
                   break;
      case (XPNH): sprintf (name, "%s/SHIP%03ld/CANTSHIP/H%03ld%s",
                            cidstr, cnum, item, strito36 (std, resp, 4));
                   break;
      case (XPNT): sprintf (name, "%s/SHIP%03ld/CANTSHIP/T%03ld%s",
                            cidstr, cnum, item, strito36 (std, resp, 4));
                   break;
      case (XPAK): sprintf (name, "%s/SHIP%03ld/a%03ld%s",
                            cidstr, cnum, item, strito36 (std, resp, 4));
                   break;
      case (XRXC): sprintf (name, "%s/RECV%03ld/CHUNK/x%s",
                            cidstr, cnum, strito36 (std, item, 7));
                   break;
      case (XRCC): sprintf (name, "%s/RECV%03ld/chunkctr", cidstr, cnum);
                   break;
      case (XRCH): sprintf (name, "%s/RECV%03ld/chunkhis", cidstr, cnum);
                   break;
      case (XRRX): sprintf (name, "%s/RECV%03ld/RERECV/r%03ld%s",
                              cidstr, cnum, item, strito36 (std, resp, 4));
                   break;
      case (XRRL): sprintf (name, "%s/RECV%03ld/recvlock", cidstr, cnum);
                   break;

   /*-------------------File Attachments------------------------------------*/
      case (XACD): sprintf (name, "%s/ATTACH/C%03ld", cidstr, cnum);
                  break;
      case (XAUD): sprintf (name, "%s/ATTACH/USER%03d/%s",
                                   cidstr, udnum, wordstr);
                  break;
      case (XARF):
         /*** Handle 10K Item and response attachments in this way (each):
         /    Use 000-999 (decimal), then A00 (12960) to GXZ (21959). */
         if (item > 999) strito36 (item_str, item + TEN_K_OFFSET, 3);
         else            sprintf  (item_str, "%03ld", item);

         if (resp > 999) strito36 (resp_str, resp + TEN_K_OFFSET, 3);
         else            sprintf  (resp_str, "%03ld", resp);

         sprintf (name, "%s/ATTACH/C%03ld/%s%s", cidstr, cnum,
                  item_str, resp_str);          break;

      case (XART):
         /*** Only one temporary file per user. */
         sprintf (name, "%s/ATTACH/C%03ld/%s.tmp", cidstr, cnum, wordstr);
         break;

      case (XAMF):
         sprintf (name, "%s/ATTACH/USER%03d/%s/m%03ld",
                        cidstr, udnum, wordstr, resp);

      case (XAMT):
         /*** Only one temporary file per sender/receiver pair. */
         sprintf (name, "%s/ATTACH/USER%03d/%s/xxx.tmp",
                        cidstr, udnum, wordstr);
         break;

      default:
         buglist (bugtell ("Bad case in unitname!", n, item, wordstr));
         sysexit (1);
      }
   }
#endif


/*============================Vax/VMS====================================*/
#if VV

#define USERNUM(q,n) sprintf(name, "%s.USER.%s]%s%03d.;1", cidstr, uidstr, q, n)
#define USER(q)      sprintf(name, "%s.USER.%s]%s.;1",     cidstr, uidstr, q)
#define USERAU(q)    sprintf(name, "%s.USER.%s]%s.au;1",   cidstr, uidstr, q)
#define TEXT(q)      sprintf(name, "sys$login:%s.%s.1",    uidstr, q)
#define CONFNUM(q,n) sprintf(name, "%s.C%03d]%s%03d.;1",   cidstr, cnum,q, n)
#define CONF(q)      sprintf(name, "%s.C%03d]%s.;1",       cidstr, cnum,q);
#define CONFAU(q)    sprintf(name, "%s.C%03ld]%s.au;1",    cidstr, cnum,q);
#define SYSTNUM(q,n) sprintf(name, "%s.MISC]%s%03d.;1",    cidstr, q, n)
#define SYST(q)      sprintf(name, "%s.MISC]%s.;1",        cidstr, q)
#define HOST(q)      sprintf(name, "%s.H%03ld]%s.;1",      cidstr, cnum,q)
#define HOSTAU(q)    sprintf(name, "%s.H%03ld]%s.au;1",    cidstr, cnum,q)

   ENTRY ("unitname", "");

   switch (n) {

   /*-----------I/O files--------------------------------------------*/
      case (XIRE):
      case (XICF):
      case (XIMF):
      case (XISF):
      case (XIAL):
      case (XIOF):
      case (XIPT):  strcpy  (name, wordstr);    break;
      case (XIBB):  strcpy  (name, "xxx");      break;

      case (XITX):  TEXT    ("cau1");           break;
      case (XITY):  TEXT    ("cau2");           break;
      case (XICE):  USER    ("caucused");       break;
      case (XILM):  TEXT    ("cau0");           break;

   /*-----------User files-------------------------------------------*/
      case (XURG):  USER    ("register");       break;
      case (XUOU):  USER    ("onlyuser");       break;
      case (XUND):  USER    ("nmsubddd");       break;
      case (XUNN):  USERNUM ("nmsub", item);    break;
      case (XUOD):  USER    ("alsubddd");       break;
      case (XUON):  USERNUM ("alsub", item);    break;
      case (XUXF):  USER    ("xactions");       break;
      case (XUXT):  USER    ("tmpxact");        break;
      case (XAUCO): USERAU  ("confnet");        break;
      case (XAUMO): USERAU  ("mailout");        break;
      case (XAUMI): USERAU  ("mailin");         break;

      case (XUDD):
         sprintf (name, "%s.USER.%s]", cidstr, wordstr);
         break;

      case (XUPA):
         sprintf (name, "%s.USER.%s]p%03d0000.;1", cidstr, wordstr, cnum);
         break;

      case (XUSD):  USER    ("dirsubjs");       break;
      case (XUSF):
         sprintf (name, "%s.USER.%s]s%03d%04d.;1", cidstr, wordstr, resp, lnum);
         break;

      case (XUMD):  USER    ("dirmessg");       break;
      case (XUMF):
         sprintf (name, "%s.USER.%s]m%03d%04d.;1", cidstr, wordstr, resp, lnum);
         break;

      case (XULG):
         if (first_xulg) {
            sysymd  (systime(), &year, &month, &day, &hour, &minute, &sec);
            sprintf (xulgname, "%s.USER.%s]lg%02d%02d%02d.;1",
                               cidstr, wordstr, year, month, day);
         }
         strcpy (name, xulgname);
         first_xulg = 0;
         break;

   /*-----------Conference files-------------------------------------*/
      case (XCMD):  CONF    ("membrddd");       break;
      case (XCMN):  CONFNUM ("membr", item);    break;
      case (XCTD):  CONF    ("titleddd");       break;
      case (XCTN):  CONFNUM ("title", item);    break;
      case (XCAD):  CONF    ("authrddd");       break;
      case (XCAN):  CONFNUM ("authr", item);    break;
      case (XCND):  CONF    ("nmsubddd");       break;
      case (XCNN):  CONFNUM ("nmsub", item);    break;
      case (XCOD):  CONF    ("alsubddd");       break;
      case (XCON):  CONFNUM ("alsub", item);    break;

      case (XCMA):  CONF    ("masteres");       break;
      case (XCIN):  CONF    ("introduc");       break;
      case (XCGR):  CONF    ("greeting");       break;
      case (XCUS):  CONF    ("userlist");       break;
      case (XCXT):  CONF    ("xaction");        break;
      case (XCUX):  CONF    ("userxact");       break;
      case (XCNB):  CONF    ("neighbor");       break;
      case (XCTC):  CONF    ("trueconf");       break;
      case (XACCI): CONFAU  ("netin");          break;
      case (XACCO): CONFAU  ("netout");         break;

      case (XCED):  CONF    ("itimeddd");       break;
      case (XCEN):  CONFNUM ("itime", item);    break;

      case (XCFD):
         if (item > 999) strito36 (item_str, item + TEN_K_OFFSET, 3);
         else            sprintf  (item_str, "%03ld", item);
         sprintf (name, "%s.C%03ld]rt%sddd.;1", cidstr, cnum, item_str);
         break;

      case (XCFN):
         if (resp > 999) strito36 (item_str, resp + TEN_K_OFFSET, 3);
         else            sprintf  (item_str, "%03ld", resp);
         sprintf (name, "%s.C%03ld]rt%s%03ld.;1", cidstr, cnum, item_str, item);
         break;

      case (XCRD):  CONFNUM ("direc", item);    break;
      case (XCRF):
         /*** Handle 10K Items and responses in the following way (each):
         /    Name is in pre-2.4 format (all decimal digits, iiirrrllll)
         /    Item & Response parts are in STD (base36), from
         /    A00 (12960) to GXZ (21959). */
         if (item > 999) strito36 (item_str, item + TEN_K_OFFSET, 3);
         else            sprintf  (item_str, "%03ld", item);
 
         if (resp > 999) strito36 (resp_str, resp + TEN_K_OFFSET, 3);
         else            sprintf  (resp_str, "%03ld", resp);
 
         sprintf (name, "%s.C%03ld]%s%s%04ld.;1", cidstr, cnum,
                  item_str, resp_str, lnum);
         break;

      case (XCSD):  CONF    ("dirsubjs");       break;
      case (XCSF):
         sprintf (name, "%s.C%03d]s%03d%04d.;1", cidstr, cnum, resp, lnum);
         break;

      case (XCDD): sprintf (name, "%s.C%03d",         cidstr, cnum);   break;
      case (XCLG): sprintf (name, "%s]log%03d.trk;1", cidstr, cnum);   break;


   /*-----------System files-----------------------------------------*/
      case (XSMD):  SYSTNUM ("dicti", item);    break;
      case (XSMU):  SYSTNUM ("mxusr", item);    break;
      case (XSPC):  SYST    ("printcod");       break;
      case (XSBG):  SYST    ("bugslist");       break;
      case (XSLG):  SYST    (ascquick(flag.logfile));   break;
      case (XSCK):  SYST    ("confsmsk");       break;

      case (XSND):  SYST    ("namesddd");       break;
      case (XSNN):  SYSTNUM ("names", item);    break;
      case (XSPD):  SYST    ("passwddd");       break;
      case (XSPN):  SYSTNUM ("passw", item);    break;
      case (XSCD):  SYST    ("confsddd");       break;
      case (XSCN):  SYSTNUM ("confs", item);    break;
      case (XSPV):  SYST    ("privlist");       break;
      case (XSMO):  sprintf (name, "%s]%s",        cidstr, wordstr);  break;
      case (XSUP):  sprintf (name, "%s]%s",        cidstr, wordstr);  break;
      case (XSUG):  sprintf (name, "%s.GROUPS]%s", cidstr, wordstr);  break;

      case (XSPB):  sprintf (name, "%s.parsebay]x%s.;1", cidstr,
                                   strito36 (std, cnum, 7));       break;
      case (XSRP):  sprintf (name, "%s.reparse]x%s.;1",  cidstr,
                                   strito36 (std, cnum, 7));       break;
      case (XSR2):  sprintf (name, "%s.reparse]x%s.;1",  cidstr,
                                   strito36 (std, cnum, 7));       break;
      case (XSSS):  sprintf (name, "%s.scripts]shipm%03ld.;1", cidstr, cnum);
                    break;
      case (XSRS):  sprintf (name, "%s.scripts]recvm%03ld.;1", cidstr, cnum);
                    break;
      case (XSHD):  SYST    ("hostddd");        break;
      case (XSHN):  SYSTNUM ("host", item);     break;
      case (XSNF):  SYST    ("sysinfo");        break;
      case (XSSM):  SYST    ("shipmeth");       break;
      case (XSRM):  SYST    ("recvmeth");       break;
      case (XSPL):  SYST    ("xfplock");        break;
      case (XSUL):  SYST    ("xfulock");        break;
      case (XSRD):  SYST    ("routeddd");       break;
      case (XSRN):  SYSTNUM ("route", item);    break;
      case (XSAM):  SYST    (wordstr);          break;

   /*-----------Host files-----------------------------------------*/
      case (XHI1):  HOST    ("hostinfo");       break;
      case (XHI2):  HOST    ("hostinfo");       break;
      case (XHML):  HOST    ("outmail");        break;
      case (XHSL):  HOST    ("shipper");        break;
      case (XAHMI): HOSTAU  ("mailin");         break;
      case (XAHMO): HOSTAU  ("mailout");        break;
      case (XAHCI): HOSTAU  ("confin");         break;
      case (XAHCO): HOSTAU  ("confout");        break;
      case (XAHPI): HOSTAU  ("persin");         break;
      case (XAHPO): HOSTAU  ("persout");        break;
      case (XHDD):  sprintf (name, "%s.H%03ld]",   cidstr, cnum);   break;
      case (XHSV):  sprintf (name, "%s.H%03ld]shipv%03ld.;1", cidstr, cnum,
                             item);                                 break;
      case (XHRV):  sprintf (name, "%s.H%03ld]recvv%03ld.;1", cidstr, cnum,
                             item);                                 break;
      case (XHXA):  sprintf (name, "%s.H%03ld]xact%s.;1",     cidstr, cnum,
                             strito36 (std, item, 4));              break;

   /*-----------Other CaucusLink units-------------------------------*/
      case (XPXH): sprintf (name, "%s.ship%03ld]H%03ld%s.;1", cidstr,
                                   cnum, item, strito36 (std, resp, 4));
                   break;
      case (XPXT): sprintf (name, "%s.ship%03ld]T%03ld%s.;1", cidstr,
                                   cnum, item, strito36 (std, resp, 4));
                   break;
      case (XPXJ): sprintf (name, "%s.ship%03ld]J%03ld%s.%s;1",
                            cidstr, cnum, item, strito36 (std,  resp, 4),
                                                strito36 (std2, lnum, 3));
                   break;
      case (XPRH): sprintf (name, "%s.ship%03ld.reship]H%03ld%s.;1",
                            cidstr, cnum, item, strito36 (std, resp, 4));
                   break;
      case (XPRT): sprintf (name, "%s.ship%03ld.reship]T%03ld%s.;1",
                            cidstr, cnum, item, strito36 (std, resp, 4));
                   break;
      case (XPNH): sprintf (name, "%s.ship%03ld.cantship]H%03ld%s.;1",
                            cidstr, cnum, item, strito36 (std, resp, 4));
                   break;
      case (XPNT): sprintf (name, "%s.ship%03ld.cantship]T%03ld%s.;1",
                            cidstr, cnum, item, strito36 (std, resp, 4));
                   break;
      case (XPAK): sprintf (name, "%s.ship%03ld]a%03ld%s.;1",
                            cidstr, cnum, item, strito36 (std, resp, 4));
                   break;
      case (XRXC): sprintf (name, "%s.recv%03ld.chunk]x%s.;1",
                            cidstr, cnum, strito36 (std, item, 7));
                   break;
      case (XRCC): sprintf (name, "%s.recv%03ld]chunkctr.;1", cidstr, cnum);
                   break;
      case (XRCH): sprintf (name, "%s.recv%03ld]chunkhis.;1", cidstr, cnum);
                   break;
      case (XRRX): sprintf (name, "%s.recv%03ld.rerecv]r%03ld%s.;1",
                              cidstr, cnum, item, strito36 (std, resp, 4));
                   break;
      case (XRRL): sprintf (name, "%s.recv%03ld]recvlock.;1", cidstr, cnum);
                   break;

   /*-----------------File Attachments----------------------------------------*/
      case (XACD): sprintf (name, "%s.ATTACH.C%03ld]", cidstr, cnum);
                   break;
      case (XAUD): sprintf (name, "%s.ATTACH.USER.%s]",
                                  cidstr, udnum, wordstr);
      case (XARF):
         /*** Handle 10K Items and responses in the following way (each):
         /    Name is in pre-2.4 format (all decimal digits, iiirrrllll)
         /    for items and responses 1-999.
         /    Item & Response parts are in STD (base36), from
         /    a00 (12960) to gy0 (21960). */
         if (item > 999) strito36 (item_str, item + 12960, 3);
         else            sprintf  (item_str, "%03ld", item);

         if (resp > 999) strito36 (resp_str, resp + 12960, 3);
         else            sprintf  (resp_str, "%03ld", resp);

         sprintf (name, "%s.ATTACH.C%03ld]%s%s.;1", cidstr, cnum,
                  item_str, resp_str);       break;

      case (XART):
         /*** Only one temporary file per user. */
         sprintf (name, "%s.ATTACH.C%03ld]%s.tmp", cidstr, cnum, wordstr);
         break;

      case (XAMF):
         sprintf (name, "%s.ATTACH.USER.%s]m%03ld.;1", cidstr, wordstr, resp);
         break;

      case (XAMT):
         /*** Only one temporary file per sender/receiver pair. */
         sprintf (name, "%s.ATTACH.USER.%s]xxx.tmp",
                        cidstr, udnum, wordstr);
         break;

      default:
         buglist (bugtell ("Bad case in unitname!", n, (long) item, wordstr));
         sysexit (1);
   }
#endif


/*============================PRIME======================================*/
#if PR

#define USERNUM(q,n) sprintf (name, "%s>USER!>%s>%s%03d",  cidstr, uidstr, q, n)
#define USER(q)      sprintf (name, "%s>USER!>%s>%s",      cidstr, uidstr, q)
#define TEXT(q)      sprintf (name, "t$%s.%s",             uidstr, q)
#define CONFNUM(q,n) sprintf (name, "%s>C!>C%03d>%s%03d",  cidstr, cnum,q, n)
#define CONF(q)      sprintf (name, "%s>C!>C%03d>%s",      cidstr, cnum,q);
#define SYSTNUM(q,n) sprintf (name, "%s>MISC!>%s%03d",     cidstr, q, n)
#define SYST(q)      sprintf (name, "%s>MISC!>%s",         cidstr, q)
  {
   extern char primepass[], primekey[];
   char        pwstr[40], pwfile[80];
   FILE       *fp, *fopen();

   /*** If first call to UNIT_NAME, load the encrypted password
   /    from the file EPW and decrypt it. */
   if (NULLSTR(primepass)  &&  NOT NULLSTR(cidstr)) {
      sprintf  (pwfile, "%s>epw", cidstr);
      fp = fopen (pwfile, "r");
      fgets    (pwstr, 10, fp);
      fclose   (fp);
      sysclean (pwstr);
      strdecry (primepass, pwstr, primekey);
   }

   switch (n) {

   /*-----------I/O files--------------------------------------------*/
      case (XIRE):
      case (XICF):
      case (XIMF):
      case (XISF):
      case (XIAL):
      case (XIOF):
      case (XIPT):  strcpy  (name, wordstr);    break;
      case (XIBB):  strcpy  (name, "xxx");      break;

      case (XITX):  TEXT    ("cau1");           break;
      case (XITY):  TEXT    ("cau2");           break;
      case (XICE):  USER    ("caucused");       break;
      case (XILM):  TEXT    ("cau0");           break;

   /*-----------User files-------------------------------------------*/
      case (XURG):  USER    ("register");       break;
      case (XUOU):  USER    ("onlyuser");       break;
      case (XUND):  USER    ("nmsubddd");       break;
      case (XUNN):  USERNUM ("nmsub", item);    break;
      case (XUOD):  USER    ("alsubddd");       break;
      case (XUON):  USERNUM ("alsub", item);    break;

      case (XUDD):
         sprintf (name, "%s>USER!>\%s", cidstr, wordstr);
         break;

      case (XUPA):
         sprintf (name, "%s>USER!>%s>p%03d0000", cidstr, wordstr, cnum);
         break;

      case (XUSD):  USER    ("dirsubjs");       break;
      case (XUSF):
         sprintf (name, "%s>USER!>%s>s%03d%04d", cidstr, wordstr, resp, lnum);
         break;

      case (XUMD):  USER    ("dirmessg");       break;
      case (XUMF):
         sprintf (name, "%s>USER!>%s>m%03d%04d", cidstr, wordstr, resp, lnum);
         break;

      case (XULG):
         if (first_xulg) {
            sysymd  (systime(), &year, &month, &day, &hour, &minute, &sec);
            sprintf (xulgname, "%s>USER!>%s>lg%02d%02d%02d",
                               cidstr, wordstr, year, month, day);
         }
         strcpy (name, xulgname);
         first_xulg = 0;
         break;

   /*-----------Conference files-------------------------------------*/
      case (XCMD):  CONF    ("membrddd");       break;
      case (XCMN):  CONFNUM ("membr", item);    break;
      case (XCTD):  CONF    ("titleddd");       break;
      case (XCTN):  CONFNUM ("title", item);    break;
      case (XCAD):  CONF    ("authrddd");       break;
      case (XCAN):  CONFNUM ("authr", item);    break;
      case (XCND):  CONF    ("nmsubddd");       break;
      case (XCNN):  CONFNUM ("nmsub", item);    break;
      case (XCOD):  CONF    ("alsubddd");       break;
      case (XCON):  CONFNUM ("alsub", item);    break;

      case (XCMA):  CONF    ("masteres");       break;
      case (XCIN):  CONF    ("introduc");       break;
      case (XCGR):  CONF    ("greeting");       break;
      case (XCUS):  CONF    ("userlist");       break;

      case (XCED):  CONF    ("itimeddd");       break;
      case (XCEN):  CONFNUM ("itime", item);    break;

      case (XCFD):
         if (item > 999) strito36 (item_str, item + TEN_K_OFFSET, 3);
         else            sprintf  (item_str, "%03ld", item);
         sprintf (name, "%s>C!>C%03ld>rt%sddd", cidstr, cnum, item_str);
         break;

      case (XCFN):
         if (resp > 999) strito36 (item_str, resp + TEN_K_OFFSET, 3);
         else            sprintf  (item_str, "%03ld", resp);
         sprintf (name, "%s>C!>C%03ld>rt%s%03ld", cidstr, cnum, item_str, item);
         break;

      case (XCRD):  CONFNUM ("direc", item);    break;
      case (XCRF):
         /*** Handle 10K Items and responses in the following way (each):
         /    Name is in pre-2.4 format (all decimal digits, iiirrrllll)
         /    Item & Response parts are in STD (base36), from
         /    A00 (12960) to GXZ (21959). */
         if (item > 999) strito36 (item_str, item + TEN_K_OFFSET, 3);
         else            sprintf  (item_str, "%03ld", item);
 
         if (resp > 999) strito36 (resp_str, resp + TEN_K_OFFSET, 3);
         else            sprintf  (resp_str, "%03ld", resp);
 
         sprintf (name, "%s>C!>C%03ld>R%s%s%04ld", cidstr, cnum,
                  item_str, resp_str, lnum);    break;

      case (XCSD):  CONF    ("dirsubjs");       break;
      case (XCSF):
         sprintf (name, "%s>C!>C%03d>s%03d%04d", cidstr, cnum, resp, lnum);
         break;

      case (XCDD): sprintf (name, "%s>C!>C%03d",   cidstr, cnum);   break;
      case (XCLG): sprintf (name, "%s>C!>log%03d", cidstr, cnum);   break;


   /*-----------System files-----------------------------------------*/
      case (XSMD):  SYSTNUM ("dicti", item);    break;
      case (XSMU):  SYSTNUM ("mxusr", item);    break;
      case (XSPC):  SYST    ("printcod");       break;
      case (XSBG):  SYST    ("bugslist");       break;
      case (XSLG):  SYST    (ascquick(flag.logfile));   break;
      case (XSCK):  SYST    ("confsmsk");       break;

      case (XSND):  SYST    ("namesddd");       break;
      case (XSNN):  SYSTNUM ("names", item);    break;
      case (XSPD):  SYST    ("passwddd");       break;
      case (XSPN):  SYSTNUM ("passw", item);    break;
      case (XSCD):  SYST    ("confsddd");       break;
      case (XSCN):  SYSTNUM ("confs", item);    break;
      case (XSPV):  SYST    ("privlist");       break;
      case (XSMO):  sprintf (name, "%s>%s",         cidstr, wordstr);   break;
      case (XSUP):  sprintf (name, "%s>%s",         cidstr, wordstr);   break;
      case (XSUG):  sprintf (name, "%s>GROUPS!>%s", cidstr, wordstr);   break;

      default:
         buglist (bugtell ("Bad case in unitname!", n, (long) item, wordstr));
         sysexit (1);
   }

   /*** Insert decrypted passwords in pathnames that need it. */
   sprintf  (pwstr, " %s>", primepass);
   stralter (name,  "!>",   pwstr);
  }
#endif


/*============================IBM PC=====================================*/
#if IM | PN

#define USERNUM(q,n) sprintf(name, "%s\\USER\\%s\\%s%03ld", cidstr, uidstr, q,n)
#define USER(q)      sprintf(name, "%s\\USER\\%s\\%s",      cidstr, uidstr, q)
#define TEXT(q)      sprintf(name, "%s\\USER\\%s\\%s",      cidstr, uidstr, q)
#define CONFNUM(q,n) sprintf(name, "%s\\C%03ld\\%s%03ld",   cidstr, cnum,   q,n)
#define CONF(q)      sprintf(name, "%s\\C%03ld\\%s",        cidstr, cnum,   q)
#define SYSTNUM(q,n) sprintf(name, "%s\\MISC\\%s%03ld",     cidstr,         q,n)
#define SYST(q)      sprintf(name, "%s\\MISC\\%s",          cidstr,         q)

   ENTRY ("unitname", "");

   /*** If Caucus is installed in the root directory, remove the
   /    solitary "\" from cidstr. */
   if (streq (cidstr, "\\"))  cidstr[0] = '\0';

   switch (n) {

   /*-----------I/O files--------------------------------------------*/
      case (XIRE):
      case (XICF):
      case (XIMF):
      case (XISF):
      case (XIAL):
      case (XIOF):
      case (XIPT):  strcpy  (name, wordstr);    break;
      case (XIBB):  strcpy  (name, "xxx");      break;

#if IM
      case (XITX):  TEXT    ("textmsc1");       break;
      case (XITY):  TEXT    ("textmsc2");       break;
      case (XICE):  TEXT    ("caucused");       break;
      case (XILM):  TEXT    ("lastmess");       break;
#endif

#if PN
      case (XITX):  sprintf (name, "%s.t1", uidstr);   break;
      case (XITY):  sprintf (name, "%s.t2", uidstr);   break;
      case (XICE):  sprintf (name, "%s.ce", uidstr);   break;
      case (XILM):  sprintf (name, "%s.lm", uidstr);   break;
#endif

   /*-----------User files-------------------------------------------*/
      case (XURG):  USER    ("register");       break;
      case (XUOU):  USER    ("onlyuser");       break;
      case (XUND):  USER    ("nmsubddd");       break;
      case (XUNN):  USERNUM ("nmsub", item);    break;
      case (XUOD):  USER    ("alsubddd");       break;
      case (XUON):  USERNUM ("alsub", item);    break;

      case (XUDD):
         sprintf (name, "%s\\USER\\%s", cidstr, wordstr);
         break;

      case (XUPA):
         sprintf (name, "%s\\USER\\%s\\p%03ld0000", cidstr, wordstr, cnum);
         break;

      case (XUSD):  USER    ("dirsubjs");       break;
      case (XUSF):
         sprintf (name, "%s\\USER\\%s\\s%03ld%04ld", cidstr, wordstr,
                  resp, lnum);
         break;

      case (XUMD):  USER    ("dirmessg");       break;
      case (XUMF):
         sprintf (name, "%s\\USER\\%s\\m%03ld%04ld", cidstr, wordstr,
                  resp, lnum);
         break;

      case (XULG):
         if (first_xulg) {
            sysymd  (systime(), &year, &month, &day, &hour, &minute, &sec);
            sprintf (xulgname, "%s\\USER\\%s\\lg%02d%02d%02d",
                               cidstr, wordstr, year, month, day);
         }
         strcpy (name, xulgname);
         first_xulg = 0;
         break;

   /*-----------Conference files-------------------------------------*/
      case (XCMD):  CONF    ("membrddd");       break;
      case (XCMN):  CONFNUM ("membr", item);    break;
      case (XCTD):  CONF    ("titleddd");       break;
      case (XCTN):  CONFNUM ("title", item);    break;
      case (XCAD):  CONF    ("authrddd");       break;
      case (XCAN):  CONFNUM ("authr", item);    break;
      case (XCND):  CONF    ("nmsubddd");       break;
      case (XCNN):  CONFNUM ("nmsub", item);    break;
      case (XCOD):  CONF    ("alsubddd");       break;
      case (XCON):  CONFNUM ("alsub", item);    break;

      case (XCMA):  CONF    ("masteres");       break;
      case (XCIN):  CONF    ("introduc");       break;
      case (XCGR):  CONF    ("greeting");       break;
      case (XCUS):  CONF    ("userlist");       break;

      case (XCED):  CONF    ("itimeddd");       break;
      case (XCEN):  CONFNUM ("itime", item);    break;

      case (XCFD):
         if (item > 999) strito36 (item_str, item + TEN_K_OFFSET, 3);
         else            sprintf  (item_str, "%03ld", item);
         sprintf (name, "%s\\C%03ld\\rt%sddd", cidstr, cnum, item_str);
         break;

      case (XCFN):
         if (resp > 999) strito36 (item_str, resp + TEN_K_OFFSET, 3);
         else            sprintf  (item_str, "%03ld", resp);
         sprintf (name, "%s\\C%03ld\\rt%s%03ld", cidstr, cnum, item_str, item);
         break;

      case (XCRD):  CONFNUM ("direc", item);    break;
      case (XCRF):
         /*** Handle 10K Items and responses in the following way (each):
         /    Name is in pre-2.4 format (all decimal digits, iiirrrllll)
         /    Item & Response parts are in STD (base36), from
         /    A00 (12960) to GXZ (21959). */
         if (item > 999) strito36 (item_str, item + TEN_K_OFFSET, 3);
         else            sprintf  (item_str, "%03ld", item);
 
         if (resp > 999) strito36 (resp_str, resp + TEN_K_OFFSET, 3);
         else            sprintf  (resp_str, "%03ld", resp);
 
         sprintf (name, "%s\\C%03ld\\%s%s%02ld.%02ld", cidstr, cnum,
                  item_str, resp_str, lnum/100, lnum%100);    break;
         sprintf (name, "%s\\C%03ld\\%03ld%03ld%02ld.%02ld", cidstr, cnum,
                         item, resp, lnum/100, lnum%100);     break;

      case (XCSD):  CONF    ("dirsubjs");       break;
      case (XCSF):
         sprintf (name, "%s\\C%03ld\\s%03ld%04ld", cidstr, cnum, resp, lnum);
         break;

      case (XCDD): sprintf (name, "%s\\C%03ld",   cidstr, cnum);   break;
      case (XCLG): sprintf (name, "%s\\log%03ld", cidstr, cnum);   break;


   /*-----------System files-----------------------------------------*/
      case (XSMD):  SYSTNUM ("dicti", item);    break;
      case (XSMU):  SYSTNUM ("mxusr", item);    break;
      case (XSPC):  SYST    ("printcod");       break;
      case (XSBG):  SYST    ("bugslist");       break;
      case (XSLG):  SYST    (ascquick(flag.logfile));   break;
      case (XSCK):  SYST    ("confsmsk");       break;

      case (XSND):  SYST    ("namesddd");       break;
      case (XSNN):  SYSTNUM ("names", item);    break;
      case (XSPD):  SYST    ("passwddd");       break;
      case (XSPN):  SYSTNUM ("passw", item);    break;
      case (XSCD):  SYST    ("confsddd");       break;
      case (XSCN):  SYSTNUM ("confs", item);    break;
      case (XSPV):  SYST    ("privlist");       break;
      case (XSMO):  sprintf (name, "%s\\%s",         cidstr, wordstr);  break;
      case (XSUP):  sprintf (name, "%s\\%s",         cidstr, wordstr);  break;
      case (XSUG):  sprintf (name, "%s\\GROUPS\\%s", cidstr, wordstr);  break;
      case (XSAM):  SYST    (wordstr);          break;

   /*---------------------------------------------File Attachments------------*/
      case (XACD): sprintf (name, "%s\\ATTACH\\C%03ld", cidstr, cnum);
                   break;
      case (XAUD): sprintf (name, "%s\\ATTACH\\USER\\%s",
                   cidstr, udnum, wordstr);
                   break;
      case (XARF):
         /*** Handle 10K Items and responses in the following way (each):
         /    Name is in pre-2.4 format (all decimal digits, iiirrrllll)
         /    for items and responses 1-999.
         /    Item & Response parts are in STD (base36), from
         /    a00 (12960) to gy0 (21960). */
         if (item > 999) strito36 (item_str, item + 12960, 3);
         else            sprintf  (item_str, "%03ld", item);

         if (resp > 999) strito36 (resp_str, resp + 12960, 3);
         else            sprintf  (resp_str, "%03ld", resp);

         sprintf (name, "%s\\ATTACH\\C%03ld\\%s%s", cidstr, cnum,
                  item_str, resp_str);       break;

      case (XART):
         /*** Only one temporary file per user. */
         sprintf (name, "%s\\ATTACH\\C%03ld\\%s.tmp", cidstr, cnum, wordstr);
         break;

      case (XAMF):
         sprintf (name, "%s\\ATTACH\\USER\\%s\\m%03ld", cidstr, wordstr, resp);
         break;

      case (XAMT):
         /*** Only one temporary file per sender/receiver pair. */
         sprintf (name, "%s\\ATTACH\\USER\\%s\\xxx.tmp",
                        cidstr, udnum, wordstr);
         break;

      default:
         buglist (bugtell ("Bad case in unitname!", n, item, wordstr));
         sysexit (1);
   }
#endif

/*============================AOS/VS=====================================*/
#if AOS
  
#define USERNUM(q,n)  sprintf (name, "%s:USER:%s:%s%03d", cidstr, uidstr, q, n)
#define USER(q)       sprintf (name, "%s:USER:%s:%s",     cidstr, uidstr, q)
#define TEXT(q)       sprintf (name, "%s:TEXT:%s:%s",     cidstr, uidstr, q)
#define CONFNUM(q,n)  sprintf (name, "%s:C%03d:%s%03d",   cidstr, cnum,q, n)
#define CONF(q)       sprintf (name, "%s:C%03d:%s",       cidstr, cnum,q);
#define SYSTNUM(q,n)  sprintf (name, "%s:MISC:%s%03d",    cidstr,         q, n)
#define SYST(q)       sprintf (name, "%s:MISC:%s",        cidstr,         q)
 
   switch (n) {
 
   /*-----------I/O files--------------------------------------------*/
      case (XIRE):
      case (XICF):
      case (XIMF):
      case (XISF):
      case (XIAL):
      case (XIOF):
      case (XIPT):  strcpy  (name, wordstr);    break;
      case (XIBB):  strcpy  (name, "xxx");      break;
 
      case (XITX):  TEXT    ("textmsc1");       break;
      case (XITY):  TEXT    ("textmsc2");       break;
      case (XICE):  USER    ("caucused");       break;
      case (XILM):  USER    ("lastmess");       break;

#if AOS                        /* Only for non-server/client systems */
     /*-----------User files-------------------------------------------*/
      case (XURG):  USER    ("register");       break;
      case (XUOU):  USER    ("onlyuser");       break;
      case (XUND):  USER    ("nmsubddd");       break;
      case (XUNN):  USERNUM ("nmsub", item);    break;
      case (XUOD):  USER    ("alsubddd");       break;
      case (XUON):  USERNUM ("alsub", item);    break;
      case (XUPA):
         sprintf (name, "%s:USER:%s:p%03d0000", cidstr, wordstr, cnum);
         break;
      case (XUSD):  USER    ("dirsubjs");       break;
      case (XUSF):
         sprintf (name, "%s:USER:%s:s%03d%04d", cidstr, wordstr, resp, lnum);
         break;
 
      case (XUMD):  USER    ("dirmessg");       break;
      case (XUMF):
         sprintf (name, "%s:USER:%s:m%03d%04d", cidstr, wordstr, resp, lnum);
         break;
 
      case (XULG):
         if (first_xulg) {
            sysymd  (systime(), &year, &month, &day, &hour, &minute, &sec);
            sprintf (xulgname, "%s:USER:%s:lg%02d%02d%02d",
                               cidstr, wordstr, year, month, day);
         }
         strcpy (name, xulgname);
         first_xulg = 0;
         break;

    /*-----------Conference files-------------------------------------*/
       case (XCMD):  CONF    ("membrddd");       break;
       case (XCMN):  CONFNUM ("membr", item);    break;
       case (XCTD):  CONF    ("titleddd");       break;
       case (XCTN):  CONFNUM ("title", item);    break;
       case (XCAD):  CONF    ("authrddd");       break;
       case (XCAN):  CONFNUM ("authr", item);    break;
       case (XCND):  CONF    ("nmsubddd");       break;
       case (XCNN):  CONFNUM ("nmsub", item);    break;
       case (XCOD):  CONF    ("alsubddd");       break;
       case (XCON):  CONFNUM ("alsub", item);    break;
 
       case (XCMA):  CONF    ("masteres");       break;
       case (XCIN):  CONF    ("introduc");       break;
       case (XCGR):  CONF    ("greeting");       break;
       case (XCUS):  CONF    ("userlist");       break;
 
       case (XCRD):  CONFNUM ("direc", item);    break;
       case (XCRF):
         /*** Handle 10K Items and responses in the following way (each):
         /    Name is in pre-2.4 format (all decimal digits, iiirrrllll)
         /    Item & Response parts are in STD (base36), from
         /    A00 (12960) to GXZ (21959). */
         if (item > 999) strito36 (item_str, item + TEN_K_OFFSET, 3);
         else            sprintf  (item_str, "%03ld", item);
 
         if (resp > 999) strito36 (resp_str, resp + TEN_K_OFFSET, 3);
         else            sprintf  (resp_str, "%03ld", resp);
 
         sprintf (name, "%s:C%03ld:%s%s%04ld", cidstr, cnum,
                  item_str, resp_str, lnum);    break;
 
       case (XCSD):  CONF    ("dirsubjs");       break;
       case (XCSF):
          sprintf (name, "%s:C%03d:s%03d%04d", cidstr, cnum, resp, lnum);
          break;
 
       case (XCDD): sprintf (name, "%s:C%03d",   cidstr, cnum);  break;
       case (XCLG): sprintf (name, "%s:log%03d", cidstr, cnum);  break;
 
 
    /*-----------System files-----------------------------------------*/
/*        case (XSMD):  SYSTNUM ("dicti", item);    break;  */
       case (XSMD):  sprintf (name, "MISC:dicti%03d", item);  break;
       case (XSMU):  SYSTNUM ("mxusr", item);    break;
       case (XSPC):  SYST    ("printcod");       break;
       case (XSBG):  SYST    ("bugslist");       break;
       case (XSLG):  SYST    (ascquick(flag.logfile));   break;
 
       case (XSND):  SYST    ("namesddd");       break;
       case (XSNN):  SYSTNUM ("names", item);    break;
       case (XSPD):  SYST    ("passwddd");       break;
       case (XSPN):  SYSTNUM ("passw", item);    break;
       case (XSCD):  SYST    ("confsddd");       break;
       case (XSCN):  SYSTNUM ("confs", item);    break;
       case (XSCK):  SYST    ("confsmsk");       break;
       case (XSPV):  SYST    ("privlist");       break;
#endif

       default:
          buglist (bugtell ("Bad case in unitname!", n, (long) item, wordstr));
          sysexit (1);
    }
#endif


/*===================Client code!=========================*/
/*** The systems that follow are all server-based.  Note that they
/    specify names only for local files. */

#if SRV

#define SYSTNUM(q,n) sprintf(name, "%s:MISC:%s%03d",    cidstr,         q,n)
  {
   char  filemode[40];
   char *strtoken();

   switch (n) {
      case (XIBB):  strcpy  (name, "xxx");      break;

#if CVM
      case (XIRE):
      case (XICF):
      case (XIMF):
      case (XIOF):
      case (XISF):
      case (XIPT):
         strcpy (name, wordstr);
         if (strtoken (filemode, 3, name) == NULL)  strcat (name, " a");
         break;

      case (XITX):  strcpy  (name, "txtmsc1 cau a");   break;
      case (XITY):  strcpy  (name, "txtmsc2 cau a");   break;
      case (XICE):  strcpy  (name, "txtedit cau a");   break;
      case (XILM):  strcpy  (name, "lastmes cau a");   break;
#endif

#if CX | CA
      case (XIRE):
      case (XIOF):
      case (XICF):
      case (XIMF):
      case (XIPT):  strcpy  (name, wordstr);            break;
      case (XITX):  sprintf (name, "%s.t1",  uidstr);   break;
      case (XITY):  sprintf (name, "%s.t2",  uidstr);   break;
      case (XICE):  sprintf (name, "%s.ced", uidstr);   break;
      case (XILM):  sprintf (name, "%s.lm",  uidstr);   break;
#if CA
      case (XSMD):  SYSTNUM ("dicti", item);       break;
      case (XSMO):  sprintf (name, "%s:%s", cidstr, wordstr);
         printf ("XSMO: %s.\n", name); break;
#endif
#endif
      default:
         buglist (bugtell ("Bad case in unitname!", n, (long) item, wordstr));
         sysexit (1);
   }
  }
#endif

#if CVM
   chxupper (name);
#endif

   chxfree (uid);

   RETURN  (1);
}
