/*** UNIT_NEWLINE.  Display newline on user's terminal, pausing appropriately.
/
/    unit_newline();
/
/    Parameters:
/
/    Purpose:
/       Display a newline on the user's terminal.  Pause after each
/       screenful, as appropriate.
/
/    How it works:
/       If screen has a definite size, pause after that many lines
/       have been written, and prompt the user to press RETURN.
/       Overwrite the prompt with blanks so that it does not appear
/       in the text after the prompt has been answered.
/       
/    Returns:
/
/    Error Conditions:
/
/    Side effects:
/
/    Related functions:
/
/    Called by:  unit_put() only!
/
/    Operating system dependencies: none
/
/    Known bugs:
/
/    Home:  unit/unitnewl.c
/ */

/*** Copyright (C) 1992 Camber-Roth.  All rights reserved.  */

/*: CX  5/14/92 11:33 New function. */
/*: CX  6/05/92 13:16 flag.infile -> u_keybd; use unit_read. */
/*: TP 10/19/92 16:06 Use input_control() to determine source of input. */
/*: CL 12/10/92 15:22 Long chxindex args. */
/*: JV  9/26/93 13:27 Add ENTRY. */

#include <stdio.h>
#include "caucus.h"
#include "inputcon.h"

extern struct screen_template screen, used;
extern struct termstate_t     term_in_caucus;
extern int                    debug;

FUNCTION  unit_newline()
{
   static Chix ss_pause = nullchix;
   static Chix ss_blank = nullchix;
   static Chix ss_abort = nullchix;
   Chix        pause, blank, abort;
   int         save_debug;

   ENTRY ("unit_newline", "");

   /*** Actually display a newline.  After *that*, we'll figure out
   /    if we are at the end of a screenful, and what to do about it. */
   sysputnl();
   used.width = 0;

   /*** Do we need to pause after a screenful? */
   if (screen.lines==0  ||
       input_control (IC_ISUNIT | IC_SESS, nullchix) != XKEY  ||
       ++used.lines < screen.lines)
      RETURN(1);

   /*** Clear any outstanding interrupts, zero the amount of screen
   /    used, and temporarily turn off debugging. */
   sysbrkclear();
   used.lines = 0;
   save_debug = debug;
   debug      = 0;

   /*** Get three strings: "Press RETURN for more", the string of
   /    blanks used to cover it up, and what the user can type to
   /    abort further output. */
   pause = chxalloc (100L, THIN, "unit_newl pause");
   blank = chxalloc (100L, THIN, "unit_newl blank");
   abort = chxalloc ( 40L, THIN, "unit_newl abort");
   mdstr (pause, "ss_pause", &ss_pause);
   mdstr (blank, "ss_blank", &ss_blank);
   mdstr (abort, "ss_abort", &ss_abort);

   /*** Flush any outstanding input, turn off echo, and display the
   /    "press return for more" prompt. */
   sysflushin();
   systurn   (&term_in_caucus, CAUCUS_ECHO + OS_ECHO, OFF);
   unit_term (pause);

   /*** Wait for the user to press RETURN.  An 's' (ss_abort) means
   /    "act as if ^C was pressed. */
   chxclear (pause);
   if (NOT unit_read (XKEY, pause, 0L))   sysbrkset();
   if (chxindex (abort, 0L, pause) == 0)  sysbrkset();

   /*** Write over the prompt with blanks. */
   sysputstr ("\015");
   unit_term (blank);
   sysputstr ("\015");

   /*** Restore everything to normal. */
   systurn   (&term_in_caucus, CAUCUS_ECHO + OS_ECHO, ON);
   debug = save_debug;
   used.lines = 0;

   chxfree (pause);
   chxfree (blank);
   chxfree (abort);

   RETURN (1);
}
