
/*** UNIT_READ.  Read a line from file N into STR.
/
/    ok = unit_read (n, str, option);
/
/    Parameters:
/       int   ok;      (success?)
/       int   n;       (unit number of open file)
/       Chix  str;     (read line of text into here)
/       long  option;  (input option mask)
/
/    Purpose:
/       Read a line of text from the file open on unit N into STR.
/       Trailing newline, if any, is removed.
/
/    How it works:
/       Unit_read() calls unit_control() to find out what kind of
/       character set is used to read the input.
/
/       OPTION controls several input options, according to its value:
/          UREAD_MAX + N    read maximum of N characters
/          UREAD_PFK        turn on PF key mapping where appropriate
/
/       If UREAD_MAX is not specified, unit_read() chooses its own maximum
/       (see BUFMAX below).
/       
/    Returns: 1 on success
/             0 on end-of-file, interrupt, hangup, or error
/
/    Error Conditions:
/
/    Side effects:
/
/    Related functions:
/
/    Called by:  Application.
/
/    Operating system dependencies: lots
/
/    Known bugs:
/       We may wish to have unit_read() return some indication
/       of an error other than the return value 0.
/
/       UREAD_MAX option is ignored for japanese character input.
/       Someday we may wish to apply a maximum *width* limit, 
/       measured in number of single-width characters.
/
/    Home:  unit/unitread.c
/ */

/*** Copyright (C) 1983 Camber-Roth.  All rights reserved.  */

/*: CX  5/27/92 14:26 Begin chixification. */
/*: CX  5/29/92 15:18 Complete rewrite of function. */
/*: CR 10/21/92 15:19 Close all comments! */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"
#include "units.h"
#include "diction.h"

FUNCTION  unit_read  (n, str, option)
   int  n;
   Chix str;
   long option;
{
#define   BUFMAX   256
   char   buf[BUFMAX], outseq[BUFMAX];
   int    maxlen, lang, uselen, code;

   ENTRY ("unitread", "");

   chxclear (str);

   /*** Determine the max length we really want to use... */
   maxlen = (option & UREAD_MAX ? option & 0xFFFF : BUFMAX);
   uselen = (maxlen < BUFMAX ? maxlen : BUFMAX);

   /*** And the default language representation for this unit. */
   lang   = unit_control (n, UNIT_GET);

   /*** And read the line of bytes from 3270 terminal... */
#if CVM | PX | PS
   if (n==XKEY  &&  flag.fullscreen)
      vmiread (uselen, buf, (option & UREAD_PFK ? 1 : 0));
   else
#endif

   /*** ... or japanese terminal... */
   if (n==XKEY  &&  (lang==LANG_EUC || lang==LANG_SJIS)) {
      code = jixread (str, lang);
      RETURN (code > -2);
   }

   /*** ... or vanilla ascii terminal... */
   else if (n==XKEY) {
      code = sysrdline (buf, uselen);
      /*** special handling for eof stuff? Maybe not! */
      if (code <= -2)  RETURN (0);
   }

   /*** ... or disk file. */
   else {
      if (NOT sysrdfile (n, buf, uselen))  RETURN (0);
   }

   /*** Convert the bytes, as selected by the default unit i/o character
   /    representation, to chix. */
   switch (lang) {
      case (LANG_NORM):
      case (LANG_ISO):  code = chxofascii (str, buf) - 1;          break;
      case (LANG_EUC):  code = jixofeuc   (str, buf, 0, outseq);   break;
      case (LANG_SJIS): code = jixofsjis  (str, buf, 0, outseq);   break;
      default:          code = -1;                                 break;
   }

   RETURN (code >= 0);
}
