/*** UNIT_REWIND.  Rewind a unit to the beginning.
/
/  n = unit_rewind (unit);
/
/  Parameters:
/     int unit;
/
/  Purpose:
/    Unit_rewind() tells the operating system that the next unit_read()
/    from this unit should start at the beginning of the file.
/
/    Assumes that the caller has already locked and opened the
/    file of type UNIT.
/
/  How it works:
/    On most systems it calls lseek().
/    Other systems just call unit_close() and unit_view().
/    NOTE that on these systems you shouldn't call unit_rewind()
/    if you're writing.
/
/    
/    Returns 1 on success and 0 otherwise. */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved.  */

/*: JV  6/03/91 16:57 Created function. */
/*: JV  7/04/91 19:16 Declare bugtell(). */

#include <stdio.h>
#include "handicap.h"
#include "common.h"
#include "systype.h"
#include "units.h"

extern struct unit_template units[];

FUNCTION  unit_rewind (n)
   int  n;
{
   char *bugtell();
#if UNIX | IM | JM | VV | PR | PN | PX | PS
   int  stat;

   ENTRY  ("unit_rewind", "");
   if (units[n].access != LOCKED && units[n].access != READ)
      buglist (bugtell ("Unit_rewind: ", n, 0L,"file not locked & open\n"));

   stat = fseek (units[n].fp, 0L, 0);
   RETURN (stat==0);
#endif


#if SRV
   ENTRY  ("unitrewind", "");

   unit_close (n);
   unit_view (n);
   RETURN ();
#endif
}
