/*** UNIT_UNLK.   Unlock the file of type N that was locked with UNIT_LOCK. */

/*** Copyright (C) 1985 Camber-Roth.  All rights reserved.  */

/*: AA  7/29/88 17:55 Source code master starting point */
/*: CR  8/04/88 13:13 VM changes only */
/*: JV  9/02/88 11:20 Fixed 'lack of close of lockname' bug in AOS. */
/*: CR  9/21/88 15:42 Add "HU" system type for HP/UX. */
/*: CR  9/23/88 12:59 Add "IX" system type. */
/*: CR  9/28/88 12:50 #define F_ULOCK for PU. */
/*: CR  9/30/88 15:06 Add server code, "#if SRV" section. */
/*: CR 11/09/88 15:43 Add #include "errchk.h" to SRV code. */
/*: CR 11/22/88 17:25 Add new PN code, clear locknum in UNIX code. */
/*: CR 12/16/88 14:58 PN needs LK_UNLCK, not LK_UNLK. */
/*: CR  3/21/89 15:26 Add use of servername to #if SRV code. */
/*: JV  4/05/89 21:41 Added code to stop client if server is dead. */
/*: CR  4/27/89 16:16 Add lock-consistency check code. */
/*: CR  4/29/89 14:10 Break SU into SS (lockf) and SN (guardian files). */
/*: CR  6/09/89 12:21 Add "SV" system type code. */
/*: CR  7/02/89 10:41 Add use of GUARD macro for BSD locking on sys V. */
/*: CW  5/08/89 14:56 Add PX system code. */
/*: CW  5/19/89 12:51 Add XW unit codes. */
/*: CR 10/12/89 15:43 Add DU system type code. */
/*: JV  7/08/90 18:35 Add MIPS RISC/os (RO). */
/*: CR  9/20/90 16:55 Add PS system type. */
/*: CR  9/27/90 12:23 Add U3, AU system types. */
/*: CJ 11/26/90 21:46 Add LA system type. */
/*: CR  2/09/91 21:44 Split long #if line in two (SYSV1, SYSV2). */
/*: SB 11/29/90 13:04 Add DY system code*/
/*: CR  3/11/91 14:10 Add AX and UL system type codes. */
/*: JV  5/14/91 19:46 Add unittag use. */
/*: CR  6/21/91 17:38 Ensure unittag[n], n>=0 ! */
/*: JV  7/30/91 09:59 Report if file open (Unix only so far). */
/*: CR  7/30/91  0:11 Add missing ENTRY! */
/*: JV  8/13/91 09:32 Add debug.h stuff (Unix only). */
/*: CR  1/06/92 19:24 Add DP system type. */
/*: CR  1/22/92 14:45 Paren SYSV stuff. */
/*: JV  7/17/92 15:26 Add U4. */
/*: JV  8/18/92 12:57 Report if file open (VMS). */
/*: CR  8/21/92 13:37 Move MV into UNISTD group. */
/*: DE  4/09/92 13:11 Chixified */
/*: CI 10/08/92 10:20 Chix Integration. */
/*: CP 11/21/93 19:09 Test debug anded with DEB_LOCK. */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"

/*** SysV types use lockf() routine.  UNISTD types use lockf() *and*
/    need <unistd.h> included. */
#define  SYSV1   A3 | LA | SV | FU | MU | SX | SS | TX | HU | IX
#define  SYSV2   PX | DU | RO | PS | U3 | AU | DY | AX | DP | U4 | MV
#define  SYSV3   PU | TX
#define  SYSV    (SYSV1 | SYSV2 | SYSV3)
#define  UNISTD  (SYSV1 | SYSV2)

#if UNISTD
#include <unistd.h>
#endif

#if PU
#define  F_ULOCK   0      /* Unlock */
#endif

#if PN
#include <sys\locking.h>
#endif

#include "units.h"

extern struct unit_template units[];
extern char                *unittag[];
extern int                  debug;
extern Chix                 confid;

FUNCTION  unit_unlk (n)
   int  n;
{
   char  *bugtell();

/*-------------------------------Server-based systems--------------------*/
#if SRV
#include "actions.h"
#include "errchk.h"
   extern int   vcernum;
   extern char *vcernam;
   extern Flag  localfile[],
                shouldlock[];
   extern Chix  servername;
   short  class;

   ENTRY ("unitunlk", "");

   if (debug & DEB_LOCK)  bugtell ("Unlk", n, 0L, (n>=0 ? unittag[n] : ""));

   if (n <= XITT)  RETURN (1);

   if (units[n].access != LOCKED)
      buglist (bugtell ("Unitunlk: ", n, 0L, "Unlocking unlocked file!\n"));

   if (localfile[n]  ||  NOT shouldlock[n]) {
      units[n].access = UNLKED;
      RETURN (1);
   }

   class = AUNLK + n;
   if (NOT vcsend (servername, class, NULL, 0, NULL, 0)) {
      ERRCHK ("unitunlk");
      unit_write (XWER, CQ("Caucus Server has terminated.\n"));
      unit_write (XWER, CQ("Caucus is now aborting.\n"));
      sysexit (0);
 }
#endif

 
/*---------------------------AOS, MV/UX-------------------------------*/
#if AOS
   extern Flag              shouldlock[];
   char file[80];

   ENTRY ("unitunlk", "");

   if (debug & DEB_LOCK)  bugtell ("Unlk", n, 0L, (n>=0 ? unittag[n] : ""));

   if (n <= XITT)  RETURN (1);

   if (units[n].access != LOCKED)
      buglist (bugtell ("Unitunlk: ", n, 0L, "Unlocking unlocked file!\n"));

   if (NOT shouldlock[n]) { units[n].access = UNLKED;   RETURN (1); }
   close (units[n].locknum);

   strconc (file, units[n].nameis, "_l");
   unlink  (file);
#endif

/*--------------------------Most UNIXes----------------------------------*/
#if (SYSV & (1-GUARD)) | PN
   extern struct locktype_t locktype[];
   extern Flag              shouldlock[];
   int    fd;

   ENTRY ("unitunlk", "");
   if (debug & DEB_LOCK)  bugtell ("Unlk", n, 0L, (n>=0 ? unittag[n] : ""));

   if (n <= XITT)  RETURN (1);

   if (units[n].access == UNLKED)
      buglist (bugtell ("Unitunlk: ", n, 0L, "Unlocking unlocked file!\n"));
   else if (units[n].access != LOCKED)
      buglist (bugtell ("Unitunlk: ", n, 0L, "Unlocking open file!\n"));

   if (NOT shouldlock[n]  ||  units[n].locknum < 0) {
      units[n].access = UNLKED;
      RETURN (1);
   }

   fd = locktype [ units[n].locktype ].fd;
   lseek (fd, units[n].locknum, 0);

#define LK1  A3 | LA | SV | FU | MU | MV | PU | SX | SS | HU | IX
#define LK2  PX | DU | RO | PS | U3 | AU | DY | AX | U4
#if LK1 | LK2
   lockf (fd, F_ULOCK, 1L);
#endif

#if TX
   locking (fd, 0, 1L);
#endif

#if PN
   locking (fd, LK_UNLCK, 1L);
#endif

   units[n].locknum = -1L;
#endif

/*---------------------------------------IBM PC--------------------------*/
#if IM | JM
   ENTRY ("unitunlk", "");
   if (debug & DEB_LOCK)  bugtell ("Unlk", n, 0L, (n>=0 ? unittag[n] : ""));
   if (n <= XITT)  RETURN (1);

   if (units[n].access != LOCKED)
      buglist (bugtell ("Unitunlk: ", n, 0L, "Unlocking unlocked file!\n"));

   if (n == XSMU)  close ( (int) units[n].locknum);
#endif

/*---------------------------------------VAX-----------------------------*/
#if VV
   extern Flag              shouldlock[];

   ENTRY ("unitunlk", "");

   if (debug & DEB_LOCK)  bugtell ("Unlk", n, 0L, (n>=0 ? unittag[n] : ""));

   if (n <= XITT)  RETURN (1);

   if (units[n].access != LOCKED)
      buglist (bugtell ("Unitunlk: ", n, 0L, "Unlocking unlocked file!\n"));

   if (NOT shouldlock[n]) { units[n].access = UNLKED;   RETURN (1); }

   sys$deq (units[n].locknum, 0, PSL$C_USER, 0);
#endif

/*---------------------------------------PRIME---------------------------*/
#if PR
   extern  Flag  shouldlock[];
   fortran clos$a();
   short   funit;

   ENTRY ("unitunlk", "");

   if (debug & DEB_LOCK)  bugtell ("Unlk", n, 0L, (n>=0 ? unittag[n] : ""));

   if (n <= XITT)  RETURN (1);

   if (units[n].access != LOCKED)
      buglist (bugtell ("Unitunlk: ", n, 0L, "Unlocking unlocked file!\n"));

   if (NOT shouldlock[n]) { units[n].access = UNLKED;   RETURN (1); }

   /*** Close the guardian file.  This will let someone else open
   /    it for "locking".  Don't delete the file now; UNIT_WIPE will
   /    take care of it later. */
   funit = units[n].locknum;
   clos$a (&funit);
#endif

/*-----------------------------Berkeley Unix--------------------------*/
#if BU | SN | GUARD | UL
#if BU | SN
#include <sys/file.h>
#endif
   extern Flag  shouldlock[];
   char file[80];

   ENTRY ("unitunlk", "");

   if (debug & DEB_LOCK)  bugtell ("Unlk", n, 0L, (n>=0 ? unittag[n] : ""));

   if (n <= XITT)  RETURN (1);

   if (units[n].access == UNLKED)
      buglist (bugtell ("Unitunlk: ", n, 0L, "Unlocking unlocked file!\n"));
   else if (units[n].access != LOCKED)
      buglist (bugtell ("Unitunlk: ", n, 0L, "Unlocking open file!\n"));

   if (NOT shouldlock[n]) { units[n].access = UNLKED;   RETURN (1); }

#if BU
   flock   ( (int) units[n].locknum, LOCK_UN);
#endif
   close   ( (int) units[n].locknum);
   strconc (file, units[n].nameis, "_l");
   unlink  (file);
#endif

   units[n].access = UNLKED;
   RETURN (1);
}
