/*** UNIT_VIEW.  Open the file of TYPE for read only.
/
/    Returns 1 on success, 0 on failure.
/ */

/*** Copyright (C) 1984 Camber-Roth.  All rights reserved.  */

/*: AA  7/29/88 17:55 Source code master starting point */
/*: CR  8/04/88 13:16 VM changes only */
/*: CR  8/24/88 13:26 Add diagnostics if VV unitview fails. */
/*: JV  9/02/88 11:21 Added debug to UNIX and AOS code.*/
/*: CR  9/21/88 14:48 Declare "extern int errno" in UNIX code. */
/*: CR  9/21/88 14:48 Add missing ";" in 2nd line of UNIX code. */
/*: CR  9/30/88 15:19 Add server code in "#if SRV" section. */
/*: CR 11/09/88 15:43 Add #include "errchk.h" to SRV code. */
/*: CR 12/07/88 15:14 VV code only: include <errno.h> */
/*: JV  2/10/89 15:46 Casted vcsend parms in SRV. */
/*: CR  3/21/89 15:26 Add use of servername to #if SRV code. */
/*: CR  3/22/89 12:23 Use MEMBUF for server data transfer buffering. */
/*: JV  4/05/89 21:44 Added code to stop client if server is dead. */
/*: CR  4/27/89 16:23 Add lock-consistency check code. */
/*: CR  6/20/89 17:37 Add correct decl of bugtell(). */
/*: CR  7/18/89 10:11 If backup found, report if original found. */
/*: CW  5/08/89 14:56 Add PX system code. */
/*: CW  5/19/89 12:51 Add XW unit codes. */
/*: CW  6/08/89 11:08 Use bugtell() instead of perror(). */
/*: CR 11/14/89 17:10 VV code: Turn sysprv off for XIRE *and* XICF. */
/*: JV  2/27/90 08:22 SRV code: better error handling. */
/*: JV  3/14/90 15:49 Removed use of shouldlock. */
/*: WC  3/23/90 12:16 Surround fopen's with sysprv(0/1). */
/*: CR  9/20/90 16:55 Add PS system type. */
/*: CR  9/26/90 12:19 Make Lint happy when not SRV. */
/*: CJ 11/26/90 21:46 Add LA system type. */
/*: CR  4/23/91 16:13 Add ultrix set-gid fixes. */
/*: JV  5/24/91 16:14 Make units[n].access check smarter. */
/*: JV  8/13/91 09:29 Add debug.h code to Unix (only) */
/*: JV  8/27/92 13:36 Fix VV errno handling. */
/*: DE  4/09/92 13:11 Chixified*/
/*: CI 10/08/92 10:28 Chix Integration. */
/*: CL 12/11/92 15:52 Long sysmem arg. */
/*: CP  8/16/93 15:59 Set units[n].fd to -1. */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"
#include "units.h"

#if SRV
#include "actions.h"
#include "errchk.h"
extern int   vcernum;
extern char *vcernam;
extern Flag  localfile[];
extern Chix  confid;
extern char  servername[];
#endif

#if VV
#include <errno>
#endif

extern struct unit_template  units[];
extern int                   debug;

FUNCTION  unit_view (n)
   int  n;
{
   char  backup[80];
   char *sysmem();
   int   file_exists, bkup_exists;
   FILE *fopen();
   char  format[160];
   char *bugtell();

   ENTRY ("unitview", "");
 
   if (debug & DEB_FILE)
      bugtell ("VIEW", n, (long) units[n].access, units[n].nameis);

   if (units[n].access != LOCKED) {
      if (units[n].access == READ)
         buglist (bugtell ("Unitview: ", n, 0L, "file already open\n"));
      else
         buglist (bugtell ("Unitview: ", n, 0L, "file not locked!\n"));
      RETURN  (0);
   }

#if SRV
   if (localfile[n]) {
      if ( (units[n].fp = fopen (units[n].nameis, "r")) == NULL) {
        if (debug & DEB_FILE)
           buglist (bugtell ("Unitview: ", n, 0L, "couldn't open.\n"));
        RETURN (0); }
   }
   else {
      /*** Allocate space for text sent back by server. */
      short class;
      char  stat;
      if ( (units[n].buffer = sysmem (L(MEMBUF))) == NULL)  {
         buglist (bugtell ("Unitview: ", n, 0L, "couldn't get memory.\n"));
         RETURN(0);
      }
      units[n].p = units[n].buffer;
     *units[n].p = '\0';

      class = AVIEW + n;
      if (NOT vcsend (servername, class, NULL, 0, (char *) &stat, 1)) {
         ERRCHK("unitview");
         unit_write (XWER, CQ("Caucus Server has terminated.\n"));
         unit_write (XWER, CQ("Caucus is now aborting.\n"));
         sysexit (0);
      }
      if (NOT stat) {
         sysfree (units[n].buffer);
        if (debug & DEB_FILE)
           buglist (bugtell ("Unitview: ", n, 0L, "couldn't open.\n"));
        RETURN (0); }
   }
#endif


#if UNIX | IM | JM | PN | PR | VV | PX | PS

   /*** Ensure Caucus privilege is not used for uncontrolled file access. */
   if (n < XURG)  sysprv(0);
   if (NOT sysaccess (units[n].nameis))  {
      if (n < XURG)  sysprv(1);
      RETURN (0);
   }

   /*** If a backup file exists, something aborted while writing a file.
   /    Restore the backup to its proper place, and continue. */
   sysbackup (backup, units[n].nameis);
   bkup_exists = syscheck (backup);
   file_exists = (bkup_exists ? syscheck (units[n].nameis) : 1);

   if (bkup_exists) {
      if (n < XURG)  sysprv(1);
      sprintf (format, "%s\n  %d '%s'\n",
        "Unitview: abort found, backup file used.", n, units[n].nameis);
      buglist (format);
      if (file_exists) {
         sprintf (format, "%s\n    %d %s\n",
            "Unitview: abort addendum: original file found: ", n,
            units[n].nameis);
         buglist (format);
      }
      if (n < XURG)  sysprv(0);
      sysunlink (units[n].nameis);
      sysrename (backup, units[n].nameis);
   }

/*-----------------------FORTUNE & MASSCOMP & A3 | LA-------------------------*/
#if UNIX | AOS | PX | VV | PS
   units[n].fp = fopen (units[n].nameis, "r");
   units[n].fd = -1;
   if (n < XURG)  sysprv(1);
   if (units[n].fp == NULL) {
      if (debug & DEB_FILE) {
#if !VV
         extern int errno;
#endif
         bugtell ("unitview: open failure", errno, 0L, units[n].nameis);
      }
      RETURN (0);
   }
#endif
 

/*---------------------------------------IBM PC--------------------------*/
#if IM | JM | PN
   units[n].fp = fopen (units[n].nameis, "r");
   units[n].fd = -1;
   if (n < XURG)  sysprv(1);
   if (units[n].fp == NULL) {
      if (debug)
         bugtell ("unitview: open failure", 0, 0L, units[n].nameis);
      RETURN (0);
   }
#endif


/*-----------------------NEW PRIME--------------------------------------*/
#if PR
   {
      fortran tsrc$$();
      extern  struct buf_template bufhead;
      struct  buf_template *p, *q;
      short   key, funit, type, code, chrpos[2];
      char   *malloc();

      /*** Open the file with tsrc$$. */
      key = KREAD + KGETU;
      chrpos[0] = 0;
      chrpos[1] = strlen(units[n].nameis);
      tsrc$$ (&key, (char []) units[n].nameis, &funit, chrpos, &type, &code);
      if (n < XURG)   sysprv(1);
      if (code != 0)  RETURN(0);
      units[n].fd = funit;

      /*** Allocate a read buffer for this file, and initialize it. */
      q = (struct buf_template *) malloc (sizeof(bufhead));
      q->next   = NULL;
      q->buf    = malloc (258);
      q->buflen = 0;
      q->bufptr = 0;
      q->n      = n;

      /*** Find the end of the linked list of read buffers,
      /    and attach this read buffer to the end of the list. */
      for (p=&bufhead;   p->next != NULL;   p = p->next) ;
      p->next   = q;
   }
#endif
#endif

   units[n].access = READ;
   RETURN (1);
}
