/*** UNIT_WIPE.  Clear specific types of UNIT I/O.
/
/    Unit_wipe removes all locks of a particular TYPE established
/    by unit_lock().  Unit_wipe is generally called immediately
/    after leaving a conference, or on exiting Caucus.
/
/    Generally, there should be no need to call unit_wipe(),
/    in that every unit_lock() has it's corresponding unit_unlk().
/    However, some systems, such as Primos, leave guardian lock
/    files lying around that ought to be cleaned up.  Unit_wipe
/    also does this cleanup.
/
/    TYPE must be one of:
/       LCNF  --  clean up locks associated with a conference
/       LUSR  --  clean up locks associated with a user
/ */

/*** Copyright (C) 1985 Camber-Roth.  All rights reserved.  */

/*: AA  7/29/88 17:55 Source code master starting point */
/*: CR  9/21/88 15:42 Add "HU" system type for HP/UX. */
/*: CR  9/23/88 12:59 Add "IX" system type. */
/*: CR 11/22/88 17:41 Add new PN code. */
/*: CR 12/16/88 14:59 PN needs extern units structure. */
/*: CR  4/29/89 14:10 Break SU into SS (lockf) and SN (guardian files). */
/*: CR  6/09/89 12:21 Add "SV" system type code. */
/*: CR  7/02/89 10:41 Add use of GUARD macro for BSD locking on sys V. */
/*: CR  7/14/89 18:00 Do not unlink lockmap files for SYSV! */
/*: CW  5/08/89 14:56 Add PX system code. */
/*: CR 10/12/89 15:43 Add DU system type code. */
/*: CR  6/21/90 12:23 Add type argument, new header comments. */
/*: JV  7/08/90 18:59 Add MIPS RISC/os (RO). */
/*: CR  9/20/90 16:55 Add PS system type. */
/*: CR  9/27/90 12:23 Add U3, AU system types. */
/*: CJ 11/26/90 21:46 Add LA system type. */
/*: CR  2/09/91 21:44 Split long #if line in two (SYSV1, SYSV2). */
/*: SB 11/30/90 11:06 Add DY system code*/
/*: CR  3/11/91 14:10 Add AX and UL system type codes. */
/*: CR  4/08/91 19:18 Use new confnum argument to unit_name(). */
/*: UL 12/11/91 20:12 Make unit_name args 3-6 long.*/
/*: CR  1/06/92 18:26 Add DP system type. */
/*: JV  7/17/92 15:27 Add U4. */
/*: DE  4/09/92 13:10 Chixified */
/*: CX  6/05/92 17:28 Include handicap.h to get L(). */
/*: CI 10/08/92 10:30 Chix Integration. */
/*: CL 12/10/92 17:07 Long ascofchx args. */

#include <stdio.h>
#include "caucus.h"
#include "handicap.h"
#include "systype.h"
#include "units.h"

#define SYSV1  A3 | LA | SV | FU | MU | MV | PU | SX | SS | TX | HU | IX
#define SYSV2  PX | DU | RO | PS | U3 | AU | DY | AX | DP | U4

extern short confnum;

FUNCTION  unit_wipe (type)
   int    type;
{

/*-------------------------------FORTUNE & MASSCOMP & A3-----------------*/
#if (SYSV1 | SYSV2) & (1-GUARD)
   extern struct locktype_t locktype[];

   ENTRY ("unitwipe", "");

   if (locktype[type].fd >= 0) {
      close (locktype[type].fd);
      locktype[type].fd = -1;
   /* Infamous corrupted masteres file */
/*    unlink (locktype[type].file); */
      locktype[type].nconf = locktype[type].user[0] = 0;
   }
   RETURN (1);
#endif

/*-------------------------------IBM PC DOS 3.x Network------------------*/
#if PN
   extern struct unit_template  units[];
   extern struct locktype_t     locktype[];

   ENTRY ("unitwipe", "");

/*
   for (i=0;   i<XLAST;   ++i)
      if (units[i].locknum > 0)   unit_unlk (i);
*/

   if (locktype[type].fd >= 0) {
      close (locktype[type].fd);
      locktype[type].fd = -1;
      locktype[type].nconf = locktype[type].user[0] = 0;
   }
   RETURN (1);
#endif

/*-------------------------------PRIMOS---------------------------------*/
#if PR
   extern Chix confid;
   extern char primepass[];

   Chix   userid;
   char   dir[90], useridstr[40], confidstr[200];

   ENTRY ("unit_wipe", "");

   userid      = chxalloc (80L, THIN, "unitwipe userid");
   ascofchx (confidstr, confid, 0L, 200L);

   /*** For PRIMOS, UNIT_WIPE deletes all of the guardian files left
   /    lying around, for this conference and for this user. */
   if (type == LCNF) {
      unit_name (dir, XCDD, L(confnum), 0L, 0L, 0L, nullchix);
      rmlocks   (dir);
   }

   if (type == LUSR) {
      sysuserid (userid);
      ascofchx  (useridstr, userid, 0L, 40L);
      sprintf   (dir, "%s>USER %s>%s", confidstr, primepass, useridstr);
      rmlocks   (dir);
   }

   RETURN (1);
}

FUNCTION  rmlocks (dir)
   char   dir[];
{
   char   file[40], pathname[90];
   char  *ptr, *lsdir();
   int    fd;

   if ( (fd = open (dir, 0)) < 0) {
      strconc (pathname, "unitwipe: ", dir);
      buglist (pathname);
      return;
   }

   while ( (ptr = lsdir(fd)) != NULL) {
      if (*ptr=='_') {
         sprintf (pathname, "%s>%s", dir, ptr);
         delete  (pathname);
      }
   }
   close (fd);
#endif
}
