/*** DISKFREE.  This function checks to see how much disk space is left
/    on the disk (or partition) containing PATH.
/    It returns a long integer which is the number of disk blocks left.
/    We assume that each OS version here, together with the MINDISK
/    macro in sysdist.h, take care of the knowledge of how big a block is.
/
/    Since checking free disk space is very expensive on some OS's,
/    we don't really check each time.  The Distributed Caucus programs
/    check disk very frequently.  So, instead, the caller puts a
/    zero into ROUND on the initial DISKFREE, and then increments
/    this value for each 'round' of calls to diskfree.
/    DISKFREE can decide whether or not it's worth it to actually
/    figure out the disk size this time, or just return the previous
/    value it returned (which seemed to be big enough last time...).
/
/    At some point in the future we may track how much disk the program
/    has used up since the last call and gamble on that.
/
/   NOTE: The Sun code has been changed to work over a network.
/    The other code (and new code) should take network-mounts into
/    consideration, but I'm not sure if the statfs() call is available
/    in System V et al.
*/

/*: JV  3/02/91 19:04 Created function. */
/*: JV  6/28/91 13:28 Pre-Integration CARVMark*/
/*: JV  7/22/91 09:15 Added non-system RETURN (SUCCESS). */
/*: CR  7/24/91 22:04 Make Lint happier. */
/*: JV 11/13/91 13:02 Make Sun code work over NFS. */

#include <stdio.h>
#include "systype.h"
#include "handicap.h"
#include "xfp.h"
#include "entryex.h"

#define SYSV A3 | SX | TX 
#if SYSV
#include <sys/types.h>
#include <sys/stat.h>
#include <ustat.h>
#endif

#if SU
#include <sys/types.h>
#include <sys/vfs.h>
#endif

FUNCTION long diskfree (round, path)
int round;
char *path;
{
#if SYSV
   char bugstr[80];
   struct stat  statbuf;
   struct ustat ustatbuf;
   ENTRY ("diskfree", "");
   
   if (stat (path, &statbuf)) { /* Succeed and complain if bad directory */
      sprintf (bugstr, "Sysdiskfull: bad directory: %s.\n", path);
      bugstr [79]='\0'; bug (bugstr, 0);
      RETURN (0);
   }
   if (ustat (statbuf.st_dev, &ustatbuf)) {      /* Succeed and complain if */
      sprintf (bugstr, "Sysdiskfull: bad device: %s.\n", path);  /* confusd */
      bugstr [79]='\0'; bug (bugstr, 0);
      RETURN (0);
   }
   RETURN (ustatbuf.f_tfree);
   /*NOTREACHED*/
#endif

#if SU
   char bugstr[80];
   struct statfs statbuf;

   ENTRY ("Deiskfree", "");

   if (statfs (path, &statbuf)) {
      sprintf (bugstr, "Sysdiskfull: bad device: %s.\n", path);  /* confusd */
      bugstr [79]='\0'; bug (bugstr, 0);
      RETURN (0);
   }
   RETURN (statbuf.f_bavail);
   /*NOTREACHED*/
#endif

   RETURN (MINDISK+1); 
}

