/*** CAUCHK.   Caucus Check program.    CV2.
/
/    Check activity on a set of conferences, and display a brief report
/    to the user.
/
/    CAUCHK is eminently suited for use in a login-file, so that the
/    user can check on activity in his/her conferences each time the
/    user logs in.  CAUCHK is invoked as:
/
/         CAUCHK   caucusid   {userid | -}  [servername]  [options]
/
/    where
/      caucusid   is the path name of the Caucus user id,
/                 e.g. "/usr/caucus" under Unix.
/
/      userid     if specified, checks the activity for that user.
/
/      options    are command line options as parsed by argchk().
/
/    CAUCHK automatically determines which conferences the
/    user is a member of, and checks the status of each of them.   */

/*** Copyright (C) 1988 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:57 Source code master starting point */
/*: CR 11/10/88 13:39 Move sysexit() to top. */
/*: AM  2/07/89 10:42 Jcaucus changes*/
/*: CR  4/05/89 17:32 Integration with Jcaucus. */
/*: JV  8/13/89 20:59 Added new argchk(), optchk(); mssgd commandline uname. */
/*: CW  7/26/89 13:35 Rearrange command line parsing. */
/*: CR 10/09/89 11:35 Use XW unit codes. */
/*: CR  4/10/91 15:04 Add term_original, term_in_caucus initialization. */
/*: CR  7/02/91 17:51 globals() is now mainglobals(). */
/*: CR  7/24/91 22:06 Make Lint happier with return. */
/*: CR  7/30/91  0:18 Comment out ENTRY, not needed in main. */
/*: JV 12/16/91 16:44 Remove pdir def. */
/*: JV  8/30/92 12:59 Check return from mainglob, add utilglob and mdglob. */
/*: JX  7/22/92 17:00 Chixify, add chx_init(). */
/*: CI 10/07/92 22:10 Chix Integration. */
/*: CR 11/16/92 14:24 Exit immediately if this user is not registered. */

#include "caucus.h"
#include "systype.h"
#include "sysinit.h"
#include "text.h"

extern struct screen_template  used, screen;
extern struct flag_template    flag;
extern struct termstate_t      term_original, term_in_caucus;
extern Chix                    confid, confname;
extern Chix                    servername;
extern short                   confnum;
extern union  null_t           null;

FUNCTION  main (argc, argv)
   int    argc;
   char  *argv[];
{
   Chix   name;
   int    i;
  
   SYSINIT;

/* ENTRY ("main", ""); */

   screen.lines =  0;    used.lines = 0;
   screen.width = 80;    used.width = 0;

   name =   chxalloc (40L, THIN, "cauchk name");
   confid = chxalloc (40L, THIN, "cauchk confid");
   confname = chxalloc (40L, THIN, "cauchk confname");

   /*** Get the conference id string from the first argument. */
   if (argc <  2)   exit (1);
   chxofascii (confid, argv[1]);
   confnum = 0;
   i = 2;

   /*** Get userid if specified. */
   if (i < argc  &&  *argv[i] != '-') {
      flag.login = 1;
      strcpy (flag.userid, argv[i]);
      ++i;
   }
   sysuserid (name);

   /*** Make sure this user exists! */
   if (NOT sysudnum (name, -1, 0))  return (0);

   /*** Get servername if it's there. */
   if (i < argc  &&  streq (argv[i], "-"))  ++i;
   if (i < argc  &&  *argv[i] != '-') {
      chxofascii (servername, argv[i]);
      ++i;
   }

   /*** Initialize break-catching and any remaining system-dependent
   /    stuff.   Process command-line flags.   Verify expiration date
   /    and cpu identity. */
   systurn (&term_original,  GET_TERM, 0);
   systurn (&term_in_caucus, GET_TERM, 0);
   argchk  (argc, argv, i);
   utilglobals ();
   optchk();         /* Process options file */

   /*** Choose Internal Text Type. */
   if (flag.japan) chx_init (JIX);
   else            chx_init (ASCII);

   sysexit (-1);
   sysglobals();
   unitglobals();
   if (NOT mainglobals()) sysexit(1);
   sysbrkinit();
  
   /*** Check mail for this user. */
   mailcheck (name);

   /*** Check all the conferences for this user. */
   if (NOT allconfs (name)  &&  NOT sysbrktest())
      mdwrite (XWTX, "chk_Tnonew", null.md);

   chxfree (name);

   sysexit (0);
   return  (0);
}
