
/*** Make Master Dictionary.  Compile dictionary source file to binary.
/
/    Invoked as:
/       makemd  dicroot  textfile  number  [options]
/
/    Arguments:
/       dicroot    initial part of binary file name, e.g. "/u/caucus/MISC/dicti"
/       textfile   top level dictionary file, e.g "diction0".
/       number     number of resulting dictionary.
/       options    -v turns on verbose output.
/                  -d turns on full debug mode.
/
/    Purpose:
/       Compile a dictionary source text file (and all of its ":included" files)
/       into the binary form of a Caucus dictionary.
/
/    How it works:
/       The output file has two sections: a hash table of mdrec_t's, and
/       a text area.  As each string is read from TEXTFILE (and its 
/       :included children), its text is added to the end of the text area.
/       When the string is complete, an entry is made in the hash table,
/       including a pointer to where the string begins in the text area.
/
/    Exit status:
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  xdiction/makemd.c
/ */

/*** Copyright (C) 1987 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:57 Source code master starting point */
/*: CR  8/04/88 15:00 Change arguments & invokation of MAKEMD */
/*: JV  8/02/88  9:52 Added '(long) sizeof(mdrec) for portability. */
/*: JV  8/02/88  9:52 Added (char *) on mdwrite for portability. */
/*: CR 09/27/88 15:26 Correct "Usage: ..." message. */
/*: JV 11/30/88 13:54 Changed printf on dic.open.fail to point to rite place!*/
/*: CR  1/12/89 11:30 Generalize file names with MDNAME(). */
/*: AM  2/07/89 10:42 Jcaucus changes*/
/*: JV  2/02/89 16:45 Added debug. */
/*: CR  4/12/89 12:27 Remove jargchk(); not needed! */
/*: CR  4/17/89 14:36 Change sysrename() --> dicrename(). */
/*: CS 11/30/90 13:39 Add top_level arg to procfile() call. */
/*: CR  6/07/91 17:19 Replace mdwrite() with mdput(). */
/*: CX  4/10/92 17:32 Begin chixification. */
/*: CP  8/09/93 12:19 Debug also turns on verbose. */

int debug;

#include <stdio.h>
#include "caucus.h"
#include "diction.h"

FUNCTION  main (argc, argv)
   int    argc;
   char  *argv[];
{
   FILE  *in, *fopen();
   static struct mdrec_t mdrec;
   int    i, fd, number, verbose;
   long   text;
   int    strings, collisions;
   char   temp[80], real[80], textfile[80];

   if (argc < 4) {
      printf ("Usage: makemd_x dictionary_root textfile dictionary_number\n");
      exit   (1);
   }

   /*** Parse optional flags. */
   verbose = debug = 0;
   for (i=4;   i<argc;   ++i) {
      if (streq (argv[i], "-d"))  { debug   = -1;    verbose = 1; }
      if (streq (argv[i], "-v"))    verbose =  1;
   }
      
   mdname (textfile, 1, argv[1], argv[2], 0);
   if ( (in = fopen (textfile, "r")) == NULL) {
      printf ("Can't open text file '%s'!\n", textfile);
      exit   (1);
   }
   fclose (in);

   sscanf    (argv[3], "%d",   &number);
   mdname    (temp, 2, argv[1], "temp", number);
   sysunlink (temp);
   if ( (fd = mdcreate (temp)) < 0) {
      printf ("Can't create dictionary file '%s'!\n", temp);
      exit   (1);
   }

   /*** Write 0's (empty MDREC's) into the hash table. */
   for (i=0;   i<HASHSIZE;   ++i) {
      mdput (fd, (char *)&mdrec, (int) sizeof(mdrec));
   }

   text    = HASHSIZE * (long) sizeof(mdrec);
   strings = collisions = 0;

   procfile (textfile, fd, &text, &collisions, &strings, 1, LANG_NORM, verbose);

   mdclose  (fd);

   if (verbose) printf ("Hash size=%d,  strings=%d,  collisions=%d\n",
                         HASHSIZE, strings, collisions);

   mdname    (real, 3, argv[1], argv[2], number);
   sysunlink (real);
   dicrename (temp, real);
}
