
/*** MDNAME.  Make names for various master dictionary files.
/
/    MDNAME contains the system-dependent code that assembles
/    the name for four different kinds of files used by the
/    MAKEMD program.  MDNAME puts the assembled name into NAME.
/    The input arguments are:
/
/    TYPE:  1:  dictionary text file 
/           2:  temporary output file for dictionary
/           3:  actual XSMD dictionary file
/           4:  dictionary ":<" include file
/
/    STR1:  "dictionary root path", i.e. "/usr/caucus/MISC"
/
/    STR2:  input file name (type 1 or 4) or temporary name (type 2)
/
/    NUM:   dictionary number.
/ */

/*** Copyright (C) 1989 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:57 Source code master starting point */
/*: CR  1/12/89 11:31 New function for makemd program. */
/*: CR  9/24/90 14:14 Add type 4, rewrite header comments. */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"

FUNCTION  mdname (name, type, str1, str2, num)
   char   name[], str1[], str2[];
   int    type, num;
{

#if UNIX | AOS | IM | PN | PR | VV | CX | CA
   switch (type) {
      case (1):
      case (4):   strcpy  (name, str2);                  break;
      case (2):   sprintf (name, "%s%03d", str2, num);   break;
      case (3):   sprintf (name, "%s%03d", str1, num);   break;
   }
#endif

#if CVM
   char p1[80], p2[80], p3[80];

   switch (type) {
      case (1):   sprintf (name, "%s %s a",       str1, str2);        break;
      case (2):   sprintf (name, "%s %s%03d a",   str1, str2, num);   break;
      case (3):   sprintf (name, "%s dict%03d a", str1,       num);   break;

      /*** For ":<" included files: any dots become spaces.  If there
      /    is no filemode, "a" is assumed.  If there is no filetype,
      /    "dic" is assumed.   Thus ":<america0" reads "america0 dic a". */
      case (4):
         strcpy   (name, str2);
         while (stralter (name, ".", " ")) ;
         strtoken (p1, 1, name);
         if (strtoken (p2, 2, name) == NULL)  strcpy (p2, "dic");
         if (strtoken (p3, 3, name) == NULL)  strcpy (p3, "a");
         sprintf  (name, "%s %s %s", p1, p2, p3);
         break;
   }
#endif

}
