/*** MOVEUXACT.  Move a user's transactions to the correct conference file(s).
/
/    int n = moveuxact (user)
/
/    Parameters:
/       Chix *user;
/
/    Purpose:
/       This is the function that takes a file of user's conference
/       xactions and moves them to a temp conference xaction file, XCUX.
/       It's argument, USER, is a Caucus userid.  The XFP will append
/       the xactions in XCXT to this file when it makes a pass thru
/       each conference, and turn the whole thing into a transport file.
/
/       Moveuxact() logs any problems to the distributed bugfile.
/
/    How it works:
/    1) Lock the xaction file, rename it to a tempfile,
/       unlock the real, lock the tempfile.
/    2) For each xaction in the tempfile:
/       1) If there isn't yet a temp conf xaction file for the conf
/          that the xaction belongs to, create a new tmp conf xaction
/          file for the conf.
/       2) Append the xaction to the tmp conf xaction file.
/    3) Delete and unlock the tempfile.
/
/   Returns:
/      1 on success
/      0 on failure.  In this case it logs the error in the bugfile.
/
/   Error Conditions:
/      Can't lock XUXF or XUXT.
/      Can't move XUXF to XUXT.  Can't open new XUXT.
/
/   Related Functions:
/
/   Called by:
/
/   Home: xfp/moveuxac.c
/ */

/*: JV  4/02/91 11:51 Create this. */
/*: JV  6/28/91 13:28 Pre-Integration CARVMark*/
/*: JV  7/04/91 18:08 Remove lint. */
/*: JV  7/17/91 14:16 Fix bugs. */
/*: JV  7/18/91 22:50 Begin transition to chixes. */
/*: JV  7/23/91 14:50 Check XUXT before opening. */
/*: CR  7/28/91 15:48 chxalloc() first argument is long. */
/*: JV  7/30/91 16:06 Remove third arg from getnextxact. */
/*: JV 10/17/91 13:31 Change diagnostics. */
/*: JV 12/07/91 20:33 Deallocate chixen and text fields. */
/*: UL 12/11/91 18:45 Make unit_lock args 3-6 long.*/
/*: JV  1/20/92 11:50 Change XCXT to XCUX. */
/*: DE  6/09/92 11:45 Chixified */
/*: JX  9/02/92 14:14 Allocate xaction. */
/*: JX  9/24/92 10:10 Don't free chixen between getnextxact() calls. */
/*: JX  9/30/92 17:26 Clear XACTION between getnextxact() calls. */
/*: CP  3/04/93 23:16 Remove nested comment. */

#include <stdio.h>
#include "handicap.h"
#include "common.h"
#include "xaction.h"
#include "unitcode.h"

FUNCTION moveuxact (user)
   Chix user;
{
   Xactionptr xaction, make_xaction();
   int   i;
   Chix  carray[TEXTFIELDS], chxalloc();
   ENTRY ("moveuxac", "");

   for (i=0; i < TEXTFIELDS; i++)
      carray[i] = chxalloc (100L, THIN, "moveu carray1");

   xaction = make_xaction();
                                        /* Note "user" in unit_lock calls. */
   if (NOT unit_lock (XUXF, READ, 0L, 0L, 0L, 0L, user)) {
      bug ("MOVEUXACT: can't lock XUXF.", 0); RETURN (0); }
   if (NOT unit_check (XUXF)) {
      unit_unlk (XUXF);   RETURN (0); }       /* No xact's for this user.  */
   if (NOT unit_lock (XUXT, READ, 0L, 0L, 0L, 0L, user)) {
      bug ("MOVEUXACT: can't lock XUXF.", 0); 
      unit_unlk (XUXF);   RETURN (0); }

   if (NOT unit_move (XUXF, XUXT)) {
      bug ("MOVEUXACT: can't move XUXF to XUXT.", 0); 
      unit_unlk (XUXF); unit_unlk (XUXT); RETURN (0); }
   unit_unlk (XUXF);

   /*** Now process the tmp version of the xaction file. */
   if (NOT unit_check (XUXT)) {
      bug ("MOVEUXACT: XUXT doesn't exist.", 0);
      unit_unlk (XUXT);  RETURN (0); }
   if (NOT unit_view (XUXT)) {
      bug ("MOVEUXACT: can't open XUXT.", 0);
      unit_unlk (XUXT);  RETURN (0); }

   while (getnextxact (XUXT, xaction)) {
      /* Add the xaction to the correct conf xaction file. */
      if (NOT unit_lock (XCUX, WRITE,  xaction->conflocnum, 0L, 0L, 0L, 
                         nullchix)) {
         bug ("MOVEUXACT: can't lock XCUX.", 0); 
         unit_close (XUXT); unit_unlk (XUXT); RETURN (0); }
      unit_append (XCUX);
      for (i=0; i<TEXTFIELDS; i++)
         chxcpy (carray[i], xaction->data[i]);

      putnextxact (XCUX, xaction, carray);
      clear_xaction (xaction);

      unit_close (XCUX);
      unit_unlk (XCUX);
   }

   unit_close (XUXT);
   unit_kill  (XUXT);
   unit_unlk  (XUXT);

   free_xaction (xaction);

   RETURN (1);
}   

