/*** XP_SHIP.   Create a xport file from a conf xaction file (XCXT) and
/               copy it into the correct shipping bay(s).
/
/   int n = xp_ship (confnum)
/
/   Parameters:
/      int confnum;
/
/   Purpose:
/      The XFP must combine the user conference xaction file (XCUX) and
/      the conference xaction file (XCXT) and turn the result into two
/      xport files:
/      1) Transport Header-File
/      2) Transport Text-File
/      and put them in a place where the shipper knows to look.  
/      XP_SHIP makes one copy of each transport file for each neighbor
/      that should receive the file.
/
/   How it works:
/
/   Returns:
/      1 on success
/      0 on failure.  In this case it logs the error in the bugfile.
/
/   Error Conditions:
/
/   Warnings:
/      1) Since we don't copy XCXT over to a temporary file before processing
/         it, a user who is changing conference text may wait longer than
/         usual for a new prompt if they make the change while the XFP is
/         processing that conf.  This time has been minimized.
/
/   Related Functions:
/      moveuxact(), xpthru_ship()
/
/   Called by: xfp (main)
/
/   Home: xfp/xpship.c
/ */

/*: JV  4/08/91 18:24 Create this. */
/*: JV  6/28/91 13:28 Pre-Integration CARVMark */
/*: JV  7/19/91 13:44 Add more debug printf's. */
/*: JV  7/23/91 14:56 Check XCNB before opening. */
/*: UL 12/11/91 18:45 Make unit_lock args 3-6 long.*/
/*: JV  1/20/92 11:54 Append XCXT to XCUX before crexport. */
/*: DE  6/09/92 15:31 Chixified */
/*: CP  7/15/93 15:47 Remove nested comments. */
/*: JV  8/25/93 14:31 Don't complain if XCNB not there: not a linked conf. */

#include <stdio.h>
#include "chixuse.h"
#include "handicap.h"
#include "unitcode.h"
#include "xaction.h"
#include "xfp.h"

extern int debug;
extern union null_t null;

FUNCTION xp_ship (confnum)
   int confnum;
{
   Chix naybor;
   long  lhnum;
   ENTRY ("xp_ship", "");

   if (NOT unit_lock (XCNB, READ, L(confnum), 0L, 0L, 0L, nullchix)) {
      bug ("XP_SHIP: can't lock XCNB.", 0);  RETURN (0); }
   if (NOT unit_check (XCNB)) {
      unit_unlk (XCNB);  RETURN (0); }
   if (NOT unit_view (XCNB)) { unit_unlk (XCNB);
      bug ("XP_SHIP: can't open XCNB.", 0);  RETURN (0); }

   if (NOT unit_lock (XCUX, WRITE, L(confnum), 0L, 0L, 0L, null.chx)) {
      unit_close (XCNB); unit_unlk (XCNB);
      bug ("XP_SHIP: can't lock XCXT.", 0);  RETURN (0); }
   if (NOT unit_lock (XCXT, READ, L(confnum), 0L, 0L, 0L, null.chx)) {
      unit_close (XCNB); unit_unlk (XCNB);
      unit_unlk  (XCUX);
      bug ("XP_SHIP: can't lock XCXT.", 0);  RETURN (0); }
   if (unit_check (XCXT)) {
      append_file (XCUX, XCUX, XCXT);
      unit_kill (XCXT);
   }
   unit_unlk (XCXT);

   naybor   = chxalloc (40L, THIN, "xpship naybor");

   if (unit_check (XCUX)) {
      unit_view (XCUX);
      while (unit_read (XCNB, naybor, 0L)) {
         if (NOT chxnum (naybor, &lhnum)) {
            bug ("XP_SHIP: bad num in XCNB.", 0); continue; }
         if (debug) printf ("Got neighbor: %d.\n", (int) lhnum);
         if (NOT unit_lock (XHI1, WRITE, lhnum, 0L, 0L, 0L, null.chx)) {
            bug ("XP_SHIP: can't lock XHI1.", (int) lhnum);  continue; }
         crexport ((int) lhnum, XCUX, NEWHEADER, confnum, (XPORTHDR *) NULL);

         unit_rewind (XCUX);
         unit_unlk (XHI1);
         if (debug) printf ("Finished neighbor: %d.\n", (int) lhnum);
      }
      unit_close (XCUX);
      unit_kill (XCUX);
   }
   unit_unlk (XCUX);
   unit_close (XCNB);  unit_unlk (XCNB);

   if (debug) printf ("Finished neighbors of conf: %d.\n", confnum);

   chxfree ( naybor );

   RETURN (1);
}   
