/*** xpthru_ship.   Copy xactions passing thru this host to shipping bays.
/
/   int n = xpthru_ship (srchostptr);
/
/   Parameters:
/      HOSTPTR srchostptr;
/
/   Purpose:
/      When a neighbor sends a xport file of conf xactions to this host,
/      and other hosts share this conf thru this host, the XFP
/      has the job of taking a copy of the original xport file
/      and using it to create new xport files.
/      This copy was created and placed in HOSThhh/xactpppp (where hhh is
/      the LH# of the xport file's *sender*) by the XFU.
/      We must find all of these files and change their headers to
/      reflect the new destinations, depending on what conf they're for
/      and which neighbor(s) are sharing that conf.  Make sure we
/      don't send it to the original sender!!
/
/   How it works:
/      First it checks the SRCHOSTPTR structure.  If the two SEQNUM_THRU...
/      fields are the same, there are no new XHXP files to process.
/      Otherwise it finds the files and processes them one by one,
/      incrementing the lower sequence number after each, and
/      storing the incremented value *after* processing the associated file.
/      (It will increment the number as it exists in the seqnum file
/       *after* it processes the file to make sure it gets processed
/       at least once!)
/
/      Note: if for any reason the XHXA file described by seqnum_thru_low
/      doesn't exist, we increment ...thru_low so it won't get checked
/      again next time.  If (n=upper-lower) and many XHXA files are missing,
/      it will take 'n' runs of the XFP before it starts processing XHXA
/      files again.  However, this does allow it to eventually "catch up"
/      instead of just sitting there.
/
/   Returns:
/      1 on success
/      0 on failure.  In this case it logs the error in the bugfile.
/
/   Error Conditions:
/
/   Related Functions:
/      nextneighbor()
/
/   Called by: xfp (main)
/
/   Home: xfp/xpthrush.c
/
/ */

/*: JV  4/15/91 11:51 (Tax Day!) Create this. */
/*: JV  6/28/91 13:28 Pre-Integration CARVMark*/
/*: JV  7/19/91 13:53 Change call to crexport. */
/*: JV  7/22/91 16:25 Add ENTRY, check XHXA before opening. */
/*: JV  8/09/91 10:27 Kill XHXA after processing it, update seqnum. */
/*: JV  8/25/91 12:33 Fix sequence number incrementing. */
/*: JV  9/17/91 09:54 Update thru_low if XHXA doesn't exist. */
/*: JV  9/19/91 10:12 Use seqnum macro operators. */
/*: CR  9/19/91 13:12 Lower and Upper must be longs. */
/*: JV 10/11/91 16:00 Fix sequence number updating on success. */
/*: JV 10/28/91 14:50 Added 2nd arg to cnumber().*/
/*: JV 11/25/91 14:46 Prevent CNUMBER from changing header.trueconf. */
/*: WC 11/29/91 21:14 Add 3rd arg to cnumber(). */
/*: UL 12/11/91 18:45 Make unit_lock args 3-6 long.*/
/*: CR 12/15/91 16:50 rewind XHXA each time; put confnum in crexport call. */
/*: JV  1/03/92 15:20 Declare strcpy(). */
/*: JX  6/04/92 14:58 Chixify. */
/*: CP  3/04/93 23:16 Remove nested comment. */

#include <stdio.h>
#include "handicap.h"
#include "common.h"
#include "chixuse.h"
#include "done.h"
#include "xaction.h"
#include "xfp.h"
#include "unitcode.h"
#include "sequence.h"
#include "conf.h"

FUNCTION xpthru_ship (srchostptr)
   HOSTPTR srchostptr;
{
   Chix line, confname;
   long lower, upper;
   int  confnum, v, success=0;
   long destlhnum;
   XPORTHDR *header, *make_xporthdr();
   Hinfoptr  hinfo;

   ENTRY ("xpthru_ship", "");

   hinfo = srchostptr->info;
   lower = hinfo->seqnum_thru_low;
   upper = hinfo->seqnum_thru_hi;
   if (lower == upper)   RETURN (1);
   
   line     = chxalloc (100L, THIN, "xpthrush line");
   confname = chxalloc (100L, THIN, "xpthrush confname");
   header = make_xporthdr();

   /* Now process each xport file that is waiting for us. */
   while (lower != upper) {
      if (NOT unit_lock (XHXA, READ, L(srchostptr->lhnumber),  lower, 0L, 0L,
          nullchix)) {
         bug ("XPTHRU_SHIP: can't lock input file.", 0); FAIL; }
      if (NOT unit_check (XHXA)) {
         bug ("XPTHRU_SHIP: XHXA doesn't exist, host=",
               srchostptr->lhnumber);
         hinfo->seqnum_thru_low = lower;
         hinfo->seqnum_thru_low = INC(hinfo->seqnum_thru_low, SEQ_THRU_MOD);
	 unit_unlk (XHXA); FAIL; }
      if (NOT unit_view (XHXA)) {
         bug ("XPTHRU_SHIP: can't open XHXA.", srchostptr->lhnumber);
         bug ("XPTHRU_SHIP: possible database error.", 0);
         unit_unlk (XHXA); FAIL; }

      /* First we need to figure out which host(s) this is going to. */
      if (NOT get_xpheader (XHXA, header)) {
         bug ("XPTHRU_SHIP: got bad header.", 0);
         unit_close (XHXA); unit_unlk (XHXA); FAIL; }
      chxcpy (confname, header->trueconf);
      confnum = cnumber (confname, TRUECONF, &v);  /* CNUMBER adds OCHN! */
      if (confnum == -1) {
         hinfo->seqnum_thru_low = lower;
         hinfo->seqnum_thru_low = INC(hinfo->seqnum_thru_low, SEQ_THRU_MOD);
	 unit_close (XHXA); unit_kill (XHXA); unit_unlk (XHXA); FAIL;
      }
      if (NOT unit_lock (XCNB, READ, L(confnum), 0L, 0L, 0L, nullchix)) {
         bug ("XPTHRU_SHIP: can't lock XCNB.", confnum); 
         unit_close (XHXA); unit_unlk (XHXA); FAIL; }
      unit_view (XCNB);
      while (unit_read (XCNB, line, 0L)) {
         chxnum (line, &destlhnum);
         if (destlhnum != srchostptr->lhnumber) {       /* Don't send back!*/
            unit_lock (XHI1, READ, L(destlhnum), 0L, 0L, 0L, nullchix);
/*          get_neighbor (destlhnum, desthptr, XHI1); */

            /*** Before writing an export file with crexport(), we must
            /    first rewind XHXA to just after the header.   Otherwise
            /    2nd & subsequent neighbors (from XCNB) will not get their
            /    copy of the transactions in XHXA! (CR) */
            unit_rewind  (XHXA);
            get_xpheader (XHXA, header);
            crexport  ((int) destlhnum, XHXA, NOADDHEADER, confnum, header);
            unit_unlk (XHI1);
         }
      }
      unit_close (XHXA);  unit_kill (XHXA);  unit_unlk (XHXA);
      unit_close (XCNB);  unit_unlk (XCNB);
      lower = INC(lower, SEQ_THRU_MOD);
      hinfo->seqnum_thru_low = lower;
   }
   SUCCEED;

done:
   chxfree (line);
   chxfree (confname);
   free_xporthdr (header);
   RETURN (success);
}   
