/*** CONF_PARSE.  Parse a xaction that the XFU has decided is cust stuff.
/
/   int n = conf_parse (xptr)
/
/   Parameters:
/     XACTIONPTR xptr;
/
/   Purpose:
/      After the XFU has decided that an incoming xaction is the result
/      of a customize command,
/      conf_parse() further parses it and applies it to the local db.
/      Note that the XFU deals with routing the info on to other hosts
/      if necessary.
/
/   How it works:
/
/   Returns:
/      1 on success
/      0 on failure.  In this case it logs the error in the bugfile.
/
/   Error Conditions:
/
/   Related Functions: mess_parse() resp_parse()
/
/   Called by: parse_xaction()
/
/   Home: xfu/confpars.c
/ */

/*: JV  5/21/91 14:35 Create this. */
/*: JV  6/28/91 13:28 Pre-Integration CARVMark*/
/*: JV  7/10/91 15:38 De-Lint. */
/*: JV  7/15/91 21:42 Add multiple text fields. */
/*: JV  7/27/91 18:34 Fix call to cus_dattr(). */
/*: CR  7/28/91 14:52 Cast conflocnum to (int) in cus_dattr() call. */
/*: CR  7/28/91 15:48 chxalloc() 1st argument is long.  Fix cus_dtext() call. */
/*: JV  7/29/91 12:35 Cast 1st arg to get_trueconf. */
/*: JV 10/31/91 12:14 Add "break"s. */
/*: JV 11/13/91 22:25 Cast conflocnum. */
/*: JV 11/14/91 15:16 Add make_master(), free_master(). */
/*: JV 11/15/91 14:01 Strip NL off of "ON". */
/*: CR 11/19/91 17:27 Send whole "id@chn" to cus_dporg(). */
/*: CR 12/26/91 11:55 Remove unused variables. */
/*: CR  1/02/92 15:09 Add case SUBSUBJECT. */
/*: DE  6/10/92 11:27 ReChixified */
/*: CR 12/14/92 15:04 Long chxalsub args. */
/*: CP  3/04/93 21:54 Fix apparent nested comments. */

#include <stdio.h>
#include "handicap.h"
#include "derr.h"
#include "maxparms.h"
#include "master.h"
#include "chixuse.h"
#include "done.h"
#include "xaction.h"

#define BUG printxact (xptr)

FUNCTION  conf_parse (xptr)
XACTIONPTR xptr;
{
   Chix   tempchx;
   int    error, value, success;
   Chix   chn, clocname, ctname;
   struct master_template master;

   ENTRY ("conf_parse", "");

   chn       = chxalloc (80L, THIN, "confpars chn");
   clocname  = chxalloc (80L, THIN, "confpars clocname");
   ctname    = chxalloc (80L, THIN, "confpars ctname");
   success   = 0;

   switch (xptr->actnum) {
   case XT_CHANGE:
      switch (xptr->subnum) {
      case SUBUSER:
         cus_dusers ((int) xptr->conflocnum, xptr->data[0], &error); 
         switch (error) {
            case DNOCONF: tellmgr (xptr, DNOCONF);
            case NOERR:   SUCCEED;
            default:      FAIL;
         }
         break;
      case SUBPORG:
         make_master (&master, MAXITEMS);
         cus_dporg ((int) xptr->conflocnum, xptr->uid, &master, &error); 
         free_master (&master);
         switch (error) {
            case DNOCONF: tellmgr (xptr, DNOCONF);
            case NOERR:   SUCCEED;
            default:      FAIL;
         }
         break;
      case SUBADD:
      case SUBCHANGE:
      case SUBSUBJECT:
      case SUBVIS:
         tempchx = chxalsub(xptr->data[0], 0L, chxlen (xptr->data[0])-1);
                                                             /* Strip NL */
         chxcpy (xptr->data[0], tempchx);
         chxfree (tempchx);
         value = chxeq (xptr->data[0], CQ("ON")) ? ON : OFF;
         get_trueconf ((int)xptr->conflocnum, clocname, ctname, chn);
         cus_dattr ( (int) xptr->conflocnum, clocname, xptr->subnum,
                    value, &error);
         switch (error) {
            case DNOCONF: tellmgr (xptr, DNOCONF);
            case NOERR:   SUCCEED;
            default:      FAIL;
         }
         break;
      case SUBGREET:
      case SUBINTRO:
         cus_dtext ((int) xptr->conflocnum, xptr->subnum, xptr->data[0], &error);
         switch  (error) {
            case DNOCONF: tellmgr (xptr, DNOCONF);
            case NOERR:   SUCCEED;
            default:      FAIL;
         }
         break;
      default:
         BUG; FAIL;
      }
   }
   SUCCEED;
done:
   chxfree ( chn );
   chxfree ( clocname );
   chxfree ( ctname );

   RETURN ( success );
}   
