/*** HOST_PARSE.  Parse a xaction that the XFU has decided is HOST stuff.
/
/   int n = host_parse (xptr)
/
/   Parameters:
/     XACTIONPTR xptr;
/
/   Purpose:
/      After the XFU has decided that an incoming xaction is the result
/      of using NetMaint to Attach, Detach, etc,
/      host_parse() further parses it and applies it to the local db.
/
/   How it works:
/
/   Returns:
/      1 on success
/      0 on failure.  In this case it logs the error in the bugfile.
/
/   Error Conditions:
/
/   Warning: for Version 1.2 the ATTACK xaction should only send
/            the routing tables if the status of that naybor was
/            UNATTACHED or PENDING.
/
/   Related Functions: item_parse conf_parse mess_parse() resp_parse()
/
/   Called by: parse_xaction()
/
/   Home: xfu/hostpars.c
/ */

/*: JV 12/13/91 10:43 Create this. */
/*: CR 12/26/91 12:10 Declare sysmem(). */
/*: JV  1/08/92 15:23 Send routing table to requestor/requestee. */
/*: JV  1/15/92 11:00 Append and Close XHML. */
/*: JV  1/24/92 11:37 Add ATTACK warning to header. */
/*: DE  6/10/92 12:43 Chixified */
/*: JX  7/24/92 15:45 Use make_hinfo(). */
/*: JX  8/09/92 16:45 Chixify hnumber arg. */
/*: JX  9/01/92 13:38 Change empty_xact to make_xaction() */
/*: CP  3/04/93 23:13 Fixed nested comments. */

#include <stdio.h>
#include "handicap.h"
#include "derr.h"
#include "unitcode.h"
#include "xfp.h"
#include "chixuse.h"
#include "xaction.h"
#include "done.h"

extern union null_t null;

FUNCTION  host_parse (xptr)
XACTIONPTR xptr;
{
   int  lhnum, success;
   Chix chn, mgrid;
   Chix carray[2];
   HOSTPTR  host, myptr, make_host();
   Hinfoptr make_hinfo();
   Xactionptr  xact, make_xaction();
   char    *sysmem();

   ENTRY ("host_parse", "");

   host       = make_host();
   host->info = make_hinfo();
   myptr      = make_host();
   myptr->info = make_hinfo();

   chn      = chxalloc (80L, THIN, "hostpars chn");
   mgrid    = chxalloc (40L, THIN, "hostpars mgrid");
   xact     = make_xaction();

   switch (xptr->actnum) {
   case XT_ATTREQ:
      /*** First change the status of this neighbor to ATTACHED. */
      chxbreak (xptr->uid, mgrid, chn, CQ("@"));
      if ((lhnum = hnumber (chn)) < 1) {
         tellmgr (xptr, 0); FAIL; }
      unit_lock (XHI1, WRITE, L(lhnum), 0L, 0L, 0L, null.chx);
      get_neighbor (lhnum, host, XHI1);
      host->info->status = ATTACHED;
      set_neighbor (host, XHI1);
      unit_unlk (XHI1);

      /*** Then reply with ATTACK. */
      xact->typenum = TT_HOST;
      xact->actnum  = XT_ATTACK;
      unit_lock (XSNF, READ, 0L, 0L, 0L, 0L, null.chx);
      get_neighbor  (0, myptr, XSNF);
      unit_unlk (XSNF);
      chxclear  (xact->uid);
      chxformat (xact->uid, CQ("%s@%s"), 0L, 0L, myptr->info->mgrid, 
                 myptr->chn);
      unit_lock (XHML, WRITE, L(lhnum), 0L, 0L, 0L, null.chx);
      unit_append (XHML);
      carray[0] = carray[1] = chxalloc (0L, THIN, "attach carray");
      putnextxact (XHML, xact, carray);
      unit_close  (XHML);
      unit_unlk   (XHML);
      chxfree (carray[0]);

      /*** Finally send the routing table. */
      send_routes ();
      break;
   case XT_ATTACK:
      /*** Change the status of this neighbor to ATTACHED. */
      chxbreak (xptr->uid, mgrid, chn, CQ("@"));
      if ((lhnum = hnumber (chn)) < 1) {
         tellmgr (xptr, 0); FAIL; }
      unit_lock (XHI1, WRITE, L(lhnum), 0L, 0L, 0L, null.chx);
      get_neighbor (lhnum, host, XHI1);
      host->info->status = ATTACHED;
      set_neighbor (host, XHI1);
      unit_unlk (XHI1);

      /*** Finally send the routing table. */
      send_routes ();
      break;

   default:
      tellmgr (xptr, 0); FAIL;
   }
   SUCCEED;

done:
   free_host (host);
   free_host (myptr);
   free_xaction (xact);

   chxfree (chn);
   chxfree (mgrid);

   RETURN (success);
}   


