/*** ITEM_PARSE.  Parse a xaction that the XFU has determined to be item stuff.
/
/   int n = item_parse (xptr)
/
/   Parameters:
/     XACTIONPTR xptr;
/
/   Purpose:
/      After the XFU has decided that an incoming xaction is item stuff,
/      item_parse() further parses it and applies it to the local db.
/      Note that the XFU deals with routing the info on to other hosts
/      if necessary.
/
/   How it works:
/
/   Warning: if iorig_to_local fails, item_parse succeeds in hopes of
/            not making a problem any worse.  This could be made more
/            intelligent by returning more info from iorig_...
/   Returns:
/      1 on success
/      0 on failure.  This really shouldn't happen except in one of the
/        rare (assumed "impossible") conditions described below.
/
/   Error Conditions:
/
/   Related Functions: mess_parse() resp_parse()
/
/   Called by: parse_xaction()
/
/   Home: xfu/itempars.c
/ */

/*: JV  5/21/91 12:03 Create this. */
/*: JV  6/28/91 13:28 Pre-Integration CARVMark*/
/*: JV  7/04/91 11:37 Remove lint. */
/*: JV  7/15/91 21:44 Add multiple text fields. */
/*: CR  7/24/91 22:09 Make Lint happier. */
/*: CR  7/28/91 15:48 chxalloc() first argument is long. */
/*: JV  7/29/91 12:39 Remove 2nd arg to reparse */
/*: JV  8/23/91 16:24 Remove return() notreached. */
/*: JV 10/31/91 12:07 Free chixen, add "break"s.. */
/*: JV 11/22/91 11:56 Succeed if iorig fails. */
/*: JV 11/29/91 11:56 Fix return-checking on iorig... */
/*: JV 12/04/91 14:05 Add FREEZE. */
/*: JV  1/13/92 16:08 Fix DELETE. */
/*: DE  6/10/92 13:50 Chixified */
/*: CP  3/04/93 21:54 Fix apparent nested comments. */
/*: CK  3/18/93 17:02 SUCCEED after successful freeze or thaw. */
/*: JV  4/05/93 17:15 Add attach arg to chg_ditem(). */

#include <stdio.h>
#include "handicap.h"
#include "derr.h"
#include "chixuse.h"
#include "null.h"
#include "xaction.h"
#include "done.h"

#define BUG printxact (xptr)

extern union null_t null;

FUNCTION  item_parse (xptr)
XACTIONPTR xptr;
{
   int  error, localinum, success;
   Chix date, time;

   ENTRY ("item_parse", "");

   date    = chxalloc (20L, THIN, "itempars date");
   time    = chxalloc (20L, THIN, "itempars time");
   success = 0;

   switch (xptr->actnum) {
   case XT_NEW:
      chxbreak (xptr->date, date, time, CQ(" "));
      chg_ditem (XT_NEW, (int)xptr->conflocnum, (int)xptr->itemid, &localinum,
                 xptr->itemchn, xptr->uname, xptr->uid, xptr->data[0], 
                 xptr->title, date, time, xptr->attach, &error);
      switch (error) {
         case DNOCONF: reparse (xptr);  SUCCEED;
         case DCFULL:  tellmgr (xptr, DCFULL);
         case DIEXISTS: 
         case NOERR:        SUCCEED;
         default:      logx ("Couldn't add to DB via chg_ditem:", xptr);
                       FAIL;
      }
      break;

   case XT_CHANGE:
      switch (xptr->subnum) {
      case  SUBTITLE:
         if (NOT iorig_to_local ((int)xptr->conflocnum, (int)xptr->itemid,
                                 xptr->itemchn, &localinum))  SUCCEED; 
         chg_dtitle ((int)xptr->conflocnum, localinum, xptr->title, &error);
         switch (error) {
            case DNOCONF:
            case DINOITEM:  reparse (xptr);
            case NOERR:                    SUCCEED;
            default:       logx ("Couldn't chg title via chg_dtitle:", xptr);
                           FAIL;
         }
         break;

      case SUBTEXT:
         if (NOT iorig_to_local ((int)xptr->conflocnum, (int)xptr->itemid, 
                         xptr->itemchn, &localinum))  SUCCEED; 
         chg_ditem (XT_CHANGE, (int) xptr->conflocnum, (int)xptr->itemid,
                    &localinum, xptr->itemchn, nullchix, nullchix, 
                    xptr->data[0], nullchix, nullchix, nullchix, 
                    xptr->attach, &error);
         switch (error) {
            case DNOCONF:
            case DINOITEM: reparse (xptr);
            case NOERR:                   SUCCEED;
            default:   logx ("Couldn't change item in DB via chg_ditem:", xptr);
                       FAIL;
         }
         break;

      case SUBFREEZE:
         if (NOT iorig_to_local ((int)xptr->conflocnum, (int)xptr->itemid, 
                         xptr->itemchn, &localinum))  SUCCEED; 
         frz_ditem ((int)xptr->conflocnum, localinum, 1);
         SUCCEED;
         break;

      case SUBTHAW:
         if (NOT iorig_to_local ((int)xptr->conflocnum, (int)xptr->itemid, 
                         xptr->itemchn, &localinum))  SUCCEED; 
         frz_ditem ((int)xptr->conflocnum, localinum, 0);
         SUCCEED;
         break;

      default:   logx ("ITEMPARSE: couldn't parse:", xptr);
                 FAIL;
      }
      break;

   case XT_DELETE:
      if (NOT iorig_to_local ((int)xptr->conflocnum, (int)xptr->itemid,
                      xptr->itemchn, &localinum))  SUCCEED;
      chg_ditem (XT_DELETE, (int)xptr->conflocnum, (int)xptr->itemid,
                 &localinum,
                 nullchix, nullchix, nullchix, nullchix, nullchix, nullchix,
                 nullchix, xptr->attach, &error);
      switch (error) {
         case DNOCONF:
         case DINOITEM: reparse (xptr);
         case NOERR:                   SUCCEED;
         default:   logx ("Couldn't delete item in DB via chg_ditem:", xptr);
                    FAIL;
      }
      break;

   default:  /* Switch: Action type=New/Change/Delete */
      BUG; FAIL;
   }

done:
   chxfree (date);
   chxfree (time);

   RETURN (success);
}   
