/*** PERS_PARSE.  Parse a xaction that the XFU has decided is people stuff.
/
/   int n = pers_parse (xptr)
/
/   Parameters:
/     XACTIONPTR xptr;
/
/   Purpose:
/      After the XFU has decided that an incoming xaction is people
/      stuff item_parse() further parses it and applies it to the local db.
/      Note that the XFU deals with routing the info on to other hosts
/      if necessary.
/
/   How it works:
/
/   Returns:
/      1 on success
/      0 on failure.  In this case it logs the error in the bugfile.
/
/   Error Conditions:
/
/   Related Functions: mess_parse() resp_parse() item_parse()
/
/   Called by: parse_xaction()
/
/   Home: xfu/perspars.c
/ */

/*: JV  5/21/91 14:20 Create this. */
/*: JV  6/28/91 13:28 Pre-Integration CARVMark*/
/*: JV  7/04/91 19:27 Remove lint. */
/*: JV  7/15/91 17:28 Add reparse args. */
/*: CR  7/24/91 22:13 Make Lint happier. */
/*: JV  7/29/91 12:54 Add arg to add_dperson, chg to chg_dperson. */
/*: JV  8/23/91 16:25 Remove return() notreached. */
/*: JV 10/31/91 12:21 Add "break"s. */
/*: CR 11/19/91 16:39 Add RETURN(0) at end to make Lint happy. */
/*: DE  6/10/92 16:49 Chixified */
/*: CP  3/04/93 23:13 Fixed nested comments. */

#include <stdio.h>
#include "handicap.h"
#include "derr.h"
#include "chixuse.h"
#include "xaction.h"
#include "done.h"

#define BUG printxact (xptr)

FUNCTION  pers_parse (xptr)
XACTIONPTR xptr;
{
   Chix id, host;
   int  error, success;

   ENTRY ("pers_parse", "");

   id     = chxalloc (80L, THIN, "perspars id");
   host   = chxalloc (80L, THIN, "perspars host");
   success = 0;

   switch (xptr->actnum) {
      case XT_NEW:
      case XT_CHANGE:
         chxbreak (xptr->uid, id, host, CQ("@"));
         chg_dperson (xptr->unewname, xptr->uname, id, host, xptr->phone,
                      &error);
         switch (error) {
            case DNOPERS:  if (xptr->actnum == XT_CHANGE) reparse (xptr);
                           break;
            case DNOALIEN:
            case DPEXIST:
            case NOERR:    SUCCEED;
            default:       logx ("Couldn't add person to DB:", xptr);
                           FAIL;
         }
         break;
      default:
         BUG; FAIL;
   }

   FAIL;
done:
   chxfree ( id );
   chxfree ( host );

   RETURN ( success );
}   
