/*** PROC_REPARSE.   Parse a ReParseBay file of xactions.
/
/   int n = proc_reparse (seqnum)
/
/   Parameters:
/      long seqnum;
/
/   Purpose:
/      In a perfect world, all xactions would arrive at all destinations
/      in the same order they were created.  Since Murphy is still alive
/      and well, we have to handle cases such as a response from one host
/      to an item that has come from another host, but arrives before the
/      item does.  The function reparse() stored xactions like these.  Here
/      we try again to digest them.
/
/   How it works:
/      Note that if a parse_xaction can't parse it this time either, it
/      calls reparse() on the xaction to "re-store" it for a later parse
/      pass.  This allows us to always throw away this copy of the
/      xport file.
/
/   Returns:
/      1 on success
/      0 on failure.  In this case it logs the error in the bugfile.
/
/   Error Conditions:
/
/   Related Functions:  reparse()
/
/   Called by:
/
/   Home: xfp/procrepa.c
/ */

/*: JV  7/15/91 20:21 Create this. */
/*: JV  7/26/91 16:29 Fix diagnostics. */
/*: JV  7/30/91 16:07 Remove 3rd arg of getnextxact. */
/*: JV  9/12/91 12:51 Add docs, toss file when done.. */
/*: JV 10/14/91 12:16 Use XSR2 instead of XSRP. */
/*: JV 12/07/91 20:49 Deallocate text fields. */
/*: UL 12/11/91 18:45 Make unit_lock args 3-6 long.*/
/*: DE  6/10/92 16:57 Chixified */
/*: JX  9/01/92 14:16 Allocate xaction. */
/*: CP  3/04/93 23:16 Remove nested comment. */

#include <stdio.h>
#include "handicap.h"
#include "unitcode.h"
#include "chixuse.h"
#include "xaction.h"

FUNCTION proc_reparse (seqnum)
long seqnum;
{
   int i;
   Xactionptr xaction, make_xaction();

   ENTRY ("proc_reparse", "");

   xaction = make_xaction();

   if (NOT unit_lock (XSR2, READ, seqnum, 0L, 0L, 0L, nullchix)) {
      bug ("XFU: Can't lock XSR2.", 0); RETURN (0); }
   if (NOT unit_view (XSR2)) {
      bug ("XFU: Can't view XSR2.", 0);
      unit_unlk (XSR2); RETURN (0);
   }
   while (getnextxact (XSR2, xaction)) {
      parse_xaction (xaction);
      for (i=0; i<TEXTFIELDS; i++)
         if (xaction->datasize[i]) chxfree (xaction->data[i]);
   }

   unit_close (XSR2);
   unit_kill  (XSR2);
   unit_unlk  (XSR2);

   free_xaction (xaction);
   RETURN (1);
}   
