/*** REPARSE.  Stash this xaction away for later digestion.
/
/   int n = reparse (xptr)
/
/   Parameters:
/      XACTIONPTR xptr;
/
/   Purpose:
/      In a perfect world, all xactions would arrive at all destinations
/      in the same order they were created.  Since Murphy is still alive
/      and well, we have to handle cases such as a response from one host
/      to an item that has come from another host, but arrives before the
/      item does.  We'll stash it away and parse it again later.
/
/   How it works:
/      First it gets the next seqnuence number for a ReParse file from
/      XSNF:seqnum_thru_hi.  It opens the file for append and writes the
/      xaction to the file.  It also adds the DATERECVD field to the xaction.
/      PROC_REPARSE uses this to decide when a xaction has gotten moldy...
/
/   Returns:
/      1 on success
/      0 on failure.  In this case it logs the error in the bugfile.
/
/   Error Conditions:
/
/   Related Functions:
/
/   Called by:
/
/   Home: xfp/reparse.c
/ */

/*: JV  7/15/91 17:36 Create this. */
/*: JV  7/27/91 16:56 Remove cnum param, fix hostinfo. */
/*: CR  7/28/91 15:25 Add missing argument to unit_lock(). */
/*: CR  7/28/91 15:48 chxalloc() first argument is long. */
/*: JV  7/29/91 08:18 Fix locking of XSNF. */
/*: JV  7/30/91 20:49 Add How it Works. */
/*: JV  9/12/91 13:16 Update sequence number. Use thru_hi. */
/*: JV  9/19/91 10:29 Use seqnum macro ops */
/*: JV 12/07/91 20:42 Deallocate chixes. */
/*: UL 12/11/91 18:45 Make unit_lock args 3-6 long.*/
/*: JV 12/16/91 10:34 Neaten code. */
/*: DE  6/11/92 11:51 Chixified */
/*: JX  7/24/92 15:50 Use make_hinfo(). */
/*: CP  3/04/93 23:16 Remove nested comment. */

#include <stdio.h>
#include "handicap.h"
#include "unitcode.h"
#include "chixuse.h"
#include "xfp.h"
#include "xaction.h"
#include "sequence.h"
#include "done.h"

FUNCTION reparse (xptr)
XACTIONPTR xptr;
{
   Chix cdata[2], chxalloc();
   Chix day, time, daytime;
   long systime(), seqnum;
   int  success = 0;
   HOSTPTR host, make_host();
   Hinfoptr make_hinfo();

   ENTRY ("reparse", "");

   host = make_host();
   host->info = make_hinfo();

   unit_lock (XSNF, READ, 0L, 0L, 0L, 0L, nullchix);
   if (NOT get_neighbor (0, host, XSNF)) {
      bug ("REPARSE: can't get my hostinfo.", 0); 
      unit_unlk (XSNF); FAIL; }
   seqnum = host->info->seqnum_thru_hi;
   unit_unlk (XSNF);

   if (NOT unit_lock (XSRP, WRITE,  seqnum, 0L, 0L, 0L, nullchix)) {
      bug ("REPARSE: couldn't lock XSRP.", 0); FAIL; }
   if (NOT unit_append (XSRP)) {
      bug ("REPARSE: couldn't open XSRP.", 0);
      unit_unlk (XSRP); FAIL;
   }
   cdata[0] = chxalloc ((long) chxlen (xptr->data[0]), THIN,
                          "reparse cdata[0]");
   cdata[1] = chxalloc ((long) chxlen (xptr->data[1]), THIN,
                          "reparse cdata[1");
   day      = chxalloc (30L, THIN, "reparse day");
   time     = chxalloc (30L, THIN, "reparse time");
   daytime  = chxalloc (60L, THIN, "reparse daytime");

   chxcpy (cdata[0], xptr->data[0]);
   chxcpy (cdata[1], xptr->data[1]);

   sysdaytime (day, time, 0, systime());
   chxformat (xptr->daterecvd, CQ("%s %s"), 0L, 0L, day, time);

   putnextxact (XSRP, xptr, cdata);

   unit_close (XSRP);
   unit_unlk  (XSRP);

   chxfree (cdata[0]); chxfree (cdata[1]);
   chxfree (day);
   chxfree (time);
   chxfree (daytime);

   unit_lock (XSNF, READ, 0L, 0L, 0L, 0L, nullchix);
   if (NOT get_neighbor (0, host, XSNF)) {
      bug ("REPARSE: can't get my hostinfo.", 0); 
      unit_unlk (XSNF); FAIL; }
   host->info->seqnum_thru_hi = INC(host->info->seqnum_thru_hi, SEQ_THRU_MOD);
   set_neighbor (host, XSNF);
   unit_unlk (XSNF);

   SUCCEED;
done:
   free_host(host);

   RETURN (success);
}   
