/*** REQ_REPORT.   Report a the return from a request to the Caucus Manager.
/
/   int n = req_report (xptr)
/
/   Parameters:
/      XACTIONPTR xptr;    (A pointer to the xaction)
/
/   Purpose:
/     When one host requests something from another (e.g. ATTREQ, SUBREQ)
/     the request may succeed or fail.  Both conditions should be reported
/     to the manager that made the request.  These codes should be handled
/     here: ATTACK, ATTNAK, SUBACK, SUBNAK.
/
/   How it works:
/
/   Returns:
/      1 on success
/      0 on failure.  In this case it logs the error in the bugfile.
/
/   Error Conditions:
/
/   Related Functions:
/
/   Called by:
/
/   Home: xfu/reqrepor.c
/
/   Dictionary strings:
/    net_Treqreport: CaucusLink has received a response to your recent request.
/    net_Tsubnak1:   Your request for subscription to conference $
/    net_Tsubnak2:   \nfailed $
/    net_Tsubnaknoc: because the conference doesn't exist.
/    net_Tsubnaknop: becuase you aren't allowed to subscribe to this conference.
/    net_Tsubnakunk: for unknown reasons.
/    net_Tsubnak3:   \nPlease contact the CaucusLink Manager at
/                    the neighbor to determine the cause of the problem.
/    net_Tnotype:    The response is in an incorrect format.
/                    There is probably a problem with the transport system.
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: JV 12/23/91 10:25 Create this. */
/*: CR 12/26/91 11:55 Remove unused variables, fix mdstr, free ctext2. */
/*: JV  1/01/92 10:05 Add SUBACK, use sendmgr(), rm conf on SUBNAK. */
/*: JV  1/15/92 16:28 Remove unused var's. */
/*: DE  6/11/92 12:09 ReChixified */
/*: CP  3/04/93 23:16 Remove nested comment. */

#include <stdio.h>
#include "handicap.h"
#include "mess.h"
#include "chixuse.h"
#include "null.h"
#include "xaction.h"
#include "xfp.h"
#include "conf.h"

extern union null_t null;
extern Chix  errtable;

FUNCTION req_report (xptr)
XACTIONPTR xptr;
{
   int      confnum, open;
   char    *reason;
   Chix     ctext, ctext2, title, tname;

   ENTRY ("req_report", "");

   /*** Build the message. */
   ctext    = chxalloc (100L, THIN, "reqrepo ctext");
   ctext2   = chxalloc ( 50L, THIN, "reqrepo ctext2");
   title    = chxalloc ( 50L, THIN, "reqrepo title");
   tname    = chxalloc (100L, THIN, "reqrepo tname");

   mdstr (ctext, "net_Treqreport", null.md);
   chxcpy (tname, xptr->conftname);
   chxupper (tname);

   switch (xptr->typenum) {
   case TT_SUBACK:
      mdstr  (ctext2, "net_Tsuback1", null.md);
      chxcat (ctext, ctext2);
      chxcat (ctext, tname);
      mdstr  (ctext2, "net_Tsuback2", null.md);
      chxcat (ctext, ctext2);
      break;

   case TT_SUBNAK:
      mdstr  (ctext2, "net_Tsubnak1", null.md);
      chxcat (ctext, ctext2);
      chxcat (ctext, tname);
      mdstr  (ctext2, "net_Tsubnak2", null.md);
      chxcat (ctext, ctext2);

      confnum = cnumber (xptr->conftname, TRUECONF, &open);
      if (confnum < 1) {
         mdstr  (ctext2, "net_Tsubnakbad", null.md);
         chxcat (ctext, ctext2);
         mdstr  (ctext2, "net_Tsnbad2", null.md);
         chxcat (ctext, ctext2);
         break;
      }
      sub_unlink (confnum);

      switch (tablematch (errtable, xptr->title) + 1) {
      case NOCONF:
         reason = "net_Tsubnaknoc"; break;
      case NOPERM:
         reason = "net_Tsubnaknop"; break;
      default:
         reason = "net_Tsubnakunk"; break;
      }
      mdstr  (ctext2, reason, null.md);
      chxcat (ctext, ctext2);
      mdstr  (ctext2, "net_Tsubnak3", null.md);
      chxcat (ctext, ctext2);
      break;

   default:
      mdstr  (ctext, "net_Tnotype", null.md);
   }

   mdstr (title, "net_Treqtitle", null.md);
   sendmgr (ctext, title);

   chxfree (ctext);  
   chxfree (ctext2);
   chxfree (title);
   chxfree (tname);
   RETURN  (1);
}   
