/*** RESP_PARSE.  Parse a xaction that the XFU has determined to be resp stuff.
/
/   int n = resp_parse (xptr)
/
/   Parameters:
/     XACTIONPTR xptr;
/
/   Purpose:
/      After the XFU has decided that an incoming xaction is resp stuff,
/      resp_parse() further parses it and applies it to the local db.
/      Note that the XFU deals with routing the info on to other hosts
/      if necessary.  Change xactions should only be processed if this
/      is the original host for the conf.
/
/   How it works:
/
/   Returns:
/      1 on success
/      0 on failure.  In this case it logs the error in the bugfile.
/     -1 if the item is frozen.  This should usually be treated as "success".
/
/   Error Conditions:
/
/   Related Functions: mess_parse() item_parse()
/
/   Called by: parse_xaction()
/
/   Home: xfu/resppars.c
/ */

/*: JV  5/21/91 12:13 Create this. */
/*: JV  6/28/91 13:28 Pre-Integration CARVMark*/
/*: JV  7/04/91 11:38 Fix DNOITEM, DNORESP, remove lint. */
/*: JV  7/15/91 22:44 Add multiple text fields. */
/*: CR  7/24/91 22:13 Make Lint happier. */
/*: CR  7/28/91 15:48 chxalloc() first argument is long. */
/*: JV  7/29/91 12:40 Remove 2nd arg from reparse */
/*: JV  8/05/91 11:11 Add check for In Transit Change. */
/*: JV  8/23/91 16:26 Remove return() notreached. */
/*: JV 10/31/91 11:56 Add XT_DELETE code. */
/*: CR 11/18/91 17:49 Add RETURN at bottom to keep Lint happy. */
/*: JV 11/22/91 11:22 Fix XT_DELETE. */
/*: JV 11/29/91 13:45 Fix return-checking of *_orig_to_local. */
/*: JV  1/21/92 10:33 Check if item is frozen before adding new resp's. */
/*: DE  6/11/92 12:50 ReChixified */
/*: CP  3/04/93 23:13 Fixed nested comments. */
/*: CK  7/27/93 17:40 Add Attachment arg to chg_dresp() calls. */

#include <stdio.h>
#include "handicap.h"
#include "derr.h"
#include "chixuse.h"
#include "null.h"
#include "xaction.h"
#include "done.h"

#define BUG printxact (xptr)

extern union null_t null;

FUNCTION  resp_parse (xptr)
XACTIONPTR xptr;
{
   Chix date, time;
   int  error, localinum, localrnum, success;
   Chix chxalloc();

   ENTRY ("resp_parse", "");

   date    = chxalloc (20L, THIN, "resppars date");
   time    = chxalloc (20L, THIN, "resppars time");
   success = 0;

   switch (xptr->actnum) {
   case XT_NEW:                   /* Ignore xaction if can't parse. */
      if (NOT iorig_to_local ((int)xptr->conflocnum, (int)xptr->itemid,
                              xptr->itemchn, &localinum))  SUCCEED;
      chxbreak (xptr->date, date, time, CQ(" "));
      if (frozen ((int)xptr->conflocnum, localinum, 0)) DONE (-1);
      chg_dresp (XT_NEW, (int)xptr->conflocnum, localinum, (int)xptr->respid,
                 &localrnum, xptr->respchn, xptr->uname, xptr->uid,
                 xptr->data[0], date, time, (Attachment) NULL, &error);
      switch (error) {
         case DINOITEM:
         case DNOCONF:   reparse (xptr); SUCCEED;
         case DIFULL:    tellmgr (xptr, error);
         case NOERR:     SUCCEED;
         default:        FAIL;
      }
      break;
   case XT_DELETE:             /* Ignore xaction if can't parse. */
      if (NOT iorig_to_local ((int)xptr->conflocnum, (int)xptr->itemid,
                          xptr->itemchn, &localinum))  SUCCEED;
      if (NOT rorig_to_local ((int)xptr->conflocnum, localinum,
                          (int)xptr->respid, xptr->respchn, &localrnum))
          SUCCEED;

      chg_dresp (XT_DELETE, (int)xptr->conflocnum, localinum,
                 (int)xptr->respid, &localrnum, xptr->respchn,
                 null.chx, null.chx, null.chx, null.chx, null.chx, 
                 (Attachment) NULL, &error);
      switch (error) {
         case DNOCONF:
         case DINOITEM:
         case DINORESP:  reparse (xptr);  SUCCEED;
         case NOERR:     SUCCEED;
         case DBADDB:
         case DBADACT:   tellmgr (xptr, error);
         default:        FAIL;
      }
      break;
   case XT_CHANGE:              /* Ignore xaction if can't parse. */
      if (NOT iorig_to_local ((int)xptr->conflocnum, (int)xptr->itemid,
                          xptr->itemchn, &localinum))  SUCCEED;
      if (NOT rorig_to_local ((int)xptr->conflocnum, localinum,
                      (int)xptr->respid, xptr->respchn, &localrnum))
         SUCCEED;
      chxbreak (xptr->date, date, time, CQ(" "));
      chg_dresp (XT_CHANGE, (int)xptr->conflocnum, localinum, 
                 (int)xptr->respid, &localrnum, xptr->respchn, xptr->uname, 
                 xptr->uid, xptr->data[0], date, time, 
                 (Attachment) NULL, &error);
      switch (error) {
         case DNOCONF:
         case DINOITEM:
         case DINORESP:  reparse (xptr);  SUCCEED;
         case NOERR:     SUCCEED;
         case DBADDB:
         case DBADACT:   tellmgr (xptr, error);
         default:        FAIL;
      }
      break;
   default:
      BUG; FAIL;
   }

   FAIL;
done:
   chxfree ( date );
   chxfree ( time );
 
   RETURN ( success );
}   
