/* SUB_UNLINK:   Remove a shared conference and warn neighbors.
/
/  int n = sub_unlink(confnum)
/
/  Parameters:
/     int confnum;   (The Local Conference Number)
/
/  Purpose:
/     If a Caucus Mgr was allowed to remove a shared conf without
/     telling any neighbors, the network would have serious problems.
/
/  How it works:
/
/  Returns:
/    0: Failed to remove the conf due to DB problems or conf not shared..
/    1: Removed the conference and sent the SUBDEL xaction to all naybors.
/
/  Error conditions:
/
/  Related functions: 
/
/  Called by: req_report()
/
/  Home: netmaint/subunlin.c
/
/  Dictionary strings:
/     net_Ebaddb:  There is an inconsistency within the Caucus database.
/                  Please report this to the Camber-Roth Support staff.
*/

/*: JV  1/01/92 12:27 Create this. */
/*: DE  6/11/92 14:25 Chixified */
/*: JX  7/31/92 16:59 Fix Chix. */
/*: JX 10/09/92 18:05 Fix pointer. */
/*: CR 10/27/93 16:18 Declare carry[TEXTFIELDS]; don't free it at done: ! */

#include <stdio.h>
#include "handicap.h"
#include "derr.h"
#include "unitcode.h"
#include "chixuse.h"
#include "null.h"
#include "xaction.h"
#include "conf.h"
#include "done.h"

extern union null_t null;

FUNCTION sub_unlink (confnum)
int confnum;
{
   int     i, success; 
   long    lhnum;
   long    systime();
   Chix    lname, tname, orighost, naybor, date, time;
   Xactionptr xaction, make_xaction();
   Chix    carray[TEXTFIELDS];

   ENTRY ("sub_unlink", "");

   lname     = chxalloc (40L, THIN, "subunlin lname");
   tname     = chxalloc (40L, THIN, "subunlin tname");
   orighost  = chxalloc (80L, THIN, "subunlin orighost");
   naybor    = chxalloc (10L, THIN, "subunlin naybor");
   date      = chxalloc (30L, THIN, "subunlin date");
   time      = chxalloc (30L, THIN, "subunlin time");
   success   = 0;

   /*** First validate that this is a shared conference. */
   get_trueconf (confnum, lname, tname, orighost);
   if (EMPTYCHX (tname)) {
      mdwrite (XWER, "net_Enotshared", null.md);
      FAIL;
   }

   /*** Now remove it. */
   rmconf (confnum);

   /*** Now build a xaction and use it to warn each of the neighbors. */
   xaction = make_xaction();
   xaction->typenum = TT_SUBDEL;
   chxclear  (xaction->conftname);
   chxformat (xaction->conftname, CQ("%s@%s"), 0L, 0L, tname, orighost);
   sysdaytime (date, time, 0, systime());
   chxclear   (xaction->date);
   chxformat  (xaction->date, CQ("%s %s"), 0L, 0L, date, time);
 
   if (NOT unit_lock (XCNB, READ, L(confnum), 0L, 0L, 0L, nullchix)) {
      bug ("SUB_UNLINK: can't lock XCNB.", 0);  FAIL; }
   if (NOT unit_check (XCNB)) {
      bug ("SUB_UNLINK: XCNB doesn't exist for conf:", confnum);
      unit_unlk (XCNB);  FAIL; }
   if (NOT unit_view (XCNB)) { unit_unlk (XCNB);
      bug ("SUB_UNLINK: can't open XCNB.", 0);  FAIL; }

   carray[0] = chxalloc (0L, THIN, "sub_unlink carray");
   for (i=1; i< TEXTFIELDS; i++)  carray[i] = carray[0];

   while (unit_read (XCNB, naybor, 0L)) {
      if (NOT chxnum (naybor, &lhnum)) {
         bug ("SUB_UNLINK: bad num in XCNB.", confnum); continue; }
      unit_lock   (XCXT, WRITE, L(confnum), 0L, 0L, 0L, nullchix);
      unit_append (XCXT);
      putnextxact (XCXT, xaction, carray);
      unit_close  (XCXT);
      unit_unlk   (XCXT);
   }
   unit_close (XCNB);
   unit_unlk (XCNB);
   chxfree (carray[0]);

   SUCCEED;

done:
   chxfree (lname);
   chxfree (tname);
   chxfree (orighost);
   chxfree (naybor);
   chxfree (date);
   chxfree (time);
   free_xaction (xaction);

   RETURN (success);
}
