/*** TELLMGR.   Report a bad xaction to the Caucus Manager.
/
/   int n = tellmgr (xptr, error)
/
/   Parameters:
/      XACTIONPTR xptr;    (A pointer to the xaction)
/      int error;          (An error value from derr.h)
/
/   Purpose:
/
/     Sometimes something is just going to go wrong.  Somebody will do
/     something Caucus wasn't prepared for, or there's a bug or whatever.
/     But we don't want this to go undetected, or even undetected for a 
/     long time.  In that case, we should report the bad xaction to
/     *somebody*, and there's no interactive user for the CaucusNet daemons.
/     Instead we'll send a Caucus message to the Caucus Manager.
/
/   How it works:
/
/   Returns:
/      1 on success
/      0 on failure.  In this case it logs the error in the bugfile.
/
/   Error Conditions:
/
/   Related Functions:  sendmgr
/
/   Called by: confpars.c hostpars.c itempars.c messforw.c messpars.c
/              parsexac.c reqrepor.c 
/
/   Known bugs:
/      The 'error' argument is not used.  Should it be?
/
/   Home: xfu/tellmgr.c
/ */

/*: JV  7/01/91 16:46 Create this. */
/*: CR  7/24/91 22:14 Make spec agree with actual function. */
/*: JV  8/05/91 08:13 Fix description of "Home:" */
/*: JV 12/07/91 18:07 Start to add real code. */
/*: UL 12/11/91 18:45 Make unit_lock args 3-6 long.*/
/*: CR 12/11/91 23:55 Declare strtoken, fix chxalloc() call. */
/*: CR 12/21/91 14:54 Use nochix in add_dmess(), and then free it! */
/*: JV 12/23/91 10:15 Change header. */
/*: CR 12/23/91 23:30 Longs must have %ld in sprintf(). */
/*: JV 12/31/91 16:08 Use sendmgr; just build text here. */
/*: JV  1/17/91 16:28 Init ERROR. */
/*: DE  6/11/92 14:56 ReChixified */
/*: JX  9/24/92 09:38 Fixify. */
/*: CP  3/04/93 23:12 Fixed nested comments. */
/*: CN  3/15/93 11:34 Fix chix fields that were treated as char strings. */

#include <stdio.h>
#include "handicap.h"
#include "mess.h"
#include "chixuse.h"
#include "xaction.h"
#include "null.h"
#include "xfp.h"

extern union null_t null;

FUNCTION tellmgr (xptr, error)
XACTIONPTR xptr;
int error;
{
   char     temp[60];
   Chix     ctext, subject;

   ENTRY ("tellmgr", "");

   ctext   = chxalloc (100L, THIN, "tellmgr ctext");
   subject = chxalloc (64L,  THIN, "tellmgr subject");

   error=0;
   printf ("Tellmgr:\n");
   printxact (xptr);
   printf ("\n");

   /*** Turn xaction into a string. */
   mdstr (ctext, "net_Emgrhead", null.md);

   sprintf (temp, "   TYPE     = %-8s  (%d) %s\n", xptr->type, xptr->typenum,
                                          xptr->transit ? "TRANSIT" : "");
   chxcat  (ctext, CQ(temp));
   sprintf (temp, "   ACTION   = %-8s  (%d)\n", xptr->action, xptr->actnum);
   chxcat  (ctext, CQ(temp));
   sprintf (temp, "   SUBTYPE  = %-8s  (%d)\n", xptr->subtype, xptr->subnum);
   chxcat  (ctext, CQ(temp));

   chxformat (ctext, CQ("   CONF     = %-20s  (%ld)\n"), 
                     xptr->conflocnum, 0L, xptr->conftname, nullchix);
   chxformat (ctext, CQ("   UID      = %s\n"), 0L, 0L, xptr->uid, nullchix);
   chxformat (ctext, CQ("   NAME     = %s   PHONE=%s\n"),
                     0L, 0L, xptr->uname, xptr->phone);
   chxformat (ctext, CQ("   IID      = %ld@%s\n"),
                     xptr->itemid, 0L, xptr->itemchn, nullchix);
   chxformat (ctext, CQ("   RID      = %ld@%s\n"),
                     xptr->respid, 0L, xptr->respchn, nullchix);
   chxformat (ctext, CQ("   RECID    = %s@%s\n"),
                     0L, 0L, xptr->recid, xptr->rechost);
   chxformat (ctext, CQ("   RECNAME  = %s\n   TITLE    = %s\n"),
                     0L, 0L, xptr->recname, xptr->title);
   chxformat (ctext, CQ("   DATE     = %s\n"), 0L, 0L, xptr->date, nullchix);

   sprintf (temp, "   MTYPE    = %s\n",     xptr->mtype);
   chxcat  (ctext, CQ(temp));
   sprintf (temp, "   M_OPARTS = %d\n",     xptr->m_oparts);
   chxcat  (ctext, CQ(temp));
   sprintf (temp, "   RRLEN    = %d\n",     xptr->rrlength);
   chxcat  (ctext, CQ(temp));
   sprintf (temp, "   MAILCTRL = %d\n",     xptr->mailctrl);
   chxcat  (ctext, CQ(temp));
   sprintf (temp, "   DATALEN1 = %ld\n",    xptr->datasize[0]);
   chxcat  (ctext, CQ(temp));
   sprintf (temp, "   DATALEN2 = %ld\n",    xptr->datasize[1]);
   chxcat  (ctext, CQ(temp));

   mdstr   (subject, "net_Ttelltitle", null.md);

   sendmgr (ctext, subject);

   chxfree (ctext);
   chxfree (subject);

   RETURN  (1);
}   
