
/*** LNUMPERMUTE.  Permute licensing password with a number.
/
/    lnumpermute (result, number);
/
/    Parameters:
/       char   *result;      (result of permutation)
/       long    number;      (permute with this number)
/
/    Purpose:
/       Lnumpermute is used to compute the "licensing password" for
/       the LICE2 and LICEPASS programs.
/
/    How it works:
/       Lnumpermute permutes the string in RESULT, according to the
/       value of NUMBER.
/
/       RESULT will always contain printable ascii characters.
/
/    Returns: 1
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  licensing programs.
/
/    Operating system dependencies:
/
/    Known bugs:      none
/
/    Home:  xlice2/lnumperm.c
/ */

/*** Copyright (C) 1993 Camber-Roth.  All rights reserved. */

/*: CR  1/24/93 12:03 1st carvmark */
/*: CK  8/07/93 19:02 Comment out debugging output! */
/*: CK 10/13/93 19:31 Use DEBUG macro to control debugging output. */

#include <stdio.h>
#include "lice2.h"
#define  DEBUG     0

FUNCTION  lnumpermute (result, number)
   char  *result;
   long   number;
{
   long   len, pos, val;

   if (number == 0)  return;

   len = strlen (result);
   pos = number % len;
   val = result[pos] - 33;
   val = val + number;
   val = val % 95;
   result[pos] = val + 33;

#if DEBUG
   fprintf (stderr, "Permuting by  %20d , new result is '%s'\r\n", 
                     number, result);
#endif
   return;
}
