
/*** LSTRPERMUTE.  Permute licensing password with a string.
/
/    lstrpermute (result, str);
/
/    Parameters:
/       char   *result;      (result of permutation)
/       char   *str;         (permute with this string)
/
/    Purpose:
/       Lstrpermute is used to compute the "licensing password" for
/       the LICE2 and LICEPASS programs.
/
/    How it works:
/       Lstrpermute permutes the string in RESULT, according to the
/       value of string STR.
/
/       RESULT will always contain printable ascii characters.
/
/    Returns: 1
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  licensing programs.
/
/    Operating system dependencies:
/
/    Known bugs:      none
/
/    Home:  xlice2/lstrperm.c
/ */

/*** Copyright (C) 1993 Camber-Roth.  All rights reserved. */

/*: CR  1/24/93 12:03 1st carvmark */
/*: CR  1/27/93 16:34 Remove ' from universe of results. */
/*: CR  2/02/93 11:28 Force 7 bits, modulus 94. */
/*: CK  8/07/93 19:02 Comment out debugging output! */
/*: CK 10/13/93 19:31 Use DEBUG macro to control debugging output. */

#include <stdio.h>
#include "lice2.h"
#define  DEBUG      0

FUNCTION  lstrpermute (result, str)
   char  *result, *str;
{
   char  *s;
   short  byte7;
   long   len, pos, val;

   if (NULLSTR(str))  return;

   len = strlen (result);

   for (s=str;   *s;   ++s) {
      byte7 = *s & 0x7F;
      if (byte7 == ' ')    continue;
      pos = (byte7) % len;
      val = result[pos] - 33;
      val = val + (byte7);
      val = val % 94;
      result[pos] = val + 33;
      if (result[pos] == '\'')  result[pos] = '\"';
   }

#if DEBUG
   fprintf (stderr, "Permuting by '%20s', new result is '%s'\r\n", 
                     str, result);
#endif
   return;
}
