/*** REG_PARSE_FILE.  Parse a file of pre-registration info.
/
/    reg_parse_user (fname)
/
/  Parameters:
/     Chix fname;     // Name of the file
/
/  Purpose:
/    REG_PARSE_FILE is one of the functions that make up the pre-registration
/    feature.
/
/  How it works:
/     One of our famous state tables, this one has the following states:
/     FIRST: Processing user's "first line"
/     IN:    Processing a new user's other info
/     OUT:   Not processing a person at all
/
/  Returns: 1 if we registered all users that were specified
/           0 if we couldn't process one or more users
/
/  Error Conditions:
/
/  Side Effects:
/
/  Related to: reg_user()
/
/  Called by: 
/
/  Home: xmaint/regparse.c
*/

/*** Copyright (C) 1988 Camber-Roth.  All rights reserved. */

/*: JX  8/05/92 09:25 Creat this. */
/*: JV 12/04/92 13:47 Add sitevar arg to storereg(). */
/*: CL 12/11/92 11:18 Long Chxcatsub args. */
/*: CL 12/11/92 13:48 Long Chxvalue N. */
/*: CR  1/13/93 13:41 Add make_partic(), free_partic() calls. */
/*: CN  4/07/93 15:46 Replace clear_reg() chxalloc's with chxclear's. */
/*: CP  5/06/92 12:41 Add 0L as 'item' arg to modnames() call. */
/*: JV  9/26/93 13:33 Add ENTRY to most functions here. */

#include <stdio.h>
#include "caucus.h"

#define OUT   0
#define FIRST 1
#define IN    2

#define BAD_USER {state=OUT;if (NOT bad_user(line)) FAIL;continue;}

extern struct flag_template    flag;
extern union null_t            null;
extern Chix                    sitevar[];
extern Chix                    newuseropts;

FUNCTION  reg_parse_file (fname)
   Chix fname;
{
   int     i, state=0, var, success=1;
   Flag    captive=0;
   Chix    aft, str, line, rest, passwd, colon;
   Userreg reg, make_userreg();
   struct  namlist_t *nlnode();

   ENTRY ("reg_parse_file", "");

   aft    = chxalloc (80L, THIN, "regparse aft");
   str    = chxalloc (80L, THIN, "regparse str");
   rest   = chxalloc (80L, THIN, "regparse line");
   line   = chxalloc (80L, THIN, "regparse rest");
   colon  = chxalloc (10L, THIN, "regparse colon");
   passwd = chxalloc (80L, THIN, "regparse passwd");

   reg = make_userreg();
   reg->briefs = nlnode(1);
   reg->print = nlnode(1);
   reg->export_mail = 0;  /* Kludge til sys_Tset0 has value for this. */
   for (i=0; i<=9; i++) sitevar[i] = chxalloc (80L, THIN, "regparse site");
   chxcpy (colon, CQ(":"));

   unit_lock (XSUP, READ, 0L, 0L, 0L, 0L, fname);
   if (NOT unit_view (XSUP)) { unit_unlk (XSUP); FAIL; }

   while (unit_read (XSUP, line, 0L)) {
      if (chxvalue (line, 0L) == '#') continue;
      if (state == OUT) {
         state = FIRST;
         if (NOT chxbreak (line, reg->name, aft, colon)) 
            BAD_USER                          /* Fatal error in file? */
         chxcpy (rest, aft);
         if (NOT chxbreak (rest, reg->lastname, aft, colon)) 
            BAD_USER                          /* Fatal error in file? */
         chxcpy (rest, aft);
         if (NOT chxbreak (rest, reg->id, aft, colon)) 
            BAD_USER                          /* Fatal error in file? */
         chxcpy (rest, aft);
         if (NOT chxbreak (rest, reg->phone, aft, colon)) 
            BAD_USER                          /* Fatal error in file? */
         if (NOT EMPTYCHX (aft)) {             /* This is a captive user. */
            chxcpy (rest, aft);
            if (NOT chxbreak (rest, passwd, aft, colon)) 
               BAD_USER                          /* Fatal error in file? */
            captive = 1;
         }
         if (NOT EMPTYCHX (passwd))       /* Captive user. */
            chxcat (reg->id, CQ("_"));
         if (NOT reg_user (reg, captive, passwd)) {  /* If fail, skip rest*/
            while (unit_read (XSUP, line, 0L))       /* of entry. */
               if (chxvalue (line, 0L) == 'E') break;
            state = OUT;
            clear_reg (reg);
            chxclear (aft); chxclear (line); chxclear (passwd);
         }
         else {
            chxclear  (str);
            chxformat (str, CQ("Adding user: %s.\n"), 0L, 0L,
                       reg->id, nullchix);
            unit_write (XITT, str);
         }
         continue;
      }

      /*** We're already processing a person, so grab lines and process
      /    each individually. */
      switch (chxvalue (line, 0L)) {
      case ('S'):                              /* SETtings */
         chxclear  (reg->setopts);
         chxcatsub (reg->setopts, line, L(1), ALLCHARS);
         decode_setopts (reg, reg->setopts, nullchix, newuseropts);
         state = IN;
         break;
      case ('I'):                              /* Brief Intro */
         chxclear  (str);
         chxcatsub (str, line, L(1), ALLCHARS);
         nlcat (reg->briefs, str, 0);
         break;
      case ('U'):                              /* User Var's */
         var = chxvalue (line, L(1)) - '0';
         chxcatsub (sitevar[var], line, L(2), ALLCHARS);
         break;
      case ('C'):
         add_user_conf (reg, line);
         break;
      case ('E'):                              /* End of User */
         storereg (reg, 0, sitevar);
         clear_reg (reg);
         chxclear (aft); chxclear (line); chxclear (passwd);
         for (i=0; i<10; i++) chxclear (sitevar[i]);
         state = OUT;
         break;
      default:
         chxclear (str);
         chxformat (str, CQ("Incorrect line: \n%s.\n"), 0L, 0L,
                    line, nullchix);
         unit_write (XITT, str); unit_write (XITT, CQ("Continuing.\n"));
         break;
      }
   }
   unit_close (XSUP);
   unit_unlk  (XSUP);

done:
   chxfree (aft);
   chxfree (str);
   chxfree (line);
   chxfree (rest);
   chxfree (passwd);
   free_userreg (reg);

   RETURN (success);
}

/*** We found some bad info in the first line.
/    Report it and try to recover. */
FUNCTION bad_user (line)
   Chix line;
{
   int  success;
   Chix message;

   ENTRY ("bad_user", "");

   message = chxalloc (80L, THIN, "baduser message");

   chxformat (message,
         CQ("The following user information was not specified correctly:\n%s"),
         0L, 0L, line, nullchix);
   unit_write (XITT, message);
   unit_write (XITT, CQ("Ignoring that user and continuing.\n"));
   while (unit_read (XSUP, line, 0L)) 
      if (chxvalue (line, 0L) == 'E') break;

   /*** At end of record or file.  Which is it? */
   if (chxvalue (line, 0L) != 'E') {
      unit_write (XITT, CQ("Could not find next user.\nAborting.\n"));
      FAIL;
   }
   SUCCEED;

done:
   chxfree (message);
   RETURN (success);
}

extern short confnum;

/*** Add a user to a conf, no questions asked. */
FUNCTION add_user_conf (reg, line)
   Userreg reg;
   Chix    line;
{
   int     openval, success, i;
   Chix    confname, message, name;
   struct  partic_t thisconf;
   Namelist add, del, nlnode();

   ENTRY ("add_user_conf", "");

   make_partic (&thisconf, MAXITEMS);
   name     = chxalloc (20L, THIN, "adduconf name");
   confname = chxalloc (20L, THIN, "adduconf confname");
   message  = chxalloc (80L, THIN, "adduconf message");

   chxclear  (confname);
   chxcatsub (confname, line, L(1), ALLCHARS);
   confnum = cnumber (confname, LOCALCONF, &openval);
   
   if (confnum < 1) {
      chxformat (message, CQ("Conference %s does not exist.\nContinuing.\n"),
                 0L, 0L, confname, nullchix);
      unit_write (XITT, message);
      FAIL;
   }
   if (NOT allowed_on (reg->id, null.nl, confnum)) {
      chxformat (message, CQ("User %s is not allowed in conference %s.\n"),
                 0L, 0L, reg->id, confname);
      unit_write (XITT, message);
      unit_write (XITT, CQ("Continuing.\n"));
      FAIL;
   }

   /*** Create participation file for user. */
   thisconf.items = 0;
   for (i=0;  i<MAXITEMS;     ++i)   thisconf.responses[i] = -1;
   if (unit_lock (XUPA, WRITE, L(confnum), 0L, 0L, 0L, reg->id)) {
      store_user (&thisconf);
      unit_unlk  (XUPA);
   }

   /*** Add user to membership list for this conf. */
   tomember (name, reg->lastname, reg->name, reg->id);
   add = nlnode(1);
   del = nlnode(1);
   nladd  (add, name, 0);
   modnames (XCMD, XCMN, nullchix, confnum, 0L, add, del, NOPUNCT);
   nlfree (add);
   nlfree (del);

   SUCCEED;

done:
   chxfree (name);
   chxfree (confname);
   chxfree (message);
   free_partic (&thisconf);
   RETURN (success);
}


/*** Empty out a Userreg. */
FUNCTION clear_reg (u)
   Userreg u;
{
   Namelist nlnode();

   ENTRY ("clear_reg", "");

   chxclear ( u->name );
   chxclear ( u->lastname );
   chxclear ( u->id );
   chxclear ( u->setopts );
   chxclear ( u->sysopts );
   chxclear ( u->phone );
   chxclear ( u->laston );
   chxclear ( u->editor );
   chxclear ( u->eot );
   chxclear ( u->startmenu );

   if (u->briefs != null.nl) nlfree (u->briefs); u->briefs = nlnode(1);
   if (u->print  != null.nl) nlfree (u->print);  u->print  = nlnode(1);

   RETURN (1);
}
