
/*** ASSEMBLE_CHUNKS.   Assemble a set of chunks into a transport file.
/
/    ok = assemble_chunks (u, recmeth, p);
/
/    Parameters:
/       int       ok;        (success?)
/       int       u;         (unit number of target transport file)
/       int       recmeth;   (number of receiving bay)
/       Trans     p;         (Trans node containing list of chunks)
/
/    Purpose:
/       Assemble a set of chunk files into a single transport file.
/
/    How it works:
/       Read the chunk files listed in Trans node P, in order, and write
/       their contents (skipping the chunk header for each file) to the
/       transport file specified by U, SEQ, RECMETH, and HNUM.
/
/       Delete the chunk files as they are read.
/
/       SEQ may be modified.  (Should it always be incremented once?)
/
/    Returns: 1 on success, 0 on error.
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by: 
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  xrec/assemble.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: CR  8/20/91 15:51 New function. */
/*: UL 12/11/91 18:45 Make unit_lock args 3-6 long.*/
/*: JX  7/29/92 16:06 Chixify. */

#include <stdio.h>
#include "chixuse.h"
#include "entryex.h"
#include "handicap.h"
#include "unitcode.h"
#include "xaction.h"
#include "recv.h"

FUNCTION  assemble_chunks (u, recmeth, p)
   int    u, recmeth;
   Trans  p;
{
   int    i, j;
   Chix   temp;

   ENTRY ("assemble_chunks", "");

   temp = chxalloc (100L, THIN, "assemble temp");

   /*** Read through all of the chunk files, (skipping the 4 line chunk
   /    header in each one), and write them to the transport file. */
   for (i=1;   i<=p->highchunk;   ++i) {
      if (NOT unit_lock (XRXC, WRITE, L(recmeth),  p->xnums[i],
                        0L, 0L, nullchix )) 
                                                        break;
      if (NOT unit_view (XRXC))   { unit_unlk (XRXC);   break; }
      for (j=0;   j<4;   ++j)  unit_read (XRXC, temp, 0L);
      while (unit_read (XRXC, temp, 0L)) {
         unit_write (u, temp);
         unit_write (u, CQ("\n"));
      }
      unit_close (XRXC);
      unit_kill  (XRXC);
      unit_unlk  (XRXC);
   }

   chxfree (temp);
   RETURN ( i > p->highchunk );
}
