/*** DO_RERECEIVE.   Process any files ready to use in the rereceive bay.
/
/    ok = do_rereceive (from, parseseq, hnum, recmeth);
/
/    Parameters:
/       int       ok;
/       long     *from;      (next transport file # ready to receive)
/       long     *parseseq;  (next slot for transport file in parse bay)
/       int       hnum;      (host transport file came from)
/       int       recmeth;   (receiving bay number)
/
/    Purpose:
/       Sometimes do_transport() receives transport files higher than
/       the one it's expecting.  Those files are placed in the re-receive
/       bay.
/
/       When do_transport() *does* get the file it's expecting, then it
/       can look back in the re-receive bay and see if it is now expecting
/       any of those files.
/
/       That's what do_rereceive() does.  If it finds any such files, it
/       officially receives them, moving them into the parse bay.
/
/    How it works:
/       Look for XRRX file, specified by FROM, RECMETH, and HNUM.
/       If found, move it to first empty XSPB slot (starting at PARSSEQ).
/       Increment FROM and PARSESEQ and try again.
/
/    Returns: 1 normally (whether an XRRX file is found or not), or 0
/       in case of locking errors.
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  do_transport()
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  xrec/dorerece.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: CR  8/21/91 14:23 New function. */
/*: JV  9/19/91 11:25 Use seqnum macro ops. */
/*: UL 12/11/91 18:45 Make unit_lock args 3-6 long.*/
/*: JX  7/29/92 16:39 Chixify. */

#include <stdio.h>
#include "entryex.h"
#include "handicap.h"
#include "unitcode.h"
#include "chixuse.h"
#include "xaction.h"
#include "recv.h"
#include "sequence.h"

FUNCTION  do_rereceive (from, parseseq, hnum, recmeth)
   long  *from, *parseseq;
   int    hnum, recmeth;
{
   ENTRY ("do_rereceive", "");

   /*** Look in the re-receive bay for a transport file with sequence
   /    number FROM.  If it's there, we can officially receive it now,
   /    by moving it to the parse bay.  If this succeeds, increment
   /    FROM, and repeat. */
   while (1) {
      if (NOT unit_lock  (XRRX, WRITE, L(recmeth), L(hnum),  *from,
                          0L, nullchix)) 
                                                        RETURN (0);
      if (NOT unit_check (XRRX))  { unit_unlk (XRRX);   break; }

      /*** We have an XRRX file, now get the first open transport file
      /    slot in the parse bay. */
      while (1) {
         if (NOT unit_lock  (XSPB, WRITE,  *parseseq, 0L, 0L, 0L, nullchix))
            { unit_unlk (XRRX);   RETURN (0); }

         if (NOT unit_check (XSPB))  break;
         unit_unlk (XSPB);
         *parseseq = INC (*parseseq, SEQ_PARS_MOD);
      }

      unit_move (XRRX, XSPB);
      unit_unlk (XSPB);
      unit_unlk (XRRX);
      *from = INC (*from, SEQ_FROM_MOD);
   }

   RETURN (1);
}
