/*** EXPAND_TRANS.  Expand the size of a Trans node to contain more chunks.
/
/    expand_trans (node, chunk);
/
/    Parameters:
/       Trans     node;        (expand this node in a Trans list)
/       int       chunk;       (expand to include this chunk number)
/
/    Purpose:
/       A Trans node contains an array of information about the chunks 
/       received for a particular transport file.  Expand that array
/       to be able to hold at least CHUNK chunks.
/
/    How it works:
/
/    Returns: 1
/
/    Error Conditions
/  
/    Side effects:
/
/    Related functions:
/
/    Called by: build_trans_list()
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  xrec/expandtr.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: CR  8/14/91 18:33 New function. */
/*: CR 12/14/92 16:00 Sysmem arg is long. */

#include <stdio.h>
#include "entryex.h"
#include "handicap.h"
#include "recv.h"

FUNCTION  expand_trans (node, chunk)
   Trans  node;
   int    chunk;
{
   int    i;
   long  *ynums;
   char  *sysmem();

   ENTRY ("expand_trans", "");

   /*** Is this chunk lower than the highest chunk we've seen? */
   if (chunk <= node->highchunk)  RETURN (1);
   node->highchunk = chunk;

   /*** Is any expansion of the XNUMS array really needed? */
   if (chunk <= node->xsize)      RETURN (1);
   chunk += 10;

   ynums = (long *) sysmem (L(chunk) * sizeof(long));
   for (i=0;               i<=node->xsize;   ++i)  ynums[i] = node->xnums[i];
   for (i=node->xsize+1;   i<=chunk;         ++i)  ynums[i] = -1L;
   sysfree ( (char *) node->xnums);
   node->xnums = ynums;
   node->xsize = chunk;

   RETURN (1);
}
