
/*** FIND_TRANS.   Find an entry for a transport file in a Trans list.
/
/    ptr = find_trans (head, seq, host);
/
/    Parameters:
/       Trans     ptr;         (returned node in Trans list)
/       Trans     head;        (search this Trans list)
/       long      seq;         (sequence number)
/       Chix      xhost;       (host name)
/
/    Purpose:
/       Search the linked Trans list HEAD for an entry for the transport
/       file with sequence number SEQ and chn HOST.  Return that entry.
/
/    How it works:
/
/    Returns: node containing this entry, or NULL if no such node.
/
/    Error Conditions
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  build_trans_list()
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  xrec/findtran.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: CR  8/14/91 18:33 New function. */
/*: JX  7/29/92 16:59 Chixify. */

#include <stdio.h>
#include "entryex.h"
#include "handicap.h"
#include "chixuse.h"
#include "recv.h"

FUNCTION Trans find_trans (head, seq, host)
   Trans  head;
   long   seq;
   Chix   host;
{
   Trans  p;

   ENTRY ("find_trans", "");

   if (seq < 0)  RETURN (NULL);

   for (p=head->next;   p!=NULL;   p = p->next)
      if (p->seq == seq  &&  chxeq (p->chn, host))  RETURN (p);

   RETURN (NULL);
}
