/*** HOST_NUMBER.   Return the host number of a given CHN.
/
/    hnum = host_number (chn);
/
/    Parameters:
/       int       hnum;      (returned host number)
/       Chix      chn;       (caucus host name)
/
/    Purpose:
/       Given a host name, return its host number.
/
/    How it works:
/       Host_number calls hnumber() to do the actual work, but then
/       caches the result in a chix for future calls.  This saves
/       opening the (rarely changing) hostddd/hostNNN files every
/       time we need to do this mapping.
/
/    Returns: host number
/             -1 if CHN is not a recognized host.
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by: 
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  xrec/hostnum.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: CR 10/03/91 20:11 New function. */
/*: CR 10/09/91 17:34 Correct 2nd arg to chxlong(). */
/*: JX  7/29/92 17:17 Chixify. */
/*: CR 12/14/92 15:12 Long last arg to chxnextline(). */

#include <stdio.h>
#include "entryex.h"
#include "handicap.h"
#include "chixuse.h"
#include "xfp.h"
#include "unitcode.h"
#include "xaction.h"
#include "derr.h"
#include "recv.h"

FUNCTION  host_number (chn)
   Chix   chn;
{
   static Chix map = nullchix;
   static Chix line, name, chxchn;
   int         hnum;
   long        i, pos;

   ENTRY ("host_number", "");

   /*** Initialize static chix on first call. */
   if (map == nullchix) {
      map    = chxalloc (80L, THIN, "host_number map");
      line   = chxalloc (40L, THIN, "host_number line");
      name   = chxalloc (40L, THIN, "host_number name");
      chxchn = chxalloc (40L, THIN, "host_number chxchn");
   }

   /*** Scan MAP for entry that contains CHN.  If found, return the
   /    matching host number. */
   chxcpy (chxchn, chn);
   for (i=pos=0;   chxnextline (line, map, &i);   ) {
      chxtoken  (name, nullchix, 2, line);
      if (chxeq (name, chxchn))   RETURN ((int) chxlong (line, &pos));
   }

   /*** If not found, call hnum() to go look at the hostddd/hostnnn data
   /    files to find the host number.  Then save this mapping in MAP. */
   if ( (hnum = hnumber (chn)) < 0)  RETURN (-1L);
   chxclear  (map);
   chxformat (map, CQ("%d %s\n"), L(hnum), 0L, chn, nullchix);
   RETURN  (hnum);
}
