/*** NEIGHBOR.  Return the host information for a particular neighbor.
/
/    host = neighbor (hnum);
/
/    Parameters:
/       HOST     *host;      (pointer to host information)
/       int       hnum;      (number of desired host)
/
/    Purpose:
/       Given a host number, return a pointer to the information about
/       that host.
/
/    How it works:
/       Neighbor() always returns a pointer to a static cache.
/       If hnum is the number of the last (most recent) host loaded,
/       neighbor returns immediately.
/
/       Otherwise, we call get_neighbor() to actually load the host
/       info into the cache.
/
/    Returns: pointer to host information
/
/    Error Conditions:
/       If the host cannot be locked or loaded, neighbor() still
/       returns the pointer to the (empty) host structure, but
/       the CHN field is set to the empty string.
/  
/    Side effects:
/
/    Related functions:
/
/    Called by: 
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  xrec/neighbor.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: CR 10/03/91 17:20 New function. */
/*: UL 12/11/91 18:45 Make unit_lock args 3-6 long.*/
/*: JX  7/30/92 10:33 Chixify. */

#include <stdio.h>
#include "entryex.h"
#include "handicap.h"
#include "xfp.h"
#include "unitcode.h"
#include "xaction.h"
#include "derr.h"
#include "recv.h"

FUNCTION  HOST * neighbor (hnum)
   int    hnum;
{
   static int       last_hnum = -1;
   static HOSTPTR   host = (HOSTPTR) NULL;
   HOSTPTR          make_host();
   Hinfoptr         make_hinfo();
   int              ok;

   ENTRY ("neighbor", "");

   if (host == (HOSTPTR) NULL) {
      host = make_host();
      host->info = make_hinfo();
   }

   if (hnum == last_hnum)   RETURN (host);

   if (ok = (hnum >= 0  &&  unit_lock (XHI1, READ, L(hnum), 0L, 0L, 0L,
          nullchix))) {
      ok = get_neighbor (hnum, host, XHI1);
      last_hnum = hnum;
      unit_unlk (XHI1);
   }

   if (NOT ok) { chxclear (host->chn);   last_hnum = -1; }

   RETURN (host);
}
