/*** OPEN_TRANSPORT.   Create an empty transport file.
/
/    ok = open_transport (u, recmeth, hnum, seq);
/
/    Parameters:
/       int       ok;        (success?)
/       int       u;         (unit number of target transport file)
/       int       recmeth;   (number of receiving bay containing chunks)
/       int       hnum;      (number of host chunks received from)
/       long     *seq;       (sequence number of created transport file)
/
/    Purpose:
/       Lock/create a transport file in the re-receive or parse bay.
/
/    How it works:
/       U==XRRX creates the transport file in the re-receive bay,
/       specified by RECMETH, HNUM, and SEQ.
/
/       U==XSPB creates the transport file in the parse bay, specified
/       by SEQ.  If such a file already exists, keep incrementing SEQ
/       until an empty slot is found.
/
/    Returns: 1 on success, 0 on error.
/
/    Error Conditions:
/ 
/    Side effects:
/
/    Related functions:
/
/    Called by:  do_transport()
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  xrec/opentran.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: CR  8/21/91 11:44 New function. */
/*: JV  9/19/91 11:26 Use seqnum macro ops. */
/*: UL 12/11/91 18:45 Make unit_lock args 3-6 long.*/
/*: JV  1/22/92 16:00 Log which XSPB file we're creating. */
/*: JX  7/31/92 11:44 Declare strito36(). */

#include <stdio.h>
#include "entryex.h"
#include "handicap.h"
#include "unitcode.h"
#include "xaction.h"
#include "recv.h"
#include "sequence.h"

FUNCTION  open_transport (u, recmeth, hnum, seq)
   int    u, recmeth, hnum;
   long  *seq;
{
   char buf[16], *strito36();

   ENTRY ("open_transport", "");

   /*** Creating the XRRX transport file is straight-forward. */
   if (u==XRRX) {
      if (NOT unit_lock (u, WRITE, L(recmeth), L(hnum),  *seq, 0L, nullchix))
                                                    RETURN (0);
      if (NOT unit_make (u))   { unit_unlk (u);     RETURN (0); }
      RETURN (1);
   }

   /*** The XSPB is trickier.  Starting with SEQ, look for the first
   /    unused transport file slot.   SEQ may be incremented. */
   while (1) {
      strito36 (buf, *seq, 8);
      printf ("Receiving file PARSEBAY/x%7s.\n", buf);
      if (NOT unit_lock  (u, WRITE,  *seq, 0L, 0L, 0L, nullchix))  RETURN (0);
      if (NOT unit_check (u))  break;
      unit_unlk (u);
      *seq = INC (*seq, SEQ_PARS_MOD);
   }
   if (NOT unit_make (u))   { unit_unlk (u);     RETURN (0); }

   RETURN (1);
}
