/*** RECMAN.   Program to move manually received files into RECVnnn/CHUNK.
/
/    Invoked as:
/       recman  confid  recmeth  [-d]  <list
/
/    Arguments:
/       confid     full path name of Caucus home directory.
/       recmeth    receiving method number
/       [-d]       option debug flag
/       list       file (or pipe) containing list of manually received files.
/
/    Purpose:
/       This program is used by the CaucusNet RECVM002 receiver-method
/       script, to move files received "manually", from RECVmmm/MANUAL
/       into a set of transport "chunk" files in RECVmmm/CHUNK.
/
/    How it works:
/
/    Exit status: 0 on success, 1 on failure.
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  xrec/recman.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: CR  9/15/91 13:37 New program. */
/*: CR  9/17/91 13:32 Requires use of mainglobals(). */
/*: CR  9/17/91 15:02 Remove unused variables. */
/*: UL 12/11/91 18:45 Make unit_lock args 3-6 long.*/
/*: CR 12/26/91 17:40 Extra unlock of XIRE should be XRXC. */
/*: JX  7/31/92 11:14 Chixify, utilglob before mainglob. */
/*: CR 12/14/92 15:20 Long chxlong 2nd arg. */
/*: CP  3/04/93 23:59 Make 'debug' extern. */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"
#include "xaction.h"
#include "recv.h"

extern struct screen_template  used, screen;
extern Chix                    confid;
extern int                     debug;

FUNCTION  main (argc, argv)
   int    argc;
   char  *argv[];
{
   char   name[202];
   int    ok, recmeth;
   long   chunk, index, increment();
   Chix   temp;

   screen.lines =  0;    used.lines = 0;
   screen.width = 80;    used.width = 0;

   /*** Check arguments and initialize CONFID. */
   if (argc < 3  ||  argc > 4) {
      printf ("Usage: recman  confid  recmeth  [-d]  <list\n");
      exit   (1);
   }
   if (argc == 4)  debug = -1;
   sscanf (argv[2], "%d", &recmeth);

   confid = chxalloc (40L, THIN, "recman confid");
   chxofascii (confid, argv[1]);
   utilglobals();
   mainglobals();

   temp = chxalloc (100L, THIN, "main temp");
   /*** Get the current chunk counter. */
   if (ok = unit_lock (XRCC, READ, L(recmeth), 0L, 0L, 0L, nullchix)) {
      if (ok = unit_view (XRCC)) {
         if (ok = unit_read (XRCC, temp, 0L)) {
            index = 0; chunk = chxlong (temp, &index); }
         unit_close (XRCC);
      }
      unit_unlk (XRCC);
   }
   if (NOT ok)  chunk = 0L;

   /*** For each file listed in the standard input... */
   while (fgets (name, 160, stdin) != NULL) {
      name [strlen(name)-1] = '\0';
      if (NOT unit_lock (XIRE, WRITE, 0L, 0L, 0L, 0L, CQ(name)))  exit (1);

      /*** Get the next unused chunk number. */
      while (1) {
         chunk = increment (chunk, SEQ_MOD);
         if (NOT unit_lock (XRXC, WRITE, L(recmeth),  chunk, 0L, 0L, nullchix))
            exit (1);
         if (NOT unit_check (XRXC))  break;
         unit_unlk (XRXC);
      }

      unit_move (XIRE, XRXC);
      unit_unlk (XRXC);
      unit_unlk (XIRE);
   }

   /*** ... and rewrite the chunk counter. */
   if (unit_lock (XRCC, WRITE, L(recmeth), 0L, 0L, 0L, nullchix)) {
      if (unit_make (XRCC)) {
         chxclear  (temp);
         chxformat (temp, CQ("%d\n"), chunk, 0L, nullchix, nullchix);
         unit_write (XRCC, temp);
         unit_close (XRCC);
      }
      unit_unlk (XRCC);
   }
   exit (0);
}
