
/*** RECVERROR.  Report errors during receipt of files.
/
/    fail = recverror (error, recvmeth, seq);
/
/    Parameters
/       int  fail;      (serious enough to stop?)
/       int  error;     (error number)
/       int  recvmeth;  (receiving method)
/       long seq;       (transport file sequence number)
/
/    Purpose:
/       Report receiver errors in a comprehensible way.  Determine if 
/       error is serious enough to stop further processing of received
/       files.
/
/    How it works:
/
/    Returns: 1 if error is serious (enough to stop)
/             0 otherwise
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  xrec/recverro.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: CR  8/16/91 13:17 New function. */
/*: CR 12/23/91 23:34 Longs must have %ld in sprintf(). */
/*: JV 10/26/92 11:29 Add DR_BADUU & DR_BADCPT. */

#include <stdio.h>
#include "handicap.h"
#include "entryex.h"
#include "derr.h"

FUNCTION  recverror (error, recvmeth, seq)
   int    error, recvmeth;
   long   seq;
{
   char   msg[400];
   int    fail;

   ENTRY ("recverror", "");

   fail = 0;
   switch (error) {

      case (DR_NOXRXC):
         sprintf (msg, "Receiver: method=%d.  Can't lock/open chunk x%07ld!\n",
                        recvmeth, seq);
         bug (msg, error);
         break;

      case (DR_BADCHK):
         sprintf (msg, "Receiver: method=%d.  Deleting impostor chunk x%07ld.\n",
                        recvmeth, seq);
         bug (msg, error);
         break;

      case (DR_BADUU):
         sprintf (msg, "Receiver: method=%d. Bad uuencoded chunk.\n", recvmeth);
         bug (msg, error);
         break;

      case (DR_BADCPT):
         sprintf (msg, "Receiver: method=%d. Bad encrypted chunk.\n", recvmeth);
         bug (msg, error);
         break;

      default:
         bug ("Receiver: unknown error!\n", error);
         break;
   }

   RETURN (fail);
}
