
/*** SEND_ACK.   Send an ACK or NAK to a host.
/
/    ok = send_ack (hnum, shipm, seqnum, ackseq, type);
/   
/    Parameters:
/       int       ok;        (success?)
/       int       hnum;      (host to send to)
/       int       shipm;     (shipping method to use)
/       long      seqnum;    (sequence number of transport file being ACK'd)
/       long     *ackseq;    (number of ACK file)
/       int       type;      (ACK or NAK?)
/
/    Purpose:
/       Send an ACK or NAK to host HNUM for transport file SEQNUM.
/
/    How it works:
/       ACKSEQ is the number of the next ACK file to send to this host.
/       (If there already is an ACK file by that number, increment ACKSEQ).
/       Increment ACKSEQ (again) once the ACK file is written.
/
/    Returns: 1 on success, 0 if the ACK file could not be locked or written.
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by: 
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  xrec/sendack.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: CR  8/20/91 15:51 New function. */
/*: CR  9/05/91 12:30 Always increment ACKSEQ. */
/*: JV  9/19/91 11:30 Use seqnum macro ops. */
/*: UL 12/11/91 18:45 Make unit_lock args 3-6 long.*/
/*: JX  7/30/92 14:50 Chixify. */

#include <stdio.h>
#include "entryex.h"
#include "handicap.h"
#include "unitcode.h"
#include "xaction.h"
#include "recv.h"
#include "sequence.h"
#include "done.h"

FUNCTION  send_ack (hnum, shipm, seqnum, ackseq, type)
   int    hnum, shipm, type;
   long   seqnum;
   long  *ackseq;
{
   Chix   temp, hostname, acktype;
   int    success

   ENTRY ("send_ack", "");

   temp     = chxalloc (100L, THIN, "sendack temp");
   hostname = chxalloc (100L, THIN, "sendack hostname");
   acktype  = chxalloc (4L,   THIN, "sendack acktype");

   getmyhost (hostname);

   /*** Find an unused ACKSEQ number. */
   while (1) {
      if (NOT unit_lock (XPAK, WRITE, L(shipm), L(hnum),  *ackseq, 0L, nullchix))
                                                    FAIL;
      if (NOT unit_check (XPAK))  break;
      unit_unlk (XPAK);
      *ackseq = INC (*ackseq, SEQ_ACK_MOD);
   }
   if (NOT unit_make (XPAK))  { unit_unlk (XPAK);   FAIL; }

   chxformat  (temp, CQ("CHUNKHOST=%s\n"), 0L, 0L, hostname, nullchix);
   unit_write (XPAK, temp);
   unit_write (XPAK, CQ("CHUNKSEQN=-1\n"));
   unit_write (XPAK, CQ("CHUNKPART=1\n"));
   unit_write (XPAK, CQ("CHUNKLAST=1\n"));
   chxclear   (temp);
   chxcpy  (acktype, CQ(type==ACK ? "ACK" : "NAK"));
   chxformat  (temp, CQ("ACKTYPE=%s\n"), 0L, 0L, acktype, nullchix);
   unit_write (XPAK, temp);
   chxclear   (temp);
   chxformat  (temp, CQ("ACKSEQN=%d\n"), seqnum, 0L, nullchix, nullchix);
   unit_write (XPAK, temp);

   unit_close (XPAK);
   unit_unlk  (XPAK);

   *ackseq = INC (*ackseq, SEQ_ACK_MOD);
   SUCCEED;

done:
   chxfree (temp);
   chxfree (hostname);
   chxfree (acktype);
   RETURN (success);
}
