
/*** GETHOSTMAP.   Get a map of active hosts.
/
/    map = gethostmap (hosts);
/
/    Parameters:
/       char  *map;         (Returned host map)
/       int    hosts;       (highest host number)
/
/    Purpose:
/       Return a char array (MAP) of which hosts are active.
/       MAP[n] is 1 if host N is active; otherwise, it contains 0.
/
/    How it works:
/       Gethostmap() allocates MAP, a char array with HOSTS+1 elements.
/       It then scans the XSHN host names file(s) and directly reads
/       and parses the list of hosts, filling in the appropriate element
/       in MAP.
/
/       The caller must sysfree (map) when finished with the map.
/
/    Returns: pointer to MAP on success
/             NULL on failure
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  SHIPALL program.
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  xship/gethostm.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: CR 12/16/91 12:49 New function. */
/*: JX  7/30/92 15:01 Chixify. */
/*: CR 12/14/92 15:15 Long chxvalue arg. */
/*: CR 12/14/92 16:01 Long sysmem arg. */

#include <stdio.h>
#include "caucus.h"

FUNCTION  char * gethostmap (hosts)
   int    hosts;
{
   long   i, n;
   char  *map, *sysmem();
   Chix   line, number, temp;

   ENTRY ("gethostmap", "");

   line   = chxalloc (80L, THIN, "gethost line");
   temp   = chxalloc (80L, THIN, "gethost temp");
   number = chxalloc (100L, THIN, "gethost number");

   if (hosts < 10)  hosts = 10;
   if ( (map = sysmem (L(hosts+5))) == NULL)  goto done;

   for (i=0;   i<=hosts;   ++i)  map[i] = 0;

   for (i=1;   (1);   ++i) {
      if (NOT unit_lock (XSHN, READ, 0L, i, 0L, 0L, nullchix))  break;
      if (NOT unit_view (XSHN))           { unit_unlk (XSHN);   break; }

      while (unit_read (XSHN, line, 0L)) {
         if (chxvalue (line, 0L) == '!')                continue;
         if (chxtoken (temp, nullchix, 2, line) == -1)  continue;
         chxclear  (number);
         chxcatsub (number, temp, 1L, ALLCHARS);
         if (NOT chxnum (number, &n))                 continue;
         map[n] = 1;
      }
      unit_close (XSHN);
      unit_unlk  (XSHN);
   }

done:
   chxfree (line);
   chxfree (temp);
   chxfree (number);
   RETURN (map);
}
