/*** LOAD_XHEADER.   Load the transport header for a (re)ship file.
/
/    ok = load_xheader (shipmeth, hostnum, seqnum, reship, head,
/                       shiporig, shiplast);
/
/    Parameters:
/       int       ok;         (Success?)
/       int       shipmeth;   (shipping method number)
/       int       hostnum;    (shipping file to this host)
/       long      seqnum;     (sequence number of transport file)
/       int       reship;     (is this a reshipment?)
/       XPORTHDR *head;       (transport header data structure)
/       long     *shiporig;   (time file was originally shipped)
/       long     *shiplast;   (time file was last shipped)
/
/    Purpose:
/       Attempt to load the transport header (specified by SHIPMETH,
/       HOSTNUM, and SEQNUM) for shipment or reshipment.
/
/    How it works:
/       If RESHIP is false, attempts to open XPXH and read the header
/       information into HEAD.
/
/       If RESHIP is true, attempts to open XPRH.  Gets the time originally
/       shipped into SHIPORIG, and time last shipped into SHIPLAST.  Reads
/       the header information into HEAD.
/
/    Returns: 1 on success, 0 if file does not exist.
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  ship_it(), reship_it()
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  xship/loadxhea.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: CR  7/31/91 18:18 New function. */
/*: CR  8/08/91 14:09 Add disk space checking. */
/*: CR  8/08/91 20:58 New function. */
/*: UL 12/11/91 18:45 Make unit_lock args 3-6 long.*/
/*: JX  7/30/92 15:31 Chixify. */
/*: CR 12/14/92 14:55 Long ascofchx arg; replace ALLCHARS. */

#include <stdio.h>
#include "entryex.h"
#include "handicap.h"
#include "unitcode.h"
#include "xaction.h"

FUNCTION  load_xheader (shipmeth, hostnum, seqnum, reship, head,
                        shiporig, shiplast)
   int       shipmeth, hostnum, reship;
   XPORTHDR *head;
   long      seqnum;
   long     *shiporig, *shiplast;
{
   int       unit, ok;
   Chix      temp;
   char      tempstr[200];

   ENTRY ("load_xheader", "");

   temp = chxalloc (100L, THIN, "loadxhead temp");

   unit = (reship ? XPRH : XPXH);

   if (ok = unit_lock (unit, READ, L(shipmeth), L(hostnum),  seqnum, 0L,
          nullchix)) {
      if (ok = unit_view (unit)) {
         if (reship) {
            unit_read (unit, temp, 0L); ascofchx (tempstr, temp, 0L, 200L);
                                    sscanf (tempstr, "SHIPORIG=%ld", shiporig);
            unit_read (unit, temp, 0L); ascofchx (tempstr, temp, 0L, 200L);
                                    sscanf (tempstr, "SHIPLAST=%ld", shiplast);
         }
         ok = get_xpheader (unit, head);
         unit_close(unit);
      }
      unit_unlk (unit); 
   }

   chxfree (temp);
   RETURN (ok);
}
