/*** MOVETRANS.   Move a transport file from one area to another.
/
/    ok = movetrans (u1, u2, hostnum, shipmeth, seqnum);
/
/    Parameters:
/       int    ok;          (Success?)
/       int    u1;          (move from unit u1...)
/       int    u2;          (...to unit u2)
/       int    hostnum;     (destination host of this file)
/       int    shipmeth;    (shipping method number)
/       long   seqnum;      (sequence number of transport file)
/
/    Purpose:
/       Movetrans() encapsulates the locking/killing/moving of
/       a transport file from one place to another.  It just provides
/       a shorter, simpler call than using the sequence of unit...()
/       calls.
/
/    How it works:
/
/    Returns: 1 on success, 0 on error.
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  ship_it(), reship_it()
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  xship/movetran.o
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: CR  8/08/91 22:25 New function. */
/*: UL 12/11/91 18:45 Make unit_lock args 3-6 long.*/
/*: JX  7/30/92 15:35 Chixify. */

#include <stdio.h>
#include "entryex.h"
#include "handicap.h"
#include "unitcode.h"
#include "chixuse.h"

FUNCTION  movetrans (u1, u2, hostnum, shipmeth, seqnum)
   int    u1, u2, hostnum, shipmeth;
   long   seqnum;
{
   int    ok;

   ENTRY ("movetrans", "");

   if (ok = unit_lock (u2, WRITE, L(shipmeth), L(hostnum),  seqnum, 0L,
          nullchix)) {
      unit_kill (u2);
      if (ok = unit_lock (u1, WRITE, L(shipmeth), L(hostnum),  seqnum, 0L,
          nullchix)) {
         ok  = unit_move (u1, u2);
         unit_unlk (u1);
      }
      unit_unlk (u2);
   }

   RETURN (ok);
}
