
/*** SHIP_ACK.   Ship ACKnowledgement files to a given neighbor.
/
/    ship_ack (hostnum, shipmeth);
/
/    Parameters:
/       int    hostnum;    (number of host to ship to)
/       int    shipmeth;   (shipping method to use)
/
/    Purpose:
/       Ship_ack() ships all outstanding "ACK" acknowledgement files
/       for a given host number / shipping method pair.
/
/    How it works:
/       Call syspdir() to get the list of all of the ACK files in the
/       shipping bay.  Ship them individually, then delete them.
/
/    Returns: 1
/
/    Error Conditions:
/  
/    Side effects:
/
/    Called by: shiphost().
/
/    Related functions:
/
/    Operating system dependencies:
/
/    Known bugs:
/       May not ship all ACK files if the # is > PDIRMAX.  (But they
/       will be picked up on the next call to ship_ack().)
/
/    Home:  xship/shipack.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: CR  8/07/91 14:04 New function. */
/*: CR  8/13/91  9:52 Clean up. */
/*: CR  9/18/91 14:35 Allocate space for pdir, & free when done. */
/*: CR  9/19/91 13:08 Cast arg to sysfree(). */
/*: UL 12/11/91 18:45 Make unit_lock/name args 3-6 long.*/
/*: JV 12/16/91 16:54 Change pdir.lnum to long.*/
/*: JV 12/31/91 15:47 Rerun sysscript() if fails on tmp resource limit */
/*: CR  1/08/92 12:30 Add missing 3rd arg to mdstr() call. */
/*: JX  7/30/92 16:03 Chixify. */
/*: CR 12/14/92 16:01 Long sysmem arg. */

#include <stdio.h>
#include "caucus.h"

#define MAXLOOP 4

extern union null_t null;

FUNCTION  ship_ack (hostnum, shipmeth)
   int    hostnum, shipmeth;
{
   struct pdir_t  pdir;
   char   shipscript[160], varsfile[160], ackfile[160], runscript[400];
   int    i, loop, scrret;
   char  *sysmem();
   Chix   ctext, ctitle;

   ENTRY ("ship_ack", "");

   /*** Get the pdir list of all ACK files for a given shipmeth/hostnum. */
   pdir.type  = -1;
   pdir.parts =  0;
   pdir.resp  = (short *) sysmem (sizeof(short) * L(PDIRMAX + 3));
   pdir.lnum  = (long  *) sysmem (sizeof(long ) * L(PDIRMAX + 3));
   syspdir (&pdir, XPAK, shipmeth, hostnum, nullchix);

   /*** Get the names of the shipping method script and the host variables
   /    file.  We'll use these to build the command line that actually
   /    ships an ACK file. */
   unit_name (shipscript, XSSS, L(shipmeth), 0L, 0L, 0L, nullchix);
   unit_name (varsfile,   XHSV, L(hostnum), L(shipmeth), 0L, 0L, nullchix);

   /*** For each ACK file, build the command line to ship it, then 
   /    execute it, and finally delete it. */
   for (i=0;   i<pdir.parts;   ++i) {
      unit_name (ackfile, XPAK, L(shipmeth), L(hostnum), pdir.lnum[i], 0L,
                  nullchix);
      sprintf   (runscript, "%s %s %s", shipscript, varsfile, ackfile);

      for (loop=0; (scrret = sysscript (runscript)) < 0 && loop < MAXLOOP;
           loop++) {
         switch (scrret) {
            case -1:
            case -2: sleep (10); continue;    /* Try again */
            case  1:                          /* Success */
            case  0:                          /* Unknwon Failure */
            default: loop = MAXLOOP; break;   /* Terminate loop */
         }
      }
      if (scrret != 1) {
         ctitle = chxalloc (100L, THIN, "shipack ctitle");
         mdstr (ctitle, "net_Ttelltitle", null.md);
         ctext = chxalloc (100L, THIN, "shipack ctext");
         mdstr (ctext,  "net_Ebadscript", null.md);
         sendmgr (ctext, ctitle);
         chxfree (ctitle);
         chxfree (ctext);
      }

      if (unit_lock (XPAK, WRITE, L(shipmeth), L(hostnum), pdir.lnum[i],
                      0L, nullchix)) {
         unit_kill  (XPAK);
         unit_unlk  (XPAK);
      }
   }

   sysfree ( (char *) pdir.resp);
   sysfree ( (char *) pdir.lnum);
   RETURN  (1);
}
