/*** SHIPALL.   Program to ship all files in all SHIPnnn bays.
/
/    Invoked as:
/       shipall  confid   type  shiphost  [-d]
/
/    Arguments:
/       confid     full path name of Caucus home directory.
/       type       either "ship" or "reship"
/       shiphost   full path name of "shiphost" script
/       [-d]       optional debugging flag
/
/    Purpose:
/       Shipall ships all outstanding transport files for all neighbors
/       of this host.  It does this once, and then exits.  It is up to
/       the caller to make sure this happens on a regular basis.
/
/       The TYPE argument determines if Shipall is acting as the CaucusNet
/       "shipper" or "reshipper".
/
/    How it works:
/       Shipall gets the number of neighbor hosts, and for each such
/       host, runs the script SHIPHOST.  That script in turn runs the
/       shiphost_x program, which does most of the real work.
/
/    Exit status:
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Operating system dependencies:
/
/    Known bugs:      none
/
/    Home:  xship/shipall.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: CR  7/29/91 18:52 New program. */
/*: CR  7/30/91 23:45 Initialize screen info. */
/*: CR  8/02/91 17:22 Add diagnostics. */
/*: CR  8/13/91 11:43 Invoke new form of shiphost. */
/*: CR  9/12/91 16:21 Add new "shiphost" argument. */
/*: CR  9/16/91 19:49 Must explicitly call mainglobals(). */
/*: CR 10/28/91 11:01 Make confid[] extern. */
/*: JV 12/31/91 14:50 Rerun sysscript() if fails on tmp resource limit */
/*: JX  7/30/92 15:12 Chixify. */
/*: JX  9/02/92 12:07 Utilglobals must be before mainglobals. */
/*: CP  3/04/93 23:34 Make 'debug' extern. */

#include <stdio.h>
#include "caucus.h"

#define MAXLOOP 4

extern struct screen_template  used, screen;
extern Chix                    confid;
extern union null_t            null;
extern int                     debug;

FUNCTION  main (argc, argv)
   int    argc;
   char  *argv[];
{
   char   dohost[200];
   int    hosts, i, loop, scrret;
   char  *hostmap;
   char  *gethostmap();
   Chix   banghosts, ctext, ctitle, chxalloc();

   ENTRY ("main", "");

   screen.lines =  0;    used.lines = 0;
   screen.width = 80;    used.width = 0;

   /*** Check arguments and initialize CONFID. */
   if (argc < 4) {
      printf ("Usage: shipall confid type shiphost\n");
      exit   (1);
   }
   if (argc == 5)  debug = -1;

   confid = chxalloc (40L, THIN, "shipall confid");
   chxofascii (confid, argv[1]);
   utilglobals();
   mainglobals();

   banghosts = chxalloc (20L, THIN, "shipall banghost");
   chxofascii (banghosts, "!hosts");
   hosts = hnumber (banghosts);
   if (hosts < 0)  bug ("Shipall: cannot get # of hosts from '!hosts'.", hosts);
   chxfree (banghosts);

   hostmap = gethostmap (hosts);
   for (i=1;   i<=hosts;   ++i)   if (hostmap[i]) {
      sprintf (dohost, "%s %s %d %s", argv[3], argv[1], i, argv[2]);
      if (debug)  printf ("Shipall: '%s'\n", dohost);

      for (loop=0; (scrret = sysscript(dohost)) < 0 && loop < MAXLOOP;
           loop++) {
         switch (scrret) {
            case -1:
            case -2: sleep (10); continue;    /* Try again */
            case  1:                          /* Success */
            case  0:                          /* Unknwon Failure */
            default: loop = MAXLOOP; break;   /* Terminate loop */
         }
      }
      if (scrret != 1) {
         ctitle = chxalloc (100L, THIN, "recvall ctitle");
         mdstr (ctitle, "net_Ttelltitle", null.md);
         ctext = chxalloc (100L, THIN, "recvall ctext");
         mdstr (ctext, "net_Ebadscript", null.md);
         sendmgr (ctext, ctitle);
         chxfree (ctitle);
         chxfree (ctext);
      }

   }
   sysfree (hostmap);
}
