
/*** SHIPERROR.  Report errors during shipment of a file.
/
/    fail = shiperror (error, hostn, seq);
/
/    Parameters
/       int  fail;      (serious enough to stop shipments?)
/       int  error;     (error number)
/       int  hostn;     (host number)
/       long seq;       (transport file sequence number)
/
/    Purpose:
/       Report shiphost errors in a comprehensible way.  Determine if 
/       error is serious enough to stop shipment of remaining transport
/       files.
/
/    How it works:
/
/    Returns: 1 if error is serious (enough to stop shipments)
/             0 otherwise
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  xship/shiperro.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: CR  8/06/91 14:16 New function. */
/*: CR  8/08/91 14:18 Add more error codes. */
/*: CR 12/26/91 14:26 Report error codes properly (DS, not DX). */

#include <stdio.h>
#include "caucus.h"
#include "derr.h"

#define  REPORT(a)  strconc (msg, a, info);  bug (msg, error)

FUNCTION  shiperror (error, hostn, seq)
   int    error, hostn;
   long   seq;
{
   char   info[100], msg[400];
   int    fail;

   ENTRY ("shiperror", "");

   sprintf (info, "  Host %d, transport sequence number %ld\n", hostn, seq);
   fail = 0;
   switch (error) {

      case (NOERR):   fail = 0;     break;

      case (DS_NOXPXH):
         REPORT ("DS_NOXPXH: can't get at transport header file.\n");
         fail = 1;
         break;

      case (DS_NOXPXT):
         REPORT ("DS_NOXPXT: can't get at transport text file.\n");
         fail = 1;
         break;

      case (DS_NOXPXJ):
         REPORT ("DS_NOXPXJ: can't create join chunk shipment file.\n");
         fail = 1;
         break;

      case (DS_NOXPRH):
         REPORT ("DS_NOXPRH: can't put transport header file in RESHIP bay.\n");
         fail = 1;
         break;

      case (DS_XTMOVE):
         REPORT ("DS_XTMOVE: can't move transport text file to RESHIP bay.\n");
         break;

      case (DS_NOTIMP):
         REPORT ("DS_NOTIMP: sysavail() or sysfsize() not implemented.\n");
         fail = 1;
         break;

      case (DS_NOTDSK):
         REPORT ("DS_NOTDSK: can't get info about free disk space.\n");
         fail = 1;
         break;

      case (DS_DSKLOW):
         REPORT ("DS_DSKLOW: insufficient free disk for this file.\n");
         fail = 1;
         break;

      default:
         REPORT ("Unknown error!\n");
         break;
   }

   RETURN (fail);
}
