/*** STORE_XHEADER.  Write a transport header in the RESHIP bay.
/
/    ok = store_xheader (shipmeth, hostnum, seqnum, head, shiporig, shiplast);
/
/    Parameters:
/       int       ok;         (Success?)
/       int       shipmeth;   (shipping method number)
/       int       hostnum;    (shipped file to this host)
/       long      seqnum;     (sequence number of transport file)
/       XPORTHDR *head;       (transport header data structure)
/       long      shiporig;   (time file was originally shipped)
/       long      shiplast;   (time file was last shipped)
/
/    Purpose:
/       Write the transport header in HEAD to a transport header file
/       in the RESHIP bay.
/
/    How it works:
/
/    Returns: 1 on success, 0 on failure.
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  xship/storexhea.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: CR  8/12/91 15:03 New function. */
/*: UL 12/11/91 18:45 Make unit_lock args 3-6 long.*/
/*: CR  1/06/91 15:45 Add shiplast argument. */
/*: JX  7/30/92 16:04 Chixify. */

#include <stdio.h>
#include "entryex.h"
#include "handicap.h"
#include "unitcode.h"
#include "xaction.h"

FUNCTION  store_xheader (shipmeth, hostnum, seqnum, head, shiporig, shiplast)
   int       shipmeth, hostnum;
   XPORTHDR *head;
   long      seqnum, shiporig, shiplast;
{
   int       ok;
   Chix      temp;
   long      systime();

   ENTRY ("store_xheader", "");

   /*** Write the header file with the time originally shipped and the
   /    time (now) last shipped. */
   if (ok = unit_lock (XPRH, WRITE, L(shipmeth), L(hostnum),  seqnum, 0L,
                             nullchix)) {
      if (NOT (ok = unit_make (XPRH)))  unit_unlk (XPRH);
   }
   if (NOT ok)  RETURN (0);

   temp = chxalloc (100L, THIN, "storexh temp");
   chxformat (temp, CQ("SHIPORIG=%d\nSHIPLAST=%d\n"), shiporig, shiplast,
              nullchix, nullchix);
   unit_write   (XPRH, temp);
   set_xpheader (XPRH, head);
   unit_close   (XPRH);
   unit_unlk    (XPRH);
   chxfree (temp);

   RETURN (1);
}
