/*** TOSSFILE.   Toss a transport header/text file that we can't ship.
/
/    ok = tossfile (hostnum, shipmeth, seqnum, delta);
/
/    Parameters:
/       int    ok;          (Success?)
/       int    hostnum;     (shipping file to this host)
/       int    shipmeth;    (shipping method number)
/       long   seqnum;      (sequence number of transport file)
/       long   delta;       (how long file has been outstanding)
/
/    Purpose:
/       If a transport header/text file pair sits in the RESHIP bay
/       too long without being shipped and ACK'd, it's time to toss it.
/       Tossfile() moves the file pair from the RESHIP bay to the CANTSHIP
/       bay, and complains to the Caucus manager.
/
/    How it works:
/
/    Returns: 1 on success, 0 on error.
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  reship_it()
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  xship/tossfile.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: CR  8/08/91 22:25 New function. */
/*: UL 12/11/91 20:12 Make unit_name args 3-6 long.*/
/*: JX  7/30/92 16:03 Chixify. */

#include <stdio.h>
#include "entryex.h"
#include "handicap.h"
#include "unitcode.h"
#include "chixuse.h"

FUNCTION  tossfile (hostnum, shipmeth, seqnum, delta)
   int    hostnum, shipmeth;
   long   seqnum, delta;
{
   int    ok1, ok2;
   char   tell1[160], tell2[160], temp[500];

   ENTRY ("tossfile", "");

   ok1 = movetrans (XPRH, XPNH, hostnum, shipmeth, seqnum);

   ok2 = movetrans (XPRT, XPNT, hostnum, shipmeth, seqnum);

   unit_name (tell1, XPNH, L(shipmeth), L(hostnum),  seqnum, 0L, nullchix);
   unit_name (tell2, XPNT, L(shipmeth), L(hostnum),  seqnum, 0L, nullchix);
   sprintf   (temp, "%s, age=%ld hours.\n  Moved to:\n   (%d) %s\n   (%d) %s\n",
                "Shiphost: Can't ship transport file",
                delta / 3600L, ok1, tell1, ok2, tell2);

/* tellmgr (NULL,  temp); */
   printf ("%s\n", temp);
   RETURN (1);
}
