/* CREAT_CONF:   Create a closed, invisible conf as a place for send_conf()
/      to send conf stuff to.
/
/  int n = do_conf ()
/
/  Purpose:
/     The process of creating a replication of a remote conf:
/     1) Create the database skeleton for the conference
/     2) Send a subscription request to a neighbor
/     3) Wait for the conf stuff.  When the userlist gets here,
/        the conf will take on the correct "openness".
/
/  How it works:
/
/  Returns:
/    0: Failed to perform any action due to input problems.
/    1: Carried out mgr's wishes as much as possible.
/
/  Error conditions:
/
/  Related functions: 
/
/  Called by: doconf()
/
/  Home: netmaint/creatcon.c
/
/  Dictionary strings:
/    net_Fbadchoice:
       "You must choose a number from the menu.
        %d is not one of those choices."
*/

/*: JV 11/06/91 12:46 Creat (sic) this. */
/*: JX  5/15/92 11:42 Chixify. */
/*: JX  8/03/92 11:55 Fixify. */

#include <stdio.h>
#include "handicap.h"
#include "derr.h"
#include "netmaint.h"
#include "unitcode.h"
#include "common.h"
#include "caucus.h"
#include "chixuse.h"
#include "done.h"

extern struct screen_template  used;
extern union null_t null;

FUNCTION creat_conf()
{
   int   error, success;
   Chix  confname;

   ENTRY ("creat_conf", "");

   confname = chxalloc (80L, THIN, "confname");

   /*** Introduction. */
   unit_write (XWTX, NEWCPAGE);
   mdwrite    (XWTX, "cst_Tintro", null.md);

   /*** Prompt the user for the name of the conference, and loop until
   /    an acceptable one is picked. */
   do {
      mdwrite (XWPM, "cst_Tcname", null.md);
      used.width = used.lines = 0;
      unit_read (XKEY, confname, 0L);
      jixreduce  (confname);
      if (sysbrktest())  FAIL;
   } UNTIL (oknewconf (confname));

   if (NOT unit_lock (XITX, WRITE, 0L, 0L, 0L, 0L, nullchix)) {
      mdwrite (XWER, "err_Ebaddata", null.md);
      FAIL;
   }
   unit_make (XITX);
   mdwrite   (XITX, "cst_Tcfempty", null.md);
   unit_close (XITX); unit_unlk (XITX);

   if (NOT make_conf (confname, nullchix, 0, XITX, &error)) {
      if (error == DBADDB)
         mdwrite (XWER, "err_Ebaddata", null.md);
      else if (error == DMKDIR)
         mdwrite  (XWTX, "cst_Tmkdir", null.md);
      FAIL;
   }

   mdwrite    (XWTX, "cst_Tcreat0", null.md);
   unit_write (XWTX, confname);
   mdwrite    (XWTX, "cst_Tcreat1", null.md);

   SUCCEED;

done:
   chxfree (confname);
   RETURN (success);
}
