/*** DTU.  Program to convert DOS text format to Unix text format.
/         
/    Invoked as:
/       dtu <dosfile >unixfile
/
/    Arguments:
/       dosfile    "input" file in DOS (or other) text format
/       unixfile   "output" file, to be produced in Unix text format
/
/    Purpose:
/       Determine if input file is in DOS (or other non-Unix) text
/       format.  If it is, convert the data to Unix text format and
/       write to the output file.
/
/    How it works:
/       Use the mappings shown below to do text conversion:
/         ^h   --> ^h    (BS)
/         ^i   --> ^i    (TAB)
/         ^j^m --> \n    (LF-CR)
/         ^j   --> \n    (LF)
/         ^m   --> \n    (CR)
/         ^l   --> ^l    (FF)
/         \0's at end of input file are ignored and thrown away
/              (may be caused by binary xmodem transfers)
/
/       If any other non-printable ascii characters appear, this is
/       not a text file.  Exit with error status.
/
/    Exit status: 0 on success
/                 1 on binary file
/                 2 on empty file
/
/    Error Conditions:
/ 
/    Side effects:
/
/    Related functions:
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  zutil/dtu.c
/ */

/*** Copyright (C) 1993 Camber-Roth.  All rights reserved. */

/*: JZ 10/07/93 16:08 Create this. */
/*: CP 11/19/93 16:14 Rewrite header comments; change c, d to int. */

#include <stdio.h>

main (argc, argv)
   int argc; char *argv[];
{
   int  c, d;
   int  binary=0, pad=0;

   /* Read 1 byte from STDIN. */
   if ((c = getchar()) == EOF) return(2);

   while ((d = getchar()) != EOF) {

      /* Write char's that aren't CR. */
      if (ascii (c))  {
         if (pad) {             /* Don't allow ASCII after NULLs */
            binary=1; break; }
         putchar (c);
      }
      else {                   /* LF & CR get special treatment. */
         if (c==10) 
            if (d==13) {
               putchar ('\n');                  /* LF & CR */
               if ((d = getchar()) == EOF) break;
            } else                              /* LF with no CR */
               putchar ('\n');

         else if (c==13) {                      /* CR */
            putchar ('\n');         
            if (d==10)                          /* ...with LF */
               if ((d = getchar()) == EOF) break;

         } else                                 /* non-ASCII */
            if (c == 0)            /* Skip terminating (padding) nulls */
               pad=1;
            else {                 /* It's *really* not ASCII */
               binary=1; break; }
      }
      /* Go back and check next char. */
      c=d;
   }

   /* Write out last character if not ^Z or padding. */
   if (c != 26 && c != 0) putchar (c);

   return (binary ? 1 : 0);
}

ascii (c)
char c;
{
   if (c == 8 || c == 9 || c == 12)         return (1);
   if (c >= ' '  &&  c <= '~')              return (1);
   return (0);
}
