/*** ENTERRESP.  Enter a response from unit IN as response RNUM.
/
/    enterresp (item, rnum, in, context);
/
/    Parameters:
/       int   item;
/       int   rnum;
/       int   in;
/       int   context;
/                          
/    Purpose:
/       Read text from unit IN, and enter it as response RNUM to ITEM.
/       If RNUM is 0, make this a new response.
/
/    How it works:
/       If IN==XITT, read the text from the user.   If IN != XITT,
/       it must be locked/unlocked by the caller.
/
/       If CONTEXT is true, it means that there is a context, i.e. text
/       already on the screen that the user would like to (continue to)
/       see, if possible, while entering the new response.  (Doing this
/       is actually handled by textenter().)
/
/    Returns: 1 on success.
/             0 on error, or cancellation by user.
/
/    Side effects:      none
/
/    Related functions:
/
/    Called by:
/
/    Operating system dependencies: none
/
/    Known bugs:      none
/
/    Home:  resp/enterres.c
/ */

/*** Copyright (C) 1987 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:51 Source code master starting point */
/*: CR  4/28/89 12:52 Check unit_lock status for failure. */
/*: CR  8/09/89 17:33 Unlock input file only if XITX. */
/*: CW  5/18/89 15:43 Add XW unit codes. */
/*: CR  3/04/90 22:39 Add CONTEXT argument, use it in textenter() call. */
/*: CR  5/09/90 16:28 Freeze an item with nearly MAXRESP responses. */
/*: CR  8/12/90 20:21 Cast textenter arg to make Lint happy. */
/*: CR  4/08/91 18:49 Add new unit_lock argument. */
/*: CR  4/09/91 13:51 Add new confnum argument to updatetext() call. */
/*: CR  6/25/91 15:27 Add 'confnum' arg to srvldmas() calls. */
/*: CR  6/26/91 14:30 Add 'confnum' argument to itemfreeze(). */
/*: CR  6/26/91 15:39 Use new chg_dresp() to do most of the work. */
/*: CR  7/08/91 12:16 Add 3rd arg to chixfile(). */
/*: CR  7/10/91 14:38 Fix call to chgxresp() to use original item #. */
/*: CR  7/27/91 13:17 Use null.str instead of NULL. */
/*: CR  7/28/91 15:48 chxalloc() first argument is long. */
/*: CR  7/30/91 13:51 Replace license check with is_distributed(). */
/*: CR 10/04/91 17:47 XT_NEW or XT_CHANGE depends on rnum. */
/*: CX 10/15/91 19:31 Chixify, clean up header comments. */
/*: CR 10/29/91 12:49 Add confnum arg to getoresp() call. */
/*: JV 10/31/91 15:20 Add dest_chn arg to chgxresp() call. */
/*: UL 12/11/91 18:43 Make unit_lock args 3-6 long.*/
/*: DE  4/20/92 14:12 Fixed missing & mistyped args, also day/time */
/*: CX  6/05/92 16:22 XKEY --> u_keybd. */
/*: JX  6/24/92 13:41 Don't write over day/time fields, don't use chxquick. */
/*: JX  8/09/92 16:38 Fixify. */
/*: TP 10/19/92 14:01 Replace u_keybd with input_control(). */
/*: CR 10/22/92 12:40 Only display ent_T... prompt if session input. */
/*: CR  1/12/93 17:54 Add chxwrap() call to wrap over-long lines. */
/*: JV  3/03/93 14:57 Add attach arg to chg_dresp(). */

#include <stdio.h>
#include "caucus.h"
#include "inputcon.h"
#include "xaction.h"
#include "derr.h"

extern struct master_template  master;
extern union  null_t           null;
extern short                   confnum;
extern Userreg                 thisuser;

FUNCTION  enterresp (item, rnum, in, context)
   int    item, rnum, in, context;
{
   Chix   ctext, date, time, itemchn, respchn;
   int    error1, error2, oitem, action, oresp;
   int    u_data, u_sess;
   long   systime();

   ENTRY ("enterresp", "");

   u_data = input_control (IC_ISUNIT | IC_DATA, nullchix);
   u_sess = input_control (IC_ISUNIT | IC_SESS, nullchix);
   if (in == u_data) {
      if (u_data == u_sess) {
         if (thisuser->verbose)  mdwrite (XWTX, "ent_Tresverb", null.md);
         else                    mdwrite (XWTX, "ent_Tenresp",  null.md);
      }
      in = XITX;
      unit_lock (XITX, WRITE, 0L, 0L, 0L, 0L, CQ(""));
      unit_kill (XITX);
      if (NOT textenter (XITX, (int) thisuser->truncate, 0, 0, context)) {
         unit_unlk (XITX);
         RETURN    (0);
      }
   }

   ctext = chxalloc (500L, THIN, "enterres ctext");
   date  = chxalloc ( 30L, THIN, "enterres date");
   time  = chxalloc ( 30L, THIN, "enterres time");
  
   sysdaytime  (date, time, 0, systime());

   /*** Load the text of the item into the chix CTEXT. */
   chixfile (in, ctext, "");
   chxwrap  (ctext, 77);

   /*** Add the response. */
   action  = (rnum==0 ? XT_NEW : XT_CHANGE);
   chg_dresp (action, confnum, item, -1, &rnum, nullchix, thisuser->name,
              thisuser->id, ctext, date, time, (Attachment) NULL, &error1);
   if (is_distributed (confnum)) {
      itemchn = chxalloc (40L, THIN, "enterres itemchn");
      respchn = chxalloc (40L, THIN, "enterres respchn");
      getoitem (&oitem, itemchn, item, &master);
      getoresp (&oresp, respchn, confnum, item, rnum);
      chgxresp (action, confnum, oitem, itemchn, oresp, respchn,
                thisuser->name, thisuser->id, ctext, date, time,
                nullchix, &error2);
      chxfree  (itemchn);
      chxfree  (respchn);
   }
   if (in == XITX)  unit_unlk (in);

   chxfree (ctext);
   chxfree (date);
   chxfree (time);

   if (error1 == DINOITEM)  mdwrite (XWTX, "res_Tdelitem", null.md);

   RETURN (error1 == NOERR);
}
