/*** GET_FROZEN: Build an Rlist of the frozen items.
/
/    ok = get_frozen (cnum, master, frozen, first);
/
/    Parameters:
/       int    ok;                       (Success?)
/       int    cnum;                     (local number of conference to be sent)
/       struct master_template *master;  (Pointer to a master template)
/       Rlist *frozen;                   (returned list of frozen items)
/       int    first;                    (1st resp # for unspec. items)
/
/    Purpose:
/       Some items in a conf may be frozen.  This function tells you which.
/
/    How it works:
/       If MASTER does not contain the item info for conference CNUM,
/       load MASTER from disk.
/
/       Allocate a new Rlist (FROZEN), and add each frozen item to it.
/       Even if no items are frozen, or if get_frozen() fails, an empty
/       Rlist is allocated.
/
/       The caller must always free the Rlist.
/
/    Returns: 1 on success
/             0 otherwise
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  cmd2/freeze.c, gen/parseran.c
/
/    Operating system dependencies:
/
/    Known bugs:      none
/
/    Home:  db/getfroze.c
/ */

/*** Copyright (C) 1992 Camber-Roth.  All rights reserved. */

/*: JV 10/19/92 12:15 New function. */
/*: CL 12/11/92 15:52 Long sysmem arg. */
/*: JV  4/02/93 11:28 Add attach arg to load_header(). */
/*: CN  5/15/93 18:06 Change frozen[] from char to short. */
/*: CT  5/28/93 13:47 Replace with Rlist. */
/*: CP  6/23/93 12:52 Always allocate Rlist FROZEN. */
/*: CP  8/01/93 10:50 Add a_mak...() name arg. */

#include <stdio.h>
#include "caucus.h"
#include "master.h"
#include "api.h"
#define   RLNULL   ( (Rlist) NULL )

extern union null_t null;

FUNCTION  get_frozen (cnum, master, frozen, first)
   int    cnum, first;
   struct master_template *master;
   Rlist *frozen;
{
   int    i, success;
   long   frnum;
   Chix   fr_str, header, title;
   char  *sysmem();
   Rlist  fptr, add_to_rlist();

   ENTRY ("get_frozen", "");

   title   = chxalloc (80L, THIN, "getfroze title");
   fr_str  = chxalloc (80L, THIN, "getfroze fr_str");
   header  = chxalloc (80L, THIN, "getfroze header");
  *frozen = fptr = a_mak_rlist("getfroze frozen");
   success = 0;

   if (master->cnum != cnum  &&  NOT srvldmas (1, master, cnum))   FAIL;

   sysbrkclear();                    /* if (NOT DELETED(i))  */
   for (i=1;   i<=master->items;   ++i) if (NOT (master->responses[i] < 0)) {
      if (sysbrktest())   break;
      load_header (cnum, i, header, title, (Attachment) NULL);
      chxtoken (fr_str, nullchix, 3, header);
      chxnum   (fr_str, &frnum);
      if (frnum)  fptr = add_to_rlist (fptr, i, first, -1);
   }
   SUCCEED;

 done:
   chxfree (title);
   chxfree (header);
   RETURN  (success);
}
