
/*** PARSE_NAMRAN.  Parse a named range of items.
/
/    PARSE_NAMRAN is handed the TYPE of a named range of items
/    (title, author, subject, mysubject) and the list of WORDS that
/    select a particular range of that type.
/
/    For any items that match TYPE and WORDS, the corresponding byte
/    in FLAGS is set to 1.
/
/    PARSE_NAMRAN returns 1 if at least one item matched, and 0 otherwise.
/
/  */

/*: AA  7/29/88 17:42 Source code master starting point*/
/*: CR  3/29/91 17:29 Use AND/OR code in matchnames() call. */
/*: CR  8/02/91 14:00 Add punctuation argument to matchnames() call. */
/*: CR 11/07/91 14:18 Add confnum arg to matchnames() call. */
/*: DE  4/01/92 12:38 Chixified */
/*: CR  2/15/93 22:31 Fix Dan-ism; chxfree itemstr/userid at end! */

#include <stdio.h>
#include "caucus.h"

extern union null_t  null;
extern short         confnum;

FUNCTION  parsenamran (type, words, flags)
   Chix   type, words;
   Flag   flags[];
{
   static Chix       indextab;
   struct namlist_t *names, *matches, *p, *nlnames(), *nlnode();
   Chix   itemstr, userid;
   short  choice, c, success;
   long   item;

   ENTRY ("parsenamran", "");

   itemstr    = chxalloc (20L, THIN, "parsenam itemstr");
   userid     = chxalloc (30L, THIN, "parsenam userid");
   success    = 0;

   mdtable ("gen_Aindex", &indextab);

   /*** Figure type: AUTHOR, TITLE, SUBJECT, MYSUBJECT. */
   choice = tablematch (indextab, type);
   
   switch (choice) {
      case (-2):
      case (-1):  FAIL;

      case ( 2):  DONE(subparse (words, flags, 0));    /* SUBJECT */
      case ( 3):  DONE(subparse (words, flags, 1));    /* MYSUBJECT */

      /*** TITLE name-range.  Just get matches against words in title. */
      case ( 1):
         names = nlnames (words, null.chx);
         c = matchnames (XCTD, XCTN, confnum, null.chx, names, 
                                     AND, &matches, NOPUNCT);
         nlfree (names);
         if (c == 0)  FAIL;
         break;

      /*** AUTHOR name-range.  Get userids of names listed.  Then match
      /    list of userids against userid/item# list. */
      case ( 0):
         /*** Get the userids that match people's names in WORDS. */
         names = nlnames (words, null.chx);
         c = matchnames (XSND, XSNN, 0, null.chx, names, AND, &matches, NOPUNCT);
         nlfree (names);
         if (c == 0)  FAIL;

         /*** Create a namelist of these userids. */
         names = nlnode (1);
         for (p=matches->next;   p!=null.nl;   p = p->next) {
            chxtoken (userid, null.chx, 2, p->str);
            nladd (names, userid, 0);
         }
         nlrdup (names, 1);
         nlfree (matches);

         /*** Match the userid's against the userid/item# partfiles. */
         c = matchnames (XCAD, XCAN, confnum, null.chx, names, OR, &matches,
                            NOPUNCT);
         nlfree (names);
         if (c == 0)  FAIL;
         break;
   }
 

   /*** For each word matched, mark the owning item number as selected. */
   for (success=0, p=matches->next;   p!=null.nl;   p = p->next) {
      chxtoken (itemstr, null.chx, 2, p->str);
      chxnum   (itemstr, &item);
      flags[(int) item] = success = 1;
   }
   nlfree (matches);

 done:

   chxfree (itemstr);
   chxfree (userid);
   RETURN  (success);
}
