
/*** PARSE_OUTPUT.   Parse output redirection from command line.
/
/    Handles output redirection (e.g. DESCRIBE ALL >file) and PRINT
/    modifiers.
/
/    Examines command line in STR for an implied output redirection
/    string, e.g. ">output", ">>file", or "PRINT".  If found,
/    open the appropriate file for output and return the unit
/    number.  Remove the redirection string from the command line.
/
/    If PRINT is found and AMBIGUOUS is true, PARSE_OUTPUT will ask
/    the user if s/he meant to print the text.  (As opposed, say, to
/    asking for a participant whose name begins with "PRINT".)
/
/    If not found, return XWTX, the unit number of the user's terminal. */

/*** Copyright (C) 1985 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:42 Source code master starting point*/
/*: CW  5/18/89 14:25 Add XW unitcodes. */
/*: CR  5/01/90 15:10 Allow output redirect if in <file. */
/*: CR  3/02/91 19:28 Add bit bucket detection. */
/*: CR  4/08/91 18:49 Add new unit_lock argument. */
/*: UL 12/11/91 18:42 Make unit_lock args 3-6 long.*/
/*: DE  4/01/92 14:46 Chixified */
/*: JX  5/27/92 15:23 chxquotify. */
/*: CX  6/05/92 15:43 Replace com_unit with u_com. */
/*: JX  6/09/92 17:37 chxtoken returns -1 on fail. */
/*: JX  6/16/92 16:46 Allocate W. */
/*: JX  7/01/92 10:08 Must manually remove modifier from command line. */
/*: CR  4/18/92 16:41 Add processing of flag.restrict_user. */
/*: CI 10/05/92 16:23 Chix Integration. */
/*: CR 10/15/92 12:38 Use new version of syspluck(), remove w. */
/*: TP 10/19/92 15:58 Use input_control(). */
/*: CL 12/10/92 14:52 Long chxalter args. */
/*: CL 12/10/92 15:22 Long chxindex args. */

#include <stdio.h>
#include "caucus.h"
#include "inputcon.h"

extern struct flag_template  flag;
extern Userreg               thisuser;
extern struct namlist_t     *sysmgrprint;
extern union  null_t         null;
extern Chix                  yesnotable;
extern Chix                  ss_outred, ss_appred;

FUNCTION parse_output (str, ambiguous)
   Chix  str;
   int   ambiguous;
{
   static Chix s_print1, s_print2, s_bb;
   struct namlist_t *printptr;
   short  redirect=0, append=0, print=0, dshprint=0;
   Chix   word;
   Chix   outred, appred, print1, print2, bucket;
   Chix   keep[2], format, newstr;
   short  out, i;
   int    success, u_comm, u_keyb;
   long   pos;

   ENTRY ("parseoutput", "");

   keep[0]   = chxalloc (80L,  THIN, "parseout keep0");
   keep[1]   = chxalloc (80L,  THIN, "parseout keep1");
   word      = chxalloc (80L,  THIN, "parseout word");
   outred    = chxalloc (10L,  THIN, "parseout outred");
   appred    = chxalloc (10L,  THIN, "parseout appred");
   print1    = chxalloc (30L,  THIN, "parseout print");
   print2    = chxalloc (30L,  THIN, "parseout print2");
   bucket    = chxalloc (20L,  THIN, "parseout bucket");
   format    = chxalloc (160L, THIN, "parseout format0");
   newstr    = chxalloc (160L, THIN, "parseout newstr");
   success   = 0;

   /*** Remove any quoted strings, but be prepared to put them back later. */
   chxquote (str, 0, keep);
  
   /*** Map "> file" into ">file" for convenience. */
   chxconc (outred, ss_outred, CQ(" "));
   chxconc (appred, ss_appred, CQ(" "));
   while   (chxalter (str, 0L, outred, ss_outred)) ;
   while   (chxalter (str, 0L, appred, ss_appred)) ;

   /*** Look at each word in STR for the telltale ">" or "PRINT". */
   mdstr (print1, "gen_Kprint1",  &s_print1);
   mdstr (print2, "gen_Kprint2",  &s_print2);
   mdstr (bucket, "gen_Kbitbuck", &s_bb);
   for (i=2;   (pos = chxtoken (word, null.chx, i, str)) != -1;   ++i) {

      redirect = (chxindex (word, 0L, ss_outred) == 0);
      append   = (chxindex (word, 0L, ss_appred) == 0);
      print    = (chxeq    (word, print1));
      dshprint = (chxeq    (word, print2));

      if (redirect || append || print || dshprint)  break;
   }

   /*** Remove the redirection file name (or PRINT modifier) from the command
   /    line, and put it into WORD. */
   if (pos >= 0)  syspluck (str, pos, word);
   chxquote (str, 1, keep);
   if (pos <  0  ||  EMPTYCHX(word))  DONE(XWTX);

   /*** Complain if redirection is not allowed. */
   u_comm = input_control (IC_ISUNIT | IC_COMM, nullchix);
   u_keyb = input_control (IC_ISUNIT | IC_SESS, nullchix);
   if (redirect  &&  NOT (flag.redirect  ||  u_comm!=u_keyb  ||
                         (flag.in_macro  &&  NOT flag.restrict_user))) {
      mdwrite (XWER, "err_Toutred", null.md);
      DONE(XWTX);
   }

   /*** If PRINT, set up for printing. */
   out = XIRE;
   if (chxeq (word, bucket))  out = XIBB;
   if (print  ||  dshprint) {
      if (NOT flag.print) {
         mdwrite (XWER, "gen_Enoprint", null.md);
         DONE(XWTX);
      }

      append   = 1;
      printptr = (flag.print == 1  ||  thisuser->print->next == null.nl
                    ? sysmgrprint : thisuser->print);
      chxcpy (word, (printptr->next)->str);
      out = XIPT;
   }

   /*** Try and open the file appropriately. */
   unit_lock (out, WRITE, 0L, 0L, 0L, 0L, word);
   if (NOT (append ? unit_append (out) : unit_make (out))) {
      unit_unlk  (out);
      mdstr      (format, "gen_Fnooutput", null.md);
      chxformat  (newstr, format, 0L, 0L, word, null.chx);
      unit_write (XWER, newstr);
      DONE(XWTX);
   }

   DONE(out);

 done:

   chxfree (word);
   chxfree (keep[0]);
   chxfree (keep[1]);
   chxfree (outred);
   chxfree (appred);
   chxfree (print1);
   chxfree (print2);
   chxfree (bucket);
   chxfree (format);
   chxfree (newstr);

   RETURN ( success );
}
