
/*** PARSE_RANGE.   Parse an item-range, aka "listofitems".
/
/    PARSE_RANGE takes a range string RANGE, and parses and removes(!)
/    any item range information from it.  This information is encoded into the
/    array FLAGS; the element corresponding to each selected item is set to 1;
/    the rest are set to 0.
/
/    WHAT tells the caller whether Items or Responses were selected:
/        2 if RESPONSES were specified
/        1 if ITEMS     were specified
/        0 if neither were specified, implies BOTH.
/
/    MOD tells the caller what modifiers were selected by
/    setting the respective bit field: NEW, UNSEEN, FRESH, BRIEF, PASS.
/
/    PARSE_RANGE returns 1 if a range containing at least one item was found,
/    and 0 otherwise.  In the latter case, FLAGS will be all 0's. */

/*** Copyright (C) 1984 Camber-Roth.  All rights reserved.  */

/*: AA  7/29/88 17:42 Source code master starting point*/
/*: JV  9/28/89 11:07 Add "SHOW ITEM n PERSON name" stuff. */
/*: CR  9/05/89 14:46 Add code for LIFO modifier. */
/*: CR 10/10/89 14:31 Remove "SHOW ITEM n PERSON name" stuff. */
/*: CR 11/03/89 12:48 Use thisitem.number for "THIS". */
/*: CR 10/22/91 14:11 Use new max_items arg in parsenum(). */
/*: DE  4/01/92 15:34 Chixified */
/*: JX  6/06/92 15:34 Remove decl of strscan. */
/*: JV 10/19/92 13:40 Add FROZEN as a modifier. */

#include "caucus.h"
#include <stdio.h>

#define  FORGOT(i)  (thisconf.responses[i] == -2  &&  NOT noforget)

extern struct master_template master;
extern struct fresh_template  fresh;
extern struct this_template   thisitem;
extern struct partic_t        thisconf;

extern union null_t           null;

static Chix reptable[14];

FUNCTION  parse_range (range, flags, what, mod)
   Chix   range;
   Flag   flags[];
   int    *what;
   struct modbit_template *mod;
{
   static Chix modtable;
   int    i, valid_range, newresp;
   int    and_new, and_unseen, newitem, noforget,
          and_forgot, and_frozen, and_fresh, fr_ok;
   Chix   last, this, all;
   Flag   modflags[20], *frozen;
   Chix   rangetype, rangewords;
   short  code;
   int    success;

   ENTRY ("parserange", "");

   last       = chxalloc (20L, THIN, "parseran last");
   this       = chxalloc (20L, THIN, "parseran this");
   all        = chxalloc (20L, THIN, "parseran all");
   rangetype  = chxalloc (40L, THIN, "parseran rangetype");
   rangewords = chxalloc (80L, THIN, "parseran rangewords");
   success    = 0;

   mdtable ("gen_Aparse", &modtable);

   /*** Initialize FLAGS to NO items selected.   VALID_RANGE remembers
   /    if what we've seen so far is syntactically valid; FOUNDSOME
   /    remembers if at least one item was selected. */
   valid_range = 0;
   for (i=0;  i<MAXITEMS;  ++i)  flags[i] = 0;

   code = pluck_namran (range, rangetype, rangewords);
   if (code == 1)  FAIL;

   /*** If a named-range was found, parse it into FLAGS. */
   if (code == 2) {
      parsenamran (rangetype, rangewords, flags);
      valid_range = 1;
   }

   /*** Set up item range replacements for THIS, LAST, and ALL keywords. */
   for (i=master.items;   i>1  &&  DELETED(i);   --i)  ;
   chxformat (last, CQ(" %d "),   L(i), 0L, null.chx, null.chx);
   chxformat (all,  CQ(" 1-%d "), L(i), 0L, null.chx, null.chx);
   chxformat (this, CQ(" %d "), L(thisitem.number), 0L, null.chx, null.chx);
   reptable[7] = this;
   reptable[8] = all;
   reptable[9] = last;


   /*** Parse, remember, and remove ITEM, RESPONSES, NEW, UNSEEN,
   /    NOFORGET, FORGOTTEN, FROZEN, FRESH, PASS  modifiers. */
   mod->none  = 1;
   mod->new   = mod->fresh = mod->unseen  = mod->brief = 0;
   mod->pass  = mod->lifo  = 0;
   *what      = and_new    = and_unseen  = noforget   = 0;
   and_forgot = and_frozen = and_fresh  = 0;
   parsewords (range, modtable, reptable, modflags);

   if (modflags[0])    *what      = 1;
   if (modflags[1])    *what      = 2;
   if (modflags[2])  { and_new    = mod->new    = 1;    mod->none = 0; }
   if (modflags[3])  { and_unseen = mod->unseen = 1;    mod->none = 0; }
   if (modflags[4])    noforget   = 1;
   if (modflags[5])    noforget   = and_forgot  = 1;
   if (modflags[6])  { and_fresh  = mod->fresh  = 1;    mod->none = 0; }
   if (modflags[10]) { mod->brief = 1;                  mod->none = 0; }
   if (modflags[11])   mod->pass  = 1;
   if (modflags[12])   mod->lifo  = 1;
   if (modflags[13])   and_frozen = 1;

   /*** Parse numeric part of item range */
   if (parsenum (range, flags, 0, master.items))  valid_range = 1;

   /*** If no valid range encountered, but these valid modifiers are found,
   /    initially select ALL items. */
   if (NOT valid_range  &&  (and_new || and_fresh || and_unseen ||
                             and_forgot || and_frozen))
      for (i=1;  i<=master.items;  ++i)  flags[i] = 1;

   /*** If the NEW modifier was found, reduce FLAGS to those items that are
   /    new, or have new responses, as selected by WHAT. */
   if (and_new) {
      for (i=1;  i<=master.items;  ++i)  if (flags[i]) {
         newitem = i>thisconf.items;
         newresp = thisconf.responses[i] < master.responses[i]
                && thisconf.responses[i] != -1;
         if      (*what==2)  flags[i] = newresp;
         else if (*what==1)  flags[i] = newitem;
         else                flags[i] = newitem || newresp;
      }
   }

   /*** If the FRESH modifier was found, reduce FLAGS to those items that
   /    were NEW or had NEW responses, when the user got on. */
   if (and_fresh) {
      for (i=1;  i<=master.items;  ++i)  if (flags[i]) {
         newitem = i>fresh.items;
         newresp = fresh.responses[i] < master.responses[i]
                && fresh.responses[i] != -1;
         if      (*what==2)  flags[i] = newresp;
         else if (*what==1)  flags[i] = newitem;
         else                flags[i] = newitem || newresp;
      }
   }

   /*** If the UNSEEN modifier was found, reduce FLAGS to those items
   /    that are old but not have been seen. */
   if (and_unseen) {
      for (i=1;  i<=master.items;  ++i) {
         flags[i] = flags[i]  &&  (thisconf.responses[i] == -1)
                              &&   i <= thisconf.items;
      }
   }

   /*** If the FORGOTTEN modifier was found, reduce FLAGS to the forgotten
   /    items only.   If not found, remove all the FORGOTTEN items. */
   if (and_forgot) {
      for (i=1;   i<=master.items;   ++i)  if (flags[i]) {
         flags[i] =  (thisconf.responses[i] == -2);
      }
   }
   else  for (i=1;   i<=master.items;   ++i) {
      if (FORGOT(i))  flags[i] = 0;
   }

   /*** If the FROZEN modifier was found, reduce FLAGS to the frozen items. */
   if (and_frozen)  {
      fr_ok = get_frozen (master.cnum, &master, &frozen);
      if (fr_ok == 1) 
         for (i=1;   i<=master.items;   ++i)
            if (flags[i])
               flags[i] = flags[i] && frozen[i];
      else                                          /* Must be DB error. */
         for (i=1;   i<=master.items;   ++i)  flags[i] = 0;
      sysfree (frozen);
   }

   /*** Remove all the deleted items. */
   for (i=1;  i<=master.items;  ++i)   if (DELETED(i))    flags[i] = 0;

   /*** See if any items actually remain after all this. */
   for (i=1;   i<=master.items;   ++i)   if (flags[i])  SUCCEED;

   FAIL;

 done:
   chxfree (last);
   chxfree (this);
   chxfree (all);
   chxfree (rangetype);
   chxfree (rangewords);

   RETURN (success);
}
