
/*** SUBPARSE.  Parse a subject name. 
/
/    subparse (words, flags, mine, mark);
/
/    Parameters:
/       int    ok;          (Success?)
/       Chix   words;       (list of words in subject name)
/       short *flags;       (array of results)
/       int    mine;        (1=psubject, 0=osubject)
/       int    mark;        (mark selected items with this value)
/
/    Purpose:
/       Parse a subject name, turn it into a marked array of item#s.
/
/    How it works:
/       WORDS selects a set of subjects.  For each item I that is
/       in that set of subjects, set FLAGS[I] to MARK.
/
/    Returns: 1 if any items were found
/             0 otherwise
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  Caucus functions
/
/    Operating system dependencies:
/
/    Known bugs:      none
/
/    Home:  subj/subparse.c
/ */

/*** Copyright (C) 1987 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:56 Source code master starting point */
/*: CR  3/29/91 17:31 Use AND/OR code for matchnames() call. */
/*: CR  8/02/91 14:00 Add punctuation argument to matchnames() call. */
/*: CR 10/29/91 11:45 Add confnum arg to getline() call. */
/*: CR 11/07/91 14:19 Add confnum arg to matchnames() call. */
/*: DE  3/24/92 12:12 Chixified */
/*: DE  4/20/92 12:30 Fixed args on 'getline' and 'chxnum' */
/*: CR  5/12/93 23:28 Add 'mark' args, make 'flags' short. */

#include <stdio.h>
#include "caucus.h"

extern Chix         confname;
extern short        confnum;
extern union null_t null;

FUNCTION  subparse (words, flags, mine, mark)
   Chix   words;
   short  flags[];
   int    mine, mark;
{
   struct namlist_t *names, *this, *matches, *nlnames();
   Chix   owner, str, word;
   long   subnum;
   short  usd, usf, utd, utf;
   short  ok, i, success;
   Chix   s, getline();

   ENTRY ("subparse", "");

   owner     = chxalloc (40L, THIN, "subparse owner");
   str       = chxalloc (80L, THIN, "subparse str");
   word      = chxalloc (60L, THIN, "subparse word");

   sysuserid (owner);

   if (mine) {
      usd = XUND;   usf = XUNN;
      utd = XUSD;   utf = XUSF;
   }
   else {
      usd = XCND;   usf = XCNN;
      utd = XCSD;   utf = XCSF;
   }

   /*** Get the list of subjects matching WORDS. */
   names = nlnames (words, null.chx);
   ok = matchnames (usd, usf, confnum, owner, names, AND, &matches, NOPUNCT);
   nlfree (names);
   if (NOT ok)  FAIL;

   /*** For each matching subject, find the subject number... */
   success = 0;
   for (this=matches->next;   this!=null.nl;   this=this->next) {
      chxtoken (str, null.chx, 2, this->str);
      chxnum   (str, &subnum);

      /*** See if the subject is in this conference. */
      if ( (s = getline (utd, utf, confnum, owner, 0, (int) subnum, 0, 
               (Textbuf) NULL)) == null.chx)
         continue;
      chxtoken (word, null.chx, 2, s);
      if (NOT chxeq (word, confname))  continue;

      /*** OR each line of item numbers into FLAGS. */
      for (i=2;
          (s = getline (utd, utf, confnum, owner, 0, (int) subnum, i, 
              (Textbuf) NULL)) != null.chx;
           ++i) {
         if (parsenum (s, flags, MAXITEMS, mark))  success = 1;
      }
   }
   nlfree (matches);

done:

   chxfree (owner);
   chxfree (str);
   chxfree (word);

   RETURN (success);
}
