/*** SYSBCOPY.   Copy unit A to unit B.  
/
/   ok = sysbcopy (src, dest)
/
/   Parameters:
/      int ok,   success?
/          src,  source file
/          dest; destination file
/
/   Purpose:
/      This function copies one file to another, possibly replacing
/      or overwriting the destination.  It retains any file format
/      and record length attributes.
/
/      It DOES:
/         copy binary data
/         retain file format and record length
/         copy from: Caucus file to: Caucus file
/                    Caucus file     User file 
/                    User file       Caucus file
/                    
/      It does NOT:
/         allow variable length records (unless data can be moved
/            independently of record structure)
/
/   How it works:
/      Both units must already be locked.
/      Unix: uses a pipe.
/      DOS:  ?
/      VMS:  ?
/
/   Returns 1 on success, 0 on failure.
/
/   Error Conditions:
/
/   Side effects:
/
/    Related Functions: unit_b_view()
/                       unit_b_read()
/                       unit_b_close()
/
/    Called by:
/
/    Operating System Dependencies:
/       Unix pipes version is being written.
/       Other versions supplied here are actually code from syscopy() and
/       *must* be rewritten.
/
/    Known Bugs: none
/
/    Home: sys/sysbcopy.c
/
/ */

/*** Copyright (C) 1993 Camber-Roth.  All rights reserved. */

/*: JV  3/05/93 10:45 Create this. */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"

#define PIPE 0

#if IM | PN
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <share.h>
#include <io.h>
#endif

#if UNIX | PX | PS
#include <signal.h>
#include <errno.h>
extern int errno;
#endif

FUNCTION  sysbcopy (src, dest)
   int src, dest;
{

/*--------------------------------Unix Pipes code-----------------------*/
#if PIPE
/******This code does not work!!  The file gets there, but the parent
/      blocks on the read after the child has closed its end of the pipe. */

/* 1) use pipe() to create the pipe
   2) fork off a child:
      C: open source file, read from it, write to the pipe
      P: open dest   file, read from the pipe, write to the file
      P: wait for child to die.
*/
#define R_PIPE 0
#define W_PIPE 1
   int  fd[2], child_pid, success, len, sig_mask;
   char buf[1024];

   pipe (fd);

   switch (child_pid = fork ()) {
   case -1:    return (0);                              /* Fork failed. */

   case 0:                                              /* Child. */
      printf ("Child.\n");
      if (src  < XURG) sysprv (0);
      while ((len = unit_b_read (src,  buf, 1024L)) > 0) {
         printf ("R");
         write (fd[W_PIPE], buf, len);
      }
      close (fd[W_PIPE]);
      exit(0);

   default:                                             /* Parent. */
      printf ("Parent.\n");
      fcntl (fd[R_PIPE], O_NDELAY, 0)
      if (dest < XURG) sysprv (0);
      while ((len = read (fd[R_PIPE],  buf, 1024)) > 0) 
         unit_b_write (dest, buf, L(len));

      close (fd[R_PIPE]);
   }

done:
  
#endif

/*--------Model for OS's which can do nothing else-----------------------*/
/*** On a Sun IPC, 24MB RAM, over NFS to a Sun 3/280, 8MB RAM, this code
/    can move just over 78KB/sec. */
#if UNIX | GENERIC_OS
   char src_prv, dest_prv, buf[1024];
   int  len;

   src_prv  = src  < XURG ? 0 : 1;
   dest_prv = dest < XURG ? 0 : 1;

   while (sysprv(src_prv), (len = unit_b_read (src, buf, 1024L)) > 0) {
      sysprv (dest_prv);
      if (unit_b_write (dest, buf, L(len)) == -1) {RETURN (0);}
   }
   
#endif


/*#if UNIX | IM | JM | PN | VV | PX | PS  */
#if 0
   int   fda, fdb, len;
   int  fd[2], read_pid, write_pid, success;
   char  buf[1024];

   ENTRY ("sysbcopy", "");

/*---------------------------------------FORTUNE & MASSCOMP & A3 | LA---------*/
#if UNIX | PX | PS

   fda = open  (a,    0);
   fdb = creat (b, 0600);

   for (len=1024;   (len = read (fda, buf, len)) > 0;)  write (fdb, buf, len);

#endif
 

/*---------------------------------------IBM PC--------------------------*/
#if IM
   fda = open (a, O_BINARY | O_RDONLY);
   fdb = open (b, O_BINARY | O_RDWR | O_CREAT, S_IREAD | S_IWRITE);

   for (len=1024;   (len = read (fda, buf, len)) > 0;)  write (fdb, buf, len);
#endif

#if PN
   fda = sopen (a, O_BINARY | O_RDONLY,         SH_DENYNO);
   fdb = sopen (b, O_BINARY | O_RDWR | O_CREAT, SH_DENYNO, S_IREAD | S_IWRITE);

   for (len=1024;   (len = read (fda, buf, len)) > 0;)  write (fdb, buf, len);
#endif

/*---------------------------------------VAX-----------------------------*/
#if VV
   fda = open  (a, 0);
   delete (b);
   fdb = creat (b, 0700, "rfm=var", "rat=cr");

   while ( (len = read (fda, buf, 1024)) > 0)  write (fdb, buf, len);
#endif

   close (fda);
   close (fdb);
#endif

/*---------------------------------------PRIME---------------------------*/
#if PR
   ENTRY  ("sysbcopy", "");
   delete (   b);
   copy   (a, b);
#endif

   RETURN (1);
}
