/*** SYSGFDIR.  Get the file names in a directory, one at a time.
/
/    ok = sysgfdir (dname, fname, start, error);
/
/    Parameters:
/       int    ok;      (success?)
/       char  *fname;   (next file in dir or NULL)
/       char  *dname;   (full pathname of the directory)
/       int    start;   (start at beginning of a listing? stop?)
/       int   *error;   (error code if failure)
/
/    Purpose:
/       Intended to be used in a loop to check all the filenames in a
/       directory.  First use of this will be to calculate the size of a dir
/       (and therefor a conference).
/
/    How it works:
/       START has three legitimate values:
/       1  Start at the beginning of the directory listing
/       0  Continue the listing from the last call
/       2  Stop this listing now (presumably before the end)
/
/       Notes: caller *must* search to the end or call this with start==2
/
/    Returns: 1 on success, 0 on failure, 2 on EOD (end-of-dir, no file)
/
/    Error Conditions:  (see syserror.h.)
/       SE_NOTIMP     Function not implemented for this system.
/       SE_BADDIR     File DNAME does not exist.
/       SE_NOINFO     Cannot get information for unknown reason.
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  application.
/
/    Operating system dependencies: considerable!
/
/    Known bugs:      none
/
/    Home:  sys/sysgfdir.c
/ */

/*** Copyright (C) 1993 Camber-Roth.  All rights reserved. */

/*: JV  3/28/93 12:00 Create this. */
/*: CP  7/16/93 16:54 Add compiler-complaint for non-implemented sys types. */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"
#include "units.h"
#include "syserror.h"

extern Chix    confid;

/*** Old style unixes, i.e. prior to new "dirent.h" */
#define  OLD2   BU | HU | DU | PS  | UL
#define  OLD3a  A3 | AU | AX | DY3 | FU | IX | LA | MU | MV | PU | PX
#define  OLD3b  SV | SX | TX | U3

#define  DIRENT RO | SU | DYP | U4

#if OLD2 | OLD3a | OLD3b
#include <sys/types.h>
#include <sys/dir.h>
#include <sys/stat.h>
#endif

#if DIRENT
#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>
#define DIRENT_T dirent
#endif

#if OLD2
#define DIRENT_T direct
#endif

FUNCTION  sysgfdir (dname, fname, start, error)
   char  *dname, *fname;
   int    start, *error;
{

/*------------------------------Most UNIXes------------------------------*/
#if UNIX
#define  DIRECTORY  0040000

   struct stat  statbuf;

#if OLD3a | OLD3b
   struct direct entry;
   static FILE  *fp;
   FILE         *fopen();
#endif
#if OLD2 | DIRENT
   static DIR    *dir;
   DIR           *opendir();
   struct DIRENT_T *dp,  *readdir();
#endif

   ENTRY ("sysgfdir", "");

   /*** Make sure the directory really exists. */
   if (stat (dname, &statbuf) != 0  ||
       (statbuf.st_mode & 0070000) != DIRECTORY) {
      *error = (SE_BADDIR);
      RETURN (0);
   }

   /*** Open the dir, if we're STARTing a list. */
   if (start == 1) {
      
#if OLD3a | OLD3b
      if ( (fp = fopen (dname, "r")) == NULL)     RETURN (SE_BADDIR);
#endif
#if DIRENT | OLD2
      if ( (dir = opendir (dname)) == (DIR *) NULL)  RETURN (SE_BADDIR);
#endif
   }

   /*** If caller is done prematurely, close the dir and return. */
   if (start == 2) {
#if OLD3a | OLD3b
      fclose (fp);
#endif
#if DIRENT | OLD2
      closedir (dir);
#endif
      RETURN (1);
   }

   /*** Find and return the next file in the list. */
#if OLD3a | OLD3b
   while (fread (&entry, sizeof(entry), 1, fp)) {
      if (entry.d_ino == 0)  continue;
      if (NOT strcmp (entry.d_name, ".") ||
          NOT strcmp (entry.d_name, ".."))    continue;
      strcpy (fname, entry.d_name);
      RETURN (1);
   }
#endif

#if DIRENT | OLD2
   while ( (dp = readdir (dir)) != (struct DIRENT_T *) NULL) {
      if (NOT strcmp (dp->d_name, ".") ||
          NOT strcmp (dp->d_name, ".."))    continue;
      strcpy (fname, dp->d_name);
      RETURN (1);
   }
#endif

   /*** At this point there wasn't another file in the list. */
#if OLD3a | OLD3b
   fclose (fp);
#endif
#if DIRENT | OLD2
   closedir (dir);
#endif

   RETURN (2);
#endif


/*------------------------------IBM PC----------------------------------*/
#if IM | PN

   ENTRY ("sysgfdir", "");

   /* JV: I didn't put the 15 minutes into writing this, since this version
   /      is pretty much dead.  If necessary, most of it exists in
   /      yim/sysdir.c. */
   5 = ;

#endif


/*------------------------------Vax/VMS---------------------------------*/
#if VV
#include <rmsdef>
#include <ssdef>
#include <descrip>
   struct dsc$descriptor_s f_desc, r_desc;
   char   pattern[200];
   static long   context;
   char  *f;

   ENTRY ("sysgfdir", "");

   sprintf (pattern, "%s]*.*", dname, match);

   /*** Initialize directory search. */
   f_desc.dsc$w_length  = strlen (pattern);
   f_desc.dsc$a_pointer = pattern;
   f_desc.dsc$b_class   = DSC$K_CLASS_S;
   f_desc.dsc$b_dtype   = DSC$K_DTYPE_T;
   r_desc.dsc$w_length  = 254;
   r_desc.dsc$a_pointer = result;
   r_desc.dsc$b_class   = DSC$K_CLASS_S;
   r_desc.dsc$b_dtype   = DSC$K_DTYPE_T;

   if (start == 1)
      context = 0L;

   if (start == 2) {
      lib$find_file_end (&context);
      RETURN (1);
   }

   /*** Get the next file. */
   f = result;
   if (lib$find_file (&f_desc, &r_desc, &context, 0, 0, 0, 0) == RMS$_NORMAL) {
      if (*f != ']')   for (f=result;   *f != ']';   ++f);
      strcpy (fname, f+1);
      RETURN (1);
   }

   /* No match. */
   lib$find_file_end (&context);
   RETURN (2);
#endif

   RETURN (1);
}
