/*** SYSMSPREP.   Prepare to transmit message through Caucus Mailer Interface.
/
/    parts = sysmsprep (mymess, file, preface, subject, allofit, thisuser);
/
/    Parameters:
/       int     parts;       (returned # parts in transmitted message)
/       int     mymess;      (message number if reply or forward)
/       int     file;        (file number for original message or reply text)
/       char   *preface;     (message preface with author name & date/time)
/       char   *subject;     (message subject if original message)
/       int     allofit;     (if reply/forward, 1=entire msg, 0=last part)
/       Userreg thisuser;
/
/    Purpose:
/      Sysmsprep() prepares a message for transmission (by sysmstrans())
/      through the CMI (Caucus Mailer Interface) to the system mailer.
/      The complete text of the message is written to file unit XITX.
/
/    How it works:
/       There are three kinds of messages that can be sent:
/       Type:          MYMESS:        FILE:
/         original          0           f>0
/         forward         n>0             0
/         reply           n>0           f>0
/
/       Sysmsprep() uses ALLOFIT to decide which parts of the message
/       to send.  It starts off with the PREFACE and SUBJECT; appends
/       the "earlier" parts of the message (n>0) if appropriate; and
/       finishes with the "new text" part of the message (f>0), if
/       appropriate.  The whole mess is written into XITX.
/
/       THISUSER is used to get the sender's name and id as needed.
/
/    Returns: # of message parts actually sent.
/
/    Error Conditions:
/ 
/    Side effects:
/
/    Related functions: sysmstrans()
/
/    Called by:  mess_send()
/
/    Operating system dependencies:
/
/    Known bugs:      none
/
/    Home:  sys/sysmspre.c
/ */

/*** Copyright (C) 1990 Camber-Roth.  All rights reserved. */

/*: JV  1/01/90 22:50 Created this function. */
/*: JV  1/27/90 18:00 Fixed Caucus Mailer Interface.*/
/*: JV  2/06/90 17:00 Remove "0) MESSAGE from" line from replies outbound */
/*: CR  4/30/90 15:14 Replace 1-(UNIX|VV) with explicit system codes. */
/*: CR  4/26/91 14:38 Expand buffer str. */
/*: CR  9/08/92 16:11 Update comments.  */
/*: JX  5/15/92 18:08 Chixify. */
/*: CI 10/07/92 14:58 Chix Integration. */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"

extern int debug;
extern union  null_t    null;

FUNCTION  sysmsprep (mymess, file, preface, subject, allofit, thisuser)
   int     mymess, file;
   Chix    preface, subject;
   int     allofit;
   Userreg thisuser;
{
   short  skip, lnum, i;
   long   parts;
   Chix   str, s, getmline();

   ENTRY ("sysmsprep", "");
  
   str = chxalloc (200L, THIN, "sysmsprep str");

   unit_kill (XITX);
   unit_make (XITX);
   parts = 0;

#if UNIX | VV
   /*** If the user is replying to or forwarding a message MYMESS, open it,
   /    and skip over all but the last part if user doesn't want ALLOFIT. */
   if (mymess > 0) {
      chxtoken (str, nullchix, 3, getmline (thisuser->id, mymess, 0, 1));
      chxnum   (str, &parts);

      /*** Reply messages re-use the SUBJECT. */
      if (file) chxcpy (subject, getmline (thisuser->id, mymess, 2, 1));

      /*** Figure how many parts to SKIP, total number of PARTS, and then
      /    skip over said parts. */
      skip  = (allofit ? 0 : parts-1);
      parts = parts - skip;
      for (lnum=3, i=0;   i<skip;   ++i) {
         do    { s = getmline (thisuser->id, mymess, lnum++, 1); }
         UNTIL ( s == nullchix  ||  chxeq (s, CQ("---"))  ||
                                    chxeq (s, CQ(" ---")));
      }
   }

   /*** An original message or reply is one "more" part. */
   if (file > 0)  ++parts;
#endif

#if IM | PN | PR | CA | CX | CVM
   /*** Write subject, original or re-used. */
   unit_write (XITX, subject);
   unit_write (XITX, CQ("\n"));
#endif

#if UNIX | VV
   /*** Write (non-skipped) parts of forwarded/reply-prompting message. */
   /*      First skip over "0) MESSAGE from " line on replies */
   if (mymess && file)
      s = getmline (thisuser->id, mymess, lnum++, 1);
   /* printf ("Skipping: %s\n", s); */
   if (mymess > 0) {
      while ( (s = getmline (thisuser->id, mymess, lnum++, 1)) != nullchix) {
         unit_write (XITX, s);
         unit_write (XITX, CQ("\n"));
      }
      if (file > 0)  unit_write (XITX, CQ("---\n"));
   }
#endif

   /*** If simple message/reply, also write what the user had to say! */
   if (file > 0) {
      unit_view  (file);
      unit_write (XITX, preface);
      unit_write (XITX, CQ("\n"));

      while (unit_read (file, str, 0L)) {
         unit_write (XITX, CQ(" "));
         unit_write (XITX, str);
         unit_write (XITX, CQ("\n"));
      }
      unit_close (file);
   }

   unit_close(XITX);
   RETURN (parts);
}
