
/*** SYSMSTRANS.   Transmit Caucus e-mail message via system mailer.
/
/   ok = sysmstrans (to_id, from_id, subject, text);
/
/   Parameters:
/     int   ok;       (Return value, see below)
/     Chix  to_id;    (recipient id/address)
/     Chix  from_id;  (sender id)
/     Chix  subject;  (message subject)
/     Chix  text;     (text of message)
/
/   Purpose:
/      Transmit Caucus e-mail message to a specified address, via the
/      operating system mailer.
/
/   How it works:
/      Places the TEXT in a temporary file, then spawns off a command
/      that runs the appropriate system mailer program, with the temporary
/      file as input.
/
/   Returns: 1 on success, 0 otherwise.
/
/   Related functions: syscmd()
/
/   Called by: add_cmimess()
/
/   Operating System Dependencies:
/      Currently uses either Unix mail or Vax/VMS MAIL.
/
/   Home: sys/sysmstra.c
/ */

/*** Copyright (C) 1990 Camber-Roth.  All rights reserved. */

/*: JV  1/01/90 22:58 Created function. */
/*: JV  1/27/90 18:00 Fixed Caucus Mailer Interface.*/
/*: CR  4/27/90 13:01 Improve subject quote handling; script is now cmi_send.*/
/*: CR  5/07/90 13:33 Update VV code. */
/*: CR  4/02/91 23:59 Expand buffer sizes for looong... file names. */
/*: CR  4/08/91 19:18 Use new confnum argument to unit_name(). */
/*: CR  4/25/91  0:54 Remove unneeded decl's. */
/*: JV 10/31/91 15:32 Declare sprintf */
/*: UL 12/11/91 20:12 Make unit_name args 3-6 long.*/
/*: JV  1/06/92 17:27 Update return from syscmd(). */
/*: JV  2/05/92 11:07 Don't remove SUBJECT: anymore. */
/*: JX  5/15/92 18:16 Chixify. */
/*: JX  9/30/92 16:32 One CQ per line. */
/*: CR 10/09/92 17:28 Remove 's', chxfree command & subj. */
/*: CR 11/20/92 18:02 Add " and @ handling to Vax/VMS code; add header. */
/*: CL 12/10/92 14:52 Long chxalter args. */
/*: CL 12/11/92 13:48 Long Chxvalue N. */
/*: CR 12/16/92 14:52 Allow Unix mail command to have kanji subjects. */
/*: CR  2/26/93 15:17 Use TEXT arg instead of XITX file for message text. */
/*: CP  7/26/93 12:55 Use new form of syscmd() call. */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"
#include "units.h"
#include "diction.h"

#define  ASCSIZE  300L

extern Chix          confid;

FUNCTION  sysmstrans (to_id, from_id, subject, text)
   Chix   to_id, from_id, subject, text;
{
   int    lang, success;
   long   value, pos;
   short  ret=0;
   Chix   command, subj;
   char   fname[300], asc_comm[ASCSIZE];

   ENTRY ("sysmstrans","");

   subj = chxalloc (200L, THIN, "sysmstra subj");

   /*** Put the TEXT into the XITX temp file.  Get the full pathname
   /    of that file in FNAME. */
   unit_name (fname,  XITX, 0L, 0L, 0L, 0L, nullchix);
   if (NOT unit_lock (XITX, WRITE, 0L, 0L, 0L, 0L, nullchix))  FAIL;
   if (NOT unit_make (XITX))            { unit_unlk (XITX);    FAIL; }
   unit_write (XITX, text);
   unit_close (XITX);

   chxcpy (subj, subject);  /* We might need to alter some of it. */

#if UNIX
   /*** ...and escape any double quotes. */
   for (pos=0;   value = chxvalue (subj, pos);   pos++)
      if (value == '"') {
         chxalter  (subj, pos, CQ("\""), chxquick("\\\"", 1));
         pos++;                                    /* Yes, *two* increments. */
      }

   /*** Build command that runs the CMI_SEND script. */
   command = chxalloc (200L, THIN, "sysmstra command");
   chxformat (command, CQ("%s/cmi_send \"%s"), 0L, 0L, confid, subj);
   chxformat (command, CQ("\" %s %s\n"),   0L, 0L, to_id, chxquick(fname,1)); 
   chmod (fname, 0755);
   lang = unit_control (XIRE, UNIT_GET);
   switch (lang) {
      case (LANG_NORM):
      case (LANG_ISO):  ascofchx  (asc_comm, command, 0L, ASCSIZE);  break;
      case (LANG_EUC):  eucofjix  (asc_comm, command, 0L, ASCSIZE);  break;
      case (LANG_SJIS): sjisofjix (asc_comm, command, 0L, ASCSIZE);  break;
      default:                     asc_comm[0] = '\0';               break;
   }
   ret = syscmd (asc_comm, SYS_WAIT) > 0;
   chxfree (command);
#endif

#if VV
 {
   char  asc_id[300], temp[300];
   char *a, *t;

   /*** Remove any double quotes from the subject. */
   while (chxalter (subj, 0L, CQ("\""), chxquick("",1))) ;

   /*** Build most of the VMS MAIL command line in ascii. */
   sprintf (asc_comm, "mail/subject=\"%s\" %s ", ascquick(subj), fname);

   /*** Process the address.  If it does not begin with a leading quote,
   /    and it contains an @, then we need to escape any quotes (by doubling
   /    them) *and* surround it in quotes.  (Ugh!) */
   strcpy  (asc_id, ascquick(to_id));
   if (asc_id[0]!='"'  &&  strindex (asc_id, "@") >= 0) {
      for (a=asc_id, t=temp;   *a;   ++a) {
         *t++ = *a;
         if (*a == '"')  *t++ = '"';
      }
      *t = '\0';
      sprintf (asc_id, "\"%s\"", temp);
   }

   strcat (asc_comm, asc_id);
   ret  = (syscmd (asc_comm, SYS_WAIT) > 0);
 }
#endif

   unit_kill (XITX);
   unit_unlk (XITX);
   DONE (ret);
   
done:
   chxfree (subj);
   RETURN  (success);
}
