
/*** SYSRDFILE.   Read a line of bytes from a disk file N.
/
/    ok = sysrdfile (n, buf, maxbuf);
/
/    Parameters:
/       int   ok;      (success?)
/       int   n;       (unit number of open file)
/       char  buf[];   (char array to hold bytes)
/       int   maxbuf;  (max size of BUF)
/
/    Purpose:
/       Read a line of bytes from the disk file open on unit N,
/       and place them (without any interpretation) into BUF.
/
/    How it works:
/       Reads up to MAXBUF-1 bytes and places them in BUF,
/       followed by a 0 byte.  Any trailing newlines are removed.
/       
/    Returns: 1 on success
/             0 on end-of-file
/
/    Error Conditions:
/
/    Side effects:
/
/    Related functions:
/
/    Called by:  Application.
/
/    Operating system dependencies: lots
/
/    Known bugs: VM manner of determining end of file on remote
/       files is pretty ugly!
/
/    Home:  sys/sysrdfil.c
/ */

/*** Copyright (C) 1983 Camber-Roth.  All rights reserved.  */

/*: CX  5/29/92 15:24 New function, split off from old unit_read(). */
/*: JX 10/08/92 11:00 Fixify. */
/*: CR  2/24/93 22:20 Implement test caching for unit XCRF. */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"
#include "units.h"

#if SRV
#include "actions.h"
#include "errchk.h"
extern int   vcernum;
extern char *vcernam;
extern char  localfile[];
extern char  confid[];
extern Chix  servername;
#endif

#define  CSIZE  2048
char  s_cache[CSIZE+2] = { "" };
char *s_ptr = s_cache;

extern struct unit_template units[];

FUNCTION  sysrdfile (n, buf, maxbuf)
   int    n;
   char   buf[];
   int    maxbuf;
{
   int    last;
   char  *bugtell();

   ENTRY ("sysrdfile", "");

   buf[0] = '\0';

   if (units[n].access != READ)
      bugtell ("Unitread: ", n, 0L, "reading from unopened file!\n");

/*------------------------------------Server-based systems----------------*/
#if SRV
  {
   short  class;
   char  *p;
   int    s;
   char  *fgets();
   
   if (localfile[n]) {
      if (fgets (buf, maxbuf, units[n].fp) == NULL)  RETURN(0);
      last = strlen (str) - 1;
      if (buf[last] == '\n')  str[last] = '\0';
      RETURN(1);
   }

   /*** Otherwise, we read stuff in big bufferfulls from the server. */

   /*** If the buffer is empty, ask the server to fill it up. */
   if (NULLSTR(units[n].p)) {
      class = AREAD + n;
      if (NOT vcsend (servername, class, NULL, 0, units[n].buffer, MEMBUF)) {
         ERRCHK("unitread");
         unit_write (XWER, CQ("Caucus Server has terminated.\n"));
         unit_write (XWER, CQ("Caucus is now aborting.\n"));
         sysexit (0);
      }
      units[n].p = units[n].buffer;
   }

   /*** Extract the "next" line from the buffer, and put it in STR. */
   for (p=units[n].p, s=0;   *p!='\n' && s<maxbuf-1;  ) buf[s++] = *p++;
   buf[s] = '\0';
   units[n].p = (*p=='\n' ? p+1 : p);

   /*** Finally, test for end-of-file. */  /* UGGHHH! */
   if ( (buf[0] & 0xFF) == 0xFF)  { buf[0] = '\0';   RETURN(0); }
   RETURN (1);
  }
#endif

/*--------------------------FORTUNE & IBM PC & MASSCOMP & PRIME------*/
#if UNIX | IM | JM | PN | VV | AOS | PX | PS
   { char  *fgets();
     char        *bptr;
     int          rsize;

   /*** Try disk caching, but for unit XCRF only for now. */
   /*
   if (n==XCRF) {
      for (bptr=buf;   (1);   ) {
         if (*s_ptr=='\n')  { *bptr   = '\0';   s_ptr++;   RETURN(1); }
         if (*s_ptr!='\0')  { *bptr++ = *s_ptr++;          continue;  }
*/
         /*** Reload s_cache. */
/*      *bptr = '\0';
         s_ptr = s_cache;
         s_cache[0] = '\0';
         rsize = fread (s_cache, 1, CSIZE, units[n].fp);
         s_cache[rsize] = '\0';
         if (rsize > 0)  continue;
         RETURN (NULLSTR(buf) ? 0 : 1);
      }
   }
   */

   if (fgets (buf, maxbuf, units[n].fp) == NULL) {
      buf[0] = '\0';
      RETURN (0);
   }

   last = strlen (buf);
   while (buf[last-1]=='\012'  ||  buf[last-1]=='\015')  buf[--last] = '\0';
   RETURN (1);
   }
#endif

/*--------------------------NEW PRIME------------------------------------*/
#if PR
   {
      extern   struct buf_template bufhead;
      struct   buf_template *p, *q;
      short    expand, bufmax, key, code, unit, i, buflen;
      short   *bufadr;
      long     zero;
      int      s;
      char     c;
      fortran  prwf$$();

      for (p = &bufhead;   p->next != NULL;   p = p->next) {
         q = p->next;
         if (q->n == n)  break;
      }

      s      =   0;
      expand =   0;       /* Expand => expand compressed blanks? */
      bufmax = 128;

      /*** Loop, moving one character at a time from the read buffer
      /    to the BUF.  Refill the read buffer when it's empty. */
      while (1) {
   
         if (q->bufptr < q->buflen) {

            if (s >= maxbuf-1) { buf[maxbuf-1] = '\0';   RETURN (1); }

            c = q->buf [ q->bufptr++ ];

            /*** Process special cases: compressed blanks, null characters,
            /    newline. */
            if (expand) {
               for (i=0;   i<c && s<maxbuf-1;   ++i)  buf[s++] = ' ';
               expand = 0;
            }

            else if (c == '\0')  ;

            else if (c == '\n') { buf[s] = '\0';         RETURN (1); }

            else if (c == 0221)    expand = 1;

            /*** Otherwise, add this char to end of STR. */
            else buf[s++] = c;
         }
   
         /*** Read buffer is empty, refill it! */
         else {
            key  = KREAD + KCONV;
            zero = 0L;
            unit = units[n].fd;
            bufadr = (short *) q->buf;

            prwf$$ (&key, &unit, bufadr, &bufmax, &zero, &buflen, &code);
            if (code != 0)  { buf[0] = '\0';   RETURN (0); }

            q->buflen = 2 * buflen;
            q->bufptr = 0;
         }
      }
   }
#endif
}
