
/*** UNIT_CLOSE.  Close a file opened with UNIT_MAKE or UNIT_VIEW. */

/*** Copyright (C) 1986 Camber-Roth.  All rights reserved.  */

/*: AA  7/29/88 17:54 Source code master starting point */
/*: CR  8/04/88 12:34 VM changes only */
/*: JV  9/13/88 16:45 Added AOS code to do acl's.*/
/*: CR  9/30/88 15:25 Add server code in "#if SRV" section. */
/*: CR 11/09/88 15:43 Add #include "errchk.h" to SRV code. */
/*: CR 12/07/88 16:08 Remove VV chmod on close. */
/*: CR  3/21/89 15:26 Add use of servername to #if SRV code. */
/*: JV  4/05/89 21:35 Added code to stop client if server is dead. */
/*: CR  4/27/89 16:31 Add lock-consistency check code. */
/*: CW  5/08/89 14:56 Add PX system code. */
/*: CW  5/19/89 12:51 Add XW unit codes. */
/*: CR  9/20/90 16:55 Add PS system type. */
/*: CR  9/26/90 12:13 Make Lint happy when not SRV. */
/*: CR  3/02/91 19:11 Add XIBB code. */
/*: JV  5/14/91 19:50 Add unittag use. */
/*: CR  6/21/91 17:38 Ensure unittag[n], n>=0 ! */
/*: JV  8/13/91 09:32 Add debug.h to Unix only */
/*: DE  4/09/92 13:18 Chixified */
/*: JX 10/08/92 11:00 Fixify. */
/*: CR  2/25/93 23:44 Put in temporary caching for XCRF. */

#include "caucus.h"
#include <stdio.h>
#include "systype.h"
#include "units.h"

#if SRV
#include "actions.h"
#include "errchk.h"
extern int   vcernum;
extern char *vcernam;
extern Flag  localfile[];
extern Chix  servername;
#endif

extern struct unit_template  units[];
extern char                 *unittag[];
extern int                   debug;

FUNCTION  unit_close (n)
   int  n;
{
   char  backup[80];
   char *bugtell();
   int   success = 0;

   ENTRY ("unitclose", "");
   if (debug & DEB_FILE)
      bugtell ("CLOSE", n, (long) units[n].access, (n>=0 ? unittag[n] : ""));

   if (n <= XITT  ||  n == XIBB)    SUCCEED;

   /*** Bug check: shouldn't be closing it if it wasn't open. */
   if (units[n].access != WRITE  &&  units[n].access != READ)
      buglist (bugtell ("Unitclos: ", n, 0L, "file is not open!\n"));

#if SRV
   if (localfile[n]) {
      fclose (units[n].fp);
      units[n].fp = NULL;
   }
   else {
      short class;
      if (units[n].access == WRITE)  unitwts (n, NULL);
      sysfree (units[n].buffer);

      class = ACLOS + n;
      if (NOT vcsend (servername, class, NULL, 0, NULL, 0)) {
         ERRCHK ("unitclose");
         unit_write (XWER, CQ("Caucus Server has terminated.\n"));
         unit_write (XWER, CQ("Caucus is now aborting.\n"));
         sysexit (0);
      }
   }
#endif

#if UNIX | IM | JM | PN | VV | AOS | PX | PS
/*
   if (n==XCRF) {
      extern char  s_cache[];
      extern char *s_ptr;
      s_ptr = s_cache;
      s_cache[0] = '\0';
   }
*/
   fclose (units[n].fp);
   units[n].fp = NULL; 
#endif

#if PR
 {
   fortran clos$a(), prwf$$();
   extern  struct buf_template bufhead;
   struct  buf_template *p, *q;
   short   funit, key, junk, code;
   short  *bufadr;
   long    zero;

   funit = units[n].fd;

   /*** If this file was open for write, truncate it here. */
   if (units[n].access == WRITE) {
      key    = KTRNC;
      bufadr = (short *) NULL;
      junk   = 0;
      zero   = 0L;
      prwf$$ (&key, &funit, bufadr, &junk, &zero, &junk, &code);
   }

   /*** Close the open file. */
   clos$a (&funit);

   /*** If this file was open for read, free the associated read 
   /    buffer in the linked list of read buffers. */
   if (units[n].access == READ) {
      for (p = &bufhead;   p->next != NULL;   p = p->next) {
         q = p->next;
         if (q->n == n) {
            free (q->buf);
            p->next = q->next;
            free (q);
            break;
         }
      }
   }

   units[n].fd = 0;
 }
#endif

#if UNIX | IM | JM | PN | VV | PR | AOS | PX | PS
   /*** If this closes a unit_make, delete the backup file, now that the
   /    current file is completely written out and finished with. */
   if (units[n].access == WRITE) {
      sysbackup (backup, units[n].nameis);
      sysunlink (backup);
#if AOS
      chmod (units[n].nameis, 0707);
#endif
   }
#endif

   units[n].access = LOCKED;
   SUCCEED;

 done:

   RETURN ( success );
}
