/*** LICENSE program, all in one file. */
/*: JV  9/24/92 13:15 Concatenate all of the license source into one file. */
/*: CR 10/21/92 15:31 Remove old carvmarks, close systime properly. */
/*: CR 11/20/92 13:52 Add copyright-notice bit. */
/*: CR 11/27/92 13:19 sysidform(): MV does have sys id. */

/*** CF_DIGIT.   Is character C a digit in the range 0 to 9? */

#include "xlice.h"
#include <stdio.h>

FUNCTION  cf_digit (c)
   int    c;
{

#if PR | CVM
#include <ctype.h>
   return (isdigit(c));
#endif

#if UNIX | IM | JM | PN | VV | AOS | CX | CA | PX | PS
   return (c >= '0'   &&   c <= '9');
#endif
}

/*** LICENSE.  Write licensing information into Caucus executable.
/
/    For CV2.    Invoked as:
/       LICENSE  caucus_x   shipcode_file  [license_address]
/  */

#define MAXSYSID 16

#if UNIX | PR | PX
#define  OPEN(x)   open (x, 2)
#endif

#if IM | PN
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>
#define  OPEN(x)   open (x, O_BINARY | O_RDWR)
#endif

#if AOS | CA
#include <fcntl.h>
#define OPEN(x)  open (x, O_RDWR)
#endif

#if CVM
#include <file.h>
#define  OPEN(x)   open (x, O_RDWR)
#endif

static struct license_t license =
   {0x11, 0x22, 0x33, 0x44, 0xAA, 0xBB, 0xCC, 0xDD};

struct flag_template flag;

FUNCTION  main (argc, argv)
   int   argc;
   char *argv[];
{
   long  start, address;
   long  search(), systime();
   int   fd, weeks, addr;
   short i;
   char  str[40], shipstr[40], caucus_x[80], shipcode[80], reldate[80];
   long  code;
   FILE *fp, *fopen();

#if UNIX | PX | PR | IM | PN | AOS | CA
   if (argc < 3) {
      printf ("Usage:  LICENSE   caucus_x   shipcode_file  [address]\n");
      exit   (1);
   }
   strcpy (caucus_x, argv[1]);
   strcpy (shipcode, argv[2]);
   addr = 3;
#endif

#if CVM
   if (argc < 7) {
      printf ("Usage: LICENSE <caucus module> <shipcode file> [address]\n");
      exit   (1);
   }
   sprintf (caucus_x, "%s %s %s", argv[1], argv[2], argv[3]);
   sprintf (shipcode, "%s %s %s", argv[4], argv[5], argv[6]);
   addr = 7;
#endif

   if ( (fd = OPEN (caucus_x)) < 0) {
      printf ("Can't open file '%s'\n", caucus_x);
      exit   (1);
   }

   address = -1L;
   if (argc == addr+1) {
      sscanf (argv[addr], "%ld", &start);
      address = search (fd, start);
   }

   if (address < 0)  address = search (fd, 0L);

   if (address < 0)  {
      printf ("Can't find locator.\n");
      exit   (1);
   }
   printf ("License structure found at address %ld\n\n", address);


   /*** Read and increment shipping code entry. */
   if ( (fp = fopen (shipcode, "r")) == NULL) {
      printf ("Can't open shipping code file '%s'\n", shipcode);
      exit   (1);
   }
   fgets  (str, 100, fp);
   fgets  (reldate, 100, fp);
   fclose (fp);
   sscanf (str+2, "%5ld", &code);
   str[2] = '\0';
   ++code;
   sprintf (shipstr, "%s%05ld", str, code);
   if ( (fp = fopen (shipcode, "w")) == NULL) {
      printf ("Can't write to shipping code file '%s'\n", shipcode);
      exit   (1);
   }
   fprintf (fp, "%s\n%s\n", shipstr, reldate);
   fclose  (fp);
   strcpy  (license.serial, shipstr);


   printf ("Enter system id(s) for this license, one per line.\n");
   printf ("When you are done entering system ids, press ENTER\n");
   printf ("one more time.\n\n");
   printf ("(If there is no sysid at all, just press ENTER.)\n");
   for (i=0;   i<15;   ++i) {
      printf ("id-%02d: ", i+1);
      fflush (stdout);
      gets   (str);
      if (NULLSTR (str))  break;
      str[MAXSYSID] = '\0';
      strlower  (str);
      sysidform (str);
      strcpy    (license.sysid[i], str);
   }
   strcpy (license.sysid[i], "");


   printf ("\nEnter maximum # of simultaneous users for this license.\n");
   printf ("0 means unlimited:  ");
   fflush (stdout);
   gets   (str);
   if (cf_digit(str[0]))  sscanf (str, "%ld", &license.maxusers);
   else     license.maxusers = 0;

   printf ("\nEnter number of weeks from now that this license expires.\n");
   printf ("0 means it does not expire:  ");
   gets   (str);
   if (cf_digit(str[0]))  {
      sscanf (str, "%d", &weeks);
      if (weeks == 0)  license.expires = 0L;
      else             license.expires = systime() + weeks * 7L * 24L * 3600L;
   }
   else  license.expires = 0L;

   /*** Now figure the function mask bits.  Currently, bit 0 controls
   /    CMI mail; bit 1 controls caucuslink functions; bit 2 controls
   /    copyright display. */
   license.function_mask = 0xFFFC;
   printf ("\nAllow CMI mail?   (yes) ");
   gets   (str);
   strlower (str);
   if (NULLSTR(str)  ||  str[0]=='y')  license.function_mask |= 1;

   printf ("Allow CaucusLink? (no) ");
   gets   (str);
   strlower (str);
   if (str[0] == 'y')  license.function_mask |= 2;

   printf ("Display copyright notice? (yes) ");
   gets   (str);
   strlower (str);
   if (str[0] == 'n')  license.function_mask &= (~ 4L);

   lseek (fd, address, 0);
   write (fd, (char *) &license, sizeof(license));

   close  (fd);

   if (license.expires != 0L)  sysdate (str, license.expires);
   else                        strcpy  (str, " (never)");
   str[9] = '\0';
   reldate [strlen(reldate)-1] = '\0';
   printf ("\nKit information:\n");
   printf ("--------------------------------------------------------------\n");
   printf ("Version  Users    Rel Date    Exp Date             Sysid  Serial Code\n");
   printf ("2.3SX     %4ld   %9s   %9s  %16s  %s\n",
            license.maxusers, reldate, str, license.sysid[0], license.serial);
   if (NOT NULLSTR(license.sysid[0])) {
      for (i=1;   NOT NULLSTR(license.sysid[i])  &&  i<16;   ++i)
         printf ("            %44s\n", license.sysid[i]);
   }
}


FUNCTION long search (fd, start)
   int   fd;
   long  start;
{
   char  buf[520];
   long  addr;
   short i, j, n;

   for (addr=start;   lseek (fd, addr, 0L), n=read (fd, buf, 520) == 520;
        addr += 512) {

      for (i=0;   i<512;   ++i) {
         for (j=0;   j<8;   ++j)   if (license.locator[j] != buf[i+j])   break;
         if  (j >= 8)  return (addr + i);
      }
   }

   return (-1L);
}
/*** STRALTER.  Alter 1st occurence of OLD in SSS to NEW. */

stralter (sss, old, new)
   char  sss[], old[], new[];
{
   int   ssslen, oldlen, newlen, pos, delta;
   register int i;
   int   strlen(), strindex();

   ssslen = strlen (sss);   oldlen = strlen (old);   newlen = strlen (new);
   if (ssslen==0 || oldlen==0)         return (0);

   if ((pos = strindex (sss, old)) < 0)  return (0);

   delta = newlen - oldlen;
   if (delta > 0) {
      for (i=ssslen;  i>= pos+oldlen;  --i)  sss[i+delta] = sss[i]; }
   if (delta < 0) {
      for (i=pos+oldlen;  i<=ssslen;   ++i)  sss[i+delta] = sss[i]; }

   for (i=0;  i<newlen;  ++i)  sss[i+pos] = new[i];

   return (1);
}
/*** STRINDEX.  Return starting position in string S of string T. */

strindex (s, t)
   char  s[], t[];
{
   register int  i, j, k;

   for (i=0;  s[i] != '\0';  i++) {
      for (j=i, k=0;  t[k] != '\0' && s[j]==t[k];  j++, k++) ;
      if (t[k] == '\0')  return (i);
   }
   return (-1);
}
/*** STRLOWER.  Convert string S to all lower case letters. */

strlower (s)
   char  *s;
{
   for (;   *s;   ++s)    if (*s>='A'  &&  *s<='Z')    *s += ('a' - 'A');

   return (1);
}

/*** STRSIMPLIFY.  Simplify a string by removing all unnecessary blanks.
/
/    Specifically, replace all multiple blanks with one blank,
/    and remove all leading and trailing blanks. */

strsimplify (str)
   char  str[];
{
   int   len;

   for (len=strlen(str);   len>0  &&  str[len-1]==' ';)  --len;
   str[len] = '\0';

   while (str[0] == ' ')  stralter (str, " ", "");

   while (stralter (str, "  ", " ")) ;
   return (1);
}

/*** STRTOKEN.  Find the POS'th token in a string.
/
/    STRTOKEN finds the POS'th token in the string LINE, and returns it
/    in WORD.  A token is a sequence of nonblank characters.  The first
/    token in a string is number 1.
/
/    STRTOKEN returns a pointer to the beginning of the found token in 
/    LINE.  If no such token was found, STRTOKEN returns NULL, and
/    sets WORD to the empty string. */

#define FALSE 0
#define TRUE  1

FUNCTION  char *strtoken (word, pos, line)
   char    word[], line[];
   int     pos;
{
   int   between, i, len;
   char  buf[100];
   char  *w, *start;

   ENTRY ("strtoken", "");

   /*** BETWEEN means we are between tokens (in the white space).  */
   between  = TRUE;
   len = strlen (line);

   for (i=0;   i<=len;   i++) {

      if (line[i]==' '  ||  line[i]=='\0') {
         if (NOT between) {
            *w = '\0';
            if ((--pos) <= 0) { strcpy (word, buf);    RETURN (start); }
         }
         between = TRUE;
      }

      else {
         if (between)  {
            w       = buf;
            start   = line + i;
            between = FALSE;
         }
         *w++ = line[i];
      }
   }

   word[0] = '\0';
   RETURN (NULL);
}

/*** SYSDATE.  Convert time in seconds to "dd-MMM-yy hh:mm". */

/*** Copyright (C) 1983 Camber-Roth.  All rights reserved.  */

static char         months[100], *m;

FUNCTION  sysdate (str, seconds)
   char   str[];
   long   seconds;
{

/*--------------------------FORTUNE & VAX & MASSCOMP & A3 & PRIME--------*/
#if UNIX | VV | PR | IM | PN | CX | CA | CVM | PX | PS

#define SYSV1 A3 | HU | FU | MU | MV | PU | SX | TX | IX | SV
#define SYSV2 PX | DU | RO | U3 | AU | DY
#define SYSV  SYSV1 | SYSV2

#if SYSV | VV | IM | PN | CX | CA
#include <time.h>
#define  CAST
struct tm *localtime();
#endif

#if PR
#include <time.h>
#define  CAST
#if PR19
struct tm *localtime();
#endif
#endif

#if BU | SU | PS
#include <sys/time.h>
#define  CAST
struct tm *localtime();
#endif

#if CVM
#include <time.h>
#define  CAST   (time_t *)
#endif

   struct tm *now;
   char   mon[20];
   char  *strtoken();

   ENTRY ("sysdate", "");

   if (NULLSTR (months)) {
      m = "JAN FEB MAR APR MAY JUN JUL AUG SEP OCT NOV DEC";
   }

   now = localtime (CAST &seconds);
   strtoken (mon, now->tm_mon+1, m);
   sprintf  (str, "%02d-%s-%2d %2d:%02d", now->tm_mday, mon,
            now->tm_year, now->tm_hour, now->tm_min);
#endif

   RETURN (1);
}

/*** SYSIDFORM.  Format user-supplied system id properly. 
/
/    SYSIDFORM is handed a system id typed by a person in STR.  It
/    reformats it according to the standard in routine SYSTEMID and
/    places it back in STR.  */

/*** Copyright (C) 1988 Camber-Roth..  All rights reserved.  */

FUNCTION  sysidform (str)
   char   str[];
{

/*-----------------------Unixes with host/node-names---------------------*/
#if BU | SU | A3 | PU | IX | HU | LA | UL | AX | MV
   strsimplify (str);
#endif

/*-----------------------Unixes w/o system id's--------------------------*/
#if FU | MU | TX
   strcpy (str, "unix");
#endif

/*------------------------SCO Xenix--------------------------------------*/
#if SX | PX
   char  word1[40], word2[40], old[40];

   strtoken (word1, 1, str);
   strtoken (word2, 2, str);
   if (NULLSTR (word2))  strcpy (word2, "386");

   while (strlen (word1) < 9) {
      strcpy (old, word1);
      strcpy (word1, "0");
      strcat (word1, old);
   }
   sprintf (str, "%s %s", word1, word2);
#endif

/*---------------------------------------VAX-----------------------------*/
#if VV
   long  n, vaxtrim();

   sscanf    (str, "%x", &n);
   sprintf   (str, "%08x", vaxtrim(n));
   strlower  (str);
#endif


/*---------------------IBM PC----------------------------------------*/
#if IM | JM | PN
   strcpy (str, "ibmpc");
#endif


/*---------------------PRIME-----------------------------------------*/
#if PR
   strcpy (str, "prime");
#endif


/*---------------------VM/CMS----------------------------------------*/
#if VM
   strcpy (str, "vm/cms");
#endif

   return (1);
}

/*** SYSTIME.  Returns absolute system time (in some constant units,
/    presumably seconds since some fixed date.) as a LONG int. */

/*** Copyright (C) 1983 Camber-Roth.  All rights reserved.  */

FUNCTION  long systime()
{

/*----------------------------------FORTUNE & VAX & MASSCOMP & A3--------*/
#if UNIX | VV | PR | IM | PN | CX | CVM | PX | CA | PS
   long time();

   ENTRY ("systime", "");
   RETURN (time ( (long *) 0) );
#endif

/*--------------------------------AOS/VS (Client/Server)--------------- */
#if CA
   int sec, bisec, min, hour, day, month, year, totaldays;
   long  todaysec;

   ENTRY ("systime", "");
   sys_gday (&day, &month, &year);                /* Compute number of days 
   sys_fday (day, month, year, &totaldays);       /* since 1-JAN-68 
   todaysec = totaldays * 86400;

   sys_gtod (&sec, &min, &hour);              /* Compute number of
   sys_ftod (sec, min, hour, &bisec);         /* biseconds since midnight. 
   RETURN (todaysec + bisec*2); */
#endif
}
