/*** PICUS.   Simple test user interface program for Caucus API.
/
/    Invoked as:
/       picus   caucus_id
/
/    Arguments:
/       caucus_id    full pathname of directory containing Caucus database.
/
/    Purpose:
/       Act as a very simple command-line user interface program to
/       a Caucus database; test the Caucus API functions in doing so.
/
/    How it works:
/
/    Exit status:
/
/    Error Conditions:
/ 
/    Side effects:
/
/    Related functions:
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  zpicus/picus.c
/ */

/*** Copyright (C) 1992 Camber-Roth.  All rights reserved. */

/*: CR 11/15/92 17:00 New program. */
/*: CR 11/16/92 12:47 Add * markings. */
/*: CR 11/27/92 18:24 Add cus primary org. */
/*: CR 11/30/92 22:27 Add all customize stuff. */
/*: CR 11/30/92 23:30 Add temp code for testing cstat. */
/*: CR 12/01/92 12:04 Fix temp code for cstat. */
/*: CR 12/01/92 15:03 Add real option 13. */
/*: JV 12/03/92 13:00 Add show person. */
/*: JV 12/04/92 10:28 Add change person. */
/*: CR 12/04/92 15:47 Add zshowresp(). */
/*: CR 12/06/92 22:23 Add delete item/resp. */
/*: CR 12/07/92 10:33 Remove unused stuff. */
/*: CR 12/08/92 11:58 Change comments. */
/*: CR 12/10/92 20:22 Add Resign, Forget. */
/*: CR 12/13/92 12:29 Add Show SET, Change SET. */
/*: JV 12/14/92 16:13 Add List Members, List People. */
/*: CR  2/22/93 22:27 Remove most code in inconf(). */
/*: JV  3/09/93 09:32 Add File Attachment: GET, DISPLAY */
/*: JV  5/03/93 13:50 Add File Attachment: DELETE */
/*: JV  5/04/93 12:50 Fix Resign code: add rest of args. */
/*: JV  6/01/93 10:04 Repeat join prompt if no conf, rm unused var's. */
/*: JJ  7/22/93 09:43 Don't inconf() on invalid conf name. */
/*: CP  7/27/93 16:56 Make exit's consistent. */
/*: CP  8/01/93 10:50 Add a_mak...() name arg. */
/*: CP  8/18/93 17:48 Remove att_ui_type. */

#include <stdio.h>
#include "chixuse.h"
#include "api.h"
#include "text.h"
#include "caucus.h"

#define P(x)  printf (x)

main (argc, argv)
   int    argc;
   char  *argv[];
{
   char     answer[20];
   int      cnum, ok;
   Initapi  init, a_mak_init();
   Clist    clist, this;
   Person   person;
   Chix     userid;

   if (argc < 2) {
      printf ("usage:  picus  caucus_dir\n");
      exit   (1);
   }

   userid = chxalloc (20L, THIN, "picus userid");

   /*** Initialize the API. */
   init = a_mak_init("picus init");
   chxcpy (init->confid, chxquick(argv[1], 0));
   a_init (init);
   chx_init (ASCII);

   person = a_mak_person("picus person");

   if (parse_attach_map (CQ("attach_map")) < 0) {
      printf ("Couldn't parse attach_map.\n");
      exit   (1);
   }

   /*** If a first-time user, register user. */
   sysuserid (userid);
   chxcpy (person->owner, userid);
   ok = a_get_person (person, P_ALL, A_WAIT);
   switch (ok) {
      case A_OK:       break;
      case A_NOPERSON: zregister ();  break;
      case A_BADARG:   printf ("Bug in picus: arg to a_get_person.\n"); break;
      case A_DBERR:    printf ("Caucus DB error: a_get_person.\n"); break;
      default:         printf ("Unknown problem in picus.\n"); break;
   }

   /*** Allocate a conference list. */
   clist = a_mak_clist("picus clist");

   /*** Conference loop. */
   while (1) {
      printf ("\n");
      if (a_get_clist (clist, P_ALL, A_WAIT) != A_OK)  break;
      printf ("---Conferences------\n");
      for (this=clist->next;   this!=NULL;   this=this->next) {
         printf ("%2d  %s\n", this->cnum, ascquick(this->lname));
      }
      printf ("--------------------\n");

      cnum = zjoin (clist, answer);
      if (cnum < 0) {
         if (streq (answer, "0"))  break;
         else                      continue;
      }

      inconf (cnum);
   }

   printf ("before a_exit\n");
   a_exit();
   printf ("before a_fre_clist\n");
   a_fre_clist (clist);
   printf ("before a_fre_init\n");
   a_fre_init  (init);
   exit (0);
}
