/*** ZATTACH.  Add or display a file attachment.
/
/    zattach (cnum, which);
/
/    Parameters:
/       int cnum;
/           which;     (add = 0, display = 1, delete = 2)
/
/    Purpose:
/       Add an attachment to an existing response or
/       display an existing attachment.
/
/    How it works:
/       
/    Returns:
/
/    Error Conditions:
/  
/    Side effects: 
/
/    Related functions:
/
/    Called by:  picus
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  zpicus/zattach.c
/ */

/*** Copyright (C) 1993 Camber-Roth.  All rights reserved. */

/*: JV  3/09/93 10:11 New function. */
/*: JV  5/03/93 13:45 Add delete. */
/*: CP  8/01/93 10:50 Add a_mak...() name arg. */
/*: CK  8/20/93 14:07 Clean up analyze_file() use, db_name use. */

#include <stdio.h>
#include "chixuse.h"
#include "unitcode.h"
#include "api.h"
#include "xaction.h"
#include "attach.h"
#include "sys.h" 
#include "null.h"

extern union null_t null;

FUNCTION  zattach (cnum, which)
   int    cnum, which;
{
   char     itemstr[8], respstr[8], file[80], com_str[200];
   int      itemnum, respnum, error;
   Chix     date, time, command, filename, filetype;
   Resp     resp;
   Attach_Mapping map;

   if (which < 0 || which > 2)  return;

   printf ("Enter item number: ");
   gets   (itemstr);
   sscanf (itemstr, "%d", &itemnum);
   if (itemnum < 1 || itemnum > 10000) return;

   printf ("Enter resp number: ");
   gets   (respstr);
   sscanf (respstr, "%d", &respnum);
   if (respnum < 0 || respnum > 10000) return;

   resp = a_mak_resp("zattach resp");
   resp->cnum = cnum;
   resp->inum = itemnum;
   resp->rnum = respnum;

   date = chxalloc (16L, THIN, "zattach date");
   time = chxalloc (16L, THIN, "zattach time");
   command  = chxalloc (16L, THIN, "zattach command");
   filename = chxalloc (80L, THIN, "zattach filename");
   filetype = chxalloc (80L, THIN, "zattach filetype");

   if (a_get_resp (resp, P_ALL, A_WAIT) != A_OK) {
      printf ("No such resp.\n"); goto done;      }
   
   chxtoken (date, nullchix, 1, resp->date);
   chxtoken (time, nullchix, 2, resp->date);

   switch (which) {
   case 0:                      /* Add attachment to existing response. */
      printf ("Enter file name: ");
      gets   (file);
      if (strlen (file) < 2) goto done;
   
      resp->attach = make_attach();

      /* Determine type of file. */
      chxofascii (filename, file);
      if ( (resp->attach->format = analyze_file (filename, filetype)) 
               == (Attach_Def) NULL) {
         printf ("Not in recognized format.\n"); goto done;     }
   
      /* Determine size of file. */
      sysfsize (file, &resp->attach->size, &error);

      chxofascii (resp->attach->db_name, file);
      chxofascii (resp->attach->name,    file);
      a_chg_resp (resp, P_ATTACH, A_WAIT);
      break;

   case 1:
      /*** Display an attachment. */
      if (a_get_resp (resp, P_ATTACH, A_WAIT) != A_OK  ||
          resp->attach->size < 1) {
         printf ("No such attachment.\n"); goto done;      }
   
      if (map = ok_to_display_attach (resp->attach)) {
         if (map == (Attach_Mapping) NULL) goto done;
         chxcpy (command, map->command);
         while (chxalter (command, 0L, CQ("$1"), resp->attach->name));
         ascofchx (com_str, command, 0L, ALLCHARS);
         printf ("Launching...\n");
         syscmd   (com_str, SYS_NOWAIT);
      }
      break;
   case 2:
      /*** Delete an attachment. */
      if (a_get_resp (resp, P_ATTACH, A_WAIT) != A_OK) {
         printf ("No such attachment.\n"); goto done;      }

      if (a_del_resp (resp, P_ATTACH, A_WAIT) != A_OK) {
         printf ("Bad database for attachment.\n"); goto done; }
      printf ("Attachment was deleted.\n");
      break;
   }

done:
   a_fre_resp  (resp);
   chxfree (date);
   chxfree (time);
   chxfree (command);
   chxfree (filename);
   chxfree (filetype);
   return;
}
