/*** ZSHOWNEW.   Display new item and response summary.
/
/    zshownew (cnum);
/
/    Parameters:
/       int       cnum;     (conference num)
/
/    Purpose:
/       Display summary of how many new items there are, and which
/       items have new responses.
/
/    How it works:
/
/    Returns:
/
/    Error Conditions:
/  
/    Side effects: 
/
/    Related functions:
/
/    Called by:  picus
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  zpicus/zshownew.c
/ */

/*** Copyright (C) 1992 Camber-Roth.  All rights reserved. */

/*: CR 12/01/92 14:52 New function. */
/*: CR 12/07/92 10:34 Make return's consistent. */
/*: CK  3/01/93 14:50 Correct unseen count. */
/*: CP  8/01/93 10:50 Add a_mak...() name arg. */

#include <stdio.h>
#include "caucus.h"
#include "chixuse.h"
#include "api.h"

FUNCTION  zshownew (cnum)
   int    cnum;
{
   Cstat  cstat;
   Pstat  pstat;
   int    ok, success, unseen, i;

   if (cnum < 0)  return (0);

   cstat = a_mak_cstat("zshownew cstat");
   pstat = a_mak_pstat("zshownew pstat");

   cstat->cnum = cnum;
   ok = a_get_cstat (cstat, P_ALL, A_WAIT);
   if (ok != A_OK) {
      printf ("a_get_cstat failed, code=%d\n", ok);
      DONE   (ok);
   }

   pstat->cnum = cnum;
   ok = a_get_pstat (pstat, P_ALL, A_WAIT);
   if (ok != A_OK) {
      printf ("a_get_pstat failed, code=%d\n", ok);
      DONE   (ok);
   }

   /*** Tell about new items. */
   if (pstat->items < cstat->items)
      printf ("New items: %d-%d\n", pstat->items+1, cstat->items);
   else
      printf ("New items: (none)\n");

   /*** Tell about Unseen items. */
   printf ("Unseen items: \n  ");
   unseen = 0;
   for (i=1;   i<=cstat->items;   ++i)
      if (pstat->resps[i] == -1) { printf ("%4d", i);   unseen = 1; }
   if (NOT unseen)  printf ("(none)");
   printf ("\n");

   /*** Tell about new responses. */
   printf ("New responses:\n");
   for (i=1;   i<=pstat->items;   ++i) {
      if (pstat->resps[i] >= 0  &&  cstat->resps[i] > pstat->resps[i]) {
         printf ("  %3d: new resps %d-%d\n", i, pstat->resps[i]+1, 
                                                cstat->resps[i]);
      }
   }

   DONE (A_OK);

done:
   a_fre_cstat (cstat);
   a_fre_pstat (pstat);
   return (success);
}
